#
#  The contents of this file are subject to the terms of the
#  Common Development and Distribution License, Version 1.0 only
#  (the "License").  You may not use this file except in compliance
#  with the License.
# 
#  You can obtain a copy of the license at 
#  http://www.opensource.org/licenses/cddl1.php
# 
#  When distributing Covered Code, include this CDDL HEADER in each
#  file and include the original License file at src/license.txt.
#  If applicable, add the following below this CDDL HEADER, with the
#  fields enclosed by brackets "[ ]" replaced with your own identifying
#  information: Portions Copyright [yyyy] [name of copyright owner]
# 

#  Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
#  Use is subject to license terms.
#  Initial Developer updates, news, and information may be found at 
#  http://sourceforge.net/projects/likewiseopenagt
#

# Eventlog Helper package (c) 2005 Centeris Corporation.
#
package Centeris::EventLogs;
use Exporter;
use Symbol;
use Time::Local;
use Data::Dumper;
@ISA = ("Exporter");
@EXPORT=qw($VERSION &LoadRulesFromFHM &SetEventlogLogfile $EVENTLOG_LOGFILE $EVENTLOG_WRITER_PGM &ProcessEventlogCmd &OpenEventWriter &OpenEventlogDB &MakeEventLogEntryString &AddEventlogDBEntry &AddEventlogEntryFromHash &CloseEventlogDB &ProcessLogLine &ProcessSMBDLine &ProcessSSHDLine &ProcessSULine &ProcessSYSLOGDLine &ProcessCronLine &SetEventlogWriter &InitSources);

#use strict ;

my $EVENTLOG_LOGFILE='/dev/null';

#
# Note --- there are some print commands that seem innocuous (e.g. "SUCCESS")
#          but are nonetheless critical  -- the code that calls some of these
#          functions expect those strings on stdout as "return values".
#
#          Please be ultra careful about tidying up those usages.
#

# file names, initialization

my $VERSION = 1.0;

my $test_mode = Symbol::qualify('$test_mode',(caller())[0]);

my $EVENTLOG_WRITER_PGM = "";

my $g_retention;
my $g_maxsize;
my %evtwriterFH;                # hash of the open handles

# 
#  starting values for message type IDs...
# 
my $sshdbase =           100;
my $subase   =           130;
my $cronbase =           200;
my $smbdbase =           300;
my $syslogdbase =        900;
my $defaultbase =       1000; # untranslated events...

my %months = (
           'Jan'=>0,
	   'Feb'=>1,
	   'Mar'=>2,
	   'Apr'=>3,
	   'May'=>4,
	   'Jun'=>5,
	   'Jul'=>6,
	   'Aug'=>7,
	   'Sep'=>8,
	   'Oct'=>9,
	   'Nov'=>10,
	   'Dec'=>11 );


#
# Auto-add source names for pretty results...
#
sub AutoAddSource ($$) {
    my ($evtlogname,$sourcename) = @_;
    if (!defined($EVENTLOG_WRITER_PGM) || ("" eq $EVENTLOG_WRITER_PGM)) {
	printf("Eventlog Writer program is not defined\n");
	return undef;
	}     
    # init them

    if (!defined(${$evtlogname}{$sourcename})) {
	# add it in 
	print "Adding a sourcename of $sourcename to $evtlogname\n"; 
        system ($EVENTLOG_WRITER_PGM,"-o","addsource",$evtlogname,
		$sourcename,"C:\\windows\\system32\\likewisemsg.dll");
	${$evtlogname}{$sourcename}++;
    } else {
	print "Source $sourcename already in $evtlogname.\n";
    }
}

#
# Initialize our sources in the eventlog registry...
#
sub InitSources () {
    if (!defined($EVENTLOG_WRITER_PGM) || ("" eq $EVENTLOG_WRITER_PGM)) {
	printf("Eventlog Writer program is not defined\n");
	return undef;
	}     
    # init them
    my @wrk;
    my @sources = ( ("System","Cron Service","C:\\windows\\system32\\likewisemsg.dll"),
		    ("System","File Server","C:\\Windows\\system32\\likewisemsg.dll"),
		    ("System","System Logger","C:\\Windows\\system32\\likewisemsg.dll"),
		    ("Security","Secure Shell Login","C:\\Windows\\system32\\likewisemsg.dll"),
		    ("Security","Switch User","C:\\Windows\\system32\\likewisemsg.dll"),
		    ("Security","Secure Shell Login","C:\\Windows\\system32\\likewisemsg.dll"),
		    ("Security","Switch User","C:\\Windows\\system32\\likewisemsg.dll")
		   );

    while (@sources) {
	(@wrk) = splice (@sources,0,3);
	print "Adding source ",$wrk[1]," to eventlog ",$wrk[0],"\n";
	# AutoAddSource($wrk[1],$wrk[0]);
        system ($EVENTLOG_WRITER_PGM,"-o","addsource",$wrk[0],$wrk[1],$wrk[2]);
    }
}




# A nice housekeeping function
#
#
sub LoadRulesFromFHM($*) {

    my ($FH,$mary) = @_ ;
    my @mm;
    my $rh;
    my $mod;
    my $caller = caller();

while (<$FH>) {
        chomp;
	@mm=split ',\s*'; 

	$mm[0]=~ s/^\"(.*)\"$/\1/;

	#get the unqualified symbols passed referenced in the caller frame 
        # but leave qualified symbols alone
        $mm[0] =~ s/\$(\w+)([^\:\w])/\$$caller\:\:$1$2/g;
	$mm[0] =~ s/\$((\w+\:\:)*\w+)/${$1}/g;

	$mm[1]=~ s/^\"(.*)\"$/\1/;
	$mm[1] =~ s/\$(\w+)/${$1}/g;

	$mm[3]=~ s/^\"(.*)\"$/\1/;
	$mm[3] =~ s/\$(\w+)/${$1}/g;
	$rh = { MATCH_PATTERN=>$mm[0], MATCH_ACTION=>$mm[1], MATCH_CONTINUE=>$mm[2], MATCH_COMMENT=>$mm[3] } ;
	if ($test_mode > 0) {
	   printf "Read [%s] [%s] [%s] [%s] \n",$mm[0],$mm[1],$mm[2],$mm[3];
        }
	if ( $$rh{MATCH_ACTION}=~ /((\w+\:\:)*(\w+))\:\:(\w)/ ) {
	    # must load this module
	    $mod = $1;
	    if (eval "require $mod") {
		$mod->import();
		# printf("$mod imported\n");
	    }
        }
        push @$mary,$rh;
    }
}
#
sub StringToTime($) {
   my ($line) = @_;
   my $mon;
   my $tmgm;
   my $nowyear;
   my @nowtime=localtime(time);
   if ( $line =~ /^(\w+)\s+(\d+)\s+(\d+)\:(\d+)\:(\d+)\s/ ) {
       $mon = $months{$1};    
       $tmgm=timelocal($5,$4,$3,$2,$mon,$nowtime[5]+1900);
       return $tmgm;
   } else {
       return undef;
   }
}


sub StringToEventHash ($){	
    my ($line) = @_;
    my $outhash;
    my $mon;
    my $tmgm;
    my @rol;
    my @str;
    my $rc; 
    my $eid;
    chomp $line;
    if ( $line =~ /^(\w+)\s+(\d+)\s+(\d+)\:(\d+)\:(\d+)\s+(\S+)\s+(.*)/ )
    {
	# get the time value
	$tmgm=StringToTime($line);
	$eid = 103;
	@rol = split (' ',$7,2);
	$str[0]=substr($rol[1],0,239);
	$outhash = { TMG=>"$tmgm", TMW=>"$tmgm", EID=>"$eid", ETP=>'INFO',USL=>0,SRC=>"$rol[0]",  SRN=>"$6", STR=>\@str, DAT=>'' };
	return $outhash;
    } else {
	return undef;
    }
}


sub pretty_sourcename(*) {
    my ($psn) = @_;
    $psn =~ s/(.*)\[\d+\]/\1/;
    $psn =~ s/\[([\w\/]+)\](\[\d+\])?/\1/; # [foo]* -> foo

    return $psn;
}
# pass in an entry and format it into a string, return the string
#
sub FmtSyslogEntry ($) {	
    my ($line) = @_;
    my $outstr;
    my $sourcename;
    my $mon;
    my $tmgm;
    my $oline;
    my @rol;
    my $rc; 
    my @nowtime=localtime(time);
    chomp $line;
    $oline='';
    if ( $line =~ /^(\w+)\s+(\d+)\s+(\d+)\:(\d+)\:(\d+)\s+(\S+)\s+(.*)/ )
    {
    $mon = $months{$1};    
    $tmgm=timegm($5,$4,$3,$2,$mon,$nowtime[5]+1900);
    @rol = split (' ',$7);
    $oline .= sprintf "LEN: 0\nRS1: 1699505740\nRCN: $rc\n";
    $oline .= sprintf "TMG: $tmgm\n";
    $oline .= sprintf "TMW: $tmgm\n";
    $oline .= sprintf "EID: 103\n";
    $oline .= sprintf "ETP: INFO\n";
    $oline .= sprintf "ECT: 0\n";
    $oline .= sprintf "RS2: 0\n";
    $oline .= sprintf "CRN: 0\n";
    $oline .= sprintf "USL: 0\n";
    $sourcename = $rol[0];
    # coerce the source name into something readable
    $sourcename = pretty_sourcename($sourcename);
    $oline .= sprintf ("SRC: %s\n",$sourcename); # 
    $oline .= sprintf "SRN: $6\n";
    if (length($rol[1])) { $oline .= sprintf "STR: %s\n",substr($rol[1],0,239); }
    if (length($rol[2])) { $oline .= sprintf "STR: %s\n",substr($rol[2],0,239); }
    $oline .= sprintf "DAT: \n";
    } else {
    # printf ("Can't understand line $line\n");
	$oline='';
     }
    return $oline;
}


#
# take a line and put it into an eventlog entry
#
sub ProcessLogLine($$) {
    my ($inline,$rule) = @_;
    my $orec;
    my $source;

    my $strref;

    if($test_mode)
    {
	print "ProcessLogLine: ",$inline,"\n";
    }
    $orec=StringToEventHash($inline);

    $strref = ${$orec}{'STR'};
    $source= ${$orec}{'SRC'};

    ${$orec}{'EID'} = $defaultbase;
    ${$orec}{'AUTOEID'} = 1;       # bug 1037 

    if ($source =~ s/(.*)\:/$1/ ) { # remove trailing colon
	${$orec}{'SRC'} = $1;
    }

    # find process id in the name
    if ($source =~ s/(.*)\[(\d+)\]\:?/$1/ ) {
       push @{$strref},"Process Id: $2";
       ${$orec}{'SRC'} = $source;
    }

    $source = pretty_sourcename($source);
    ${$orec}{'SRC'} = $source;

    if (defined($orec)) {
	AddEventlogEntryFromHash("SyslogLinux",$orec);
    }

    if($test_mode){
	print Dumper($orec),"\n";
    }
}
#
# /usr/sbin/cron lines
# 
sub ProcessCronLine($$@) {
    my ($inline,$rule,@matches) = @_;
    my $orec;
    my $strref;
    my $jobid;
    my $username;
    my $cmdline;
    if($test_mode){
	print "ProcessCronLine\n";
    }


    $orec=StringToEventHash($inline);

    ${$orec}{'EID'} = $cronbase;
    #
    # fills in default values -- so fix up what we need to.
    # print "SRC is ",${$orec}{'SRC'},"\n";
    ${$orec}{'SRC'} = "Cron Service";
    if ($inline =~ /\[(\d+)\]/) { $jobid = $1; }
    if ($inline =~ /\((.+)\) CMD/) { $username = $1; }
    if ($inline =~ /\CMD \((.*)\)/ ) { $cmdline = $1; }
    $strref = ${$orec}{'STR'};
    $$strref[0] = "Job ID : " . $jobid;;
    $$strref[1] = "User Name : ".$username;
    $$strref[2] = "Command : ".$cmdline;
    if (defined($orec)) {
	AddEventlogEntryFromHash("System",$orec);
    }
    if($test_mode){
	print "******************",Dumper($orec),"\n";
    }
}
#
# smbd lines
#
sub ProcessSMBDLine($$@) {
    my ($inline,$rule,@matches) = @_;
    my $orec;
    my $strref;
    my $jobid;
    my $username;
    my $cmdline;
    print "ProcessSMBDLine\n";
    $orec=StringToEventHash($inline);
    #
    # fills in default values -- so fix up what we need to.
    #

    ${$orec}{'EID'} = $smbdbase;

    if ($inline=~/\Werror\W/i ) {
	${$orec}{'ETP'} = 'ERROR';
    }
    ${$orec}{'SRC'} = "File Server";
    if ($inline =~ /smbd\[(\d+)\]:(.*)/) { $jobid = $1; $cmdline=$2; }
    $strref = ${$orec}{'STR'};
    $$strref[0] = "Process ID : " . $jobid;;
    $$strref[1] = "Event Information : ".$cmdline;
    if (defined($orec)) {
	AddEventlogEntryFromHash("System",$orec);
    }
    if($test_mode){
	print "**",Dumper($orec),"\n";
    }
}
sub ProcessSYSLOGDLine($$@) {
    my ($inline,$rule,@matches) = @_;
    my $orec;
    my $strref;
    my $jobid;
    my $username;
    my $cmdline;

    if($test_mode){
	print "ProcessSYSLOGDLine\n";
    }
    $orec=StringToEventHash($inline);
    
    ${$orec}{'EID'} = $syslogdbase;
    #
    # fills in default values -- so fix up what we need to.
    # print "SRC is ",${$orec}{'SRC'},"\n";
    if ($inline=~/\Werror\W/i ) {
	${$orec}{'ETP'} = 'ERROR';
    }
    ${$orec}{'SRC'} = "System Logger";
    if ($inline =~ /syslogd (\d+\.\d+\.\d+)\:(.*)/) { $jobid = $1; $cmdline=$2; }
    $strref = ${$orec}{'STR'};
    $$strref[0] = "System Logger Version " . $jobid;;
    $$strref[1] = $cmdline;
    if (defined($orec)) {
	AddEventlogEntryFromHash("System",$orec);
    }
    if($test_mode){
	print "ProcessSYSLOGDLine",Dumper($orec),"\n";
    }
}


#
# ssh lines
#

sub ProcessSSHDLine($$@) {
    my ($inline,$rule,@matches) = @_;
    my $orec;
    my $strref;
    my $jobid;
    my $username;
    my $cmdline;

    if($test_mode){
	print "ProcessSSHDLine\n";
    }

    $orec=StringToEventHash($inline);
    ${$orec}{'SRC'} = "Secure Shell Login";
    #
    # fills in default values -- so fix up what we need to.
    # print "SRC is ",${$orec}{'SRC'},"\n";

    # change the text and event types if necessary 
    
    $strref = ${$orec}{'STR'};

     if ($inline =~ /\Werror\W/) {
	 	${$orec}{'ETP'} = 'ERROR';
     }

    #Feb 24 15:46:06 vmsuse1 sshd[19782]: Invalid user rot from ::ffff:10.100.1.247


    if ($inline=~/\Werror\W/i ) {
	${$orec}{'ETP'} = 'ERROR';
    } 
	${$orec}{'EID'} = $sshdbase;
 MSGSWITCH:{
    if ($inline=~/\WInvalid user (\S+) from (.*)(\d+\.\d+\.\d+\.\d+)/i ) {
	${$orec}{'ETP'} = 'ERROR';
	${$orec}{'EID'} = 1 + $sshdbase;
        $$strref[0] = "Login attempt from $3 failed: Invalid user";
        $$strref[1] = $1;
        last MSGSWITCH;
    }

    #Feb 24 15:46:07 vmsuse1 sshd[19782]: Failed unknown for invalid user rot from ::ffff:10.100.1.247 port 4148 ssh2
    if ($inline=~/\WFailed (\S+) for invalid user (\S+) from (.*)\:(\d+\.\d+\.\d+\.\d+) (.*) /i ) {        
	${$orec}{'ETP'} = 'ERROR';
	${$orec}{'EID'} = 2+ $sshdbase;
        $$strref[0] = "Login attempt from $4 $5 failed: Invalid user ";
        $$strref[1] = $1;
        last MSGSWITCH;
    }

    #Feb 24 13:08:31 vmsuse1 sshd[18173]: Accepted keyboard-interactive/pam for root from ::ffff:10.100.1.110 port 33036 ssh2
    if ($inline=~/\WAccepted keyboard-interactive\/pam for (\S+) from (.*)\:(\d+\.\d+\.\d+\.\d+) (.*) /i ) {        
	${$orec}{'ETP'} = 'WARNING';
	${$orec}{'EID'} = 3+ $sshdbase;
        $$strref[0] = "Successful login from  $3 $4 username";
        $$strref[1] = $1;
        last MSGSWITCH;
    }
    #Mar 15 10:41:19 vmsuse1 sshd[31011]: Address 10.100.1.115 maps to bmoran_v1.corp.centeris.com, but this does not map back to the address - POSSIBLE BREAKIN ATTEMPT!
    if ($inline=~/\WAddress (\S+) maps to (\S+), but this does not map back to the address/i ) {        
	${$orec}{'EID'} = 4+ $sshdbase;
	${$orec}{'ETP'} = 'WARNING';
        $$strref[0] = "Can't match address $2 to $1, though $1 maps to $2";
        $$strref[1] = 'Possible break-in attempt';
        last MSGSWITCH;
    }
 }
    if (defined($orec)) {
	AddEventlogEntryFromHash("Security",$orec);
    }
    if($test_mode){
	print "ProcessSSHDLine",Dumper($orec),"\n";
    }
}


sub ProcessSULine($$@) {
    my ($inline,$rule,@matches) = @_;
    my $orec;
    my $strref;
    my $jobid;
    my $username;
    my $cmdline;

    if($test_mode){
	print "ProcessSULine\n";
    }

    $orec=StringToEventHash($inline);
    ${$orec}{'SRC'} = "Switch User";
    ${$orec}{'ETP'} = 'INFO';
    #
    # fills in default values -- so fix up what we need to.
    # print "SRC is ",${$orec}{'SRC'},"\n";

    # change the text and event types if necessary 
    
    $strref = ${$orec}{'STR'};

     if ($inline =~ /\Werror\W/) {
	 	${$orec}{'ETP'} = 'ERROR';
     }
    if ($inline=~/\s+\(to\s+(\S+)\)\s+(\S+)\s+on/i ) {
	${$orec}{'ETP'} = 'INFO';
        $$strref[0] = "User $1 changed to $2";	
    } 
    ${$orec}{'EID'} = $subase;

    if (defined($orec)) {
	AddEventlogEntryFromHash("Security",$orec);
    }
    if($test_mode){
	print "ProcessSULine",Dumper($orec),"\n";
    }
}
#
# open a file handle to the writer - returns a file handle that is already 
# opened or one that is created.
#

sub OpenEventWriter ($) {
    my $evtlogname = shift @_;

    printf("OpenEventWriter: Opening $evtlogname\n");
    print Dumper(%evtwriterFH);
    if (defined($evtwriterFH{$evtlogname})) {
	print "Found cached, returning \n";
	print Dumper(%evtwriterFH);
	return $evtwriterFH{$evtlogname};
    }

    if (!defined($EVENTLOG_WRITER_PGM) || ("" eq $EVENTLOG_WRITER_PGM)) {
	printf("Eventlog Writer for $evtlogname not defined\n");
	return undef;
	} 

    if (!open (FHANDLE,'|-',"$EVENTLOG_WRITER_PGM","$evtlogname")) {
	printf("Can't open eventlog writer for $evtlogname\n");
	return undef;
	} 
    # printf(" cmd is $EVENTLOG_WRITER_PGM $evtlogname\n");
    $evtwriterFH{$evtlogname} = *FHANDLE;
    # print Dumper(%evtwriterFH);
    return *FHANDLE;
}
		     
sub SetEventlogLogfile ($) {
    $EVENTLOG_LOGFILE = shift @_;
}
sub SetEventlogWriter ($) {
    $EVENTLOG_WRITER_PGM = shift @_;
}

sub OpenEventlogDB($) {
	my ($evtlogname) = @_;
	return OpenEventWriter($evtlogname);
}

#
# close the event log DB, given the hash.
#
sub CloseEventlogDB($) {
	# should untie/undef the array, get rid of the hash...
	my ($dbhandle) = @_;
	my $i;
	foreach $i (keys %evtwriterFH) {
	    if ($evtwriterFH{$i} == $dbhandle) {
		printf("Closing $i\n");
		close( $evtwriterFH{$i});
		delete $evtwriterFH{$i};
		return;
	    }
	}
	#printf("CloseEventlogDB: Handle not found\n");
}

sub CloseEventlog($) {
	# should untie/undef the array, get rid of the hash...
	my ($db) = @_;
	if (defined($db)) {
	    close( $evtwriterFH{$db});
	    undef ($evtwriterFH{$db});
	}
}

# 
# Functions used to construct and add an eventlog entry. Use Hashes for the write;
# don't use hashes for the read, since we want to go fast.
#
# given a hash, make a nice string out of it that we can pass to AddEventLogEntryDB.
# Note that we store it in the format that we're going to give it back...
# note also that the RCN (record number) field is NOT derived here, but from the DB, and at the time it's added.
# 
#
sub MakeEventLogEntryString($) {
	my ($evthash) = @_;
	my $reqd_fields="TMG TMW EID ETP USL SRC SRN STR DAT";
	my $field;


    # verify we have all that we need
	if (!defined($evthash)) { return undef; }
	foreach $field (split ' ',$reqd_fields) {
		if (!defined($$evthash{$field})) {
			return undef; #printf("Field $field must be included\n");
		}
	}
    # BUG 1037 fix - adjust event id to have number of strings
    #                if it has the AUTOEID field set to 1...
    #         

	if ($$evthash{AUTOEID} == 1) {
	    my $numstrs = $#{$$evthash{STR}};
	    if ($numstrs > 0) {
		$$evthash{EID} += $numstrs;
	    }
	}
	    

    #
    # write it to the string like time_gen time_writ evt_id evt_type src srn str[] dat
	my $outstr = sprintf("TMG: %s\nTMW: %s\nEID: %s\nETP: %s\nECT: 0\nRS2: 0\nCRN: 0\nUSL: %s\nSRC: %s\nSRN: %s\n",
						  $$evthash{TMG},$$evthash{TMW},$$evthash{EID},$$evthash{ETP},$$evthash{USL},$$evthash{SRC},$$evthash{SRN});
	my $strstr = "STR: ";
	$strstr .= join "\nSTR: ",@{$$evthash{STR}};
	$outstr .= $strstr."\n";
	$outstr .= sprintf("DAT: %s\n",$$evthash{DAT});
	return $outstr;
}

# 
#  
#
sub AddEventlogDBEntry($$) {
	my ($dbhandle,$toadd) = @_;
	my $nxtrecno;

	# 
	$nxtrecno = 0;
	# print "In AddEventlogDBEntry with", Dumper($dbhandle),"\n$toadd";
	if (defined($dbhandle) && defined($toadd) && (length($toadd) > 0)) {
	  printf { $dbhandle } "LEN: 0\nRS1: 1699505740\nRCN: %d\n%s\n",$nxtrecno,$toadd;
	  return 1; 
	}	  		
	return undef;
}

#
# Given the eventlog name and a hash, add a record to the eventlog
#
sub AddEventlogEntryFromHash($$) {

	my ($logname,$hashentry) = @_;
	my $retcode;

	if (ref($hashentry) ne 'HASH') { return 0; }

	# if auto-adding sources...
	AutoAddSource($logname,$$hashentry{SRC});
	#
  	my $addstr=MakeEventLogEntryString($hashentry);
	if (defined($addstr)) {
		my $heventlog = OpenEventlogDB($logname);
		if (defined($heventlog)) {
		    $retcode= AddEventlogDBEntry($heventlog,$addstr);
		    } else {
			print "AddEventlogEntryFromHash: can't write to handle\n";
		    }
		CloseEventlogDB($heventlog);
		return $retcode;
	    }
	return 0;
}

#
# Process the commands
#

sub ProcessEventlogCmd{
    my ($op,$logname,$handlestr,$arg1,$arg2,$arg3,$arg4,$arg5,$arg6) = @_;
   
    if (defined($EVENTLOG_LOGFILE)) {
         print $EVENTLOG_LOGFILE "ProcessEventlogCmd: At top [$op] [$logname] [$handlestr] [$arg1] [$arg2] [$arg3]\n";
     }
    #
    my $valid_cmds = "TestEventlogEntry";

    if (!($valid_cmds =~ /$op/)) {
		print "Unknown Command $op\n";
		return undef;
    }


    #
    # A test function - not called in normal course of eventlogs
    #
    if ($op eq "TestEventLogentry") {
	my @foo=('str1','str2','str3');
	my $foox = OpenEventlogDB("Application");
    	my $addstr=MakeEventLogEntryString({TMG=>time(),TMW=>time(), EID=>100,
					    ETP=>'INFO', USL=>0, SRC=>'smbd',
					    SRN=>'vmsuse1',STR=>\@foo,DAT=>''});	print "Adding an eventlog entry returns ",AddEventlogDBEntry($foox,$addstr),"\n";
	CloseEventlogDB($foox);
    }
}
1 ;

