# 
#
#  The contents of this file are subject to the terms of the
#  Common Development and Distribution License, Version 1.0 only
#  (the "License").  You may not use this file except in compliance
#  with the License.
# 
#  You can obtain a copy of the license at 
#  http://www.opensource.org/licenses/cddl1.php
# 
#  When distributing Covered Code, include this CDDL HEADER in each
#  file and include the original License file at src/license.txt.
#  If applicable, add the following below this CDDL HEADER, with the
#  fields enclosed by brackets "[ ]" replaced with your own identifying
#  information: Portions Copyright [yyyy] [name of copyright owner]
# 

#  Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
#  Use is subject to license terms.
#  Initial Developer updates, news, and information may be found at 
#  http://sourceforge.net/projects/likewiseopenagt



use Centeris::EventLogs;
use Data::Dumper;
use IO::Seekable;

# use strict;

no strict refs;

$DATE_MATCH = "^(\\w+)\\s+(\\d+)\\s+(\\d+)\:(\\d+)\:(\\d+)\\s+" ;  # match a date entry in Syslog
$MNAME_MATCH= "(\\S+)";                                    # match a machine's name in Syslog
$PREAMBLE_MATCH=$DATE_MATCH.$MNAME_MATCH;
$PREAMBLE_MATCH=~s/[\(\)]//g;
$EVENTLOG_WRITER_PGM = '/usr/bin/eventlogadm';

my $logfilename ="/var/log/messages";

my @MATCH_RULES;
#
#
# syntax of this program is 
# perl evtsyslog.pl <pidfile> <debug/init_flag> <debug_log> <ruleset1> .. <rulesetN>
#
# where debug/install_flag is 0 or 1 for debug, or 'init' to add in a list of sources from InitSource
# where debug_log is /dev/null for no log, or filename to preserve a log
# where ruleset1..N are additional match rules to use. Note that the rulesets may call in 
#       additional other modules, if the routine is specified in the form of Module::Method.
#       Foo:ProcessLogLine will cause "require foo" to be executed.
#
# ******* Note that the default mode is now to 'auto add' source names and appropriate event log files.


SetEventlogWriter($EVENTLOG_WRITER_PGM);

 my $file;
 my $line;
 my $nlines = "-n 1";
 my $rule;
 my %r;

 # pull our pidfile name from our first argument

 $pidfile = $ARGV[0];
 my $output_log = '/dev/null';

 if (defined($ARGV[1])) {
     $test_mode = $ARGV[1];
     # print "Test mode $test_mode\n";
     if ($test_mode eq "init") {
	 # note that the EVENTLOG_WRITER_PGM has been set by now...
	 InitSources();
	 exit 0;
     }
     $output_log = $ARGV[2] if defined($ARGV[2]);
 }

 

  if ($test_mode > 0 ) {
	print "Loading internal rules\n";
	}
  LoadRulesFromFHM(*main::DATA,\@MATCH_RULES);

#
# Read additional rules, with potential use of other libraries
#
if ($#ARGV > 2) {
    $i = 3;
    while ($i < ($#ARGV+1)) {
	open(FH, "<", $ARGV[$i]) or die "Can't open ruleset " . $ARGV[$i] . "\n";
	print "Loading rules from ",$ARGV[$i],"\n";
	LoadRulesFromFH(*FH,\@MATCH_RULES);
	$i++;
     }
 }

 # Setup our signal handler so that when we are killed we close
 # the messages file, remove our pidfile and return 0 to the OS

 $SIG{'KILL'} = $SIG{'QUIT'} = $SIG{'INT'} = $SIG{'TERM'} = 
     sub { close FH_MSGS; unlink $pidfile; exit 0; };

 # help our children to easily depart this mortal coil

 $SIG{'CHLD'} = 'IGNORE';

 # write our pidfile so that our calling init.d script knows how to find our proc

 # note that we open the pidfile *before* the fork, but we dont actually *write*
 # in the pid until *after* we have forked, else we get the pid of the parent.
 # we dont care about the parent's pid because it's about to die, we want the
 # child's pid!

 open FH_PID, ">$pidfile" or die " Error - Pidfile Create: $pidfile $!\n";

 # Don't fork if we're running with test mode enabled
 # Only assume we're detaching from std[in|out|err] if we're not running
 # in test mode. Otherwise assume we're being run with debugging output. 
 if($test_mode == 0)
 {
 # use a fork to allow our perl script to be started as a daemon, prior
 # implementations attempted to background the process using '&', but
 # that makes it impossible to detect nonzero return codes (ie: errors)

     die " Error - Unable To Fork: $! Exiting" if(!defined($pidC = fork()));
     exit(0) if(0!=$pidC);

 # here's that post fork pid writing mentioned in the 'pidfile open' comment

     print FH_PID $$;
     close FH_PID;

     open $devnull, '+>', '/dev/null';
     close(STDOUT);
     close(STDIN);
     close(STDERR);

     open STDOUT, '+>', $output_log;
     open STDIN, '+>', '/dev/null';
     open STDERR, '+>', $output_log;
 }

 # implement a pseudo-tail (Christiansen,Torkington '98 p283). benefit
 # of doing this instead of driving stdout from the file into the
 # script via tail(1) is that in the event of a script error we dont
 # linger on in tail and hang the caller.

 print STDOUT scalar localtime(), " - Starting \n",;

 # delayed open of /var/log/messages, bug 1237
 # need to wait for log file after forking or else service startup will hang
 while( ! -s $logfilename ){ sleep 3; }
 open(FH_MSGS, $logfilename) or die "Cannot open log file: $logfilename $!";
 seek(FH_MSGS,-1,2) or die "Cannot move to end of file: $logfilename $!";

 # print "Opened $logfilename\n";

 for (;;)   
 {
     for ($curpos = tell(FH_MSGS); $_=<FH_MSGS>;
	  $curpos = tell(FH_MSGS)) {
	 $line = $_;
	 chomp($line);
	 if($line =~ /^$/) { next; }
	 my ($MONTH,$DAY,$TIME,$HOST,$PGM,$ROL) = split " ",$line,6;
	 foreach $rule (@MATCH_RULES) {
	     %r= %{$rule};
	     # print Dumper(%r);
	     if ((@MMATCHES) = ($line =~ $r{MATCH_PATTERN})) {
		 if($test_mode)
		 {
		     print STDOUT  "Line ",$line," matched ",$r{MATCH_PATTERN}," ",$r{MATCH_COMMENT},"\n";
		     print STDOUT $1,"\n";
		 }
		 if ($test_mode == 0) {
		     eval { $r{MATCH_ACTION}($line,$r,@MMATCHES); }
		 } else {
		     printf("TEST MODE: calling %s with %s\n",$r{MATCH_ACTION},join " ",@MMATCHES);
		     printf("TEST MODE: Match action is [%s]\n",$r{MATCH_ACTION});
		     { $r{MATCH_ACTION}($line,$r,@MMATCHES); }
		 }
		 last if (!$r{MATCH_CONTINUE});
	     } else {
		 if ($test_mode) {
		     printf "Line [%s] did not match %s\n",$line,$r{MATCH_PATTERN};
		 }
	     }   
	 }
     }
     seek(FH_MSGS,$curpos,0); # same as FH_MSGS->clearerr();
     sleep 3;

     # bmoran points out that we need to follow the file name and not the filehandle
     # because syslog will eventually close the current file and reopen a new one.
     # doing a quick strace on tail, it's obvious that it just sleeps and closes
     # and sleeps and reopens the file so we need to do the same. 
     # a 9 sec (3sec sleep * 3 loop iters) cycle seems adequate absent complaints.

     ($dev,$ofi) = stat FH_MSGS;
     ($dev,$cfi) = stat $logfilename;

     # if inodes are not the same, then the file must have been rotated
     if($ofi!=$cfi)
     {
	 # close the old file, reopen the correct file name, might be the same file
	 # and then seek to the end of the file.

	 close FH_MSGS;
    
	 # make sure that the file exists and has data before attempting open
	 while( ! -s $logfilename ){ sleep 3; }
	 open(FH_MSGS, $logfilename) or die " Cannot Reopen Log File: $logfilename $!";
     }
 }

 exit 0;
 __END__
"$PREAMBLE_MATCH\s(smbd)", "ProcessSMBDLine", 1, "SMBD entries"
"$PREAMBLE_MATCH\s(sshd)", "ProcessSSHDLine", 1, "ssh entries"
"$PREAMBLE_MATCH\s(su)", "ProcessSULine", 1, "ssh entries",
"$PREAMBLE_MATCH\s(syslogd)", "ProcessSYSLOGDLine", 1, "syslogd entries"
"$PREAMBLE_MATCH\s((/\\w+)*)/(cron)", "ProcessCronLine", 1, "Pickup Cron jobs"
"$PREAMBLE_MATCH\s((/\\w+)*)/(cron)\[\d+\]", "ProcessCronLine",1, "Pickup Cron jobs"
"$PREAMBLE_MATCH\s+crond(\\S+)[\\d+]","ProcessCronLine", 1, "Pickup Cron jobs"
"$PREAMBLE_MATCH\s((/\\w+)*)/(CRON)",  "ProcessCronLine", 1, "Pickup Cron jobs"
"(.*)", "ProcessLogLine", 0, "Default rule to add to LinuxSyslog"
