#! /usr/bin/perl -w
# 
#
#  The contents of this file are subject to the terms of the
#  Common Development and Distribution License, Version 1.0 only
#  (the "License").  You may not use this file except in compliance
#  with the License.
# 
#  You can obtain a copy of the license at 
#  http://www.opensource.org/licenses/cddl1.php
# 
#  When distributing Covered Code, include this CDDL HEADER in each
#  file and include the original License file at src/license.txt.
#  If applicable, add the following below this CDDL HEADER, with the
#  fields enclosed by brackets "[ ]" replaced with your own identifying
#  information: Portions Copyright [yyyy] [name of copyright owner]
# 

#  Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
#  Use is subject to license terms.
#  Initial Developer updates, news, and information may be found at 
#  http://sourceforge.net/projects/likewiseopenagt
#


# use Data::Dumper;
use diagnostics;
# the dtype hash has the devices that we're interested in
# convert the devices we want into XML output. Note the keys should all be UPPERCASE
#
%dtype = (CPU=>0,
	  DISK=>1,
	  "SCSI"=>1,
	  "CD-ROM"=>2,
	  "CD-ROM (CD-RW)"=>2,
	  BIOS=>3,
	  "PARALLEL CONTROLLER (SPP)"=>4,
          "PARALLEL CONTROLLER (ECP)"=>4,
          "PARALLEL CONTROLLER (EPP)"=>4,
	  "MAIN MEMORY"=>5,
	  "VGA COMPATIBLE CONTROLLER (VGA)" =>6,
	  "USB CONTROLLER (UHCI)"=>7,
	  "IDE INTERFACE"=>8,
	  "ETHERNET CONTROLLER"=>9,
	  "LOOPBACK"=>9,
	  "NETWORK INTERFACE"=>9,
	  "HUB"=>7,
	  "KEYBOARD"=>10,
	  "PS/2 MOUSE"=>11);

%ptrans = (Model=>"Name");

# only put entries in here that need to be translated.
#%dname = ();
#print Dumper(%dtype);

# BUGBUG - hwinfo only comes on suse. we need to distribute our own and stick it in /opt/centeris/bin
$hwinfo = "/usr/sbin/hwinfo --cdrom --printer --bios --usb --netcard --mouse --keyboard --disk --scsi --ide --network --storage-ctrl --sys --cpu --partition --usb-ctrl --hub --memory";

open(HWINFO, "$hwinfo|") or die "ERROR: unable to execute $hwinfo";

sub ee($) {
    my ($s) = @_;
    $s =~ s/\>/\&gt;/g;
    $s =~ s/\</\&lt;/g;
    $s =~ s/\&/\&amp;/g;
    $s =~ s/\"/\&quot;/g;
#   $s =~ s/\//_/g;
    return $s;
}
# smash invalid XML_Name chars
sub ef($) {
    my ($s) = @_;

    $s =~ s/^\s*(\d+)/n$1/ ; # prefix any numerics
    $s =~ s/\s/_/g;          # turn spaces into _
    $s =~ s/\:/\./g;         # no colons, please!
    $s =~ s/\#//g;           # hash marks, nyet!
    $s =~ s/[^\w.-]/-/g;     # and, nothing that isn't in the xml charset.
    return $s;
}

print "<deviceTree>\n";
$ingroup = 0;
while (<HWINFO>) {
    chomp;
    if ($_ =~ /^\d+\:(.*)/) { # start a new device
	($id1,$id1,$id1,$id2) = split(" ",$1,4); 
	#print $id2;
	$ucid2 = uc($id2);
	if ($ingroup) {

	    print "\t\t</device>\n";
	    print "\t</group>\n";
	    $ingroup = 0;
	}
	if (exists($dtype{$ucid2})) {
		$ingroup = 1;
		$ucid2 = uc($id2);
		print "\t<group>\n";
#		print "\t\t<name>",ee($ucid2),"</name>\n";
		print "\t\t<name>",ee($id2),"</name>\n";
		print "\t\t<type>",ee($dtype{$ucid2}),"</type>\n";
		# print $id2;
		# print $_ ;
		print "\t\t<device>\n";
		if ($ucid2 eq "BIOS") { print "\t\t\t\t<Name>Standard PC BIOS</Name>";}
	    }
    } else {
    if ($ingroup) {
	if ($_ =~ /^\s+((\S+\s*)+)\:\s*(.*)/) { # look for the keyword and value here...
	    ($param_name,$param_value) =  ($1,$3);
	    if (exists($ptrans{$param_name})) {
		$param_name=$ptrans{$param_name};
	    } else {
		$param_name=ef($param_name);
	    }
		
		print "\t\t\t<",ee($param_name),">";
		$param_value =~ s/.+\"(.*)\"/$1/;
	        $param_value =~ s/^\"(.*)\"$/$1/;
		print ee($param_value);
		print "</" . ee($param_name) . ">\n";	
	    }
    }
    }
}
if ($ingroup) {

	print "\t\t</device>\n";
	print "\t</group>\n";
	$ingroup = 0;
}
print "</deviceTree>\n";

close(HWINFO);
