#!/bin/bash

#   The contents of this file are subject to the terms of the
#   Common Development and Distribution License, Version 1.0 only
#   (the "License").  You may not use this file except in compliance
#   with the License.
 
#   You can obtain a copy of the license at 
#   http://www.opensource.org/licenses/cddl1.php
 
#   When distributing Covered Code, include this CDDL HEADER in each
#   file and include the original License file at src/license.txt.
#   If applicable, add the following below this CDDL HEADER, with the
#   fields enclosed by brackets "[ ]" replaced with your own identifying
#   information: Portions Copyright [yyyy] [name of copyright owner]

#   Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
#   Use is subject to license terms.
#   Initial Developer updates, news, and information may be found at 
#   http://sourceforge.net/projects/likewiseopenagt

EVENTLOGADM_BINARY="/usr/bin/eventlogadm"

LW_DIR_PREFIX=/usr/share/likewise
LW_AGENT_DIR=$LW_DIR_PREFIX/open-agent
LW_AGENT_BIN_DIR=$LW_AGENT_DIR/bin
LW_BIN_DIR=$LW_DIR_PREFIX/bin

# Show the list of functions.

function Help
{
    echo
    echo "'initLikewise' is the function that finishes the cesm-agent install."
    echo "'Help' is the function that generated the text you are reading now."
    echo
    echo "The remaining functions in the list are post-install building blocks," 
    echo "call them with the 'h' parameter to get their argument help:"
    echo
    echo " $0 FunctionName h"
    echo
    grep "^function \+\(\w*\) *$" $0 | sed s/function//
    echo
}

# FirewallConfigure we need to penetrate the firewall using ssh and the
# appropriate smb ports and our tomcat port

function ConfigureFirewall
{

    PORTS_T="22 139 445 9106"  # tcp ports for ssh, samba and our agent
    PORTS_U="137 138"               # udp ports for netbios

    if [ 'h' = "$1" ]; then echo "Arguments Hardcoded: $PORTS"; return -1; fi


    if [ -e /sbin/SuSEfirewall2 ]; then

	sed -i.bakt "s/^\\s*\(FW_SERVICES_EXT_TCP=.*\)\"/\\1 $PORTS_T\"/" /etc/sysconfig/SuSEfirewall2
	sed -i.baku "s/^\\s*\(FW_SERVICES_EXT_UDP=.*\)\"/\\1 $PORTS_U\"/" /etc/sysconfig/SuSEfirewall2

        # Firewall MAC address BUGBUG - shouldnt screw up eth0 setups, i think
        # regexp that describes the ethernet interface that we want to replace
        # cant figure out how to describe hex in sed, so word (\w) will do

	FW_RPL_E='FW_DEV_EXT="eth-id-\w\w:\w\w:\w\w:\w\w:\w\w:\w\w"'

        # derive the correct macaddr by looking in the network directory

	ETH_IF_LS=/etc/sysconfig/network/ifcfg-
	CENT_ETH_IF=`ls /etc/sysconfig/network/ifcfg-eth-id-*`

        # if the str that holds the ls results isnt empty perform bash
        # substitution geekery to create a replacement ethernet interface

	if [ ! 0 == ${#CENT_ETH_IF} ]; then
	    FW_CENT_SERV_E="FW_DEV_EXT=${CENT_ETH_IF#$ETH_IF_LS}"

            # do the replacement

	    sed -i.bake "s/$FW_RPL_E/$FW_CENT_SERV_E/" /etc/sysconfig/SuSEfirewall2
	fi

	# check to see if the firewall is running by checking the output of
	# SuSEfirewall2 for the word 'state' in the absence of the word 
        # 'matching'. This is the only test that i have been able to discover
	# that will work on SLES 9 SuSE 9.X and OpenSuSE

	/sbin/SuSEfirewall2 status 2>&1 | grep 'state [^(matching)]'; fw_status=$?

	if [ "0" == "$fw_status" ]; then
	    /sbin/SuSEfirewall2 stop; rc=$?; if [ 0 != $rc ]; then return $rc; fi
	    /sbin/SuSEfirewall2 start; return $?
	else
	    echo "Firewall settings modified, but firewall not started."
	    return 0;
	fi
    fi

    # RHEL4 and possibly other dists use iptables directly
    if [ -e /etc/sysconfig/iptables ]; then

	INSERT_AT='-A FORWARD -j RH-Firewall-1-INPUT'
	for port in $PORTS_T; do
	  sed -i.bakt  "s/\($INSERT_AT\)/\\1\n-A RH-Firewall-1-INPUT -p tcp --dport $port -j ACCEPT/"  /etc/sysconfig/iptables
	done
	for port in $PORTS_U; do
	  sed -i.baku  "s/\($INSERT_AT\)/\\1\n-A RH-Firewall-1-INPUT -p udp --dport $port -j ACCEPT/"  /etc/sysconfig/iptables
	done
	
	# test status, restart iptables if it was running

	service iptables status 2>&1 | grep "is stopped"; fw_status=$?

	if (( $fw_status == 1 )); then
	    /etc/init.d/iptables restart; return $?
	else
	    echo "Firewall settings modified, but firewall not started."
	    return 0;
	fi
    fi

    echo "No firewall configuration file was found."
    echo "Assuming no firewall is enabled."
    return 0
}


# Configure CUPS. 
# cups /etc/cups/mime.convs /etc/cups/mime.types

function CUPSConfigure
{
    # Check for existence of mime config (convs,types) file names

    if [[ 'h' = $1 || -z $1 || -z $2 || -z $3 ]]
	then echo "cupsdname mimeconv_fullpath mimetype_fullpath"; return -1;fi


    sed -i.bak "s/^#application\(.*\)raw\(.*\)/application\1raw\2/"       $2
    rc=$?; if [ 0 != $rc ]; then return $rc; fi

    sed -i.bak "s/^#application\/octet-stream/application\/octet-stream/" $3
    rc=$?; if [ 0 != $rc ]; then return $rc; fi

    DaemonManage $1 'on'; rc=$?; return $rc
}


# Start up the printing stuff that we need to start...
function initCups
{
    # bounce cups
    /etc/init.d/cups stop
    /etc/init.d/cups start
    
}


# check SELinux state, if it's not permissive or disabled, we cant run

function isSELinuxOff
{
    # we seek out an selinux state of enforcing, if we find it with grep
    # then we return 1 and let the caller decide what to do

    if [ -e /etc/selinux/config ] ; then

	cat /etc/selinux/config | sed '/#/d' | grep -i selinux\ *=\ *enforcing;rc=$?; 

	if [ 0 == $rc ];  then echo "SELinux is enabled";  return 1
	else                   echo "SELinux is disabled"; return 0
	fi

    else echo "SELinux is not installed"; return 0
    fi
}


# starting them or stopping them and enabling and disabling them in init.d

function DaemonManage
{
    # check argument correctness

    if [[ 'h' = "$1" || -z "$1" || -z "$2" ]]
	then echo "DaemonManage daemonname daemonaction: on or off";     return -1; fi

    if [[ ! 'off' = "$2" && ! 'on' = "$2" ]];              then return -2; fi

    # set up to either start the daemon and add it to the boot process or
    # stop the daemon and remove it from the boot process

    ACTION_INITD='start'
    ACTION_CKCFG='add'

    if [ 'off' = "$2" ]; then ACTION_INITD='stop'; ACTION_CKCFG='del'; fi

    # check to see if the daemon exists before we try and do anything

    if ! [ -e /etc/init.d/"$1" ]; then
	echo "daemon: $1 not found"
	return 2
    fi
        
    # check the status of the daemon and see what's up, because some of them
    # are coded so that they fail if you ask them to do something that they
    # are already doing. thankfully, both 'status' and 'checkproc' return
    # the same status vals.

    /etc/init.d/$1 status; rc=$?;
    
    if [[ 'start' = $ACTION_INITD && -z $rc ]]; then              return 0;  fi
    if [[ 'stop'  = $ACTION_INITD && -n $rc ]]; then              return 0;  fi
    
    # if we tried to check status before doing our job and we fell out of the
    # previous tests with a nonzero error code, then we need to die

    if [ $rc -gt 4 ];  then   return $rc; fi # error conditions 5 and 6 are BAD.

    # ok, we must be able to do a job that needs doing

    /etc/init.d/$1 $ACTION_INITD $ACTION_ARGS; rc=$?; if [ 0 != $rc ];  then   return $rc; fi

    # to make a daemon ready for permanent use, it has to be
    # registered with chkconfig. After that is finished, we are part
    # of the boot process (or have been removed from the boot process
    # if 'stop' was the argument passed to us)

    /sbin/chkconfig --$ACTION_CKCFG $1; rc=$?;                   return $rc
    /sbin/chkconfig $1 $2;              rc=$?;                   return $rc
}


# aggregate calls to daemon manage for all of the samba daemons

function DaemonManageSamba
{
    if [[ 'h' = "$1" || -z "$1" ]]
	then echo "DaemonManageSamba daemonaction: on or off";     return -1; fi

    # NB: because we are only working with user/workgroup authentication we will
    # *never* want to *start* winbind, we will only want to *stop* it. so we
    # add it to the list of daemons to manage only in the case of stopping them.

    daemons="smb nmb" 

    if [ 'off' = "$1" ]; then daemons="$daemons winbind"; fi

    for sambadaemon in `echo $daemons`
      do

      # DaemonManage returns 2 for daemons that it cant find and we know that in some cases
      # not finding a daemon is a expected result (ie: will never find nmb init.d script on RHEL)

      DaemonManage $sambadaemon "$1"; rc=$?; if [[ 0 != $rc || 2 != $rc ]]; then return $rc; fi
    done
}


# do all the work required to configure and (re)start samba with a workgroup relationship

function ConfigureSamba
{
    if [[ 'h' = "$1" || -z "$1" ]]
	then echo "ConfigureSamba WORKGROUP-name";     return -1; fi

    if [ ! -f $EVENTLOGADM_BINARY ]
	then echo "$EVENTLOGADM_BINARY not found -- is samba up to date on this system?"; 
             echo "Eventlogs won't work until $EVENTLOGADM_BINARY is installed."; fi
    
    DaemonManageSamba 'off'; rc=$?; # if [[ 0 != $rc && 3 != $rc ]]; then return $rc; fi
    # set up path to jre -- this looks for the jre and writes to a variable file...
    $LW_AGENT_BIN_DIR/findjre.sh

    echo "Calling sambaShareInstaller"
    # call the share making smb.conf modifier thingy

    $LW_AGENT_BIN_DIR/sambaShareInstaller /etc/samba/smb.conf $LW_AGENT_BIN_DIR/ $1
    rc=$?; if [ 0 != $rc ]; then return $rc; fi

    # 

    DaemonManageSamba  'on'; rc=$?; if [[ 0 != $rc && 3 != $rc ]]; then return $rc; fi
}


# check if Likewise Agent successfully bound to the port

function isAgentListening
{
    if [ 'h' = "$1" ]; then
        echo "isAgentListening";
        return -1;
    fi
	# timeout in seconds before declaring failure
	timeout=30
	# port that Likewise is supposed to listen on
	port=9106

	iter=0

	while [ $iter -lt $timeout ]
	do
        	netstat -t -l | grep $port > /dev/null 2>&1
        	rc=$?
        	if [ 0 = $rc ]; then
		        echo "Likewise Agent is up and listening"
			return $rc
        	fi
        	iter=$(( $iter + 1 ))
        	sleep 1
	done
	echo "Likewise Agent is not up and listening"
	return 1
}


# aggregate calls to daemon manage for all of the likewise daemons

function ConfigureLikewise
{

    if [[ 'h' = "$1" || -z "$1" ]]
	then echo "DaemonManageLikewise daemonaction: on or off";     return -1; fi
    # create the working dirs
    # think that the w32x86 dir get's created automagically; not sure, so make it.

    /usr/bin/install -d $LW_DIR_PREFIX/{conf,data,logs,print/W32X86} $LW_DIR_PREFIX/conf
    rc=$?; if [ 0 != $rc ]; then return $rc; fi

    daemons="likewise-open" 
   

    for likewisedaemon in `echo $daemons`
      do

      # Link to our daemon if we are turning the daemons on and havent done it already.  
      # We avoid copying the script into the /etc/init.d dir so that we only have to
      # worry about one location during an upgrade.

      if [[ 'on' = "$1" && ! -f /etc/init.d/$likewisedaemon && -f $LW_AGENT_BIN_DIR/$likewisedaemon ]]; then
	  ln -s $LW_AGENT_BIN_DIR/$likewisedaemon /etc/init.d/$likewisedaemon
	  rc=$?; if [ 0 != $rc ]; then return $rc; fi
      fi
      
      DaemonManage $likewisedaemon "$1"; rc=$?; if [ 0 != $rc ]; then return $rc; fi
    done

    daemons="likewise-evtlogd likewise-perfd" 

    for likewisedaemon in `echo $daemons`
      do

      # Link to our daemon if we are turning the daemons on and havent done it already.  
      # We avoid copying the script into the /etc/init.d dir so that we only have to
      # worry about one location during an upgrade.

      if [[ 'on' = "$1" && ! -f /etc/init.d/$likewisedaemon && -f $LW_BIN_DIR/$likewisedaemon ]]; then
	  ln -s $LW_BIN_DIR/$likewisedaemon /etc/init.d/$likewisedaemon
          rc=$?; 
	  # if [ 0 != $rc ]; then return $rc; fi   # IGNORE RETURN CODES FOR NOW...
      fi
      
      DaemonManage $likewisedaemon "$1"; rc=$?; 
          # if [ 0 != $rc ]; then return $rc; fi   # IGNORE RETURN CODES FOR NOW...
    done

        
    CUPSConfigure 'cups' '/etc/cups/mime.convs' '/etc/cups/mime.types'
    rc=$?; if [ 0 != $rc ]; then return $rc; fi
    
    echo "Configured CUPS"
    # start cups
    initCups

    # Agent supposedly started successfully, check that it is listening

    isAgentListening; rc=$?; if [ 0 != $rc ]; then return $rc; fi
}


# Finish off the setup of samba and the agent, link it in to init.d and turn it on in boot

function initLikewise
{
    if [[ 'h' = "$1" || -z "$1" ]]
	then echo "initLikewise WORKGROUP-name";     return -1; fi

    # check the selinux state and complain and die if it's turned on

    isSELinuxOff; rc=$?; if [ 0 != $rc ]; then
	echo "SELinux is enabled. Please turn it off, reboot the machine and try again."
	exit 1
    fi

    # turn on the correct ports in the firewall.

    ConfigureFirewall; rc=$?; if [ 0 != $rc ]; then return $rc; fi

    # make samba work, by the time this is done, smb.conf is configured
    # and the appropriate samba daemons are running.

    ConfigureSamba "$1"; rc=$?; if [ 0 != $rc ]; then return $rc; fi

    # make likewise work, by the time this is done, all the stuff is configured
    # and the appropriate likewise daemons are running.

    ConfigureLikewise "on" ; rc=$?; if [ 0 != $rc ]; then return $rc; fi
}



# 'main' or 'the script' for want of a better way to say it :-)

if [ 0 = "${#@}" ]; then Help; exit 0; fi # if the script has no args, show help

$@                                        # execute the func specified on cmdline
rc=$?                                     # grab the return code.

# Check the return code so that we can write out Failed or Succeeded for the
# benefit of the windows side of the installer.

if [ 0 != "$rc" ]; then echo "Open Likewise Post Install Failed"
else                    echo "Open Likewise Post Install Succeeded"
fi

exit $rc                                  # return the error code to the caller.


