#! /usr/bin/perl -w
# 
#
#  The contents of this file are subject to the terms of the
#  Common Development and Distribution License, Version 1.0 only
#  (the "License").  You may not use this file except in compliance
#  with the License.
# 
#  You can obtain a copy of the license at 
#  http://www.opensource.org/licenses/cddl1.php
# 
#  When distributing Covered Code, include this CDDL HEADER in each
#  file and include the original License file at src/license.txt.
#  If applicable, add the following below this CDDL HEADER, with the
#  fields enclosed by brackets "[ ]" replaced with your own identifying
#  information: Portions Copyright [yyyy] [name of copyright owner]
# 

#  Copyright (c) 2006 by Centeris, Inc.  All rights reserved.
#  Use is subject to license terms.
#  Initial Developer updates, news, and information may be found at 
#  http://sourceforge.net/projects/likewiseopenagt
#
#      $Id: likewise-open-agent.pl,v 1.4 2006/01/25 01:12:06 bmoran Exp $

# use Data::Dumper;
# -------------------------------------------------------------------
#
#  Likewise Open Agent - SOAP Services for the Likewise Open Agent
#     (originally written using jwsdp)
# 
# -------------------------------------------------------------------

use SOAP::Lite;   #  +trace=>all; 
use SOAP::Transport::HTTP; # +trace;

# lean(er) and mean(er) SOAP services daemon for Likewise Open Agent
# TODO: eventually break the service handling out of this file.

$MINIMUM_SOAPLITE_VER = 0.66;
$LIKEWISE_AGENT_PORT = 9106;

$LW_AGENT_BIN_DIR="/usr/share/likewise/open-agent/bin";
$LW_BIN_DIR="/usr/share/likewise/bin";

if (( 0 + $SOAP::Lite::VERSION) < $MINIMUM_SOAPLITE_VER) { # make sure versions are ok...
    print STDERR "$0 : Need a newer version of SOAP::Lite, please see http://www.soaplite.com\n";
    exit -1;
}

# Note that we MAY be able to upgrade just HTTP.pm on some platforms...
if (check_for_usable_HTTPPM()) {
    exit -1;
}

my $LIKEWISE_VERSION = `head -1 $main::LW_AGENT_BIN_DIR/VERSION`;
     chomp($LIKEWISE_VERSION);

if ($LIKEWISE_VERSION eq "") {$LIKEWISE_VERSION = "(unknown)"; }

print "\n\nLikewise Open Agent SOAP server, version $LIKEWISE_VERSION\n\n";

print "Using SOAP::Lite version ",$SOAP::Lite::VERSION,"\n";

  # don't want to die on 'Broken pipe' or Ctrl-C

  if (!defined($ARGV[0]) || ("" eq $ARGV[0])) {
    print "daemon mode.\n";
      $SIG{PIPE} = $SIG{INT} = 'IGNORE';
  } else {
      print "In test mode, Ctrl-C to exit\n";
  }

  my $daemon = SOAP::Transport::HTTP::Daemon
    -> new (LocalPort => $LIKEWISE_AGENT_PORT)
    -> dispatch_to('Demo','centeriscomSystemInfo')
    ;

print "Server listening on ", $daemon->url, "\n";
$daemon->handle;

# shouldn't return...

####### rest are subs and built in packages

sub fix_HTTPPM {
    my ($fname) = $_;
    # todo - apply patch here if we can . TODO: see if we can
    return 0;
}

sub check_for_usable_HTTPPM {
#
# All of this is about handling .NET runtime's use of the expect header...
#   SOAP/Transport/HTTP.pm that handles this needs to be able to do it.
#
    if ((0 + $SOAP::Lite::VERSION) < $MINIMUM_SOAPLITE_VER) {
           print STDERR "SOAP::Lite version must be at least $MINIMUM_SOAPLITE_VER\n";
	   return 1;
       }
    return 0;
}
############ INTERNAL PACKAGE
package centeriscomSystemInfo;

 sub getIniFileInfo($$) {
     my ($section,$keyval) = @_;
     $in_section = 0;

     if (!open (INIF,"</etc/samba/smb.conf")) {
        return undef;
     }

     while (<INIF>) {
         chomp;
         if ( /^\s*\[$section\]/i ) { # we're in global section
           $in_section = 1;
         }
          elsif ( /^\s*\[/ ) {  # not in global section
           $in_section = 0;
       }
       if ($in_section && /^\s*$keyval\s*\=\s*(.*)/) {
           # handle "somestuff", somestuff, somestuff #comment,  "somestuff" # comment
           $mstr = $1;

           if ($mstr =~ /(.*)\#.*/ ) { # get rid of comments...
                $mstr=$1;
            }
           if ($mstr =~ /\"(.*)\"\#*.*/ ) { # get rid of quotes
                $mstr=$1;
            }
           close(INIF);
           return $mstr;
       }
     }
     close(INIF);
     return undef;
 }

  sub getDeviceTree {
      my $cmd = "perl $main::LW_BIN_DIR/hwinfo_filter.pl";
      my $foo = qx ($cmd) ;

      print STDERR "IN getDeviceTree\n";

      return SOAP::Data->name('result') 
                   ->type('string')
                   ->uri('urn:centeriscomSystemInfo')
                   ->value($foo);
  }


  sub likewiseOpenAgentVersion {
      my $loa_ver_num = `head -1 $main::LW_AGENT_BIN_DIR/VERSION`;
      chomp($loa_ver_num);

      print STDERR "IN likewiseOpenAgentVersion\n";

      return SOAP::Data->name('result') 
                   ->type('string')
                   ->uri('urn:centeriscomSystemInfo')
                   ->value($loa_ver_num);

  }
  sub getConfiguredDomain{
      # open /etc/samba/smb.conf, get the 'realm' from the [global] section

      print STDERR "IN getConfiguredDomain\n";

      my $retval;

      #my $secmodel = getIniFileInfo("global","security");
      
      #if ( $secmodel =~ /ads\s*/ ) {
      #    $retval="DOMAIN: ".getIniFileInfo("global","realm");
      #    } else {
      #    $retval="WORKGROUP: ".getIniFileInfo("global","workgroup");
      #}

      # note that this only shows the info from smb.conf. May not be relevant if not AD joined

      $retval=getIniFileInfo("global","workgroup");
      return SOAP::Data->name('result') 
                   ->type('string')
                   ->uri('urn:centeriscomSystemInfo')
                   ->value($retval);


  }

  sub getConfiguredDescription{
      # open /etc/samba/smb.conf, get the 'server string' from the [global] section

      print STDERR "IN getConfiguredDescription\n";

      my $retval= getIniFileInfo("global","server string");

      return SOAP::Data->name('result') 
                   ->type('string')
                   ->uri('urn:centeriscomSystemInfo')
                   ->value($retval);
  }

  sub what_release_am_i {
     my @KNOWN_RELS= ('fedora-release','gentoo-release','debian_version','SuSE-release',
                      'knoppix-version','redhat-release','redhat_version');

     foreach $fn (@KNOWN_RELS) {
	 if (-e "/etc/$fn") {
	     my $retval = `head -1 /etc/$fn`;    # cheap shot.
	     chomp $retval;
             return $retval;
         }           
     }
     return "Unknown Linux Release";
  }

  sub centerisSystemInfo {
      # look for  /etc/*-release, take the first line


      print STDERR "IN centerisSystemInfo\n";

      my $loa_ver = "Likewise Open Agent version ";

      my $release = what_release_am_i();

      # my $release = `head -1 /etc/*-release`;
      # chomp $release;

      my $hostname = `hostname`;
      chomp($hostname);

      my $cpuinfo = `cat /proc/cpuinfo`;
      my $meminfo = `cat /proc/meminfo`;
      my $loa_ver_num = `head -1 $main::LW_AGENT_BIN_DIR/VERSION`;
      chomp($loa_ver_num);

      if (!defined($loa_ver_num)) { $loa_ver_num="unknown"; }
      $loa_ver .= $loa_ver_num;

      my $cpuspeed;

      if ($cpuinfo =~ /^model name\s+\:\s+(.*)/m ) {
          $modelname = $1;
      }
      if ($cpuinfo =~ /^cpu MHz\s+\:\s+(.*)/m ) {
          $cpuspeed = $1;
      }
      if ($meminfo =~ /^MemTotal\:\s+(\d+)\s+kB/m ) {
          $memsize = $1;
      }
      $memsize = int( $memsize / 1024 );
      @si = ($loa_ver, $release,"","","root@".$hostname,"","","",$modelname,"CPU Speed ".$cpuspeed." MHz",
             $memsize." Megabytes of RAM", "","");
      #print $#si;
      #print "\n";
      return \@si;
  }
 
  1;
############ INTERNAL PACKAGE END


