/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

package com.centeris.jettyrunner;

import com.centeris.webservice.utils.SystemFileLocator;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.servlet.WebApplicationContext;
import org.apache.log4j.*;

import java.io.IOException;

public class Main
{
    /**
     * Set to true to enable mDNS announcements.
     * Temporarially disabled to work around issues with jetty being zombied.
     *
     * TODO: add this setting to the global property settings class, if it exists
     */

	private static boolean useDebug		= false ;

    static Logger logger = Logger.getLogger(Main.class);

	public static void main(String[] args) throws Exception
	{

		//
		// Process the command line args passed to us.
		//
		// TODO - make it pull this info from a "settings class" that is
		// yet to be written.
		//

		checkArgs( args ) ;

		//
		// First up, create the jetty server.
		//

		final Server server = new Server();

        String usePort;

		//
		// no SSL, just add a normal listener here.
		//

		usePort = ":" + SystemFileLocator.getSoapPortDebugString();

		server.addListener(usePort) ;

		//
		// set up the context that the jetty sever will be using.
		//

        WebApplicationContext context = null;
        context = server.addWebApplication("/", "cesm-agent.war");

		//
		// fix for bug #826 - TMPWATCH job in /etc/cron.daily can delete important files.
		//
		// The fix for this is simple. we no longer expand the WAR file out. All of the classes
		// and resource are extracted on the fly by jetty with our current bits. The expansion was
		// a hold/crossover from the windows code base.
		//

		context.setExtractWAR(false);
		
        context.setDefaultsDescriptor("") ;   // TODO: do we need one?

        server.start();

        logger.info("Adding shutdown hook...");
        {
            // add hook for daemon shutdown.  This code will be executed when the user
            // stops the daemon properly, or if we exit from System.exit().
            // This will not be called if the program exits abnormally.
            //
            Thread shutdownThread = new Thread() {
                public void run() {
                    shutdownHook(server);
                }
            };
            Runtime.getRuntime().addShutdownHook(shutdownThread);
        }
        

        logger.info("cesm-agent daemon started.");
	}

    public static void shutdownHook(final Server server)
    {
        // shut down the Jetty server
        if ((null != server) && (server.isStarted()))
        {
            try
            {
                logger.info("Shutting down jetty");
                server.stop();
            }
            catch (InterruptedException e)
            {
                logger.error("Caught exception from server.stop(): ", e);
            }
        }

        logger.info("Shutdown complete.");
    }

	/**
	 * quick function to parse args passed to us.
	 *
	 * @param args
	 */

	private static void checkArgs( String [] args )
	{
		//
		// quick check.
		//

		if( args == null )
		{
			return ; // nothing to do.
		}
		
		int iTotalArgsFound = 0 ; 		// for error checking.

		int iTotalArgs = args.length ;

		//
		// loop through the list of arguments and check for matching entries.
		//

		for( int iLoop = 0 ; iLoop < iTotalArgs; iLoop++ )
		{

			//
			// check for the different args that we care about.
			//

			if( args[iLoop].compareToIgnoreCase("-DEBUG") == 0 )
			{
				useDebug = true ;
				continue ;
			}

		}

		//
		// TODO - how to handle the end check for params. Throw error or
		// keep on skipping it. ?
		//

		if( iTotalArgs != iTotalArgsFound )
		{
			// do something useful here.
            logger.warn("Argument mismatch from command line: " + iTotalArgsFound + " args found, "+
                    iTotalArgs + " args expected");
		}

	}

}  // end of class Main
