/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

package com.centeris.webservice.samba;

import java.util.HashMap;

/**
 * @author eriks
 *
 * Keep all the useful information about an ini section
 * Useful external functions are putKey and getKeys
 */

public class IniSection {
	private int startLine;
	private int endLine;
	private HashMap keys;
	private String sectionName;
	
	/**
	 * Represents a section of an .ini file.
	 *
	 * @param sectionName	The name of the .ini section.
	 */
	public IniSection(String sectionName) {
		this.sectionName = sectionName;
		keys = new HashMap();
	}
	
	public void setStartLine(int line) {
		startLine = line;
	}
	
	public void setEndLine(int line) {
		endLine = line;
	}
	
	public int getStartLine() {
		return startLine;
	}
	
	public int getEndLine() {
		return endLine;
	}
	
	public String getName() {
		return sectionName;
	}
	
// allows you to add a key/value pair to the section
	public void putKey(String keyName, String keyValue) {
		keys.put(keyName, keyValue);
	}
	
// allows you to get a hashmap of the key/value pairs for the section
	public HashMap getKeys() {
		return keys;
	}
	
	public void removeKey(String keyName) {
		keys.remove(keyName);
	}

}
