/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

/*
 * This programis used by samba for 'share add/modify/delete command'
 * This program gets used like:
 * SambaShare add <path_to_smb_conf> <sharename> <path_for_share> <comment>
 * SambaShare change <path_to_smb_conf> <sharename> <path_for_share> <comment>
 * SambaShare delete <path_to_smb_conf> <sharename>
 */

package com.centeris.webservice.samba;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.File;

public class SambaShare {
	private static String commandName;
	private static String configFile;
	private static String shareName;
	private static String pathName;
	private static String comment;
	private static IniPreserve ini = null;
	
	
	public static void main(String[] args) {
		if (args.length < 3) {
			printSyntaxAndExit();
		}
		commandName = args[0].toLowerCase();
		configFile = args[1];
		shareName = args[2];
		if (5 == args.length) {
			pathName = args[3];
			comment = args[4];
		}
		try {
			ini = new IniPreserve(configFile);
		} catch (FileNotFoundException e) {
			System.out.println("File not found.");
			System.exit(1);
		} catch (IOException e) {
			System.out.println("Error reading from file");
			System.exit(1);
		}

		if ( commandName.equals("add") && (5 == args.length) ) { addCommand(); }
		else if ( commandName.equals("change") && (5 == args.length) ) { changeCommand(); }
		else if ( commandName.equals("delete") && (3 == args.length) ) { deleteCommand(); }
		else { printSyntaxAndExit(); }
		ini.commit();
	}
	
	
	private static void addCommand() {
        // BUG154: if the path isn't valid, we need to fail
        File ShareDirectory = new File(pathName);
        if ( ! (ShareDirectory.exists() && ShareDirectory.isDirectory()) )
        {
            System.exit(1);
        }

		IniSection newSection = null;
		
		try {
			if (! ini.legalSectionName(shareName) ) { System.exit(1); }
			newSection = ini.addSection(shareName);
		} 
		catch (IniSectionAlreadyExists e) {
			System.out.println("Share already exists, can't add again!");
			System.exit(1);
		}
		newSection.putKey("path", pathName);
		newSection.putKey("comment", comment);
		ini.replaceSection(newSection);
	}

	private static void changeCommand() {
		IniSection section = null;
		
		try {
			if (! ini.legalSectionName(shareName) ) { System.exit(1); }
			section = ini.getSection(shareName);
		} catch (IniSectionNotFound e) {
			System.out.println("Share not found");
			System.exit(1);
		}
		section.putKey("path", pathName);
		section.putKey("comment", comment);
		section.putKey("read only", "no");
		ini.replaceSection(section);
	}
	
	private static void deleteCommand() {
		if (! ini.legalSectionName(shareName) ) { System.exit(1); }
		ini.removeSection(shareName);
	}
	
	private static void printSyntaxAndExit() {
		System.out.println("Invalid Syntax");
		System.out.println("Syntax:");
		System.out.println("SambaShare add <config file> <share name> <path name> <comment>");
		System.out.println("SambaShare change <config file> <share name> <path name> <comment>");
		System.out.println("SambaShare delete <config file> <share name> <path name> <comment>");
		System.exit(1);
	}
}
