/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

package com.centeris.webservice.utils;

/**
 * Created by IntelliJ IDEA.
 * User: jmannix
 * Date: Jul 11, 2005
 * Time: 4:30:19 PM
 *
 * ExecException is to be thrown whenever Runtime.exec("command") returns an unexpected error code.
 * It should be instantiated with strings representing the command used, the stdOut and stdErr of the
 * process, as well as the return code of the process (if any of these are available) which may be
 * accessed by whoever catches the exception, to decide what to do with this issue.
 *
 */
public class ExecException extends Exception
{
    private String sCommand;
    private String sStdOut;
    private String sStdErr;
    private int iRet;

    /**
     * If initialized with no information, set the return value to -100 as a default.
     */
    public ExecException()
    {
        super();
        sCommand="";
        sStdOut="";
        sStdErr="";
        iRet=-100;
    }
    /**
     * Constructors which don't set the return value leave it at the default of -(100+numArgsInConstructor) for
     * debugging purposes
     * @param msg is passed up the super() call, and also set to be the stdErr if none is supplied
     */
    public ExecException(String msg)
    {
        super(msg);
        sCommand="";
        sStdOut="";
        sStdErr=msg;
        iRet=-101;
    }
    /**
     *
     * @param msg passed up to super(), but is no longer set into stdErr
     * @param cause yeilds its stackTrace().toString into our stdErr variable
     */
    public ExecException(String msg, Throwable cause)
    {
        super(msg, cause);
        sCommand="";
        sStdOut="";
        sStdErr="";
        iRet=-102;
    }
    /**
     *
     * @param msg passed up to super(), but is no longer set into stdErr
     * @param command The command we tried to execute.
     * @param cause yeilds its stackTrace().toString into our stdErr variable
     */
    public ExecException(String msg, String command, Throwable cause)
    {
        super(msg, cause);
        sCommand=command;
        sStdOut="";
        sStdErr="";
        iRet=-102;
    }
    public ExecException(String stdOut, String stdErr)
    {
        super("Exec() error:" + "STDOUT: " +stdOut + '\n' + "STDERR: " + stdErr);
        sCommand="";
        sStdOut = stdOut;
        sStdErr = stdErr;
        iRet=-103;
    }
    public ExecException(String command, String stdOut, String stdErr)
    {
        super("Exec() error:" + "STDOUT: " +stdOut + '\n' + "STDERR: " + stdErr +
                '\n' + "Command: " + command);
        sCommand=command;
        sStdOut = stdOut;
        sStdErr = stdErr;
        iRet=-104;
    }
    public ExecException(String command,
                         String stdOut,
                         String stdErr,
                         int retCode,
                         Throwable cause)
    {
        super("Exec() error:" + "STDOUT: " +stdOut + '\n'
                              + "STDERR: " + stdErr + '\n'
                              + "Command: " + command + '\n'
                               + "Cause: " + (null != cause ?
                                                  ( null != cause.getMessage() ? cause.getMessage() : cause.getClass().getName() )
                                                  : "") + "\n"

                              + "Return Code: " + retCode,
              cause);
        sCommand = command;
        sStdOut  = stdOut;
        sStdErr  = stdErr;
        iRet     = retCode;
    }
    public ExecException(String command, String stdOut, String stdErr, int retCode)
    {
        this(command, stdOut, stdErr, retCode, null);
    }
    public ExecException(String command, ProcessInfo info)
    {
        this(command, info.stdOut, info.stdErr, info.returnCode, info.exception);
    }
    public String getCommand() { return sCommand; }
    public String getStdOut() { return sStdOut; }
    public String getStdErr() { return sStdErr; }
    public int getRetCode() { return iRet; }

}
