/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

#include "perf.h"

#define CENTERIS_PROPERTIES_FILE "/opt/centeris/conf/systemFiles.properties"
#define SUSE_HTTPD_CONF_PATH "/etc/apache2/httpd.conf"
#define RHEL4_HTTPD_CONF_PATH "/etc/httpd/conf/httpd.conf"

void get_apacheinfo(PERF_DATA_BLOCK *data)
{
	FILE *stats;
	char buf[PROC_BUF];
	char command[PATH_MAX];
	char *key, *value;
	int i = 0;
	int ret;

	memset(command, 0, PATH_MAX);
	sprintf(command, "grep -q extended-server-status %s && wget -r 'http://localhost/extended-server-status?auto' -O /tmp/extended-server-status > /dev/null 2>&1", data->apacheInfo.confFileName);
/*	fprintf(stderr, "about to execute [%s]", command);*/
	ret = system(command);
/*	fprintf(stderr, "get_apacheifo -- system returned [%d]. status is [%d]\n", ret, WEXITSTATUS(ret));*/

	if(!(stats = fopen("/tmp/extended-server-status", "r")))
	{
		data->apacheInfo.isEnabled = FALSE;
		return;
	}
	
	data->apacheInfo.isEnabled = TRUE;   

	rewind(stats);
	fflush(stats);

	memset(buf, 0, PROC_BUF);
	while(fgets(buf, sizeof(buf), stats))
	{
		key = strtok(buf, ":");
		value = strtok(NULL, ":");

		if(strcmp(key, "Total Accesses") == 0)
		{
			data->apacheInfo.data->lastAccesses = data->apacheInfo.data->totalAccesses;
			data->apacheInfo.data->totalAccesses = atoi(value);
		}
		else if(strcmp(key, "Total kBytes") == 0)
		{
			data->apacheInfo.data->lastKBytes = data->apacheInfo.data->totalKBytes;
			data->apacheInfo.data->totalKBytes = atoi(value);
		}
		else if(strcmp(key, "Uptime") == 0)
		{
			data->apacheInfo.data->uptimeSec = atoi(value);
		}
		else if(strcmp(key, "BusyWorkers") == 0)
		{
			data->apacheInfo.data->busyWorkers = atoi(value);
		}
		else if(strcmp(key, "IdleWorkers") == 0)
		{
			data->apacheInfo.data->idleWorkers = atoi(value);
		}
	}

	fclose(stats);

	return;
}

void init_apachedata_desc(PERF_DATA_BLOCK *data)
{
	init_perf_counter(&(data->apacheInfo.apacheObjDesc),
			  &(data->apacheInfo.apacheObjDesc),
			  get_counter_id(data),
			  "Apache",
			  "The Apache performance objects consists of counters that describe the state of the Apache HTTP server",
			  0,
			  PERF_OBJECT);
	init_perf_counter(&(data->apacheInfo.totalAccesses),
			  &(data->apacheInfo.apacheObjDesc),
			  get_counter_id(data),
			  "Total Accesses",
			  "Total Accesses is the number of requests that have come in for all virtual servers since Apache has started",
			  PERF_SIZE_DWORD | PERF_TYPE_NUMBER | PERF_NUMBER_DECIMAL | PERF_DISPLAY_NO_SUFFIX,
			  PERF_COUNTER);
	init_perf_counter(&(data->apacheInfo.totalKBytes),
			  &(data->apacheInfo.apacheObjDesc),
			  get_counter_id(data),
			  "Total kBytes",
			  "Total kBytes is the total number of kBytes that have been served up by all the servers since this Apache was started",
			  PERF_SIZE_LARGE | PERF_TYPE_NUMBER | PERF_NUMBER_DECIMAL | PERF_DISPLAY_NO_SUFFIX,
			  PERF_COUNTER);
	init_perf_counter(&(data->apacheInfo.uptimeSec),
			  &(data->apacheInfo.apacheObjDesc),
			  get_counter_id(data),
			  "Server Uptime",
			  "Server Uptime is the number of seconds elapsed since the last server restart",
			  PERF_SIZE_DWORD | PERF_TYPE_NUMBER | PERF_NUMBER_DECIMAL | PERF_DISPLAY_NO_SUFFIX,
			  PERF_COUNTER);
	init_perf_counter(&(data->apacheInfo.reqPerSec),
			  &(data->apacheInfo.apacheObjDesc),
			  get_counter_id(data),
			  "Requests/second",
			  "Requests/second is the number of requests the server received in the last second",
/*			  PERF_SIZE_DWORD | PERF_TYPE_COUNTER | PERF_COUNTER_VALUE | PERF_DELTA_COUNTER | PERF_DISPLAY_PER_SEC,*/
			  PERF_SIZE_DWORD | PERF_TYPE_NUMBER | PERF_NUMBER_DECIMAL | PERF_DISPLAY_NO_SUFFIX,
			  PERF_COUNTER);
	init_perf_counter(&(data->apacheInfo.busyWorkers),
			  &(data->apacheInfo.apacheObjDesc),
			  get_counter_id(data),
			  "Busy Workers",
			  "Busy Workers is the number of workers that are currently serving requests",
			  PERF_SIZE_DWORD | PERF_TYPE_NUMBER | PERF_NUMBER_DECIMAL | PERF_DISPLAY_NO_SUFFIX,
			  PERF_COUNTER);
	init_perf_counter(&(data->apacheInfo.idleWorkers),
			  &(data->apacheInfo.apacheObjDesc),
			  get_counter_id(data),
			  "Idle Workers",
			  "Idle Workers is the number of workers that are currently serving requests",
			  PERF_SIZE_DWORD | PERF_TYPE_NUMBER | PERF_NUMBER_DECIMAL | PERF_DISPLAY_NO_SUFFIX,
			  PERF_COUNTER);

	return;
}

void get_apacheconfpath(PERF_DATA_BLOCK *data)
{
	int status;
	FILE *propFile;
	FILE *confFile;
	char *key, *value;
	char buf[PATH_MAX];

	propFile = fopen(CENTERIS_PROPERTIES_FILE, "r");
	memset((void *)&(data->apacheInfo.confFileName), 0, PATH_MAX);
	if(propFile == NULL)
	{
		/* Try default SuSE location */
		confFile = fopen(SUSE_HTTPD_CONF_PATH, "r");
		if(confFile == NULL)
		{
			/* Try default RHEL4 path */
			confFile = fopen(RHEL4_HTTPD_CONF_PATH, "r");
			if(confFile == NULL)
			{
				perror("get_apacheconfpath: Unable to locate httpd.conf file");
				exit(1);
			}
			sprintf(data->apacheInfo.confFileName, RHEL4_HTTPD_CONF_PATH);
			fclose(confFile);
			return;
		}
		sprintf(data->apacheInfo.confFileName, SUSE_HTTPD_CONF_PATH);
		fclose(confFile);
		return;
	}
	while(fgets(buf, sizeof(buf), propFile)) 
	{
		if(buf[0] == '#')
			continue;
		if(strchr(buf, '=') == NULL)
			continue;
		key = strtok(buf, "=");
		value = strtok(NULL, "\n");

		if(strncmp("HttpdConf", key, strlen("HttpdConf")) == 0)
		{
			confFile = fopen(value, "r");
			if(confFile == NULL)
			{
				perror("get_apacheconfpath: httpd.conf location specified in properties file does not exist");
				exit(1);
			}
			sprintf(data->apacheInfo.confFileName, value);
			fclose(confFile);
			fclose(propFile);
			return;
		}
	}
	
	if(strlen(data->apacheInfo.confFileName) == 0)
	{
		perror("get_apacheconfpath: no httpd.conf location specified in properties file");
		exit(1);
	}

	fclose(propFile);
	return;
}
void init_apache_data(PERF_DATA_BLOCK *data)
{
	data->apacheInfo.data = calloc(1, sizeof(*data->apacheInfo.data));
	if(!data->memInfo.data)
	{
		perror("init_apache_data: out of memory");
		exit(1);
	}

	get_apacheconfpath(data);
	get_apacheinfo(data);

	if(data->apacheInfo.isEnabled == TRUE)
	{
		init_apachedata_desc(data);
	}

	return;
}

void output_apache_desc(PERF_DATA_BLOCK *data, RuntimeSettings rt)
{
	if(data->apacheInfo.isEnabled == TRUE)
	{
		output_perf_desc(data->apacheInfo.apacheObjDesc, rt);
		output_perf_desc(data->apacheInfo.totalAccesses, rt);
		output_perf_desc(data->apacheInfo.totalKBytes, rt);
		output_perf_desc(data->apacheInfo.uptimeSec, rt);
		output_perf_desc(data->apacheInfo.reqPerSec, rt);
		output_perf_desc(data->apacheInfo.busyWorkers, rt);
		output_perf_desc(data->apacheInfo.idleWorkers, rt);
	}

	return;
}

void output_apacheinfo(PERF_DATA_BLOCK *data, RuntimeSettings rt, int tdb_flags)
{
	if(data->apacheInfo.isEnabled == TRUE)
	{
		output_perf_counter(data->apacheInfo.totalAccesses,
				    (unsigned long long)data->apacheInfo.data->totalAccesses,
				    rt, tdb_flags);
		output_perf_counter(data->apacheInfo.totalKBytes,
				    data->apacheInfo.data->totalKBytes,
				    rt, tdb_flags);
		output_perf_counter(data->apacheInfo.uptimeSec,
				    (unsigned long long)data->apacheInfo.data->uptimeSec,
				    rt, tdb_flags);
		output_perf_counter(data->apacheInfo.reqPerSec,
				    (unsigned long long)(data->apacheInfo.data->totalAccesses - data->apacheInfo.data->lastAccesses),
				    rt, tdb_flags);
		output_perf_counter(data->apacheInfo.busyWorkers,
				    (unsigned long long)data->apacheInfo.data->busyWorkers,
				    rt, tdb_flags);
		output_perf_counter(data->apacheInfo.idleWorkers,
				    (unsigned long long)data->apacheInfo.data->idleWorkers,
				    rt, tdb_flags);
	}

	return;
}
