/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

#include "perf.h"

#define MTAB "/etc/mtab"

void init_logical_diskdata_desc(PERF_DATA_BLOCK *data)
{
	init_perf_counter(&(data->logicalDiskInfo.diskObjDesc),
			  &(data->logicalDiskInfo.diskObjDesc),
			  get_counter_id(data),
			  "Logical Disk",
			  "The Logical Disk object consists of counters that show information about disks.",
			  0,
			  PERF_OBJECT);
	init_perf_counter(&(data->logicalDiskInfo.freeMegs),
			  &(data->logicalDiskInfo.diskObjDesc),
			  get_counter_id(data),
			  "Megabytes Free",
			  "The amount of available disk space, in megabytes.",
			  PERF_SIZE_LARGE | PERF_TYPE_NUMBER | PERF_NUMBER_DECIMAL | PERF_DISPLAY_NO_SUFFIX,
			  PERF_COUNTER);

	return;
}

int is_logical_disk(char *dev)
{
	char *sd, *hd, *mapper;

	if(sd = strstr(dev, "/dev/sd"))
		return 1;
	else if(hd = strstr(dev, "/dev/hd"))
		return 1;
	else if(mapper = strstr(dev, "/dev/mapper"))
		return 1;

	return 0;
}
	
void init_num_logical_disks(PERF_DATA_BLOCK *data)
{
	FILE *mtab;
	char buf[PROC_BUF];
	char *start;
	int i = 0, num;

	if(!(mtab = fopen(MTAB, "r")))
	{
		perror("init_disk_names: fopen");
		exit(1);
	}

	rewind(mtab);
	fflush(mtab);

	while(fgets(buf, sizeof(buf), mtab))
	{
		i+= is_logical_disk(buf);
		memset(buf, 0, PROC_BUF);
	}

	data->logicalDiskInfo.numDisks = i;
	fclose(mtab);

	return;
}
	
void init_logical_disk_names(PERF_DATA_BLOCK *data)
{
	FILE *mtab;
	char buf[PROC_BUF];
	char *line, *start, *stop;
	int i = 0, num;

	if(!(mtab = fopen(MTAB, "r")))
	{
		perror("init_logical_disk_names: fopen");
		exit(1);
	}

	rewind(mtab);
	fflush(mtab);

	while(fgets(buf, sizeof(buf), mtab))
	{
		if(is_logical_disk(buf))
		{
			line = buf;
			start = strsep(&line, " ");
			start = strsep(&line, " ");
			memset(data->logicalDiskInfo.mdata[i].mountpoint, 0, NAME_LEN);
			memcpy(data->logicalDiskInfo.mdata[i].mountpoint, start, strlen(start));
			i++;
		}
	}

	fclose(mtab);

	return;
}

void get_logical_diskinfo(PERF_DATA_BLOCK *data)
{
	int i;
	LogicalDiskData *p;
	struct statfs statfsbuf;
	int status;

	for(i = 0; i < data->logicalDiskInfo.numDisks; i++)
	{
		p = &(data->logicalDiskInfo.data[i]);
		status = statfs(data->logicalDiskInfo.mdata[i].mountpoint, &statfsbuf);
		p->freeMegs = (statfsbuf.f_bfree*statfsbuf.f_bsize)/1048576;
	}
	return;
}
void init_logical_disk_data(PERF_DATA_BLOCK *data)
{
	init_logical_diskdata_desc(data);
	
	init_num_logical_disks(data);

	data->logicalDiskInfo.mdata = calloc(data->logicalDiskInfo.numDisks, sizeof(LogicalDiskMetaData));
	if(!data->logicalDiskInfo.mdata)
	{
		fatal("init_disk_data: out of memory");
	}

	init_logical_disk_names(data);

	data->logicalDiskInfo.data = calloc(data->logicalDiskInfo.numDisks, sizeof(LogicalDiskData));
	if(!data->logicalDiskInfo.data)
	{
		fatal("init_disk_data: out of memory");
	}

	get_logical_diskinfo(data);

	return;
}

void output_logical_disk_desc(PERF_DATA_BLOCK *data, RuntimeSettings rt)
{
	output_perf_desc(data->logicalDiskInfo.diskObjDesc, rt);
	output_perf_desc(data->logicalDiskInfo.freeMegs, rt);
	output_num_instances(data->logicalDiskInfo.diskObjDesc, data->logicalDiskInfo.numDisks, rt);

	return;
}

void output_logical_diskinfo(PERF_DATA_BLOCK *data, RuntimeSettings rt, int tdb_flags)
{
	int i;
	
	output_perf_counter(data->logicalDiskInfo.freeMegs,
			    data->logicalDiskInfo.data[0].freeMegs,
			    rt, tdb_flags);

	for(i = 0; i < data->logicalDiskInfo.numDisks; i++)
	{
		output_perf_instance(data->logicalDiskInfo.diskObjDesc.index,
				     i,
				     (void *)&(data->logicalDiskInfo.data[i]),
				     sizeof(LogicalDiskData),
				     data->logicalDiskInfo.mdata[i].mountpoint,
				     rt, tdb_flags);
	}

	return;
}
