/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

#include "perf.h"

#define MTAB "/etc/mtab"

void init_physical_diskdata_desc(PERF_DATA_BLOCK *data)
{
	init_perf_counter(&(data->physicalDiskInfo.diskObjDesc),
			  &(data->physicalDiskInfo.diskObjDesc),
			  get_counter_id(data),
			  "Physical Disk",
			  "The Physical Disk object consists of counters that show information about underlying physical devices.",
			  0,
			  PERF_OBJECT);
	init_perf_counter(&(data->physicalDiskInfo.writesPerSec),
			  &(data->physicalDiskInfo.diskObjDesc),
			  get_counter_id(data),
			  "Writes/sec",
			  "The number of writes per second to that disk.",
			  PERF_SIZE_DWORD | PERF_TYPE_COUNTER | PERF_COUNTER_RATE | PERF_DELTA_COUNTER | PERF_DISPLAY_PER_SEC,
			  PERF_COUNTER);
	init_perf_counter(&(data->physicalDiskInfo.readsPerSec),
			  &(data->physicalDiskInfo.diskObjDesc),
			  get_counter_id(data),
			  "Reads/sec",
			  "The number of reads of that disk per second.",
			  PERF_SIZE_DWORD | PERF_TYPE_COUNTER | PERF_COUNTER_RATE | PERF_DELTA_COUNTER | PERF_DISPLAY_PER_SEC,
			  PERF_COUNTER);

	return;
}

int is_physical_disk(char *dev)
{
	char *sd, *hd;

	if(sd = strstr(dev, "/dev/sd"))
		return 1;
	else if(hd = strstr(dev, "/dev/hd"))
		return 1;

	return 0;
}
	
void init_num_physical_disks(PERF_DATA_BLOCK *data)
{
	FILE *mtab;
	char buf[PROC_BUF];
	char *start;
	int i = 0, num;

	if(!(mtab = fopen(MTAB, "r")))
	{
		perror("init_physical_disk_names: fopen");
		exit(1);
	}

	rewind(mtab);
	fflush(mtab);

	while(fgets(buf, sizeof(buf), mtab))
	{
		i+= is_physical_disk(buf);
		memset(buf, 0, PROC_BUF);
	}

	data->physicalDiskInfo.numDisks = i;
	fclose(mtab);

	return;
}
	
void init_physical_disk_names(PERF_DATA_BLOCK *data)
{
	FILE *mtab;
	char buf[PROC_BUF];
	char *line, *start, *stop;
	int i = 0, num;

	if(!(mtab = fopen(MTAB, "r")))
	{
		perror("init_physical_disk_names: fopen");
		exit(1);
	}

	rewind(mtab);
	fflush(mtab);

	while(fgets(buf, sizeof(buf), mtab))
	{
		if(is_physical_disk(buf))
		{
			line = buf;
			start = strsep(&line, " ");
			start = strrchr(start, '/');
			start +=1;
			memcpy(data->physicalDiskInfo.mdata[i].devname, start, strlen(start));
			i++;
		}
	}

	fclose(mtab);

	return;
}

void get_physical_diskinfo(PERF_DATA_BLOCK *data)
{
	int i;
	PhysicalDiskData *p;
	int status, num;
	char buf[LARGE_BUF], *start;
	FILE *diskstats;
	long reads, writes, discard;

	diskstats = fopen("/proc/diskstats", "r");
	rewind(diskstats);
	fflush(diskstats);
	status = fread(buf, sizeof(char), LARGE_BUF, diskstats);
	fclose(diskstats);

	for(i = 0; i < data->physicalDiskInfo.numDisks; i++)
	{
		p = &(data->physicalDiskInfo.data[i]);
		start = strstr(buf, data->physicalDiskInfo.mdata[i].devname);
		start += strlen(data->physicalDiskInfo.mdata[i].devname) + 1;
		num = sscanf(start, "%u %u %u %u",
			     &reads,
			     &discard, 
			     &writes, 
			     &discard);
		p->writesPerSec = writes;
		p->readsPerSec = reads;
	}
	return;
}
void init_physical_disk_data(PERF_DATA_BLOCK *data)
{
	init_physical_diskdata_desc(data);
	
	init_num_physical_disks(data);

	data->physicalDiskInfo.mdata = calloc(data->physicalDiskInfo.numDisks, sizeof(PhysicalDiskMetaData));
	if(!data->physicalDiskInfo.mdata)
	{
		fatal("init_physical_disk_data: out of memory");
	}

	init_physical_disk_names(data);

	data->physicalDiskInfo.data = calloc(data->physicalDiskInfo.numDisks, sizeof(PhysicalDiskData));
	if(!data->physicalDiskInfo.data)
	{
		fatal("init_physical_disk_data: out of memory");
	}

	get_physical_diskinfo(data);

	return;
}

void output_physical_disk_desc(PERF_DATA_BLOCK *data, RuntimeSettings rt)
{
	output_perf_desc(data->physicalDiskInfo.diskObjDesc, rt);
	output_perf_desc(data->physicalDiskInfo.writesPerSec, rt);
	output_perf_desc(data->physicalDiskInfo.readsPerSec, rt);
	output_num_instances(data->physicalDiskInfo.diskObjDesc, data->physicalDiskInfo.numDisks, rt);

	return;
}

void output_physical_diskinfo(PERF_DATA_BLOCK *data, RuntimeSettings rt, int tdb_flags)
{
	int i;
	
	output_perf_counter(data->physicalDiskInfo.writesPerSec,
			    (unsigned long long)data->physicalDiskInfo.data[0].writesPerSec,
			    rt, tdb_flags);
	output_perf_counter(data->physicalDiskInfo.readsPerSec,
			    (unsigned long long)data->physicalDiskInfo.data[0].readsPerSec,
			    rt, tdb_flags);

	for(i = 0; i < data->physicalDiskInfo.numDisks; i++)
	{
		output_perf_instance(data->physicalDiskInfo.diskObjDesc.index,
				     i,
				     (void *)&(data->physicalDiskInfo.data[i]),
				     sizeof(PhysicalDiskData),
				     data->physicalDiskInfo.mdata[i].devname,
				     rt, tdb_flags);
	}

	return;
}
