/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

#include "perf.h"

void get_processinfo(PERF_DATA_BLOCK *data)
{
  int status;
  struct sysinfo info;
  status = sysinfo(&info);

  data->processInfo.data->runningProcessCount = (unsigned int)info.procs;

  return;
}

void init_processdata_desc(PERF_DATA_BLOCK *data)
{
  init_perf_counter(&(data->processInfo.processObjDesc),
		    &(data->processInfo.processObjDesc),
		    get_counter_id(data),
		    "Processes",
		    "%The Processes performance object displays aggregate information about processes on the machine.",
		    0,
		    PERF_OBJECT);
  init_perf_counter(&(data->processInfo.runningProcessCount),
		    &(data->processInfo.processObjDesc),
		    get_counter_id(data),
		    "Process Count",
		    "Process Count is the number of processes currently on the machine.",
		    PERF_SIZE_DWORD | PERF_TYPE_NUMBER | PERF_NUMBER_DECIMAL | PERF_DISPLAY_NO_SUFFIX,
		    PERF_COUNTER);
   
  return;
}

void init_process_data(PERF_DATA_BLOCK *data)
{
  data->processInfo.data = calloc(1, sizeof(*data->processInfo.data));
  if(!(data->processInfo.data))
    {
      perror("init_process_data: out of memory");
      exit(1);
    }
  
  init_processdata_desc(data);

  get_processinfo(data);

  return;
}

void output_processinfo(PERF_DATA_BLOCK *data, RuntimeSettings rt, int tdb_flags)
{
	output_perf_counter(data->processInfo.runningProcessCount,
			    (unsigned long long)data->processInfo.data->runningProcessCount,
			    rt, tdb_flags);

	return;
}

void output_process_desc(PERF_DATA_BLOCK *data, RuntimeSettings rt)
{
	output_perf_desc(data->processInfo.processObjDesc, rt);
	output_perf_desc(data->processInfo.runningProcessCount, rt);

	return;
}
