/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Net;

namespace Centeris.MMC.DeviceSnapin
{
	/// <summary>
	/// Summary description for ConnectForm.
	/// </summary>
	public class ConnectForm : System.Windows.Forms.Form
	{
		// some constants
		protected const string sErrorCaption = "Error";
		protected const string sErrorMsg = "An error occurred while reading the device information for this server.\r\nError: {0}";
		protected const string sErrorResMsg = "{0} is not a valid Centeris-enabled server.";
		protected const string sErrorAuthMsg = "You do not have sufficient user credentials to access this server.";

		// from LMERR.H
		protected const int NERR_InvalidComputer = 2351;

		private System.Windows.Forms.Button btnOK;
		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.TextBox tbTarget;
		private System.Windows.Forms.Button btnBrowse;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.HelpProvider helpProvider1;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public ConnectForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

		}

		public ConnectForm(string sRemoteName) : this()
		{
			tbTarget.Text = sRemoteName;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		public string Target
		{
			get
			{
				return tbTarget.Text.Trim();
			}
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnOK = new System.Windows.Forms.Button();
			this.btnCancel = new System.Windows.Forms.Button();
			this.tbTarget = new System.Windows.Forms.TextBox();
			this.btnBrowse = new System.Windows.Forms.Button();
			this.label1 = new System.Windows.Forms.Label();
			this.panel1 = new System.Windows.Forms.Panel();
			this.helpProvider1 = new System.Windows.Forms.HelpProvider();
			this.SuspendLayout();
			// 
			// btnOK
			// 
			this.btnOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.btnOK.Enabled = false;
			this.helpProvider1.SetHelpString(this.btnOK, "Closes the dialog and saves any changes you have made");
			this.btnOK.Location = new System.Drawing.Point(233, 90);
			this.btnOK.Name = "btnOK";
			this.helpProvider1.SetShowHelp(this.btnOK, true);
			this.btnOK.Size = new System.Drawing.Size(87, 21);
			this.btnOK.TabIndex = 2;
			this.btnOK.Text = "OK";
			this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
			// 
			// btnCancel
			// 
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.helpProvider1.SetHelpString(this.btnCancel, "Closes the dialog discarding any changes that you have made");
			this.btnCancel.Location = new System.Drawing.Point(327, 90);
			this.btnCancel.Name = "btnCancel";
			this.helpProvider1.SetShowHelp(this.btnCancel, true);
			this.btnCancel.Size = new System.Drawing.Size(86, 21);
			this.btnCancel.TabIndex = 3;
			this.btnCancel.Text = "Cancel";
			// 
			// tbTarget
			// 
			this.helpProvider1.SetHelpString(this.tbTarget, "Provides a space for you to type in the name of the computer that you want to man" +
				"age");
			this.tbTarget.Location = new System.Drawing.Point(7, 35);
			this.tbTarget.Name = "tbTarget";
			this.helpProvider1.SetShowHelp(this.tbTarget, true);
			this.tbTarget.Size = new System.Drawing.Size(313, 20);
			this.tbTarget.TabIndex = 0;
			this.tbTarget.Text = "";
			this.tbTarget.TextChanged += new System.EventHandler(this.tbTarget_TextChanged);
			// 
			// btnBrowse
			// 
			this.helpProvider1.SetHelpString(this.btnBrowse, "Click to search for the name of a computer that you want to manage");
			this.btnBrowse.Location = new System.Drawing.Point(327, 35);
			this.btnBrowse.Name = "btnBrowse";
			this.helpProvider1.SetShowHelp(this.btnBrowse, true);
			this.btnBrowse.Size = new System.Drawing.Size(86, 20);
			this.btnBrowse.TabIndex = 1;
			this.btnBrowse.Text = "&Browse...";
			this.btnBrowse.Click += new System.EventHandler(this.btnBrowse_Click);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(7, 17);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(313, 20);
			this.label1.TabIndex = 6;
			this.label1.Text = "&Specify the Linux computer you want this snap-in to manage.";
			// 
			// panel1
			// 
			this.panel1.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.panel1.Location = new System.Drawing.Point(0, 78);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(421, 1);
			this.panel1.TabIndex = 7;
			// 
			// ConnectForm
			// 
			this.AcceptButton = this.btnOK;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.btnCancel;
			this.ClientSize = new System.Drawing.Size(421, 119);
			this.Controls.Add(this.tbTarget);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.btnOK);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.btnBrowse);
			this.Controls.Add(this.panel1);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.HelpButton = true;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "ConnectForm";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Device Manager";
			this.ResumeLayout(false);

		}
		#endregion

		private void btnBrowse_Click(object sender, System.EventArgs e)
		{
			ADObject[] rgo =  DsPicker.Show(this.Handle);
			if (rgo!=null && rgo.Length>0)
				tbTarget.Text = rgo[0].Name;
			
		}

		private void btnOK_Click(object sender, System.EventArgs e)
		{
			// do a minimal check on the name field
			string s = Target;

			if (s.StartsWith(@"\\"))
			{
				// the target must be coming from the text box. Fix it
				// there and then retrieve it again
				tbTarget.Text = s.Substring(2);

				s = Target;
			}

			// validate the name?
			if (s!="")
			{
				// TODO: A quick check to see if it has port 9105 open?

				// try to fetch data from the machine - this will throw an exception on failure
				try
				{
					DeviceNode.GetXMLFromServer(this.Handle, s);
					this.DialogResult = DialogResult.OK;
				}
				catch(WebException we)
				{
					// if we can't resolve the name, then we're toast
					if (we.Status==WebExceptionStatus.NameResolutionFailure)
					{
						string sError = string.Format(sErrorResMsg, s);
						MessageBox.Show(sError, sErrorCaption, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					}
					else if (we.Status==WebExceptionStatus.ProtocolError)
					{
						// Check for auth error
						MessageBox.Show(sErrorAuthMsg, sErrorCaption, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					}
					else
					{
						// generic failure
						string sError2 = string.Format(sErrorMsg, we.Message);
						MessageBox.Show(sError2, sErrorCaption, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					}

					this.DialogResult = DialogResult.Cancel;
				}
				catch(Exception ex)
				{
					// generic error
					string sError = string.Format(sErrorMsg, ex.Message);
					MessageBox.Show(sError, sErrorCaption, MessageBoxButtons.OK, MessageBoxIcon.Warning);

					this.DialogResult = DialogResult.Cancel;
				}
			}
			else
			{
				// blank file names are not cool
				this.DialogResult = DialogResult.Cancel;
			}
		}

		private void tbTarget_TextChanged(object sender, System.EventArgs e)
		{
			// if non-empty (and not \ or \\), enable the ok button
			string s = tbTarget.Text.Trim();
			if (s.Length>0 && s!=@"\" && s!=@"\\")
				this.btnOK.Enabled = true;
			else
				this.btnOK.Enabled = false;
		}

	}
}
