/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace Centeris.MMC.DeviceSnapin
{
	/// <summary>
	/// Form that displays a device node's properties
	/// </summary>
	public class PropPage : System.Windows.Forms.Form
	{
		#region Data

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Button btnOk;
		private System.Windows.Forms.ColumnHeader Property;
		private System.Windows.Forms.ColumnHeader Value;
		private System.Windows.Forms.ListView list;
		private System.Windows.Forms.PictureBox pbIcon;
		private System.Windows.Forms.Label labelNoProps;
		private System.Windows.Forms.Label labelName;

		// backreference to the node whose props are being shown
		TreeNode tn = null;

		#endregion

		#region Constructor, Dispose

		/// <summary>
		/// Should only be called by designer
		/// </summary>
		public PropPage()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

		}

		// called when have a valid treenode with properties
		public PropPage(TreeNode tn)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// hide/show necessary stuff
			list.Visible = true;
			pbIcon.Visible = false;
			labelName.Visible = false;
			labelNoProps.Visible = false;

			this.tn = tn;
		}

		// called when don't have a treenode or properies
		public PropPage(string sName, Image icon, bool bGroup)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// initialize
			pbIcon.Image = icon;
			labelName.Text = sName;

			// hide/show necessary stuff
			list.Visible = false;
			pbIcon.Visible = true;
			labelName.Visible = true;
			labelNoProps.Visible = !bGroup;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnOk = new System.Windows.Forms.Button();
			this.list = new System.Windows.Forms.ListView();
			this.Property = new System.Windows.Forms.ColumnHeader();
			this.Value = new System.Windows.Forms.ColumnHeader();
			this.pbIcon = new System.Windows.Forms.PictureBox();
			this.labelName = new System.Windows.Forms.Label();
			this.labelNoProps = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// btnOk
			// 
			this.btnOk.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.btnOk.Location = new System.Drawing.Point(368, 408);
			this.btnOk.Name = "btnOk";
			this.btnOk.TabIndex = 1;
			this.btnOk.Text = "OK";
			this.btnOk.Click += new System.EventHandler(this.btnOk_Click);
			// 
			// list
			// 
			this.list.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																				   this.Property,
																				   this.Value});
			this.list.Location = new System.Drawing.Point(8, 8);
			this.list.Name = "list";
			this.list.Size = new System.Drawing.Size(430, 384);
			this.list.TabIndex = 2;
			this.list.View = System.Windows.Forms.View.Details;
			this.list.Visible = false;
			// 
			// Property
			// 
			this.Property.Text = "Property";
			this.Property.Width = 140;
			// 
			// Value
			// 
			this.Value.Text = "Value";
			this.Value.Width = 265;
			// 
			// pbIcon
			// 
			this.pbIcon.Location = new System.Drawing.Point(24, 27);
			this.pbIcon.Name = "pbIcon";
			this.pbIcon.Size = new System.Drawing.Size(32, 32);
			this.pbIcon.SizeMode = System.Windows.Forms.PictureBoxSizeMode.StretchImage;
			this.pbIcon.TabIndex = 3;
			this.pbIcon.TabStop = false;
			this.pbIcon.Visible = false;
			// 
			// labelName
			// 
			this.labelName.Location = new System.Drawing.Point(64, 32);
			this.labelName.Name = "labelName";
			this.labelName.Size = new System.Drawing.Size(368, 40);
			this.labelName.TabIndex = 4;
			this.labelName.Visible = false;
			// 
			// labelNoProps
			// 
			this.labelNoProps.Location = new System.Drawing.Point(24, 104);
			this.labelNoProps.Name = "labelNoProps";
			this.labelNoProps.Size = new System.Drawing.Size(336, 23);
			this.labelNoProps.TabIndex = 5;
			this.labelNoProps.Text = "This device does not provide any visible properties.";
			this.labelNoProps.Visible = false;
			// 
			// PropPage
			// 
			this.AcceptButton = this.btnOk;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.btnOk;
			this.ClientSize = new System.Drawing.Size(450, 446);
			this.ControlBox = false;
			this.Controls.Add(this.labelNoProps);
			this.Controls.Add(this.labelName);
			this.Controls.Add(this.pbIcon);
			this.Controls.Add(this.list);
			this.Controls.Add(this.btnOk);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.KeyPreview = true;
			this.Name = "PropPage";
			this.ShowInTaskbar = false;
			this.Text = "Properties";
			this.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.PropPage_KeyPress);
			this.ResumeLayout(false);

		}
		#endregion
	
		#region Other methods
		protected ArrayList SplitString(string s)
		{
			// create the return array
			ArrayList al = new ArrayList();

			// iterate through the string
			string sCurrent = "";
			int iLastSeparator=-1;
			for(int i=0; i<s.Length; i++)
			{
				// fetch a source character
				char ch = s[i];

				// add it to the current string
				sCurrent += ch;

				// remember if this is a break char
				if (ch==' ' || ch==',')
					iLastSeparator = sCurrent.Length-1;

				// see if we've exceeded our size goal
				if (sCurrent.Length>40)
				{
					// yep, back out to last separator, if we can
					if (iLastSeparator>0)
					{
						// see if we need to lop off anything
						string sExcess = "";
						if (iLastSeparator<sCurrent.Length-1)
						{
							// yep
							sExcess = sCurrent.Substring(iLastSeparator+1);
							sCurrent = sCurrent.Substring(0, iLastSeparator+1);
						}

						// add the current stuff
						al.Add(sCurrent);

						// zero out (adding excess)
						sCurrent = "" + sExcess;

						// reset for next line
						iLastSeparator = -1;
	
					}
					else
					{
						// break arbitrarily
						al.Add(sCurrent);
						sCurrent = "";
					}
				}
			}

			// if anything left, add it
			if (sCurrent.Length>0)
				al.Add(sCurrent);

			return al;
		}

		protected override void OnLoad(EventArgs e)
		{
			// if there are any name value pairs associated with the node,
			// put them in our table.
			if (tn!=null && tn.Tag!=null)
			{
				Hashtable ht = (Hashtable) tn.Tag;

				// get the keys and sort them
				ICollection colKeys = ht.Keys;
				ArrayList arKeys = new ArrayList(colKeys);
				arKeys.Sort();

				// iterate through each key
				foreach (string sKey in arKeys)
				{
					// fetch the data for the key
					string s = (string) ht[sKey];

					// split them, if necessary
					ArrayList arData = SplitString(s);

					// add the first one
					ListViewItem lvi = new ListViewItem(sKey);
					if (arData.Count>0)
						lvi.SubItems.Add((string) arData[0]);
					list.Items.Add(lvi);

					// add more if present
					for (int i=1; i<arData.Count; i++)
					{
						lvi = new ListViewItem("");
						lvi.SubItems.Add((string) arData[i]);
						list.Items.Add(lvi);
					}
				}
			}

			base.OnLoad (e);
		}

		private void btnOk_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

		/// <summary>
		/// Note that we must handle the Esc and Enter keypresses here since the embedded treeview will otherwise
		/// swallow them
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void PropPage_KeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e)
		{
			if ( (e.KeyChar==(char)13) ||(e.KeyChar==(char)27))
			{
				e.Handled = true;
				this.Close();
			}
		}

		#endregion

	}
}
