/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace Centeris.MMC.PrinterSnapin
{
	/// <summary>
	/// Summary description for ConnectForm.
	/// </summary>
	public class ConnectForm : System.Windows.Forms.Form
	{
		// some constants
		protected const string sBadPathError = "{0} is not a valid Microsoft Windows or Centeris-enabled Linux server.";
		protected const string sErrorCaption = "Error";

		// from LMERR.H
		protected const int NERR_InvalidComputer = 2351;

		private System.Windows.Forms.RadioButton rbLocal;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.Button btnBrowse;
		private System.Windows.Forms.TextBox tbTarget;
		private System.Windows.Forms.Button btnOK;
		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.RadioButton rbRemote;
		private System.Windows.Forms.HelpProvider help;
		private System.ComponentModel.IContainer components=null;

		public ConnectForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

		}

		public ConnectForm(bool bRemoteMode, string sRemoteName) : this()
		{
			// set the initial control settings
			if (bRemoteMode)
			{
				rbLocal.Checked = false;
				rbRemote.Checked = true;
				tbTarget.Text = sRemoteName;
			}
			else
			{
				rbLocal.Checked = true;
				rbRemote.Checked = false;
				tbTarget.Text = "";
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		public string Target
		{
			get
			{
				if (rbRemote.Checked)
					return tbTarget.Text.Trim();
				else
					return "";
			}
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.rbLocal = new System.Windows.Forms.RadioButton();
			this.rbRemote = new System.Windows.Forms.RadioButton();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.tbTarget = new System.Windows.Forms.TextBox();
			this.btnBrowse = new System.Windows.Forms.Button();
			this.btnOK = new System.Windows.Forms.Button();
			this.btnCancel = new System.Windows.Forms.Button();
			this.help = new System.Windows.Forms.HelpProvider();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// rbLocal
			// 
			this.rbLocal.Checked = true;
			this.help.SetHelpString(this.rbLocal, "Specifies that you want to manage the local computer");
			this.rbLocal.Location = new System.Drawing.Point(16, 29);
			this.rbLocal.Name = "rbLocal";
			this.help.SetShowHelp(this.rbLocal, true);
			this.rbLocal.Size = new System.Drawing.Size(336, 21);
			this.rbLocal.TabIndex = 0;
			this.rbLocal.TabStop = true;
			this.rbLocal.Tag = "";
			this.rbLocal.Text = "&Local computer (the computer this console is running on)";
			this.rbLocal.CheckedChanged += new System.EventHandler(this.rbLocal_CheckedChanged);
			// 
			// rbRemote
			// 
			this.help.SetHelpString(this.rbRemote, "Specifies that you want to manage a remote computer");
			this.rbRemote.Location = new System.Drawing.Point(16, 50);
			this.rbRemote.Name = "rbRemote";
			this.help.SetShowHelp(this.rbRemote, true);
			this.rbRemote.Size = new System.Drawing.Size(136, 21);
			this.rbRemote.TabIndex = 1;
			this.rbRemote.Tag = "";
			this.rbRemote.Text = "&Another computer.";
			this.rbRemote.CheckedChanged += new System.EventHandler(this.radioButton1_CheckedChanged);
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.tbTarget);
			this.groupBox1.Controls.Add(this.btnBrowse);
			this.groupBox1.Location = new System.Drawing.Point(8, 11);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(512, 77);
			this.groupBox1.TabIndex = 3;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Select the computer you want this snap-in to manage.";
			// 
			// tbTarget
			// 
			this.tbTarget.Enabled = false;
			this.help.SetHelpString(this.tbTarget, "Provides a space for you to type in the name of the computer that you want to man" +
				"age");
			this.tbTarget.Location = new System.Drawing.Point(152, 42);
			this.tbTarget.Name = "tbTarget";
			this.help.SetShowHelp(this.tbTarget, true);
			this.tbTarget.Size = new System.Drawing.Size(248, 20);
			this.tbTarget.TabIndex = 2;
			this.tbTarget.Tag = "";
			this.tbTarget.Text = "";
			this.tbTarget.TextChanged += new System.EventHandler(this.tbTarget_TextChanged);
			// 
			// btnBrowse
			// 
			this.btnBrowse.Enabled = false;
			this.help.SetHelpString(this.btnBrowse, "Click to search for the name of a computer that you want to manage");
			this.btnBrowse.Location = new System.Drawing.Point(416, 42);
			this.btnBrowse.Name = "btnBrowse";
			this.help.SetShowHelp(this.btnBrowse, true);
			this.btnBrowse.Size = new System.Drawing.Size(87, 20);
			this.btnBrowse.TabIndex = 3;
			this.btnBrowse.Tag = "";
			this.btnBrowse.Text = "&Browse...";
			this.btnBrowse.Click += new System.EventHandler(this.btnBrowse_Click);
			// 
			// btnOK
			// 
			this.btnOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.help.SetHelpString(this.btnOK, "Closes the dialog and saves any changes you have made");
			this.btnOK.Location = new System.Drawing.Point(328, 96);
			this.btnOK.Name = "btnOK";
			this.help.SetShowHelp(this.btnOK, true);
			this.btnOK.Size = new System.Drawing.Size(87, 21);
			this.btnOK.TabIndex = 4;
			this.btnOK.Tag = "";
			this.btnOK.Text = "OK";
			this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
			// 
			// btnCancel
			// 
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.help.SetHelpString(this.btnCancel, "Closes the dialog discarding any changes that you have made");
			this.btnCancel.Location = new System.Drawing.Point(424, 96);
			this.btnCancel.Name = "btnCancel";
			this.help.SetShowHelp(this.btnCancel, true);
			this.btnCancel.Size = new System.Drawing.Size(87, 21);
			this.btnCancel.TabIndex = 5;
			this.btnCancel.Tag = "";
			this.btnCancel.Text = "Cancel";
			// 
			// ConnectForm
			// 
			this.AcceptButton = this.btnOK;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.btnCancel;
			this.ClientSize = new System.Drawing.Size(530, 126);
			this.Controls.Add(this.rbRemote);
			this.Controls.Add(this.rbLocal);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.btnOK);
			this.Controls.Add(this.btnCancel);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.HelpButton = true;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "ConnectForm";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Tag = "This dialog allows you to select the computer that you want to manage";
			this.Text = "Shared Printers";
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void radioButton1_CheckedChanged(object sender, System.EventArgs e)
		{
			if (rbRemote.Checked)
			{
				tbTarget.Enabled = true;
				btnBrowse.Enabled = true;

				// disable the ok button if we don't have a minimally valid server name
				string s = tbTarget.Text.Trim();
				if (s.Length==0 || s==@"\" || s==@"\\")
					btnOK.Enabled = false;
			}
		}

		private void rbLocal_CheckedChanged(object sender, System.EventArgs e)
		{
			if (rbLocal.Checked)
			{
				tbTarget.Enabled = false;
				btnBrowse.Enabled = false;

				// enable the ok button
				btnOK.Enabled = true;
			}
		}

		private void btnBrowse_Click(object sender, System.EventArgs e)
		{
			ADObject[] rgo =  DsPicker.Show(this.Handle);
			if (rgo!=null && rgo.Length>0)
				tbTarget.Text = rgo[0].Name;
			
		}

		private void btnOK_Click(object sender, System.EventArgs e)
		{
			// do a minimal check on the name field
			string s = Target;

			if (s.StartsWith(@"\\"))
			{
				// the target must be coming from the text box. Fix it
				// there and then retrieve it again
				tbTarget.Text = s.Substring(2);

				s = Target;
			}

			// validate the name?
			if (s!="")
			{
				// try to fetch printer info
				System.Data.DataTable dt = PrintAPI.CreateDataTable();
				try
				{
					PrintAPI.FetchPrinterData(s, dt);
				}
				catch(Exception)
				{
					// if failed, not good
					string sMessage = string.Format(sBadPathError, s);
					MessageBox.Show(sMessage, sErrorCaption);
					DialogResult = DialogResult.None;
				}
			}
			
			// NOTE: empty server names indicate localhost
		}

		private void tbTarget_TextChanged(object sender, System.EventArgs e)
		{
			// disable the ok button if we don't have a minimally valid server name
			string s = tbTarget.Text.Trim();
			if (s.Length==0 || s==@"\" || s==@"\\")
				btnOK.Enabled = false;
			else
				btnOK.Enabled = true;
		
		}

	}
}
