/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.Runtime.InteropServices;
using Ironring.Management.MMC;

///
/// This is a big ugly file just to deal with the "Browse..." button in the
/// connectform. It would be appropriate to split it up, but I hate to introduce
/// so much namespace cruft just for this stupid functionality [MV 03-23-05]
/// 
namespace Centeris.MMC.PrinterSnapin
{
	#region Interop structures
	/// <summary>
	/// The DSOP_INIT_INFO structure contains data required to initialize an object picker dialog box. 
	/// </summary>
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
	public struct DSOP_INIT_INFO
	{
		public uint cbSize;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string pwzTargetComputer;
		public uint cDsScopeInfos;
		public IntPtr aDsScopeInfos;
		public uint flOptions;
		public uint cAttributesToFetch;
		public IntPtr apwzAttributeNames;
	}


	/// <summary>
	/// The DSOP_SCOPE_INIT_INFO structure describes one or more scope types that have the same attributes. A scope type is a type of location, for example a domain, computer, or Global Catalog, from which the user can select objects.
	/// A scope type is a type of location, for example a domain, computer, or Global Catalog, from which the user can select objects. 
	/// </summary>
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto), Serializable]
	public struct DSOP_SCOPE_INIT_INFO
	{
		public uint cbSize;
		public uint flType;
		public uint flScope;
		[MarshalAs(UnmanagedType.Struct)]
		public DSOP_FILTER_FLAGS FilterFlags;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string pwzDcName; 
		[MarshalAs(UnmanagedType.LPWStr)]
		public string pwzADsPath;
		public uint hr;
	}

	/// <summary>
	/// The DSOP_UPLEVEL_FILTER_FLAGS structure contains flags that indicate the filters to use for an up-level scope. An up-level scope is a scope that supports the ADSI LDAP provider.
	/// </summary>
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
	public struct DSOP_UPLEVEL_FILTER_FLAGS
	{
		public uint flBothModes;
		public uint flMixedModeOnly;
		public uint flNativeModeOnly;
	}

	/// <summary>
	/// The DSOP_FILTER_FLAGS structure contains flags that indicate the types of objects presented to the user for a specified scope or scopes.
	/// </summary>
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
	public struct DSOP_FILTER_FLAGS
	{
		public DSOP_UPLEVEL_FILTER_FLAGS Uplevel;
		public uint flDownlevel;
	}

	/// <summary>
	/// The DS_SELECTION structure contains data about an object the user selected from an object picker dialog box. 
	/// The DS_SELECTION_LIST structure contains an array of DS_SELECTION structures.
	/// </summary>
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
	public struct DS_SELECTION
	{
		[MarshalAs(UnmanagedType.LPWStr)]
		public string pwzName;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string pwzADsPath;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string pwzClass;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string pwzUPN;
		public IntPtr pvarFetchedAttributes;
		public uint flScopeType;
	}
	/// <summary>
	/// The DS_SELECTION_LIST structure contains data about the objects the user selected from an object picker dialog box.
	/// This structure is supplied by the IDataObject interface supplied by the IDsObjectPicker::InvokeDialog method in the CFSTR_DSOP_DS_SELECTION_LIST data format.
	/// </summary>
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
	public struct DS_SELECTION_LIST
	{
		public uint cItems;
		public uint cFetchedAttributes;
		public DS_SELECTION[] aDsSelection;
	}

	/// <summary>
	/// Flags that indicate the scope types described by this structure. You can combine multiple scope types if all specified scopes use the same settings. 
	/// </summary>
	public class DSOP_SCOPE_TYPE_FLAGS
	{
		public const uint DSOP_SCOPE_TYPE_TARGET_COMPUTER = 0x00000001;
		public const uint DSOP_SCOPE_TYPE_UPLEVEL_JOINED_DOMAIN = 0x00000002;
		public const uint DSOP_SCOPE_TYPE_DOWNLEVEL_JOINED_DOMAIN = 0x00000004;
		public const uint DSOP_SCOPE_TYPE_ENTERPRISE_DOMAIN = 0x00000008;
		public const uint DSOP_SCOPE_TYPE_GLOBAL_CATALOG = 0x00000010;
		public const uint DSOP_SCOPE_TYPE_EXTERNAL_UPLEVEL_DOMAIN = 0x00000020;
		public const uint DSOP_SCOPE_TYPE_EXTERNAL_DOWNLEVEL_DOMAIN = 0x00000040;
		public const uint DSOP_SCOPE_TYPE_WORKGROUP = 0x00000080;
		public const uint DSOP_SCOPE_TYPE_USER_ENTERED_UPLEVEL_SCOPE = 0x00000100;
		public const uint DSOP_SCOPE_TYPE_USER_ENTERED_DOWNLEVEL_SCOPE = 0x00000200;
	}

	/// <summary>
	/// Flags that determine the object picker options.
	/// </summary>
	public class DSOP_INIT_INFO_FLAGS
	{
		public const uint DSOP_FLAG_MULTISELECT = 0x00000001;
		public const uint DSOP_FLAG_SKIP_TARGET_COMPUTER_DC_CHECK = 0x00000002;
	}

	/// <summary>
	/// Flags that indicate the format used to return ADsPath for objects selected from this scope. The flScope member can also indicate the initial scope displayed in the Look in drop-down list. 
	/// </summary>
	public class DSOP_SCOPE_INIT_INFO_FLAGS
	{
		public const uint DSOP_SCOPE_FLAG_STARTING_SCOPE =0x00000001;
		public const uint DSOP_SCOPE_FLAG_WANT_PROVIDER_WINNT =0x00000002;
		public const uint DSOP_SCOPE_FLAG_WANT_PROVIDER_LDAP =0x00000004;
		public const uint DSOP_SCOPE_FLAG_WANT_PROVIDER_GC =0x00000008;
		public const uint DSOP_SCOPE_FLAG_WANT_SID_PATH =0x00000010;
		public const uint DSOP_SCOPE_FLAG_WANT_DOWNLEVEL_BUILTIN_PATH =0x00000020;
		public const uint DSOP_SCOPE_FLAG_DEFAULT_FILTER_USERS =0x00000040;
		public const uint DSOP_SCOPE_FLAG_DEFAULT_FILTER_GROUPS =0x00000080;
		public const uint DSOP_SCOPE_FLAG_DEFAULT_FILTER_COMPUTERS =0x00000100;
		public const uint DSOP_SCOPE_FLAG_DEFAULT_FILTER_CONTACTS =0x00000200;
	}

	/// <summary>
	/// Filter flags to use for an up-level scope, regardless of whether it is a mixed or native mode domain. 
	/// </summary>
	public class DSOP_FILTER_FLAGS_FLAGS
	{
		public const uint DSOP_FILTER_INCLUDE_ADVANCED_VIEW = 0x00000001;
		public const uint DSOP_FILTER_USERS = 0x00000002;
		public const uint DSOP_FILTER_BUILTIN_GROUPS = 0x00000004;
		public const uint DSOP_FILTER_WELL_KNOWN_PRINCIPALS = 0x00000008;
		public const uint DSOP_FILTER_UNIVERSAL_GROUPS_DL = 0x00000010;
		public const uint DSOP_FILTER_UNIVERSAL_GROUPS_SE = 0x00000020;
		public const uint DSOP_FILTER_GLOBAL_GROUPS_DL = 0x00000040;
		public const uint DSOP_FILTER_GLOBAL_GROUPS_SE = 0x00000080;
		public const uint DSOP_FILTER_DOMAIN_LOCAL_GROUPS_DL = 0x00000100;
		public const uint DSOP_FILTER_DOMAIN_LOCAL_GROUPS_SE = 0x00000200;
		public const uint DSOP_FILTER_CONTACTS = 0x00000400;
		public const uint DSOP_FILTER_COMPUTERS = 0x00000800;
	}

	/// <summary>
	/// Contains the filter flags to use for down-level scopes
	/// </summary>
	public class DSOP_DOWNLEVEL_FLAGS
	{
		public const uint DSOP_DOWNLEVEL_FILTER_USERS = 0x80000001;
		public const uint DSOP_DOWNLEVEL_FILTER_LOCAL_GROUPS = 0x80000002;
		public const uint DSOP_DOWNLEVEL_FILTER_GLOBAL_GROUPS = 0x80000004;
		public const uint DSOP_DOWNLEVEL_FILTER_COMPUTERS = 0x80000008;
		public const uint DSOP_DOWNLEVEL_FILTER_WORLD = 0x80000010;
		public const uint DSOP_DOWNLEVEL_FILTER_AUTHENTICATED_USER = 0x80000020;
		public const uint DSOP_DOWNLEVEL_FILTER_ANONYMOUS = 0x80000040;
		public const uint DSOP_DOWNLEVEL_FILTER_BATCH = 0x80000080;
		public const uint DSOP_DOWNLEVEL_FILTER_CREATOR_OWNER = 0x80000100;
		public const uint DSOP_DOWNLEVEL_FILTER_CREATOR_GROUP = 0x80000200;
		public const uint DSOP_DOWNLEVEL_FILTER_DIALUP = 0x80000400;
		public const uint DSOP_DOWNLEVEL_FILTER_INTERACTIVE = 0x80000800;
		public const uint DSOP_DOWNLEVEL_FILTER_NETWORK = 0x80001000;
		public const uint DSOP_DOWNLEVEL_FILTER_SERVICE = 0x80002000;
		public const uint DSOP_DOWNLEVEL_FILTER_SYSTEM = 0x80004000;
		public const uint DSOP_DOWNLEVEL_FILTER_EXCLUDE_BUILTIN_GROUPS = 0x80008000;
		public const uint DSOP_DOWNLEVEL_FILTER_TERMINAL_SERVER = 0x80010000;
		public const uint DSOP_DOWNLEVEL_FILTER_ALL_WELLKNOWN_SIDS = 0x80020000;
		public const uint DSOP_DOWNLEVEL_FILTER_LOCAL_SERVICE = 0x80040000;
		public const uint DSOP_DOWNLEVEL_FILTER_NETWORK_SERVICE = 0x80080000;
		public const uint DSOP_DOWNLEVEL_FILTER_REMOTE_LOGON = 0x80100000;
	}

	/// <summary>
	/// The IDsObjectPicker.InvokeDialog result
	/// </summary>
	public class HRESULT
	{
		public const int S_OK = 0; // The method succeeded. 
		public const int S_FALSE = 1; // The user cancelled the dialog box. ppdoSelections receives NULL. 
		public const int E_NOTIMPL = unchecked((int)0x80004001); // ?
	}

	/// <summary>
	/// The CFSTR_DSOP_DS_SELECTION_LIST clipboard format is provided by the IDataObject obtained by calling IDsObjectPicker.InvokeDialog
	/// </summary>
	public class CLIPBOARD_FORMAT
	{
		public const string CFSTR_DSOP_DS_SELECTION_LIST =
			"CFSTR_DSOP_DS_SELECTION_LIST";
	}

	/// <summary>
	/// The TYMED enumeration values indicate the type of storage medium being used in a data transfer. 
	/// </summary>
	public enum TYMED
	{
		TYMED_HGLOBAL = 1,
		TYMED_FILE = 2,
		TYMED_ISTREAM = 4,
		TYMED_ISTORAGE = 8,
		TYMED_GDI = 16,
		TYMED_MFPICT = 32,
		TYMED_ENHMF = 64,
		TYMED_NULL = 0
	}

	/// <summary>
	/// The DVASPECT enumeration values specify the desired data or view aspect of the object when drawing or getting data.
	/// </summary>
	public enum DVASPECT
	{
		DVASPECT_CONTENT = 1,
		DVASPECT_THUMBNAIL = 2,
		DVASPECT_ICON = 4,
		DVASPECT_DOCPRINT = 8
	}

	#endregion

	#region Interop interfaces
	/// <summary>
	/// The object picker dialog box.
	/// </summary>
	[ComImport, Guid("17D6CCD8-3B7B-11D2-B9E0-00C04FD8DBF7")]
	public class DSObjectPicker
	{
	}

	/// <summary>
	/// The IDsObjectPicker interface is used by an application to initialize and display an object picker dialog box. 
	/// </summary>
	[ComImport, Guid("0C87E64E-3B7A-11D2-B9E0-00C04FD8DBF7"),
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	public interface IDsObjectPicker
	{
		void Initialize(ref DSOP_INIT_INFO pInitInfo);
		void InvokeDialog(IntPtr HWND, out IDataObject lpDataObject);
	}


	#endregion

	#region ADObject helper struct
	public struct ADObject
	{
		private string className;
		private string name;
		private string upn;

		public string ClassName
		{
			get
			{
				return className;
			}
			set
			{
				className = value;
			}
		}

		public string Name
		{
			get
			{
				return name;
			}
			set
			{
				name = value;
			}
		}

		public string UPN
		{
			get
			{
				return upn;
			}
			set
			{
				upn = value;
			}
		}
	}
	#endregion

	#region Interop class to wrap Global Lock/Unlock
	public class PInvoke
	{
		/// <summary>
		/// The GlobalLock function locks a global memory object and returns a pointer to the first byte of the object's memory block.
		/// GlobalLock function increments the lock count by one.
		/// Needed for the clipboard functions when getting the data from IDataObject
		/// </summary>
		/// <param name="hMem"></param>
		/// <returns></returns>
		[DllImport("Kernel32.dll")]
		public static extern IntPtr GlobalLock(IntPtr hMem);

		/// <summary>
		/// The GlobalUnlock function decrements the lock count associated with a memory object.
		/// </summary>
		/// <param name="hMem"></param>
		/// <returns></returns>
		[DllImport("Kernel32.dll")]
		public static extern bool GlobalUnlock(IntPtr hMem);
	}

	#endregion

	public class DsPicker
	{
		public static ADObject[] Show(IntPtr parentHandle)
		{		
			IDataObject dataObj = null;
			IDsObjectPicker ipicker = Initialize();		
			ipicker.InvokeDialog(parentHandle,out dataObj);					
			return ProcessSelections(dataObj);			
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		private static IDsObjectPicker Initialize()
		{
			DSObjectPicker picker = new DSObjectPicker();
			IDsObjectPicker ipicker = (IDsObjectPicker)picker;

			DSOP_SCOPE_INIT_INFO[] scopeInitInfo = new DSOP_SCOPE_INIT_INFO[2];

			// Initialize 1st search scope			
			scopeInitInfo[0].cbSize = (uint)Marshal.SizeOf(typeof(DSOP_SCOPE_INIT_INFO));
			scopeInitInfo[0].flType = DSOP_SCOPE_TYPE_FLAGS.DSOP_SCOPE_TYPE_GLOBAL_CATALOG;   //Convert.ToUInt32("3FE",16);
			scopeInitInfo[0].flScope = DSOP_SCOPE_INIT_INFO_FLAGS.DSOP_SCOPE_FLAG_DEFAULT_FILTER_COMPUTERS + 1; // Starting !?
			scopeInitInfo[0].FilterFlags.Uplevel.flBothModes = DSOP_FILTER_FLAGS_FLAGS.DSOP_FILTER_COMPUTERS + 1; // +1 = advanced view, Check MSDN for the available options
			scopeInitInfo[0].FilterFlags.flDownlevel = DSOP_DOWNLEVEL_FLAGS.DSOP_DOWNLEVEL_FILTER_COMPUTERS;
			scopeInitInfo[0].pwzADsPath = null;
			scopeInitInfo[0].pwzDcName = null;
			scopeInitInfo[0].hr = 0;

			// Initialize 2nd search scope
			scopeInitInfo[1].cbSize = (uint)Marshal.SizeOf(typeof(DSOP_SCOPE_INIT_INFO));
			scopeInitInfo[1].flType = Convert.ToUInt32("37F",16);
			scopeInitInfo[1].flScope = DSOP_SCOPE_INIT_INFO_FLAGS.DSOP_SCOPE_FLAG_DEFAULT_FILTER_USERS | 
				DSOP_SCOPE_INIT_INFO_FLAGS.DSOP_SCOPE_FLAG_DEFAULT_FILTER_GROUPS;
			scopeInitInfo[1].FilterFlags.Uplevel.flBothModes = Convert.ToUInt32("3FF",16);
			scopeInitInfo[1].FilterFlags.flDownlevel = DSOP_DOWNLEVEL_FLAGS.DSOP_DOWNLEVEL_FILTER_ALL_WELLKNOWN_SIDS + 7;
			scopeInitInfo[1].pwzADsPath = null;
			scopeInitInfo[1].pwzDcName = null;
			scopeInitInfo[1].hr = 0;
						

			// Allocate memory from the unmananged mem of the process, this should be freed later!??
			IntPtr refScopeInitInfo = Marshal.AllocHGlobal
				(Marshal.SizeOf (typeof (DSOP_SCOPE_INIT_INFO)) * 2);
			
			// Marshal structs to pointers
			Marshal.StructureToPtr (scopeInitInfo[0], 
				refScopeInitInfo,true);

			Marshal.StructureToPtr (scopeInitInfo[1],
				(IntPtr) ((int) refScopeInitInfo + Marshal.SizeOf
				(typeof (DSOP_SCOPE_INIT_INFO))),true);


			// Initialize structure with data to initialize an object picker dialog box. 
			DSOP_INIT_INFO initInfo = new DSOP_INIT_INFO (); 						
			initInfo.cbSize = (uint) Marshal.SizeOf (initInfo); 
			initInfo.pwzTargetComputer = null; // local computer			
			initInfo.cDsScopeInfos = 2; 
			initInfo.aDsScopeInfos = refScopeInitInfo;  
			// Flags that determine the object picker options. Allow user to select
			// multiple objects and specify that this is not a domain controller
			//initInfo.flOptions = DSOP_INIT_INFO_FLAGS.DSOP_FLAG_MULTISELECT | DSOP_INIT_INFO_FLAGS.DSOP_FLAG_SKIP_TARGET_COMPUTER_DC_CHECK;
			initInfo.flOptions = DSOP_INIT_INFO_FLAGS.DSOP_FLAG_SKIP_TARGET_COMPUTER_DC_CHECK;
			
			// We're not retrieving any additional attributes
			initInfo.cAttributesToFetch = 0; 
			initInfo.apwzAttributeNames = IntPtr.Zero; 
			
			// Initialize the Object Picker Dialog Box with our options
			ipicker.Initialize (ref initInfo); 
			
			return ipicker;
		}

		private static ADObject[] ProcessSelections(IDataObject dataObj)
		{
			if(dataObj == null)
				return null;			

			ADObject[] selections = null;

			// The STGMEDIUM structure is a generalized global memory handle used for data transfer operations
			STGMEDIUM stg = new STGMEDIUM();
			stg.tymed = (uint)TYMED.TYMED_HGLOBAL;
			stg.hGlobal = (int)IntPtr.Zero;
			stg.pUnkForRelease = null;

			// The FORMATETC structure is a generalized Clipboard format.
			FORMATETC fe = new FORMATETC();			
			fe.cfFormat = System.Windows.Forms.DataFormats.GetFormat("CFSTR_DSOP_DS_SELECTION_LIST").Id; //The CFSTR_DSOP_DS_SELECTION_LIST clipboard format is provided by the IDataObject obtained by calling IDsObjectPicker::InvokeDialog
			fe.ptd = (int) IntPtr.Zero;
			fe.dwAspect = 1; //DVASPECT_CONTENT    = 1,  
			fe.lindex = -1; // all of the data
			fe.tymed = (uint)TYMED.TYMED_HGLOBAL; //The storage medium is a global memory handle (HGLOBAL)


			dataObj.GetData(ref fe, ref stg);
	
			IntPtr pDsSL = PInvoke.GlobalLock((IntPtr)stg.hGlobal);

			try
			{
				// the start of our structure
				IntPtr current = pDsSL;
				// get the # of items selected
				int cnt = Marshal.ReadInt32(current);
				
				// if we selected at least 1 object
				if (cnt > 0)
				{				
					selections = new ADObject[cnt];
					// increment the pointer so we can read the DS_SELECTION structure
					current = (IntPtr)((int)current + (Marshal.SizeOf(typeof(uint))*2));
					// now loop through the structures
					for (int i = 0; i < cnt; i++)
					{
						// marshal the pointer to the structure
						DS_SELECTION s = (DS_SELECTION)Marshal.PtrToStructure(current, typeof(DS_SELECTION));
						Marshal.DestroyStructure(current, typeof(DS_SELECTION));

						// increment the position of our pointer by the size of the structure
						current = (IntPtr)((int)current + Marshal.SizeOf(typeof(DS_SELECTION)));
												
						selections[i] = new ADObject();					
						selections[i].Name = s.pwzName;
						selections[i].UPN = s.pwzADsPath;
						selections[i].ClassName = s.pwzClass.ToLower();				
					}
				}
			}			
			finally
			{
				PInvoke.GlobalUnlock(pDsSL);
			}		
			return selections;
		}
	}
}
