/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.Runtime.Serialization;
using System.IO;
using System.Text;
using Ironring.Management.MMC;

namespace Centeris.MMC.PrinterSnapin
{
	public class PrinterState : SnapinStateBase
	{
		// state information
		private MainClass parentSnapin = null;

		/// <summary>
		/// Construct the default state.
		/// </summary>
		public PrinterState(MainClass parentSnapin)
		{			
			this.parentSnapin = parentSnapin;
		}

		#region SnapinStateBase overrides

		/// <summary>
		/// Override to read-back the state of the object.
		/// </summary>
		/// <param name="stream"></param>
		public override void ReadState(ComStream stream) 
		{
			// read the stored server name
			string s = ReadString(stream);
			if (s!=null)
				((PrinterNode)parentSnapin.RootNode).SetServerTarget(s);

		}

		/// <summary>
		/// Write out this object. Override me.
		/// </summary>
		/// <param name="stream"></param>
		public override void WriteState(ComStream stream)
		{
			if (((PrinterNode)parentSnapin.RootNode).PrimaryMode)
			{
				// store the server name
				WriteString(stream, ((PrinterNode) parentSnapin.RootNode).ServerTarget);

				// clear dirtiness
				Dirty = false;
			}
		}

		#endregion

		// helpers
		protected void WriteString(ComStream stream, string s)
		{
			// encode the unicode string into an array of bytes
			UnicodeEncoding uni = new UnicodeEncoding();
			byte[] arb = uni.GetBytes(s);

			// write the length first
			int c = arb.Length;
			byte blo = (byte) (c & 0xff);
			byte bhi = (byte) (c >> 8);
			stream.WriteByte(blo);
			stream.WriteByte(bhi);

			// now write the string itself
			stream.Write(arb, 0, c);
		}

		protected string ReadString(ComStream stream)
		{
			// read the length
			int blo = stream.ReadByte();
			int bhi = stream.ReadByte();

			int c = (bhi<<8) + blo;

			// allocate the byte array
			byte[] arb = new byte[c];

			// read the bytes
			stream.Read(arb, 0, c);

			// convert to a string
			UnicodeEncoding uni = new UnicodeEncoding();
			return uni.GetString(arb);
			
		}

	}
}
