/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.Data;
using System.Runtime.InteropServices;
using System.ComponentModel;

namespace Centeris.MMC.PrinterSnapin
{
	/// <summary>
	/// Summary description for PrintAPI.
	/// </summary>
	public class PrintAPI
	{
		// error handling constants
		protected const int nErrorBadRPC = 1722;
		protected const int nErrorRPC_S_CALL_FAILED = 1726;
		protected const string sErrorMsg = "Unable to retrieve printer list from {0}. Error {1}";
		protected const string sBadServerName = "An error occurred while reading the list of printers for this server. ";

		/// <summary>
		/// Column names for created tables
		/// TODO: Put these in a resource?
		/// </summary>
		protected const string sCol1 = 	"Name";		
		protected const string sCol2 = 	"Documents";		
		protected const string sCol3 = 	"Status";		
		protected const string sCol4 = 	"Comments";		
		protected const string sCol5 = 	"Location";
		protected const string sCol6 =  "Model";
		public const string NameCol = sCol1;

		protected const string sReady = "Ready";
		protected const string sPaused = "Paused";
		protected const string sError = "Error";
		protected const string sOther = "Other";

		const int ERROR_INSUFFICIENT_BUFFER = 122;

		protected enum PEFlags
		{
			PRINTER_ENUM_DEFAULT     = 0x00000001,
			PRINTER_ENUM_LOCAL       = 0x00000002,
			PRINTER_ENUM_CONNECTIONS = 0x00000004,
			PRINTER_ENUM_FAVORITE    = 0x00000004,
			PRINTER_ENUM_NAME        = 0x00000008,
			PRINTER_ENUM_REMOTE      = 0x00000010,
			PRINTER_ENUM_SHARED      = 0x00000020,
			PRINTER_ENUM_NETWORK     = 0x00000040
		};

		protected enum PRINTER_STATUS
		{
			PRINTER_STATUS_PAUSED            = 0x00000001,
			PRINTER_STATUS_ERROR             = 0x00000002,
			PRINTER_STATUS_PENDING_DELETION  = 0x00000004,
			PRINTER_STATUS_PAPER_JAM         = 0x00000008,
			PRINTER_STATUS_PAPER_OUT         = 0x00000010,
			PRINTER_STATUS_MANUAL_FEED       = 0x00000020,
			PRINTER_STATUS_PAPER_PROBLEM     = 0x00000040,
			PRINTER_STATUS_OFFLINE           = 0x00000080,
			PRINTER_STATUS_IO_ACTIVE         = 0x00000100,
			PRINTER_STATUS_BUSY              = 0x00000200,
			PRINTER_STATUS_PRINTING          = 0x00000400,
			PRINTER_STATUS_OUTPUT_BIN_FULL   = 0x00000800,
			PRINTER_STATUS_NOT_AVAILABLE     = 0x00001000,
			PRINTER_STATUS_WAITING           = 0x00002000,
			PRINTER_STATUS_PROCESSING        = 0x00004000,
			PRINTER_STATUS_INITIALIZING      = 0x00008000,
			PRINTER_STATUS_WARMING_UP        = 0x00010000,
			PRINTER_STATUS_TONER_LOW         = 0x00020000,
			PRINTER_STATUS_NO_TONER          = 0x00040000,
			PRINTER_STATUS_PAGE_PUNT         = 0x00080000,
			PRINTER_STATUS_USER_INTERVENTION = 0x00100000,
			PRINTER_STATUS_OUT_OF_MEMORY     = 0x00200000,
			PRINTER_STATUS_DOOR_OPEN         = 0x00400000,
			PRINTER_STATUS_SERVER_UNKNOWN    = 0x00800000,
			PRINTER_STATUS_POWER_SAVE        = 0x01000000
		};

		public const int PRINTER_ALL_ACCESS = 0x000F0000 + 4 + 8;

		protected enum DSPRINT_OPTIONS
		{
			DSPRINT_PUBLISH         = 0x00000001,
			DSPRINT_UPDATE          = 0x00000002,
			DSPRINT_UNPUBLISH       = 0x00000004,
			DSPRINT_REPUBLISH       = 0x00000008
			// DSPRINT_PENDING         = 0x80000000
		};


		[ StructLayout( LayoutKind.Sequential, CharSet=CharSet.Ansi )]
		protected class PRINTER_DEFAULTS
		{
			public IntPtr pDatatype; 
			public IntPtr pDevMode; 
			public int DesiredAccess; 
		};

		[ StructLayout( LayoutKind.Sequential, CharSet=CharSet.Ansi )]
		protected class PRINTER_INFO_2
		{
			public IntPtr  pServerName; 
			public IntPtr  pPrinterName; 
			public IntPtr  pShareName; 
			public IntPtr  pPortName; 
			public IntPtr  pDriverName; 
			public IntPtr  pComment; 
			public IntPtr  pLocation; 
			public IntPtr  pDevMode; 
			public IntPtr  pSepFile; 
			public IntPtr  pPrintProcessor; 
			public IntPtr  pDatatype; 
			public IntPtr  pParameters; 
			public IntPtr  pSecurityDescriptor; 
			public int     Attributes; 
			public int     Priority; 
			public int     DefaultPriority; 
			public int     StartTime; 
			public int     UntilTime; 
			public int     Status; 
			public int     cJobs; 
			public int     AveragePPM; 
		};

		[ StructLayout( LayoutKind.Sequential, CharSet=CharSet.Ansi )]
			protected class PRINTER_INFO_7
		{
			public IntPtr  pszObjectGUID; 
			public uint    dwAction; 
		};
		
		[DllImport("winspool.drv", CharSet=CharSet.Unicode, SetLastError=true)]
		protected extern static bool EnumPrinters(	int dwFlags,
			string Name,
			int Level,
			IntPtr pPrinterEnum,
			int cBuf,
			ref int cBufNeeded,
			ref int cReturned);

		[DllImport("winspool.drv", CharSet=CharSet.Unicode, SetLastError=true)]
		protected extern static bool OpenPrinter(	string sPrinterName,
			out int hPrinter,
			IntPtr pDefault);


		[DllImport("winspool.drv", CharSet=CharSet.Unicode, SetLastError=true)]
		protected extern static bool DeletePrinter( int hPrinter );

		[DllImport("winspool.drv", CharSet=CharSet.Unicode, SetLastError=true)]
		protected extern static bool SetPrinter( int hPrinter, int nLevel, IntPtr pData, int nCommand );

		/// <summary>
		/// Creates a data table with the columns that will later be populated
		/// in FetchPrinterData
		/// </summary>
		/// <returns>A DataTable</returns>
		public static DataTable CreateDataTable()
		{
			DataTable dt = new DataTable();

			dt.Columns.Add(sCol1);
			dt.Columns.Add(sCol2);
			dt.Columns.Add(sCol3);
			dt.Columns.Add(sCol4);
			dt.Columns.Add(sCol5);
			dt.Columns.Add(sCol6);

			return dt;
		}

		/// <summary>
		/// Fetches data about the printers available on the designated server
		/// </summary>
		/// <param name="sServer"></param>
		/// <param name="dt">The DataTable into which the data should be placed. dt MUST have been created properly with CreateDataTable</param>
		public static void FetchPrinterData(string sServer, DataTable dt)
		{
			int dwFlags = (int) PEFlags.PRINTER_ENUM_NAME;
			string Name = @"\\" + sServer;
			int Level = 2;
			IntPtr pPrinterEnum = IntPtr.Zero;
			int cBufNeeded=0;
			int cReturned=0;

			///
			/// TODO: remove hack. Samba 3.0.13 fails if we pass in a null buffer. To avoid this
			/// pass in a pointer to a zero length buffer!
			///

#if SAMBAFIXED
			if (EnumPrinters( dwFlags, Name, Level, pPrinterEnum, 0, ref cBufNeeded, ref cReturned))
			{
				// Can only happen if no printers available!
				return;
			}
#else

			pPrinterEnum = Marshal.AllocHGlobal(0);
			if (EnumPrinters( dwFlags, Name, Level, pPrinterEnum, 0, ref cBufNeeded, ref cReturned))
			{
				// Can only happen if no printers available!
				return;
			}
			// free the mem
			Marshal.FreeHGlobal(pPrinterEnum);
#endif

			// Normally, we fail because we need a bigger buffer. However, if we fail
			// with RPC_S_CALL_FAILED, this means that we "succeeded" but that the remote
			// machine has nothing to tell us
			int nError = Marshal.GetLastWin32Error();

			if (nError==nErrorRPC_S_CALL_FAILED)
				return;

			if (nError!=ERROR_INSUFFICIENT_BUFFER)
			{
				// Any other error results in an exception
				string sErr;
				if (nError==nErrorBadRPC)
					sErr = sBadServerName;
				else
					sErr = string.Format(sErrorMsg, sServer, nError);
				throw new Exception(sErr);
			}

			// allocate a big enough buffer
			pPrinterEnum = Marshal.AllocCoTaskMem(cBufNeeded);

			// retry
			if (!EnumPrinters( dwFlags, Name, Level, pPrinterEnum, cBufNeeded, ref cBufNeeded, ref cReturned))
			{
				string sErr = string.Format(sErrorMsg, sServer, Marshal.GetLastWin32Error());
				throw new Exception(sErr);
			}

			// iterate through the retrieved entries, copying printer entries 
			// to the output table
			IntPtr pCur = pPrinterEnum;
			for( int i = 0; i < cReturned; i++ )
			{
				// marshal the entry into 
				PRINTER_INFO_2 pi2 = new PRINTER_INFO_2();
				Marshal.PtrToStructure(pCur, pi2);

				// add a row to our table
				DataRow dr = dt.NewRow();

				// trim off server name
				string sName = Marshal.PtrToStringUni(pi2.pPrinterName);
				if (sName.StartsWith(@"\\"))
				{
					// yep, trim off prefix
					int ibackwhack = sName.LastIndexOf(@"\");
					sName = sName.Substring(ibackwhack+1);
				}
				dr[sCol1] = sName;
				dr[sCol2] = pi2.cJobs.ToString();
				dr[sCol3] = MapPrinterStatus(pi2.Status);
				dr[sCol4] = Marshal.PtrToStringUni(pi2.pComment);
				dr[sCol5] = Marshal.PtrToStringUni(pi2.pLocation);
				dr[sCol6] = Marshal.PtrToStringUni(pi2.pDriverName);
				dt.Rows.Add(dr);

				// release any referenced data pointed to by the current info structure
				Marshal.DestroyStructure( pCur, typeof(PRINTER_INFO_2) );

				// advance to the next entry
				pCur = (IntPtr)((int)pCur + 	Marshal.SizeOf(pi2));
			}

			// free the mem
			Marshal.FreeHGlobal(pPrinterEnum);
		}

		protected static string MapPrinterStatus(int status)
		{
			if (status==0)
				return sReady;

			if ((status & (int)PRINTER_STATUS.PRINTER_STATUS_PAUSED)!=0)
				return sPaused;

			if ((status & (int)PRINTER_STATUS.PRINTER_STATUS_ERROR)!=0)
				return sError;

			return sOther;
		}

		/// <summary>
		/// Deletes a printer from the designated server. Must have administrator access to it.
		/// </summary>
		/// <param name="sServer"></param>
		/// <param name="sPrinter"></param>
		public static void DeletePrinter(string sServer, string sPrinter)
		{
			// first, open the printer
			string s = @"\\" + sServer + @"\" + sPrinter;
			int hPrinter;
			PRINTER_DEFAULTS pd = new PRINTER_DEFAULTS();
			pd.pDatatype = IntPtr.Zero;
			pd.pDevMode = IntPtr.Zero;
			pd.DesiredAccess = PRINTER_ALL_ACCESS;
			IntPtr p = Marshal.AllocCoTaskMem(Marshal.SizeOf(pd));
			Marshal.StructureToPtr(pd, p, false);
			if (!OpenPrinter(s, out hPrinter, p))
				throw new Exception("Failed to open printer. Error: " + Marshal.GetLastWin32Error());

			// now remove it from AD
			PRINTER_INFO_7 pi7 = new PRINTER_INFO_7();
			pi7.pszObjectGUID = IntPtr.Zero;
			pi7.dwAction = (uint) DSPRINT_OPTIONS.DSPRINT_UNPUBLISH;
			p = Marshal.AllocCoTaskMem(Marshal.SizeOf(pi7));
			Marshal.StructureToPtr(pi7, p, false);
			SetPrinter(hPrinter, 7, p, 0);

			// NOTE: we swallow any error since the operation may be left in a "pending" state rather
			// than returning success or failure right away.

			// TODO: do we have to delete memory?

			// finally, remove the printer
			if (!DeletePrinter(hPrinter))
				throw new Exception("Failed to delete printer. Error: " + Marshal.GetLastWin32Error());
		}

		public static int GetColumnWidth(int iCol)
		{
			switch(iCol+1)
			{
				case 1:
					return 200;

				case 2:
					return 100;

				case 3:
					return 100;

				case 4:
					return 150;

				case 5:
					return 150;

				case 6:
					return 200;

				default:
					throw new Exception("PrinterAPI.GetColumnWidth Invalid column number!");
			}
		}

	}
}
