/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.Reflection;
using System.Collections;
using System.Runtime.InteropServices;
using System.Net;
using SMUtil.SystemInfo;
using System.Web.Services.Protocols;

namespace Centeris.MMC.SMUtil
{
	[ComVisible(true)]
	[Guid("FA60B442-A981-4c5a-A29A-A83757D38D33")]
	public class SMUtil : ISystemManagerUtil
	{
		// constants
		private const int iPort = 9106;
		private const string sProtocol = "http";

		// public methods
		public int ShowShutdownDialog(IntPtr hWndParent, String hostname)
		{
			ShutdownForm form = new ShutdownForm(hostname);
			WindowWrapper iWin32Parent = new WindowWrapper(hWndParent);
			return (int)form.ShowDialog(iWin32Parent);
		}

		public string GetConfiguredDescription(string hostname)
		{
			SystemInfo s = new SystemInfo();

			// set protocol, creds, etc.
			PrepareService(s, hostname);

			return s.getConfiguredDescription();
		}

		public string GetConfiguredDomain(string hostname)
		{
			SystemInfo s = new SystemInfo();

			// set protocol, creds, etc.
			PrepareService(s, hostname);

			return s.getConfiguredDomain();
		}

		public string[] centerisSystemInfo(string hostname)
		{
			SystemInfo s = new SystemInfo();

			// set protocol, creds, etc.
			PrepareService(s, hostname);

			return s.centerisSystemInfo();
		}

		private void PrepareService(SoapHttpClientProtocol soap, string hostname)
		{
			// munge the URL
			UriBuilder uri = new UriBuilder(soap.Url);
			uri.Host = hostname;
			uri.Port = iPort;
			uri.Scheme = sProtocol;
			soap.Url = uri.ToString();

			// set a cookie jar
			soap.CookieContainer = new CookieContainer();

		}

		/// <summary>
		/// This private class is used to convert an IntPtr window handle into an object
		/// that supports IWin32Window
		/// </summary>
		[ComVisible(false)]
		private class WindowWrapper : System.Windows.Forms.IWin32Window
		{
			public WindowWrapper(IntPtr handle)
			{
				_hwnd = handle;
			}

			public IntPtr Handle
			{
				get { return _hwnd; }
			}

			private IntPtr _hwnd;
		}
	}

	/// <summary>
	/// Registers the assembly for COM interop.
	/// 
	/// This is another way of performing self-installation.
	/// This way works with the InstallUtil tool or with the InstallShield
	/// and Wise installers.
	/// </summary>
	[System.ComponentModel.RunInstaller(true)]
	[ComVisible(false)]
	public class RegisterNetClass: System.Configuration.Install.Installer
	{

		public override void Install(IDictionary savedState)
		{
			base.Install(savedState);

			RegistrationServices rs = new RegistrationServices();
			Assembly a = Assembly.GetExecutingAssembly();
			rs.RegisterAssembly(a, AssemblyRegistrationFlags.SetCodeBase);
		}

		public override void Uninstall(IDictionary savedState)
		{
			base.Uninstall(savedState);

			RegistrationServices rs = new RegistrationServices();
			Assembly a = Assembly.GetExecutingAssembly();
			rs.UnregisterAssembly(a);
		}
	}
}
