/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace Centeris.MMC.SMUtil
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	/// 
	
    [ComVisible(false)]
	internal class ShutdownForm : System.Windows.Forms.Form

	{
        private System.Windows.Forms.Label label2;
        private System.Windows.Forms.Label label3;
        private System.Windows.Forms.Label labelHostname;
        private System.Windows.Forms.GroupBox groupBox1;
        private System.Windows.Forms.Label label6;
        private System.Windows.Forms.Label label8;
        private System.Windows.Forms.Button buttonOK;
        private System.Windows.Forms.Button buttonCancel;
        private System.Windows.Forms.ComboBox comboBoxShutdownType;
        private System.Windows.Forms.TextBox textBoxShutdownComment;
        private System.Windows.Forms.Label labelIcon;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
        private System.Windows.Forms.Label labelShutdownDescription;
        private bool m_fCancelShutdown = false;

		internal class Win32 
		{
			[DllImport("advapi32.dll", CharSet=CharSet.Unicode, SetLastError=true)]
			public static extern int InitiateSystemShutdownEx(String machinename,
				String message, UInt32 timeout, Int32 forceapps, Int32 reboot,
				UInt32 reason);
				
		}

	    public ShutdownForm(string strRemoteHostname)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

            this.Text = this.Text + " - " + strRemoteHostname;
            this.labelHostname.Text = strRemoteHostname;

            // when the user changes between "shutdown" and "restart",
            // change the description text below the list box to describe
            // the action.
            //
            comboBoxShutdownType.SelectedIndexChanged +=
                new EventHandler(this.comboBoxShutDownType_OnSelectedIndexChanged);

            comboBoxShutdownType.SelectedIndex = 0;

            String userName = System.Environment.UserName.ToString();
            String machineName = System.Environment.MachineName.ToString();
            textBoxShutdownComment.Text = "Remote shut down requested by "
                + userName + " on " + machineName + ".";

            // when the user tabs to into the comment text box, we want to
            // select all the text, so that the default text can easily be
            // replaced.
            //
            textBoxShutdownComment.Enter +=
                new EventHandler(this.textBoxShutdownComment_OnEnter);
        }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(ShutdownForm));
			this.labelIcon = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.comboBoxShutdownType = new System.Windows.Forms.ComboBox();
			this.labelHostname = new System.Windows.Forms.Label();
			this.labelShutdownDescription = new System.Windows.Forms.Label();
			this.buttonOK = new System.Windows.Forms.Button();
			this.buttonCancel = new System.Windows.Forms.Button();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.textBoxShutdownComment = new System.Windows.Forms.TextBox();
			this.label8 = new System.Windows.Forms.Label();
			this.label6 = new System.Windows.Forms.Label();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// labelIcon
			// 
			this.labelIcon.Image = ((System.Drawing.Image)(resources.GetObject("labelIcon.Image")));
			this.labelIcon.Location = new System.Drawing.Point(16, 8);
			this.labelIcon.Name = "labelIcon";
			this.labelIcon.Size = new System.Drawing.Size(32, 32);
			this.labelIcon.TabIndex = 0;
			// 
			// label2
			// 
			this.label2.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label2.Location = new System.Drawing.Point(64, 8);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(88, 16);
			this.label2.TabIndex = 1;
			this.label2.Text = "Remote computer:";
			// 
			// label3
			// 
			this.label3.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label3.Location = new System.Drawing.Point(64, 32);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(256, 23);
			this.label3.TabIndex = 3;
			this.label3.Text = "&What do you want the remote computer to do?";
			// 
			// comboBoxShutdownType
			// 
			this.comboBoxShutdownType.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBoxShutdownType.Items.AddRange(new object[] {
																	  "Shut down",
																	  "Restart"});
			this.comboBoxShutdownType.Location = new System.Drawing.Point(64, 56);
			this.comboBoxShutdownType.Name = "comboBoxShutdownType";
			this.comboBoxShutdownType.Size = new System.Drawing.Size(264, 21);
			this.comboBoxShutdownType.TabIndex = 4;
			// 
			// labelHostname
			// 
			this.labelHostname.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.labelHostname.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold);
			this.labelHostname.Location = new System.Drawing.Point(160, 8);
			this.labelHostname.Name = "labelHostname";
			this.labelHostname.Size = new System.Drawing.Size(168, 16);
			this.labelHostname.TabIndex = 2;
			// 
			// labelShutdownDescription
			// 
			this.labelShutdownDescription.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.labelShutdownDescription.Location = new System.Drawing.Point(64, 88);
			this.labelShutdownDescription.Name = "labelShutdownDescription";
			this.labelShutdownDescription.Size = new System.Drawing.Size(264, 16);
			this.labelShutdownDescription.TabIndex = 5;
			// 
			// buttonOK
			// 
			this.buttonOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.buttonOK.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.buttonOK.Location = new System.Drawing.Point(192, 312);
			this.buttonOK.Name = "buttonOK";
			this.buttonOK.TabIndex = 7;
			this.buttonOK.Text = "OK";
			this.buttonOK.Click += new System.EventHandler(this.buttonOK_Click);
			// 
			// buttonCancel
			// 
			this.buttonCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.buttonCancel.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.buttonCancel.Location = new System.Drawing.Point(272, 312);
			this.buttonCancel.Name = "buttonCancel";
			this.buttonCancel.TabIndex = 8;
			this.buttonCancel.Text = "Cancel";
			this.buttonCancel.Click += new System.EventHandler(this.buttonCancel_Click);
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.textBoxShutdownComment);
			this.groupBox1.Controls.Add(this.label8);
			this.groupBox1.Controls.Add(this.label6);
			this.groupBox1.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.groupBox1.Location = new System.Drawing.Point(48, 120);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(296, 176);
			this.groupBox1.TabIndex = 6;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "&Shut Down Message (optional)";
			// 
			// textBoxShutdownComment
			// 
			this.textBoxShutdownComment.AcceptsReturn = true;
			this.textBoxShutdownComment.Location = new System.Drawing.Point(16, 80);
			this.textBoxShutdownComment.Multiline = true;
			this.textBoxShutdownComment.Name = "textBoxShutdownComment";
			this.textBoxShutdownComment.Size = new System.Drawing.Size(264, 80);
			this.textBoxShutdownComment.TabIndex = 2;
			this.textBoxShutdownComment.Text = "";
			// 
			// label8
			// 
			this.label8.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label8.Location = new System.Drawing.Point(16, 64);
			this.label8.Name = "label8";
			this.label8.Size = new System.Drawing.Size(56, 16);
			this.label8.TabIndex = 1;
			this.label8.Text = "&Message:";
			// 
			// label6
			// 
			this.label6.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label6.Location = new System.Drawing.Point(16, 24);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(264, 32);
			this.label6.TabIndex = 0;
			this.label6.Text = "This message will be displayed to any users of the machine, and will be stored in" +
				" the Event Log.";
			// 
			// ShutdownForm
			// 
			this.AcceptButton = this.buttonOK;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.buttonCancel;
			this.ClientSize = new System.Drawing.Size(354, 344);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.buttonCancel);
			this.Controls.Add(this.buttonOK);
			this.Controls.Add(this.labelShutdownDescription);
			this.Controls.Add(this.labelHostname);
			this.Controls.Add(this.comboBoxShutdownType);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.labelIcon);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "ShutdownForm";
			this.Text = "Shut Down Remote Computer";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.OnClosing);
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

        private void buttonOK_Click(object sender, System.EventArgs e)
        {
            String strAction = getShutdownTypeVerb();
			String strRemoteHostname = labelHostname.Text;

            // actually schedule the shutdown
            //
			int nErrorCode = Win32.InitiateSystemShutdownEx(
                                strRemoteHostname,              // machine name
                                textBoxShutdownComment.Text,    // message
								0,                              // timeout
                                0,                              // force apps closed
                                comboBoxShutdownType.SelectedIndex, // reboot?
                                0);                             // reason

            // display success message or failure dialog
            //

            if (nErrorCode!=0) // success
            {
                String messageBoxText = "The computer '" + strRemoteHostname + "' is being " + strAction.ToLower() + ".\n\n"
                    + "Please restart the Computer Management tool.";
                String caption = strRemoteHostname + " " + strAction;
                System.Windows.Forms.MessageBox.Show(this,
                    messageBoxText,
                    caption,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information);

                this.DialogResult = System.Windows.Forms.DialogResult.OK;
                m_fCancelShutdown = false;

                this.Close();
            }
            else
            {
				int nWin32Error = Marshal.GetLastWin32Error();
                string strError = new Win32Exception(nWin32Error).Message;

                String messageBoxText = "The computer '" + strRemoteHostname + "' could not be " + strAction.ToLower() + ".\n\n"
                    + "Error details: " + strError;
                String caption = "Could Not " + strAction + " " + strRemoteHostname;
                System.Windows.Forms.MessageBox.Show(this,
                    messageBoxText,
                    caption,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);

				this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
                
                m_fCancelShutdown = true;
            }
        }

        private void buttonCancel_Click(object sender, System.EventArgs e)
        {
            this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            m_fCancelShutdown = false;

            this.Close();
        }

	    private string getShutdownTypeDescription()
	    {
	        int idx = this.comboBoxShutdownType.SelectedIndex;
            String strDescription = "";
            switch (idx)
            {
                case 0:
                    strDescription = "Shuts down Linux, and turns off power " +
                                     "to the computer.";
                    break;
                case 1:
                    strDescription = "Shuts down Linux, and starts Linux again.";
                    break;
            }
            return strDescription;
	    }

	    private string getShutdownTypeVerb()
	    {
	        int idx = this.comboBoxShutdownType.SelectedIndex;
            String strDescription = "";
            switch (idx)
            {
                case 0:
                    strDescription = "Shut Down";
                    break;
                case 1:
                    strDescription = "Restarted";
                    break;
            }
            return strDescription;
	    }

	    public void textBoxShutdownComment_OnEnter(object sender, EventArgs e)
	    {
	        textBoxShutdownComment.SelectAll();
	    }

	    public void comboBoxShutDownType_OnSelectedIndexChanged(object sender, EventArgs e)
	    {
            labelShutdownDescription.Text = getShutdownTypeDescription();
	    }

        private void OnClosing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            e.Cancel = m_fCancelShutdown;
        }
    }
}
