using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.Collections;
using System.Globalization;
using System.Runtime.InteropServices;

using System.Threading;
using ceh = Ironring.Management.MMC.CustomExceptionHandler;

namespace Ironring.Management.MMC
{
	/// <summary>
	/// All MMC snapin Nodes derive from BaseNode to provided basic services and 
	/// communication with the SnapinBase class.
	/// </summary>
	public class BaseNode
	{

		//Added by Alexander Kachalkov
		/// <summary>
		/// Property Sheet title
		/// </summary>
		private string	_propertySheetTitle;
		public string PropertySheetTitle
		{
			get { return _propertySheetTitle; }
			set { _propertySheetTitle = value; }
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// Property Sheet show mode
		/// </summary>
		private bool _ShowModalPropertySheet;
		public bool ShowModalPropertySheet
		{
			get { return _ShowModalPropertySheet; }
			set { _ShowModalPropertySheet = value; }
		}

		//Added by Alexander Kachalkov
		protected bool m_MultiSelectMode;
		/// <summary>
		/// Indicate that we are in Multiselect Mode
		/// </summary>
		public bool MultiSelectMode
		{
			get{return m_MultiSelectMode;}
			set{m_MultiSelectMode = value;}
		}
	
		//Added by Lesley van Zijl
		protected bool m_Pasteable;
		/// <summary>
		/// Indicate that we are Pasteable, needed for Dropping (in Drag&amp;Drop)
		/// </summary>
		public bool Pasteable
		{
			get{return m_Pasteable;}
			set{m_Pasteable = value;}
		}

		//Added by Lesley van Zijl
		protected bool m_Cutable;
		/// <summary>
		/// Indicate that we are Cutable, needed for Dragging (in Drag&amp;Drop)
		/// </summary>
		public bool Cutable
		{
			get{return m_Cutable;}
			set{m_Cutable = value;}
		}

		//Added by Alexander Kachalkov
		///////////////////////////////////////////////////////////////////////
		//
		// TurnOnThemes
		//
		#region
		/// <summary>
		/// Enable support of Windows XP Theams
		/// </summary>
		/// <returns></returns>
		private IntPtr TurnOnThemes()
		{
			ACTCTX ctx;
                
			ctx.cbSize = (uint) Marshal.SizeOf(typeof(ACTCTX));
			ctx.dwFlags = ACTCTX_FLAG.RESOURCE_NAME_VALID;
			ctx.lpSource = this.GetType().Module.Assembly.Location;
			ctx.wProcessorArchitecture = 0;  
			ctx.wLangId = 0;
			ctx.lpAssemblyDirectory = null; 
			ctx.lpResourceName = "ENABLETHEMES";
			ctx.lpApplicationName = null;  
			ctx.hModule = IntPtr.Zero;
                
			IntPtr h = CreateActCtx(ref ctx);
			uint cookie = 0;
			bool b = ActivateActCtx(h, out cookie);
			return h;
		}// TurnOnThemes

		// Added by Lesley van Zijl
		// if the OS version is <= 5.0 NO THEMES AVAILABLE
		private bool HasThemes() {
			if(System.Environment.OSVersion.Version.Major >= 5) {
				if(System.Environment.OSVersion.Version.Minor > 0) {
					System.Diagnostics.Debug.WriteLine("detected a > XP OS, themes");
					return true;
				}
			}
			System.Diagnostics.Debug.WriteLine("detected a < XP OS, no themes");
			return false;
		}


		#endregion

		//Added by Alexander Kachalkov
		//////////////////////////////////////////////////////////////////////
		//
		// ConsoleVerbs
		//
		#region
		/// <summary>
		/// Get the IConsoleVerb interface from MMC
		/// </summary>
		protected IConsoleVerb GetIConsoleVerb()
		{
			IConsoleVerb icv;       
			Snapin.ResultViewConsole.QueryConsoleVerb(out icv);
			return icv;
		}

		/// <summary>
		/// Set standart MMC console verbs
		/// </summary>
		public void SetVerbState(MMC_CONSOLE_VERB cmd, MMC_BUTTON_STATE state, bool show)
		{
			IConsoleVerb icv = GetIConsoleVerb();
			icv.SetVerbState(cmd, state, Convert.ToInt32(show));
		}
		/// <summary>
		/// Get standart MMC console verbs
		/// </summary>
		public bool GetVerbState(MMC_CONSOLE_VERB cmd, MMC_BUTTON_STATE state)
		{
			IConsoleVerb icv = GetIConsoleVerb();
			int show = 0;
			icv.GetVerbState(cmd, state, ref show);
			return Convert.ToBoolean(show);
		}
		/// <summary>
		/// Set default MMC console verbs
		/// </summary>
		public void SetDefaultVerb(MMC_CONSOLE_VERB cmd)
		{
			IConsoleVerb icv = GetIConsoleVerb();
			icv.SetDefaultVerb(cmd);
		}
		/// <summary>
		/// Set default MMC console verbs
		/// </summary>
		public MMC_CONSOLE_VERB GetDefaultVerb()
		{
			IConsoleVerb icv = GetIConsoleVerb();
			MMC_CONSOLE_VERB cmd = MMC_CONSOLE_VERB.NONE;
			icv.GetDefaultVerb(ref cmd);
			return cmd;
		}
		#endregion 

		//Added by Alexander Kachalkov
		/// <summary>
		/// EditMode flag used to show Property Sheet in the Edit or New Mode
		/// </summary>
		bool m_EditMode;

		/// <summary>
		/// The mode this node represents
		/// </summary>
		public bool EditMode
		{
			get { return m_EditMode; }
			set { m_EditMode = value; }
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// The parent node that allow to add new child nodes
		/// to parent node using property sheet
		/// </summary>
		protected BaseNode m_ParentNode;

		//Added by Alexander Kachalkov
		/// <summary>
		/// back reference to parent node this node belongs to 
		/// </summary>
		public BaseNode ParentNode
		{
			get { return m_ParentNode; }
			set { m_ParentNode = value; }
		}

		/// <summary>
		/// Back reference to my snapin
		/// </summary>
		protected SnapinBase m_Snapin;

		// Added by Alexander Kachalkov
		// TBD not really used right now!
		/// <summary>
		/// The guid type for this node 
		/// Could be used for Column Persistence.
		/// </summary>
		protected Guid m_guid;
		
		// added by Roman Kiss
		/// <summary>
		/// The node tag (for general purposes) 
		/// </summary>
		protected object m_tag;

		/// <summary>
		/// collection of children nodes
		/// </summary>
		protected ArrayList m_ChildNodes;

		/// <summary>
		/// The display name appears in the scope pane (tree)
		/// </summary>
		protected string m_sDisplayName;

		/// <summary>
		/// Out opaque id we assign and pass to MMC
		/// </summary>
		protected int m_iCookie;              

		/// <summary>
		/// MMC's unique identifier for this object 
		/// </summary>
		protected int m_iHScopeItem;          

		/// <summary>
		/// MMC's unique identifier for the parent
		/// </summary>
		protected int m_iParentHScopeItem;    

		/// <summary>
		/// Collection of images to use for this node's result pane
		/// </summary>
		protected ImageList m_Images = null;

		/// <summary>
		/// Default collection of images to use for result pane
		/// </summary>
		protected static ImageList s_DefaultImages = null;


		/// <summary>
		/// Index into ImageCollection
		/// </summary>
		protected int m_nClosedImage = 0;
        
		/// <summary>
		/// Index into ImageCollection
		/// </summary>
		protected int m_nOpenImage = 1;

		/// <summary>
		/// Index into ImageCollection
		/// </summary>
		protected int m_nSmallImage  = 0;

		/// <summary>
		/// Index into ImageCollection
		/// </summary>
		protected int m_nLargeImage  = 1;
        

		/// <summary>
		/// primary index of all menuitems, All other types are duplicated here
		/// to make it easy to find them again when MMC invoke one.
		/// </summary>
		protected ArrayList m_MenuItems = null;	   

		/// <summary>
		/// All "Top" menu items
		/// </summary>
		protected ArrayList m_TopMenuItems = null;
        
		/// <summary>
		/// All "New" menu items
		/// </summary>
		protected ArrayList m_NewMenuItems = null;
        
		/// <summary>
		/// All "Task" menu items
		/// </summary>
		protected ArrayList m_TaskMenuItems = null;
        
		/// <summary>
		/// All "View" menu items
		/// </summary>
		protected ArrayList m_ViewMenuItems = null;
		
		/// <summary>
		/// Collection of property pages for this node
		/// </summary>
		protected ArrayList m_propertyPages = new ArrayList();        

		/// <summary>
		/// The optional property sheet for this node
		/// </summary>
		protected PropertySheet m_propSheet;

		/// <summary>
		/// Implements a spare collection of events to save space 
		/// when the common case is now one registers.
		/// </summary>
		protected EventHandlerSet m_events = new EventHandlerSet();

		// Added Alexander Ovchinnikov for Update Node from PP
		/// <summary>
		/// This is delegate for UpdateNode from PropertyPage
		/// </summary>
		public OnUserDelegate UpdateHandle = null;

        private bool _alwaysInsertWithPlusSign = false;
        // added by Marat Bakirov (bmf)
        /// <summary>
        /// Overwrite default InsertChildren behaviour, with this set the scopenodes will 
        /// Always insert scopenodes with the plus sign visible, even if there are no child nodes.
        /// <seealso cref="InsertChildren"/>
        /// </summary>
        public bool AlwaysInsertWithPlusSign
        {
            get {return _alwaysInsertWithPlusSign;} 
            set {_alwaysInsertWithPlusSign = value;}
        }
		
		/// <summary>
		/// The events prublished by all nodes.  These keys are used to 
		/// identify the event notification type.
		/// </summary>
		#region
		protected static object OnActivateKey = new object();
		protected static object OnAddScopePaneImagesKey = new object();
		protected static object OnAddResultPaneImagesKey = new object();
		protected static object OnBtnClickKey = new object();
		protected static object OnClickKey = new object();
		protected static object OnColumnClickKey = new object();
		protected static object OnCutOrRemoveKey = new object();
		protected static object OnDblClickKey = new object();
		protected static object OnDeleteKey = new object();
		protected static object OnDeselectAllKey = new object();
		protected static object OnExpandKey = new object();
		protected static object OnCollapseKey = new object();
		protected static object OnExpandExtensionKey = new object();
		protected static object OnCollapseExtensionKey = new object();
		protected static object OnInitOcxKey = new object();
		protected static object OnMenuBtnClickKey = new object();
		protected static object OnMaximizedKey = new object();
		protected static object OnMinimizedKey = new object();
		protected static object OnQueryPasteKey = new object();
		protected static object OnRefreshKey = new object();
		protected static object OnRemoveChildrenKey = new object();
		protected static object OnRenameKey = new object();
		protected static object OnSelectScopeKey = new object();
		protected static object OnDeselectScopeKey = new object();
		protected static object OnSelectResultKey = new object();
		protected static object OnDeselectResultKey = new object();
		protected static object OnShowKey = new object();
		protected static object OnViewChangeScopeKey = new object();
		protected static object OnViewChangeResultKey = new object();
		protected static object OnContextHelpKey = new object();
		//---added by Roman Kiss
		protected static object OnUserKey = new object();
		//---added by Roman Kiss
		protected static object OnQueryPropertiesKey = new object();
		//Added by Alexander Kachalkov
		protected static object OnControlbarCommandKey= new object();
		//Added by Alexander Kachalkov
		protected static object OnControlbarNotifyKey = new object();
		// MAM: filtering
		protected static object OnFilterChangeKey = new Object();
		// Added  by Alexander Kachalkov
		protected static object OnGetFilterMenuKey = new Object();
		// Added  by Alexander Kachalkov
		protected static object OnFilterOperatorKey = new Object();
		//Added by Attila Kapostyak
		protected static object OnPrintKey = new object();	
		//Added by Wojciech Sobocinski
		protected static object OnPasteKey = new object();

		#endregion
		////////////////////////////////////////////////////////////////////////////
		//
		// Implementation
		//
		#region
		/// <summary>
		/// Used for code recyling
		/// </summary>
		internal BaseNode()
		{	
			m_iHScopeItem = 0;
			m_iParentHScopeItem = 0;
			m_ChildNodes = new ArrayList(8);
		}

		/// <summary>
		/// Base ctor must be called by all derivatives to register ourselves 
		/// with the snapinbase.  This process assigned the cookie
		/// Uses itself as ParentNode
		/// </summary>
		/// <param name="snapin"></param>
		public BaseNode(SnapinBase snapin) : this()
		{
			m_Snapin = snapin;
			//Added by Alexander Kachalkov
			//by default no parent node
			m_ParentNode = this;
			m_iCookie = snapin.Register(this);
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// Base ctor must be called by all derivatives to register ourselves 
		/// with the snapinbase.  This process assigned the cookie
		/// </summary>
		/// <param name="snapin"></param>
		/// <param name="parentNode"></param>
		public BaseNode(SnapinBase snapin, BaseNode parentNode) : this()
		{
			m_Snapin = snapin;
			//Added by Alexander Kachalkov
			m_ParentNode = parentNode;
			m_iCookie = snapin.Register(this);
		}

		/// <summary>
		/// Base ctor must be called by all derivatives to register ourselves 
		/// with the snapinbase. This process assigned the cookie
		/// </summary>
		/// <param name="snapin"></param>
		/// <param name="parentNode"></param>
		/// <param name="displayName"></param>
		public BaseNode(SnapinBase snapin, BaseNode parentNode, String displayName) : this(snapin, parentNode)
		{
			this.DisplayName = displayName;
		}

		/// <summary>
		/// Base ctor must be called by all derivatives to register ourselves 
		/// with the snapinbase. This process assigned the cookie
		/// Uses itself as ParentNode
		/// </summary>
		/// <param name="snapin"></param>
		/// <param name="displayName"></param>
		public BaseNode(SnapinBase snapin, String displayName) : this(snapin)
		{
			this.DisplayName = displayName;
		}

		/// <summary>
		/// finalizer cleanup
		/// </summary>
		~BaseNode()
		{
			// TBD: any unmanged resources?  IDisposable
		}

		/// <summary>
		/// our unique id - opaque don't infer a thing from it! 
		/// </summary>
		public int Cookie
		{
			get { return m_iCookie; }
			set { m_iCookie = value; }
		}

		/// <summary>
		/// back reference to the snapin this node belongs to 
		/// </summary>
		public SnapinBase Snapin
		{
			get { return m_Snapin; }
		}

		/// Add virtual to this method Alexander Ovchinnikov
		/// <summary>
		/// whether or not property pages are supported, default false
		/// </summary>
		public virtual bool HavePropertyPages
		{
			get { return m_propertyPages.Count != 0; }
		}

		/// <summary>
		/// MMC's unique id for this node's parent 
		/// </summary>
		public int ParentHScopeItem
		{
			get { return m_iParentHScopeItem; }
			set { m_iParentHScopeItem = value; }
		}

		/// <summary>
		/// MMC's unique id for this node 
		/// </summary>
		public int HScopeItem
		{
			get { return m_iHScopeItem; }
			set { m_iHScopeItem = value; }
		}

		/// <summary>
		/// The name of this Node
		/// </summary>
		public string DisplayName
		{
			get { return m_sDisplayName; }
			set { m_sDisplayName = value; }
		}

		/// <summary>
		/// Set or get the result view image colleciton for this node.
		/// The images are indexed by the overridable GetResultViewImageIndex
		/// method.  Many nodes can share the same image index.
		/// </summary>
		public ImageList ResultPaneImages
		{
			get 
			{ 
				if (m_Images == null)
					return s_DefaultImages;
				return m_Images;
			}
			set { m_Images = value; }
		}

		/// <summary>
		/// Set or get the default result view image colleciton.
		/// The images are indexed by the overridable GetResultViewImageIndex
		/// method.  Many nodes can share the same image index.
		/// </summary>
		public static ImageList DefaultResultPaneImages
		{
			get { return s_DefaultImages; }
			set { s_DefaultImages = value; }
		}
        

		/// <summary>
		/// index into the ImageCollection
		/// </summary>
		public int ClosedImageIndex
		{
			get { return m_nClosedImage; }
			set { m_nClosedImage = value; }
		}

		/// <summary>
		/// index into the ImageCollection
		/// </summary>
		public int OpenImageIndex
		{
			get { return m_nOpenImage; }
			set { m_nOpenImage = value; }
		}

		/// <summary>
		/// index into the ImageCollection
		/// </summary>
		public int SmallImageIndex
		{
			get { return m_nSmallImage; }
			set { m_nSmallImage = value; }
		}

		/// <summary>
		/// index into the ImageCollection
		/// </summary>
		public int LargeImageIndex
		{
			get { return m_nLargeImage; }
			set { m_nLargeImage = value; }
		}

		/// <summary>
		/// The guid or this node type - used in DataObject
		/// </summary>
		public virtual Guid Guid
		{
			get { return m_guid; }
			set { m_guid = value; }
		}

		
		/// <summary>
		/// added by Roman Kiss
		/// The node tag for general purposes.
		/// </summary>
		public object Tag
		{
			get { return m_tag; }
			set { m_tag = value; }
		}

		/// <summary>
		/// how many children do we have, if any?
		/// </summary>
		public int NumChildren
		{
			get { return (m_ChildNodes == null) ?  0 : m_ChildNodes.Count; }
		}

		//Changed by Alexander Kachalkov
		/// <summary>
		/// Return the children collection as an array
		/// due to the VB.NET incompatibility
		/// </summary>
		//public BaseNode[] Children
		public ArrayList Children
		{
			//get { return (BaseNode[])m_ChildNodes.ToArray(); }
			get { return m_ChildNodes; }
		}
		#endregion
		//////////////////////////////////////////////////////////////////
		//
		// Methods
		//
		#region
		/// <summary>
		/// called after creation to perform one time init tasks
		/// </summary>
		public virtual void Initialize( ){
			//Added Alexander Ovchinnikov
			UpdateHandle += new OnUserDelegate( UpdateThisNode );
		}

		/// <summary>
		/// Add a node to the child colleciton
		/// </summary>
		/// <param name="node"></param>
		public void AddChild(BaseNode node)
		{
			m_ChildNodes.Add(node);
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// Delete node and release all Cookies
		/// Use this method do delete nodes from Scope view
		/// to eliminate memory leaks
		/// </summary>
		/*public void Remove(bool removeChildrenOnly)
		{
			//---delete node and its children
			if (this.HScopeItem != 0)
				Snapin.ConsoleNameSpace.DeleteItem((uint)this.HScopeItem, Convert.ToInt32(!removeChildrenOnly));

			//Added Pietro Di Gregorio : if this is the top node ParentHScopeItem is 0
			if (this.ParentHScopeItem!=0)
				//---set scope to the root
				this.Snapin.ResultViewConsole.SelectScopeItem(this.ParentHScopeItem);

			if (!removeChildrenOnly)
				Snapin.Unregister(this);

			foreach(BaseNode node in m_ChildNodes)
			{
				Snapin.Unregister(node);
				if (node.NumChildren != 0) node.Remove(true);
			}
			m_ChildNodes.RemoveRange(0, m_ChildNodes.Count);
		}*/
		//Added Alexander Ovchinnikov
		public void Remove( bool removeChildrenOnly ) {
			//---delete node and its children
			if( this.HScopeItem != 0 ){
				Snapin.ConsoleNameSpace.DeleteItem( ( uint )this.HScopeItem, Convert.ToInt32( !removeChildrenOnly ));
			}
				
			//---set scope to the root
			foreach( BaseNode node in m_ChildNodes ){
				if( node.NumChildren != 0 ){
					node.Remove(true);
				}
				Snapin.Unregister(node);				
			}
			m_ChildNodes.Clear( );

			if (!removeChildrenOnly){
				if( this.ParentHScopeItem != 0 ){
					if( this.Snapin.FindNodeByHScope( this.ParentHScopeItem ) != null ){
						this.Snapin.ResultViewConsole.SelectScopeItem(this.ParentHScopeItem);
					}else{
						System.Diagnostics.Debug.WriteLine( this.DisplayName );
					}
				}
				Snapin.Unregister(this);
			}			
		}

		

		/// <summary>
		/// insert this node into the console namespace
		/// </summary>
		/// <param name="parent"></param>
		public virtual void Insert(BaseNode parent)
		{
			Insert(parent, true);
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// insert this node into the console namespace as leaf (no childern = false)
		/// </summary>
		/// <param name="parent"></param>
		/// <param name="hasChildren"></param>
		public virtual void Insert(BaseNode parent, bool hasChildren)
		{
			Insert(parent, hasChildren, parent.HScopeItem);
		}

		/// <summary>
		/// This method is used in the special case that we have to insert the
		/// node into a "foreign" scope item. This comes up in the case
		/// that we are extending a  snapin
		/// </summary>
		public virtual void InsertAsExtension(bool hasChildren, int relativeId)
		{
			Insert(null, hasChildren, relativeId);
		}

		/// <summary>
		/// Generic method to Insert a node into a scope tree
		/// </summary>
		public virtual void Insert(BaseNode parent, bool hasChildren, int relativeId)
		{
			IConsoleNameSpace2 cns = Snapin.ConsoleNameSpace;

			SCOPEDATAITEM sdi = new SCOPEDATAITEM();
			
			sdi.mask =	(uint)SDI.STR		|   
				(uint)SDI.PARAM     |   
				(uint)SDI.PARENT    |
				(uint)SDI.IMAGE     |
				(uint)SDI.OPENIMAGE |
				(uint)SDI.CHILDREN;

			// TBD: consider doing the dynamic update of images using 
			// IConsoleNameSpace2::SetItem

			// The image index is going to be the same as the cookie value
			sdi.nImage      = ClosedImageIndex;

			// The open image is the same as the closed image
			sdi.nOpenImage  = OpenImageIndex;

			sdi.relativeID  = relativeId;

			// We set displayname to -1 to initiate a callback for the string
			// (We need to do it this way... it's an MMCism)
			sdi.displayname = (IntPtr)(-1);
    
			sdi.lParam      = Cookie;

			// The children field is set to either 0 or 1 (if it has children or not)
			sdi.cChildren   = Convert.ToInt32(hasChildren); //NumChildren == 0 ? 0 : 1; Alexander Ovchinnikov Comment
    
			cns.InsertItem(ref sdi);

			// Once the item has been inserted, we're given the HScopeItem value for the node
			// MMC uses this value to uniquely identify the node. We'll store it for future
			// use.
			HScopeItem = sdi.ID;
			
			// We'll also put in the parent's HScopeItem
			ParentHScopeItem = relativeId;

			//Added by Wojciech Sobocinski
			// reset the parent node
			this.ParentNode = parent;

		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// Update this node image in the console namespace
		/// </summary>
		public void SetScopeViewItemImage()
		{
			IConsoleNameSpace2 cns = Snapin.ConsoleNameSpace;
			SCOPEDATAITEM sdi = new SCOPEDATAITEM();
			
			sdi.mask =	(uint)SDI.IMAGE     |
				(uint)SDI.OPENIMAGE;

			// The image index is going to be the same as the cookie value
			sdi.nImage      = ClosedImageIndex;

			// The open image is the same as the closed image
			sdi.nOpenImage  = OpenImageIndex;

			sdi.ID  = HScopeItem;
    
			cns.SetItem(ref sdi);
		}

		/// <summary>
		/// This function adds all the node's children to MMC
		/// Fix for +sign fix thanks to: Marat Bakirov
		/// </summary>
		public void InsertChildren()
		{
			foreach (BaseNode node in m_ChildNodes)
				node.Insert(this, node.Children.Count > 0 || node.AlwaysInsertWithPlusSign);
		}
		
		//Added by Alexander Kachalkov
		/// <summary>
		/// This function adds all the node's children to MMC with + on the left
		/// </summary>
		public void InsertChildren(bool hasChildren)
		{
			foreach (BaseNode node in m_ChildNodes)
				node.Insert(this, hasChildren);
		}

		/// <summary>
		/// Dynamically add menu items to the context menus 
		/// </summary>
		/// <param name="piCallback"></param>
		/// <param name="pInsertionAllowed"></param>
		public virtual void AddMenuItems(IContextMenuCallback piCallback, ref int pInsertionAllowed)
		{
			// Conditionally add our menu items
			if ((pInsertionAllowed & (int)CCM.INSERTIONALLOWED_TOP) > 0)
				OnAddTopMenu(piCallback);
			
			if ((pInsertionAllowed & (int)CCM.INSERTIONALLOWED_NEW) > 0)
				OnAddNewMenu(piCallback);

			if ((pInsertionAllowed & (int)CCM.INSERTIONALLOWED_TASK) > 0)
				OnAddTaskMenu(piCallback);

			if ((pInsertionAllowed & (int)CCM.INSERTIONALLOWED_VIEW) > 0)
				OnAddViewMenu(piCallback);
		}

		/// <summary>
		/// Add a property page to the node.
		/// </summary>
		/// <param name="title"></param>
		/// <param name="userControlType"></param>
		public void AddPropertyPage(string title, Type userControlType)
		{
			PropertyPage page = new PropertyPage(title);
			page.ControlType = userControlType;
			m_propertyPages.Add(page);
		}


		/// <summary>
		/// Virtual method to provide Result view configuration information
		/// </summary>
		/// <param name="pViewOptions"></param>
		/// <returns></returns>
		public virtual string GetResultViewType(ref int pViewOptions)
		{
			// the default for std list views
			pViewOptions = 0;
			return string.Empty;		
		}

		/// <summary>
		/// Refreshes the result view by forcing a selection of the corresping scope node
		/// </summary>
		public virtual void RefreshResultView()
		{
			Snapin.SelectScopeNode(this);
		}

		/// <summary>
		/// This function will provide MMC with information on how
		/// to display data items in the result pane. This function
		/// is used to get data both for a column view and a list view.
		/// TBD: trim down for basic list view only - reporting stuff is in reportnode
		/// </summary>
		/// <param name="ResultDataItem"></param>
		public virtual void GetDisplayInfo(ref RESULTDATAITEM ResultDataItem)
		{
			// If we need a display name
			if ((ResultDataItem.mask & (uint)RDI.STR) > 0)
				ResultDataItem.str = Marshal.StringToCoTaskMemUni(m_sDisplayName);

			// return the image current index for the image on the result pane
			if ((ResultDataItem.mask & (uint)RDI.IMAGE) > 0)
				ResultDataItem.nImage = GetResultViewImageIndex(ResultDataItem);

			if ((ResultDataItem.mask & (uint)RDI.PARAM) > 0)
				ResultDataItem.lParam = m_iCookie;

			// Don't know what this field is for, MSDN isn't clear
			// on it... just set it to 0 if we need to
			if ((ResultDataItem.mask & (uint)RDI.INDEX) > 0)
				ResultDataItem.nIndex = 0;
            
			// Reserved
			if ((ResultDataItem.mask & (uint)RDI.INDENT) > 0)
				ResultDataItem.iIndent = 0;
		}

		/// <summary>
		/// provides scope pane infor to MMC  
		/// </summary>
		/// <param name="sdi"></param>
		public virtual void GetDisplayInfo(ref SCOPEDATAITEM sdi)
		{
			// See if they want the display name
			if ((sdi.mask & (uint)SDI.STR) > 0)
				sdi.displayname = Marshal.StringToCoTaskMemUni(DisplayName);

			// We shouldn't need to set this.... but if we need to...
			if ((sdi.mask & (uint)SDI.STATE) > 0)
				sdi.nState = 0; 

			// If we're inquiring about children....
			if ((sdi.mask & (uint)SDI.CHILDREN) > 0)
				sdi.cChildren = NumChildren;
		}


		//Added by Alexander Kachalkov
		/// <summary>
		/// This method is called by MMC Console when asking for Properties
		/// </summary>
		/// <param name="lpProvider"></param>
		/// <param name="handle"></param>
		//Added by Alexander Ovchinnikov virtual
		public virtual void CreatePropertyPages(IPropertySheetCallback lpProvider, IntPtr handle)
		{
			CreatePropertyPages(lpProvider, handle, null);
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// This function will create MODAL property sheet pages based
		/// on the information stored in its m_propertyPages field.
		/// It registers the property page along with the callback
		/// functions.
		/// Use it while creating NEW node
		/// </summary>
		/// <param name="lpProvider"></param>
		/// <param name="handle"></param>
		/// <param name="displayName"></param>
		public void CreatePropertyPagesModal(IPropertySheetCallback lpProvider, IntPtr handle, string displayName)
		{
			if (this.m_propSheet == null || m_propSheet.isClosed)
				CreatePropertySheet(displayName);
			else
				m_propSheet.Activate();
		}

        //Added by Alexander Kachalkov
        //Added the DisplayName for New Mode title
        /// <summary>
        /// This function will create property sheet pages based 
        /// on the information stored in its m_propertyPages field.
        /// It registers the property page along with the callback
        /// functions.
        /// </summary>
        /// <param name="lpProvider"></param>
        /// <param name="handle"></param>
        /// <param name="displayName"></param>
        public void CreatePropertyPages(IPropertySheetCallback lpProvider, IntPtr handle, string displayName)
        {
            if (this.m_propSheet == null || m_propSheet.isClosed)
            {
                // Use the BaseNode.PropertySheetTitle as the displayname if it's set. 
                if (PropertySheetTitle != null)
                    displayName = PropertySheetTitle;

                // We need to spin off a new thread for this.
                // Why?
                // For winforms forms to work correctly (including accelerators, etc)
                // we need to use a winforms message pump. We can accomplish that
                // by showing our form using Form.ShowDialog().
                // However, this blocks the thread's execution until the dialog
                // is dismissed. We'll get around this by spinning off another thread
                // to handle the form.
                //Changed by Alexander Kachalkov
                if (ShowModalPropertySheet)
                    CreatePropertySheet(displayName);
                else
                    ThreadPool.QueueUserWorkItem(new WaitCallback(CreatePropertySheet), displayName);
            }
            else
                m_propSheet.Activate();
        }

        //Added by Alexander Kachalkov
		// Changed by Alexander Kachalkov to virtual due to the request
		/// <summary>
		/// This function should be called on the non-MMC thread.  We need 
		/// to create the property sheet on the thread that will be used to host 
		/// the property sheet.
		/// </summary>
		/// <param name="displayName">PropertyPage dialog caption text</param>
		protected virtual void CreatePropertySheet(object displayName)
		{
			//if (!HavePropertyPages) return;

			m_propSheet = new PropertySheet(this);
			foreach (PropertyPage page in m_propertyPages)
				m_propSheet.Pages.Add(page);

			//m_propSheet.Text = DisplayName + " Properties";
			//Chandged by  Alexander Kachalkov
			if (displayName == null)
				displayName = PropertySheetTitle;

			m_propSheet.Text = Convert.ToString(displayName);

			//Added by Alexander Kachalkov
			//Turn ON Themes (Lesley van Zijl) first a check on OS
			IntPtr ctx = new System.IntPtr();
			if(HasThemes())
				ctx = TurnOnThemes();

			//Show Propery Sheet
			m_propSheet.ShowDialog( );
			
			//Added by Alexander Kachalkov			
			//Turn OFF Themes (Lesley van Zijl) first a check on OS
			if(HasThemes())
				ReleaseActCtx(ctx);

			//Added by Alexander Ovchinnikov
			m_propSheet.Pages.Clear( );
			m_propSheet = null;
		}

		// Added by Greg Arnold
		/// <summary>
		/// This function is called when SortComponent is used. This virtual method 
		/// allows for custom sorting implementations. For example, when you want 
		/// to handle DateTime display text vs. DateTime for sorting.
		/// </summary>
		/// <param name="nCol"></param>
		/// <param name="lParam"></param>
		/// <returns></returns>
		public virtual IComparable GetSortCompareInfo(int nCol, int lParam)
		{
			IComparable comparer = null;

			// Get the node
			BaseNode node = Snapin.FindNode(lParam);
			if (node != null)
			{
				// Request the node's display info
				RESULTDATAITEM rdi = new RESULTDATAITEM();
				rdi.mask	= (uint)RDI.STR;
				rdi.lParam	= lParam;
				rdi.nCol	= nCol;

				node.GetDisplayInfo(ref rdi);

				// Get the display text and return its IComparable
				string data = Marshal.PtrToStringAuto(rdi.str);
				comparer = (data as IComparable);
			}

			return comparer;
		}

		#endregion
		///////////////////////////////////////////////////////////////////////
		//
		// Notification methods
		//
		#region
		/// <summary>
		/// Is sent when a window is being activated or deactivated, the 
		/// flag indicates which.
		/// </summary>
		/// <param name="bActivate"></param>
		public virtual void OnActivate(bool bActivate)
		{
			m_events.Fire(OnActivateKey, this, new ActivateArgs(bActivate));
		}

		/// <summary>
		/// called to add scope (tree) pane images to the MMC 
		/// provided image list. 
		/// By default we don't add any images but delegate to our children
		/// because they might want to.
		/// if you override this method call this to delegate to children.
		/// </summary>
		/// <param name="il"></param>
		public virtual void OnAddScopePaneImages(IImageList il)
		{
			foreach(BaseNode node in m_ChildNodes)
				node.OnAddScopePaneImages(il);

			m_events.Fire(OnAddScopePaneImagesKey, this, new AddImagesArgs(il));
		}

		/// <summary>
		/// called to add result pane images to the MMC 
		/// provided image list. 
		/// </summary>
		/// <param name="il"></param>
		public virtual void OnAddResultPaneImages(IImageList il)
		{
			if (ResultPaneImages != null)
				ResultPaneImages.LoadImageList(il, m_iCookie);

			// Now add all the children images
			foreach(BaseNode node in m_ChildNodes)
				node.OnAddResultPaneImages(il);

			m_events.Fire(OnAddResultPaneImagesKey, this, new AddImagesArgs(il));
		}

		/// <summary>
		/// Called when user clicks on a ControlBar button
		/// </summary>
		/// <param name="CmdId">ID assigned to button</param>
		public virtual void OnBtnClick(int CmdId)
		{
			m_events.Fire(OnBtnClickKey, this, new CommandArgs(CmdId));
		}

		/// <summary>
		/// TBD: huh?
		/// </summary>
		public virtual void OnClick()
		{
			m_events.Fire(OnClickKey, this, null);
		}

		/// <summary>
		/// user clicks on a result listview column header
		/// </summary>
		/// <param name="nCol">column number</param>
		/// <param name="bAscending"></param>
		public virtual void OnColumnClick(int nCol, bool bAscending)
		{
			m_events.Fire(OnColumnClickKey, this, null);
		}

		//Changed by Wojciech Sobocinski
		/// <summary>
		/// Cut or remove clipboard operation
		/// </summary>
		public virtual bool OnCutOrRemove(IDataObject ido)
		{
			m_events.Fire(OnCutOrRemoveKey, this, new ObjectArgs(ido));
			return false;
		}

		/// <summary>
		/// Double clicked on a node.
		/// </summary>
		public virtual bool OnDblClick()
		{
			m_events.Fire(OnDblClickKey, this, null);
			return false;
		}

		/// <summary>
		/// Sent to inform the snapin that the user wants to delete the node. 
		/// </summary>
		public virtual void OnDelete()
		{
			m_events.Fire(OnDeleteKey, this, null);
		}

		/// <summary>
		/// This message is sent when all items of an owner-data result pane
		///	are deselected.
		/// </summary>
		public virtual void OnDeselectAll()
		{
			m_events.Fire(OnDeselectAllKey, this, null);
		}

		/// <summary>
		/// called when node is expanding 
		/// </summary>
		public virtual void OnExpand()
		{
			InsertChildren();
			m_events.Fire(OnExpandKey, this, null);
		}

		/// <summary>
		/// Called when node is collapsing
		/// </summary>
		public virtual void OnCollapse()
		{
			m_events.Fire(OnCollapseKey, this, null);
		}

		/// <summary>
		/// Called when this node's parent is a node in a snapin that we are extending.
		/// We have to insert the node into the parent's scope tree
		/// </summary>
		public virtual void OnExtensionExpand(IDataObject lpDataObject, int relativeId)
		{
			// insert the root node into the scope
			InsertAsExtension( false, relativeId);
			m_events.Fire(OnExpandExtensionKey, this, null);
		}

		public virtual void OnExtensionCollapse(IDataObject lpDataObject)
		{
			// TODO: do we have to remove ourselves from the parent's scope pane?
			m_events.Fire(OnCollapseExtensionKey, this, null);
		}
        
		/// <summary>
		/// Sent to a snap-in when its custom OCX is initialized for the first time.
		/// </summary>
		/// <param name="objOCX">The control IUnknown</param>
		/// <returns></returns>
		public virtual bool OnInitOcx(object objOCX)
		{
			m_events.Fire(OnInitOcxKey, this, new ObjectArgs(objOCX));
			return false;
		}

		// added by Roman Kiss
		/// <summary>
		/// Sent to a snap-in by user application
		/// </summary>
		/// <param name="key"></param>
		/// <param name="val"></param>
		public virtual void OnUser(string key, object val)
		{
			m_events.Fire(OnUserKey, this, new NameValueArgs(key, val));
		}

		// added by Roman Kiss
		/// <summary>
		/// Sent by a snap-in when it query for properties pages.
		/// </summary>
		public virtual void OnQueryProperties()
		{
			m_events.Fire(OnQueryPropertiesKey, this, null);
		}
		
		//Changed by Alexander Kachalkov        
		/// <summary>
		/// This message is sent when a user clicks on a button.
		/// </summary>
		/// <param name="CmdId"></param>
		/// <param name="x"></param>
		/// <param name="y"></param>
		public virtual void OnMenuBtnClick(int CmdId, int x, int y)
		{
			m_events.Fire(OnMenuBtnClickKey, this, null);
		}

		/// <summary>
		/// Window is being maximized
		/// </summary>
		public virtual void OnMaximized()
		{
			m_events.Fire(OnMaximizedKey, this, null);
		}

		/// <summary>
		/// Window is being minimized
		/// </summary>
		public virtual void OnMinimized()
		{
			m_events.Fire(OnMinimizedKey, this, null);
		}

		//Changed by Wojciech Sobocinski
		/// <summary>
		/// Can the ido item be pasted?
		/// </summary>
		/// <param name="ido">IDataObject of item to be pasted</param>
		/// <returns></returns>
		public virtual bool OnQueryPaste(IDataObject ido)
		{
			bool retval = false;
			// By Wojciech Sobocinski, implemented in BaseNode by Lesley van Zijl
			//
			// only allow another node like us to be pasted in
			//
			BaseNode nodeToBePasted = null;
			if(ido is DataObject)
				nodeToBePasted = ((Ironring.Management.MMC.DataObject)ido).Node as BaseNode;

			retval = OnQueryPasteValidator(nodeToBePasted);
			
			// Lesley: do we still want to fire this ?
			// Woj didn't in his subclassed DragDropNode I don't know why not
			m_events.Fire(OnQueryPasteKey, this, new ObjectArgs(ido));
			System.Diagnostics.Debug.WriteLine("OnQueryPaste returns: " + retval);
			return retval;
		}

		#region OnQueryPaste helper
		/// <summary>
		/// See if the node that we are wanting to copy/cut can be pasted
		/// in the node were we want it to be
		///	checks:
		///	- Don't do anything if the Node is null, or the right properties aren't set
		/// - Don't paste on ourselves
		/// - Don't paste our parent or grandparents onto us -> recursive
		/// - Don't paste if they are already a child
		/// </summary>
		/// <param name="nodeToBePasted">IDataObject of item to be pasted</param>
		/// <returns></returns>
		protected virtual bool OnQueryPasteValidator(BaseNode nodeToBePasted)
		{
			bool b = true;
			bool retval = false;
			BaseNode target = this;
			BaseNode parent = this.ParentNode;
			if(nodeToBePasted != null && nodeToBePasted.Cutable && this.Pasteable)
			{
				while(target.ParentHScopeItem != 0)
				{
					if(nodeToBePasted.HScopeItem == parent.HScopeItem)
					{
						b = false;
						break;
					}
					target = parent;
					parent = parent.ParentNode;
				}
				retval = (b && // dont paste our parent or grandparents onto us
					nodeToBePasted.HScopeItem != this.HScopeItem && // don't paste on ourselves
					nodeToBePasted.ParentHScopeItem != this.HScopeItem); // and finally don't paste if they are already a child
			}
			return retval;
		}
		#endregion

		/// <summary>
		/// Result view is refreshed
		/// </summary>
		public virtual void OnRefresh()
		{
			m_events.Fire(OnRefreshKey, this, null);
		}

		/// <summary>
		/// Informs the snapin to delete all the cookies it has added below.
		/// </summary>
		public virtual void OnRemoveChildren()
		{
			m_events.Fire(OnRemoveChildrenKey, this, null);
		}

		/// <summary>
		///  Calle dto see if the node can be renamed
		/// </summary>
		/// <returns></returns>
		public virtual bool OnTryRename()
		{
			return false;
		}

		/// <summary>
		/// Called to supply the renamed value.  Return true if successful
		/// </summary>
		/// <param name="newName"></param>
		/// <returns></returns>
		public virtual bool OnRename(string newName)
		{
			m_events.Fire(OnRenameKey, this, new RenameArgs(newName));
			return false;
		}

		/// <summary>
		/// Called when node is selected in the scope pane
		/// </summary>
		public virtual void OnSelectScope()
		{
			if(this.Cutable)
			{
				// without the copy verb MMC 1.2 can't D&D
				SetVerbState(MMC_CONSOLE_VERB.COPY, MMC_BUTTON_STATE.ENABLED, true);
				SetVerbState(MMC_CONSOLE_VERB.CUT, MMC_BUTTON_STATE.ENABLED, true);
			}

			if(this.Pasteable)
				SetVerbState(MMC_CONSOLE_VERB.PASTE, MMC_BUTTON_STATE.ENABLED, true);

			// MAM: register this node as the current node for this snapin.
			this.m_Snapin.CurrentScopeNode = this;

			// See if we need to enable then property sheets item on the popup menu
			if (HavePropertyPages)
				SetVerbState(MMC_CONSOLE_VERB.PROPERTIES, MMC_BUTTON_STATE.ENABLED, true);
			else
				SetVerbState(MMC_CONSOLE_VERB.PROPERTIES, MMC_BUTTON_STATE.ENABLED, false);

			m_events.Fire(OnSelectScopeKey, this, null);
		}

		/// <summary>
		/// Called when node is deselected in the scope pane
		/// </summary>
		public virtual void OnDeselectScope()
		{
			m_events.Fire(OnDeselectScopeKey, this, null);
		}

		/// <summary>
		/// Called when node is selected in the result pane
		/// </summary>
		public virtual void OnSelectResult()
		{

			//Added by Alexander Kachalkov for Copy&Past support for Report Node
			if(this.Cutable)
			{
				// without the copy verb MMC 1.2 can't D&D
				SetVerbState(MMC_CONSOLE_VERB.COPY, MMC_BUTTON_STATE.ENABLED, true);
				SetVerbState(MMC_CONSOLE_VERB.CUT, MMC_BUTTON_STATE.ENABLED, true);
			}

			if(this.Pasteable)
				SetVerbState(MMC_CONSOLE_VERB.PASTE, MMC_BUTTON_STATE.ENABLED, true);
			
			m_events.Fire(OnSelectResultKey, this, null);
		}
        
		/// <summary>
		/// Called when node is deselected in the result pane
		/// </summary>
		public virtual void OnDeselectResult()
		{
			m_events.Fire(OnDeselectResultKey, this, null);
		}

		/// <summary>
		/// Show the Node
		/// </summary>
		public virtual void OnShow()
		{
			m_events.Fire(OnShowKey, this, null);
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// Show the Node
		/// </summary>
		/// <param name="bSelecting">TRUE if selecting, FALSE if deselecting</param>
		/// <remarks>
		/// TRUE if selecting. Indicates that the snap-in should set up the result pane and add the enumerated items. 
		///	FALSE if deselecting. Indicates that the snap-in is going out of focus and that it should clean up all result item cookies, because the current result pane will be replaced by a new one.
		/// </remarks>
		public virtual void OnShow(bool bSelecting)
		{
			m_events.Fire(OnShowKey, this, null);
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// Called when ToolBar button pressed
		/// </summary>
		public virtual void OnControlbarCommand(IControlbar Controlbar, IToolbar Toolbar, MMC_CONSOLE_VERB verb)
		{
			m_events.Fire(OnControlbarCommandKey, this, new ToolbarCommandArgs(Controlbar, Toolbar, verb));
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// Called when ToolBar created
		/// </summary>
		public virtual void OnControlbarNotify(IControlbar Controlbar, IToolbar Toolbar, bool bScope, bool bSelect)
		{
			m_events.Fire(OnControlbarNotifyKey, this, new SetToolbarArgs(Controlbar, Toolbar, bScope, bSelect));
		}

		public virtual void OnViewChangeScope(BaseNode node)
		{
			m_events.Fire(OnViewChangeScopeKey, this, null);
		}
        
		public virtual void OnViewChangeResult(BaseNode node)
		{
			m_events.Fire(OnViewChangeResultKey, this, null);
		}
        
		public virtual void OnContextHelp()
		{
			m_events.Fire(OnContextHelpKey, this, null);
		}

		//Added by Attila Kapostyak
		public virtual void OnPrint()							
		{														
			m_events.Fire(OnPrintKey, this, null);				
		}														

		//Added by Wojciech Sobocinski
		/// <summary>
		/// Paste the ido item
		/// </summary>
		/// <param name="ido">IDataObject of item to be pasted</param>
		/// <returns></returns>
		public virtual bool OnPaste(IDataObject ido)
		{
			bool retval = false;
			BaseNode nodeToBePasted;
			if(!(ido is DataObject))
				return false;
				
			nodeToBePasted = ((Ironring.Management.MMC.DataObject)ido).Node;

			// By Wojciech Sobocinski, implemented in BaseNode by Lesley van Zijl
			// is this check required ?
			if(nodeToBePasted.Cutable && this.Pasteable)
			{
							
				// calling node.Remove(false) caused a trap when inserting back with an invalid handle
				// so we will do it manually
				Snapin.ConsoleNameSpace.DeleteItem((uint)nodeToBePasted.HScopeItem, 1);

				// need to remove from parent child
				BaseNode parent = nodeToBePasted.ParentNode;
				parent.Children.Remove(nodeToBePasted);
				this.AddChild(nodeToBePasted);
				nodeToBePasted.Insert(this);
			
				retval = true;
			}

			m_events.Fire(OnPasteKey, this, new ObjectArgs(ido));
			return retval;
		}

		// MAM: added column filtering
		/// <summary>
		/// Column filtering 
		/// </summary>
		/// <param name="code"></param>
		/// <param name="col"></param>
		public virtual void OnFilterChange(MMC_FILTER_CHANGE_CODE code, int col) 
		{
			m_events.Fire(OnFilterChangeKey, this, null);
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// Get the filter operators menu for a result view column
		/// </summary>
		public virtual bool OnGetFilterMenu(int col, RECT data)
		{
			m_events.Fire(OnGetFilterMenuKey, this, new FilterArgs(col, data));
			return false;
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// User selects an entry from a filter operator menu
		/// </summary>
		public virtual void OnFilterOperator() 
		{
			m_events.Fire(OnFilterOperatorKey, this, null);
		}

		#endregion
		//////////////////////////////////////////////////////////////////////
		//
		// Published Events
		//
		#region
		public event NodeNotificationHandler OnActivateEvent
		{
			add { m_events.AddHandler(OnActivateKey, value); }
			remove { m_events.RemoveHandler(OnActivateKey, value); }
		}

		public event NodeNotificationHandler OnAddScopePaneImagesEvent
		{
			add { m_events.AddHandler(OnAddScopePaneImagesKey, value); }
			remove { m_events.RemoveHandler(OnAddScopePaneImagesKey, value); }
		}

		public event NodeNotificationHandler OnAddResultPaneImagesEvent
		{
			add { m_events.AddHandler(OnAddResultPaneImagesKey, value); }
			remove { m_events.RemoveHandler(OnAddResultPaneImagesKey, value); }
		}

		public event NodeNotificationHandler OnBtnClickEvent
		{
			add { m_events.AddHandler(OnBtnClickKey, value); }
			remove { m_events.RemoveHandler(OnBtnClickKey, value); }
		}

		public event NodeNotificationHandler OnClickEvent
		{
			add { m_events.AddHandler(OnClickKey, value); }
			remove { m_events.RemoveHandler(OnClickKey, value); }
		}

		public event NodeNotificationHandler OnColumnClickEvent
		{
			add { m_events.AddHandler(OnColumnClickKey, value); }
			remove { m_events.RemoveHandler(OnColumnClickKey, value); }
		}

		public event NodeNotificationHandler OnCutOrRemoveEvent
		{
			add { m_events.AddHandler(OnCutOrRemoveKey, value); }
			remove { m_events.RemoveHandler(OnCutOrRemoveKey, value); }
		}

		public event NodeNotificationHandler OnDblClickEvent
		{
			add { m_events.AddHandler(OnDblClickKey, value); }
			remove { m_events.RemoveHandler(OnDblClickKey, value); }
		}

		public event NodeNotificationHandler OnDeleteEvent
		{
			add { m_events.AddHandler(OnDeleteKey, value); }
			remove { m_events.RemoveHandler(OnDeleteKey, value); }
		}

		public event NodeNotificationHandler OnDeselectAllEvent
		{
			add { m_events.AddHandler(OnDeselectAllKey, value); }
			remove { m_events.RemoveHandler(OnDeselectAllKey, value); }
		}

		public event NodeNotificationHandler OnExpandEvent
		{
			add { m_events.AddHandler(OnExpandKey, value); }
			remove { m_events.RemoveHandler(OnExpandKey, value); }
		}

		public event NodeNotificationHandler OnCollapseEvent
		{
			add { m_events.AddHandler(OnCollapseKey, value); }
			remove { m_events.RemoveHandler(OnCollapseKey, value); }
		}

		public event NodeNotificationHandler OnInitOcxEvent
		{
			add { m_events.AddHandler(OnInitOcxKey, value); }
			remove { m_events.RemoveHandler(OnInitOcxKey, value); }
		}

		//--added by Roman Kiss
		public event NodeNotificationHandler OnUserEvent
		{
			add { m_events.AddHandler(OnUserKey, value); }
			remove { m_events.RemoveHandler(OnUserKey, value); }
		}

		//--added by Roman Kiss
		public event NodeNotificationHandler OnQueryPropertiesEvent
		{
			add { m_events.AddHandler(OnQueryPropertiesKey, value); }
			remove { m_events.RemoveHandler(OnQueryPropertiesKey, value); }
		}
		
		public event NodeNotificationHandler OnMenuBtnClickEvent
		{
			add { m_events.AddHandler(OnMenuBtnClickKey, value); }
			remove { m_events.RemoveHandler(OnMenuBtnClickKey, value); }
		}

		public event NodeNotificationHandler OnMaximizedEvent
		{
			add { m_events.AddHandler(OnMaximizedKey, value); }
			remove { m_events.RemoveHandler(OnMaximizedKey, value); }
		}

		public event NodeNotificationHandler OnMinimizedEvent
		{
			add { m_events.AddHandler(OnMinimizedKey, value); }
			remove { m_events.RemoveHandler(OnMinimizedKey, value); }
		}

		public event NodeNotificationHandler OnQueryPasteEvent
		{
			add { m_events.AddHandler(OnQueryPasteKey, value); }
			remove { m_events.RemoveHandler(OnQueryPasteKey, value); }
		}

		public event NodeNotificationHandler OnRefreshEvent
		{
			add { m_events.AddHandler(OnRefreshKey, value); }
			remove { m_events.RemoveHandler(OnRefreshKey, value); }
		}

		public event NodeNotificationHandler OnRemoveChildrenEvent
		{
			add { m_events.AddHandler(OnRemoveChildrenKey, value); }
			remove { m_events.RemoveHandler(OnRemoveChildrenKey, value); }
		}

		public event NodeNotificationHandler OnRenameEvent
		{
			add { m_events.AddHandler(OnRenameKey, value); }
			remove { m_events.RemoveHandler(OnRenameKey, value); }
		}

		public event NodeNotificationHandler OnSelectScopeEvent
		{
			add { m_events.AddHandler(OnSelectScopeKey, value); }
			remove { m_events.RemoveHandler(OnSelectScopeKey, value); }
		}

		public event NodeNotificationHandler OnDeselectScopeEvent
		{
			add { m_events.AddHandler(OnDeselectScopeKey, value); }
			remove { m_events.RemoveHandler(OnDeselectScopeKey, value); }
		}

		public event NodeNotificationHandler OnSelectResultEvent
		{
			add { m_events.AddHandler(OnSelectResultKey, value); }
			remove { m_events.RemoveHandler(OnSelectResultKey, value); }
		}

		public event NodeNotificationHandler OnDeselectResultEvent
		{
			add { m_events.AddHandler(OnDeselectResultKey, value); }
			remove { m_events.RemoveHandler(OnDeselectResultKey, value); }
		}

		public event NodeNotificationHandler OnShowEvent
		{
			add { m_events.AddHandler(OnShowKey, value); }
			remove { m_events.RemoveHandler(OnShowKey, value); }
		}

		public event NodeNotificationHandler OnViewChangeScopeEvent
		{
			add { m_events.AddHandler(OnViewChangeScopeKey, value); }
			remove { m_events.RemoveHandler(OnViewChangeScopeKey, value); }
		}

		public event NodeNotificationHandler OnViewChangeResultEvent
		{
			add { m_events.AddHandler(OnViewChangeResultKey, value); }
			remove { m_events.RemoveHandler(OnViewChangeResultKey, value); }
		}

		public event NodeNotificationHandler OnContextHelpEvent
		{
			add { m_events.AddHandler(OnContextHelpKey, value); }
			remove { m_events.RemoveHandler(OnContextHelpKey, value); }
		}

		//Added by Alexander Kachalkov
		public event NodeNotificationHandler OnControlbarCommandEvent
		{
			add { m_events.AddHandler(OnControlbarCommandKey, value); }
			remove { m_events.RemoveHandler(OnControlbarCommandKey, value); }
		}

		//Added by Alexander Kachalkov
		public event NodeNotificationHandler OnControlbarNotifyEvent
		{
			add { m_events.AddHandler(OnControlbarNotifyKey, value); }
			remove { m_events.RemoveHandler(OnControlbarNotifyKey, value); }
		}

		//Added by Wojciech Sobocinski
		public event NodeNotificationHandler OnPasteEvent
		{
			add { m_events.AddHandler(OnPasteKey, value); }
			remove { m_events.RemoveHandler(OnPasteKey, value); }
		}

		#endregion
		//////////////////////////////////////////////////////////////////////
		//
		// Context Menu Management
		//
		#region
		//---added by Roman Kiss
		public MenuItem GetMenuItem(int intCCM, string name) 
		{
			ArrayList selArraList = null;

			//---find the properly menu container
			switch((CCM)intCCM)
			{
				case CCM.INSERTIONALLOWED_TOP:
					selArraList = m_TopMenuItems;
					break;

				case CCM.INSERTIONALLOWED_NEW:
					selArraList = m_NewMenuItems;
					break;

				case CCM.INSERTIONALLOWED_TASK:
					selArraList = m_TaskMenuItems;
					break;

				case CCM.INSERTIONALLOWED_VIEW:
					selArraList = m_ViewMenuItems;
					break;

				default:
                    return null;
			}

			//---find the item in the menu
			foreach(object obj in selArraList) 
			{
				MenuItem mi = obj as MenuItem;
				if(mi.Name == name)
					return mi;
			}

			//---no exist
			return null;
		}

		//changed to virtual: requested by medyanik
		public virtual void Command(int nCommandId)
		{		
			if (nCommandId >= 0 && nCommandId < MenuItemIndex.Count)
			{
				MenuItem item = (MenuItem)MenuItemIndex[nCommandId];
				item.Command(this);
			}
		}

		public void AddTopMenuItem(MenuItem item)
		{
			if (m_TopMenuItems == null)
				m_TopMenuItems = new ArrayList();

			// set the command Id so we can retreive this item later
			item.CommandId = MenuItemIndex.Count;
			MenuItemIndex.Add(item);

			m_TopMenuItems.Add(item);
		}
		
		public void AddNewMenuItem(MenuItem item)
		{
			if (m_NewMenuItems == null)
				m_NewMenuItems = new ArrayList();

			item.CommandId = MenuItemIndex.Count;
			MenuItemIndex.Add(item);

			m_NewMenuItems.Add(item);
		}
		public void AddTaskMenuItem(MenuItem item)
		{
			if (m_TaskMenuItems == null)
				m_TaskMenuItems = new ArrayList();

			item.CommandId = MenuItemIndex.Count;
			MenuItemIndex.Add(item);

			m_TaskMenuItems.Add(item);
		}
		public void AddViewMenuItem(MenuItem item)
		{
			if (m_ViewMenuItems == null)
				m_ViewMenuItems = new ArrayList();

			item.CommandId = MenuItemIndex.Count;
			MenuItemIndex.Add(item);

			m_ViewMenuItems.Add(item);
		}
		
		protected ArrayList MenuItemIndex 
		{
			get 
			{
				if (m_MenuItems == null)
					m_MenuItems = new ArrayList();
	
				return m_MenuItems;
			}
		}

		protected virtual void OnAddTopMenu(IContextMenuCallback piCallback)
		{
			if (m_TopMenuItems != null)
			{
				foreach(MenuItem item in m_TopMenuItems)
				{
					if(item.Visible) 
					{
						// Let's add "Refresh Page" the the "view" menu
						CONTEXTMENUITEM newitem = new CONTEXTMENUITEM();

						newitem.strName = item.Name;
						newitem.strStatusBarText = item.StatusText;
						newitem.lCommandID = item.CommandId;
						newitem.lInsertionPointID = (uint)CCM.INSERTIONPOINTID_PRIMARY_TOP;
						newitem.fFlags = (int)item.Flags;
						newitem.fSpecialFlags = (int)item.SpecialFlags;

						// Now add this item through the callback
						piCallback.AddItem(ref newitem);
					}
				}
			}
		}

		/// <summary>
		/// Called by the framework wo add menu items to the MMC "Top Menu"
		/// </summary>
		/// <param name="piCallback"></param>
		protected virtual void OnAddNewMenu(IContextMenuCallback piCallback)
		{
			if (m_NewMenuItems != null)
			{
				foreach(MenuItem item in m_NewMenuItems)
				{
					if(item.Visible) 
					{
						// Let's add "Refresh Page" the the "view" menu
						CONTEXTMENUITEM newitem = new CONTEXTMENUITEM();

						newitem.strName = item.Name;
						newitem.strStatusBarText = item.StatusText;
						newitem.lCommandID = item.CommandId;
						newitem.lInsertionPointID = (uint)CCM.INSERTIONPOINTID_PRIMARY_NEW;
						newitem.fFlags = (int)item.Flags;
						newitem.fSpecialFlags = (int)item.SpecialFlags;

						// Now add this item through the callback
						piCallback.AddItem(ref newitem);
					}
				}
			}
		}

		/// <summary>
		/// Called by the framework wo add menu items to the MMC "Task Menu"
		/// </summary>
		/// <param name="piCallback"></param>
		protected virtual void OnAddTaskMenu(IContextMenuCallback piCallback)
		{
			if (m_TaskMenuItems != null)
			{
				foreach(MenuItem item in m_TaskMenuItems)
				{
					if(item.Visible) 
					{
						// Let's add "Refresh Page" the the "view" menu
						CONTEXTMENUITEM newitem = new CONTEXTMENUITEM();

						newitem.strName = item.Name;
						newitem.strStatusBarText = item.StatusText;
						newitem.lCommandID = item.CommandId;
						newitem.lInsertionPointID = (uint)CCM.INSERTIONPOINTID_PRIMARY_TASK;
						newitem.fFlags = (int)item.Flags;
						newitem.fSpecialFlags = (int)item.SpecialFlags;

						// Now add this item through the callback
						piCallback.AddItem(ref newitem);
					}
				}
			}
		}

		/// <summary>
		/// Called by the framework to add menu items to the MMC "View Menu"
		/// </summary>
		/// <param name="piCallback"></param>
		protected virtual void OnAddViewMenu(IContextMenuCallback piCallback)
		{
			if (m_ViewMenuItems != null)
			{
				foreach(MenuItem item in m_ViewMenuItems)
				{
					if(item.Visible) 
					{
						// Let's add "Refresh Page" the the "view" menu
						CONTEXTMENUITEM newitem = new CONTEXTMENUITEM();

						newitem.strName = item.Name;
						newitem.strStatusBarText = item.StatusText;
						newitem.lCommandID = item.CommandId;
						newitem.lInsertionPointID = (uint)CCM.INSERTIONPOINTID_PRIMARY_VIEW;
						newitem.fFlags = (int)item.Flags;
						newitem.fSpecialFlags = (int)item.SpecialFlags;

						// Now add this item through the callback
						piCallback.AddItem(ref newitem);
					}
				}
			}
		}

		//Changed by Alexander Kachalkov
		/// <summary>
		/// Return Image index for this node.
		/// Many nodes can share the same image index.
		/// </summary>
		protected virtual int GetResultViewImageIndex(RESULTDATAITEM rdi)
		{
			int offset = 0;
			if (IsUseSmallIcons())
				offset = SmallImageIndex;
			else
                offset = LargeImageIndex;

			return (Cookie << 16) + offset;
		}

		/// <summary>
		/// Helper method to determine if the result view for this node
		/// is in large icon or small icon view.  Based on what the component
		/// IConsole2 interface says
		/// </summary>
		/// <returns></returns>
		protected bool IsUseSmallIcons()
		{
			IConsole2 console = Snapin.ResultViewConsole;

			if (console != null)
			{
				IResultData rdata = console as IResultData;
				if (rdata != null)
				{
					int nViewType = 0;
					rdata.GetViewMode(out nViewType);
					return nViewType != 0;
				}
			}
			// default to small icons
			return true;
		}
		#endregion

		// MAM: more property page support.
		/// <summary>
		/// Clear the set of propertypages for this node.		
		/// </summary>
		public virtual void ClearPropertyPages() {
			this.m_propertyPages.Clear();
		}

		//Added by Alexander Kachalkov
		//////////////////////////////////////////////////////////////////////////
		// P/Invoke Helpers for Themes
		[DllImport("kernel32.dll", CharSet=CharSet.Auto)]
		public static extern IntPtr CreateActCtx(ref ACTCTX  pActCtx);
    
		[DllImport("kernel32.dll")]
		public static extern bool ActivateActCtx(IntPtr hActCtx,  out uint lpCookie);

		[DllImport("kernel32.dll")]
		public static extern bool ReleaseActCtx(IntPtr hActCtx);

		//Summary Changes Alexander Ovchinnikov
		#region
		public void RemovePropertySheet( ){
			if( this.m_propSheet != null && !m_propSheet.isClosed ){
				m_propSheet.Invoke( m_propSheet.m_CloseHandle, new object[]{ string.Empty, this } );
			}
		}

		public bool IsSelected( ){
			RESULTDATAITEM rdi = new RESULTDATAITEM();
			this.Snapin.GetSelectedResultDataItem( ref rdi );
			return rdi.lParam == this.Cookie;
		}

		public virtual void UpdateThisNode( string key, object val ){
			DisplayName = key;

			IConsoleNameSpace2 console = this.Snapin.ConsoleNameSpace;
			SCOPEDATAITEM sdi = new SCOPEDATAITEM();
			
			sdi.mask =	(uint)SDI.STR;
			sdi.displayname = (IntPtr)(-1);
			sdi.ID = this.HScopeItem;
			
			console.SetItem( ref sdi );			
		}

        /// <summary>
        /// Authors can override this method to support their own custom data formats.
        /// 
        /// </summary>
        /// <param name="cf">The clipboard format, as specified in the FORMATETC
        ///                  struct passed to GetDataHere.
        /// </param>
        /// <param name="writer">An output stream to which the data representing
        ///                  this format should be written.  If the format is not
        ///                  understood, then nothing should be written.
        /// </param>
        /// <returns>"true" if the clipboard format is understood.  When true is returned,
        ///             the conents of this format should have been written into "writer".
        ///          "false" is the clipboard format is unknown.
        ///          </returns>
        public virtual bool HandleCustomClipboardFormat(ushort cf, BinaryWriter writer)
        {
            // override this if you're interested in handing your own
            // clipboard formats
            //
            return false;
        }

	    #endregion
		
	}

}
