using System;
using System.IO;
using System.Runtime.InteropServices;


namespace Ironring.Management.MMC
{
	/// <summary>
	/// UCOMIStream to .NET wrapper.
	/// Why do I have to implement this anyway?
	/// </summary>
	public class ComStream : Stream {

		protected int m_readCount = 0;
		protected int m_writeCount = 0;

		protected UCOMIStream m_comStream;
		public UCOMIStream comStream {
			get { return m_comStream; }
			set { m_comStream = value; }
		}
	
		public ComStream(UCOMIStream stream) : base() {
			comStream = stream;			
		}

		#region Stream interface
		public override bool CanRead {
			get { return true; }
		}

		public override bool CanSeek {
			get { return true; }
		}

		public override bool CanWrite {
			get { return true; }
		}

		public override long Length {
			get { 
				System.Runtime.InteropServices.STATSTG pstatstg;
				this.comStream.Stat(out pstatstg, 0);
				return pstatstg.cbSize;
			}
		}

		public override long Position {
			get { 
				System.IntPtr ptr = new System.IntPtr();
				this.comStream.Seek(0, (int)SeekOrigin.Current, ptr);
				return ptr.ToInt64();
			}
			set { 
				System.IntPtr ptr = new System.IntPtr();
				this.comStream.Seek(value, (int)SeekOrigin.Begin, ptr); 
			}
		}

		public override void Close() {
			if(this.comStream != null) {
				this.comStream.Commit(0);
				// Make sure we really unwrap.
			}
			this.comStream = null;
		}

		public override void Flush() {
			if(this.comStream != null) {
				this.comStream.Commit(0);
			}
		}

		public override long Seek(long offset, SeekOrigin origin) {
			System.IntPtr newPos = new System.IntPtr();
			this.comStream.Seek(offset, (int)origin, newPos);
			return newPos.ToInt64();
		}
		
		public override int Read([In,Out] byte[] buffer, int offset, int count) {
			byte[] buf = new byte[count];
			System.IntPtr lenPtr = new System.IntPtr();
			this.comStream.Read(buf, count, lenPtr);
			int len = lenPtr.ToInt32();
			m_readCount += count;
			System.Diagnostics.Debug.WriteLine("Read: " + m_readCount + " bytes of the requested: " + count + " bytes");
			int pos=0;
 			//while(pos < len) {
			while(pos < count) {
				buffer[offset++] = buf[pos++];
			}			
			//return lenPtr.ToInt32();
			return count;
		}

		public override void SetLength(long value) {
			this.comStream.SetSize(value);
		}

		public override void Write(byte[] buffer, int offset, int count) {
			System.IntPtr writtenPtr = new System.IntPtr();
			byte[] buf = new byte[count];
			m_writeCount += count;
			System.Diagnostics.Debug.WriteLine("Wrote: " + m_writeCount + " bytes");
			for(int i=0; i<count; i++) {
				buf[i] = buffer[offset+i];
			}

			this.comStream.Write(buf, count, writtenPtr);			
		}

		#endregion
	}
}
