using System;
using System.Threading;
using System.Windows.Forms;

namespace Ironring.Management.MMC
{
	/// <summary>
	/// Summary description for ExceptionHandler.
	/// </summary>
	// Creates a class to handle the exception event.
	public class CustomExceptionHandler 
	{

		static bool	_showThreadException;
		public static bool ShowThreadException
		{
			get { return _showThreadException; }
			set { _showThreadException = value; }
		}
		 
		// Handles the exception event.
		public static void OnThreadException(object sender, ThreadExceptionEventArgs t) 
		{
			if (!ShowThreadException) return;

			DialogResult result = DialogResult.Cancel;
			try
			{
				if (t.Exception.InnerException == null)
					result = ShowThreadExceptionDialog(t.Exception);
				else
					result = ShowThreadExceptionDialog(t.Exception.InnerException);
			}
			catch
			{
				try
				{
					MessageBox.Show("Fatal Error", "Fatal Error", MessageBoxButtons.AbortRetryIgnore, MessageBoxIcon.Stop);
				}
				finally
				{
					Application.Exit();
				}
			}

			// Exits the program when the user clicks Abort.
			if (result == DialogResult.Abort) 
				Application.Exit();
		}
 
		// Creates the error message and displays it.
		private static DialogResult ShowThreadExceptionDialog(Exception e) 
		{
			string errorMsg = "An error occurred please contact the adminstrator with the following information:\n\n";
			errorMsg = errorMsg + e.Message + "\n\nStack Trace:\n" + e.StackTrace;
			return MessageBox.Show(errorMsg, "Application Error", MessageBoxButtons.AbortRetryIgnore, MessageBoxIcon.Stop);
		}
	}
}
