using System;
using System.IO;
using System.Collections;
using System.Runtime.InteropServices;

using System.Threading;
using ceh = Ironring.Management.MMC.CustomExceptionHandler;

namespace Ironring.Management.MMC
{
	/// <summary>
	/// Summary description for FolderNode.
	/// </summary>
	public class FolderNode : BaseNode
	{
		protected string path;
		protected ArrayList Files = new ArrayList();

		public FolderNode(SnapinBase snapin, string path) : base(snapin)
		{			
			this.path = path;
			CreateChildFiles(path);
		}
		protected void CreateChildFiles(string path)
		{
			DirectoryInfo directoryInfo = new DirectoryInfo(path);
			foreach(FileInfo fileInfo in directoryInfo.GetFiles()) 
			{
				FileNode file = new FileNode(this.Snapin, this);
				file.DisplayName = fileInfo.Name;

				Files.Add(file);
			}
		}

		/// <summary>
		/// This method is called by the MMC snap to populate the data for listview 
		/// in the right pane, for the respective node item selected in the left pane.
		/// </summary>
		public override void OnShow() 
		{
			IConsole2 console = Snapin.ResultViewConsole;

			IHeaderCtrl2 header = console as IHeaderCtrl2;
			OnShowHeader(header);

			IResultData rdata = console as IResultData;
			OnShowData(rdata);

			rdata.SetViewMode((int)ViewMode.Report);

			base.OnShow();
		}

		/// <summary>
		/// This method is called by the OnShow() method to display header for the listview 
		/// in the right pane, for the respective node item selected in the left pane.
		/// </summary>
		public virtual void OnShowHeader(IHeaderCtrl2 header) 
		{
			try 
			{
				header.InsertColumn(0, "Name", (int)ColumnHeaderFormat.LEFT, 250);					
			}
			catch(COMException e) 
			{
				System.Diagnostics.Debug.WriteLine(e.Message);
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
			catch(Exception e) 
			{
				System.Diagnostics.Debug.WriteLine(e.Message);
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
		}

		/// <summary>
		/// This method is called by the OnShow() method to display data for the listview 
		/// in the right pane, for the respective node item selected in the left pane.
		/// </summary>
		public virtual void OnShowData(IResultData ResultData) 
		{
			int nRow = 1;

			RESULTDATAITEM rdi = new RESULTDATAITEM();
			foreach (FileNode node in Files) 
			{
				node.ToString();

				rdi.mask = (uint)RDI.STR | (uint)RDI.IMAGE | (uint)RDI.PARAM;     
				rdi.nImage = 0; 
				rdi.str = (IntPtr)(-1);
				rdi.nCol = 0;
				rdi.lParam = node.Cookie | (nRow <<  16);

				ResultData.InsertItem(ref rdi);
				nRow++;
			}
		}
	}


	public class FileNode : BaseNode
	{
		protected FolderNode parentFolder = null;

		public FileNode(SnapinBase snapin) : base(snapin)
		{			

		}

		public FileNode(SnapinBase snapin, FolderNode parFolder) : base(snapin)
		{
			parentFolder = parFolder;
		}

		public override void GetDisplayInfo(ref RESULTDATAITEM ResultDataItem) 
		{
			bool bCallbase = true;

			int nRow = (ResultDataItem.lParam >> 16) - 1;
			int nCol = ResultDataItem.nCol;

			if ((ResultDataItem.mask & (uint)RDI.STR) > 0) 
			{				
				string data = DisplayName;

				ResultDataItem.str = Marshal.StringToCoTaskMemUni(data);
			}

			if ((ResultDataItem.mask & (uint)RDI.IMAGE) > 0) 
			{
				int offset = LargeImageIndex;
				if (IsUseSmallIcons())
					offset = SmallImageIndex;

				ResultDataItem.nImage = (parentFolder.Cookie << 16) + offset;
			}

			if (bCallbase)
				base.GetDisplayInfo(ref ResultDataItem);
		}
	}
}
