using System;
using SHDocVw;
using mshtml;

using System.Threading;
using System.Runtime.InteropServices;

using ceh = Ironring.Management.MMC.CustomExceptionHandler;

namespace Ironring.Management.MMC
{
	/// <summary>
	/// HTML node class provides a web browser for the result view.  
	/// Set the Url to set the starting page.
	/// </summary>
	public class HTMLNode : BaseNode
	{
        /// <summary>
        /// The Url to navigate to on startup.
        /// </summary>
		protected string m_strUrl;

        //////////////////////////////////////////////////////////////////
        //
        // Implementation
        //
        
        /// <summary>
        /// ctor.
        /// </summary>
        /// <param name="snapin"></param>
		public HTMLNode(SnapinBase snapin) : base(snapin)
		{
			m_strUrl = "about:blank";
			this.OnSelectScopeEvent += new NodeNotificationHandler(OnSelectScope);
		}

        /// <summary>
        /// Set or get the url to navigate to on startup.
        /// </summary>
		public string Url
		{
			get { return m_strUrl; }
            set { m_strUrl = value; }
		}
		
		/// <summary>
		/// Provide MMC with the startup Url when called
		/// </summary>
		/// <param name="pViewOptions"></param>
		/// <returns></returns>
		public override string GetResultViewType(ref int pViewOptions)
		{
			// the default for std list views
			pViewOptions = 0;
			return Url;		
		}

		/// <summary>
		/// When root is selected, make the Print action available.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected virtual void OnSelectScope(object sender, NodeEventArgs e)
		{
			SetVerbState(MMC_CONSOLE_VERB.PRINT, MMC_BUTTON_STATE.ENABLED, true);
		}

		/// <summary>
		/// Allows printing of the web page using the browser's print functionality.
		/// </summary>
		public override void OnPrint()
		{
			try 
			{
				object objWebBrowser;
				m_Snapin.ResultViewConsole.QueryResultView(out objWebBrowser);

				// Did we find the browser object?
				if ((objWebBrowser as SHDocVw.IWebBrowser2) != null)
				{
					SHDocVw.IWebBrowser2 webBrowser = (SHDocVw.IWebBrowser2)objWebBrowser;

					// Did we find the document object?
					if ((webBrowser.Document as IHTMLDocument2) != null)
					{
						IHTMLDocument2 htmlDoc = (IHTMLDocument2)webBrowser.Document;
						htmlDoc.execCommand("Print", true, null);
					}
				}
			}
			//Added by Alexander Kachalkov
			catch(COMException e) 
			{
				System.Diagnostics.Debug.WriteLine(e.Message);
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
			catch(Exception ex)
			{
				string method = this.GetType().FullName + "." + System.Reflection.MethodBase.GetCurrentMethod().Name + "(): ";
				System.Diagnostics.Trace.WriteLine(method + ex.Message, "EXCEPTION");

				//Added by Alexander Kachalkov
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(ex));
			}
		}
	}
}
