using System;

namespace Ironring.Management.MMC
{	
	public delegate void NodeNotificationHandler(object sender, NodeEventArgs args);

	/// <summary>
	/// Summary description for NodeEvents.
	/// </summary>
	public class NodeEventArgs : EventArgs
	{
		public NodeEventArgs()
		{
		}
	}

	public class ActivateArgs : NodeEventArgs
	{
		public bool m_activate;

		public ActivateArgs(bool activate)
		{
			m_activate = activate;
		}
	}

	public class AddImagesArgs : NodeEventArgs
	{
		public IImageList m_il;
        
		public AddImagesArgs(IImageList il)
		{
			m_il = il;
		}
	}

	public class CommandArgs : NodeEventArgs
	{
		public int m_command;
        
		public CommandArgs(int command)
		{
			m_command = command;
		}
	}

	public class ObjectArgs : NodeEventArgs
	{
		public object m_object;
        
		public ObjectArgs(object obj)
		{
			m_object = obj;
		}
	}

	//---added by Roman Kiss
	public class NameValueArgs : NodeEventArgs
	{
		private string m_key;
		private object m_val;
    
		public string Key {get{ return m_key; } set {m_key = value;}}
		public object Val {get{ return m_val; } set {m_val = value;}}
		public NameValueArgs() { m_key = null; m_val = null; }
		public NameValueArgs(string key, object val){	m_key = key;	m_val = val; }
	}

	//Added by Alexander Kachalkov
	public class ToolbarBaseArgs : NodeEventArgs
	{
		private IToolbar m_toolbar;
		private IControlbar m_ctrlbar;

		public IToolbar Toolbar {get{return m_toolbar;}set{m_toolbar=value;}}
		public IControlbar ControlBar {get{return m_ctrlbar;}set{m_ctrlbar=value;}}
		
		public ToolbarBaseArgs(IControlbar ControlBar, IToolbar Toolbar) {m_ctrlbar = ControlBar; m_toolbar = Toolbar;}
	}

	//Added by Alexander Kachalkov
	public class SetToolbarArgs : ToolbarBaseArgs
	{
		private bool m_bScope;
		private bool m_bSelect;

		public bool bScope {get{return m_bScope;}}
		public bool bSelect {get{return m_bSelect;}}

		public SetToolbarArgs(IControlbar Controlbar, IToolbar Toolbar, bool bScope, bool bSelect):base(Controlbar, Toolbar) {m_bScope = bScope;; m_bSelect = bSelect;}
	}

	//Added by Alexander Kachalkov
	public class ToolbarCommandArgs : ToolbarBaseArgs
	{
		private MMC_CONSOLE_VERB m_verb;
		public MMC_CONSOLE_VERB verb {get{return m_verb;}}

		public ToolbarCommandArgs(IControlbar Controlbar, IToolbar Toolbar, MMC_CONSOLE_VERB verb):base(Controlbar, Toolbar)  {m_verb = verb;}
	}

	//Added by Alexander Kachalkov
	public class RenameArgs: NodeEventArgs
	{
		private string m_newName;
    
		public string NewName {get{ return m_newName; } set {m_newName = value;}}
		public RenameArgs(string NewName) { m_newName = NewName;}
	}

	//Added by Alexander Kachalkov
	public class FilterArgs: NodeEventArgs
	{
		private int m_col;
		private RECT m_param;

		public FilterArgs(int col, RECT data) { m_col = col;m_param = data;}
	}
}
