using System;


namespace Ironring.Management.MMC
{
	/// <summary>
	/// Node with a Resultview that contains ActiveX controls.  Provide the
	/// Guid for the OCX and the rest is magic.  Currently doesn't support
	/// sinking events generated by the control.
	/// </summary>
	public class OCXNode : BaseNode
	{
        /// <summary>
        /// The guid for the ActiveX Control
        /// </summary>
        protected Guid m_guidOCX;

        /// <summary>
		/// Create a new ActiveX control for resultviews 
		/// of this type or reuse the existing one
		/// </summary>
		protected bool m_bCreateNew;

		/// <summary>
		/// ctor
		/// </summary>
		/// <param name="snapin">Link back to the SnapinBase of this node</param>
		public OCXNode(SnapinBase snapin) : base(snapin)
		{
			m_bCreateNew = false;
		}

		/// <summary>
		/// ctor
		/// </summary>
		/// <param name="snapin">Link back to the SnapinBase of this node</param>
		/// <param name="displayName">DisplayName of this node</param>
		public OCXNode(SnapinBase snapin, String displayName) : this(snapin)
		{
			this.DisplayName = displayName;
		}

		/// <summary>
		/// ctor 
		/// </summary>
		/// <param name="snapin">Link back to the SnapinBase of this node</param>
		/// <param name="displayName">DisplayName of this node</param>
		/// <param name="OCXGuid">The OCX Guid that will be displayed on this node</param>
		public OCXNode(SnapinBase snapin, String displayName, Guid OCXGuid) : this(snapin, displayName)
		{
			this.OCXGuid = OCXGuid;
		}

        /// <summary>
        /// Access the ActiveX control Guid
        /// </summary>
        public Guid OCXGuid
        {
            get { return m_guidOCX; }
            set { m_guidOCX = value; }
        }

		/// <summary>
		/// This is an MMC concept.  Should it create a new activeX control for each 
		/// result view of this type or a new one.
		/// </summary>
		public bool CreateNew
		{
			get { return m_bCreateNew; }
			set { m_bCreateNew = value; }
		}

		/// <summary>
		/// Provides the dring representation fo the guid to MMC 
		/// to create on its terms.
		/// </summary>
		/// <param name="pViewOptions">Pointer to the value that provides the console with options specified by the owning snap-in.</param>
		/// <returns>A string that specifies the view to display for the specified cookie (see IComponent::GetResultViewType). </returns>
		public override string GetResultViewType(ref int pViewOptions)
		{			
			pViewOptions = 0;			

			// If set, tell MMC to destroy the cached OCX and creates (then caches) a 
			// new OCX every time an item requests the OCX view.
			if (CreateNew)
				pViewOptions |= (int)MMC_VIEW_OPTIONS.CREATENEW;

			// Since we are showing an OCX in the result view, we can tell MMC to disable the 
			// standard listview options from the node's 'View' context menu.
			pViewOptions |= (int)MMC_VIEW_OPTIONS.NOLISTVIEWS;

			// Return the Guid in bracket format i.e. {xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}
			return OCXGuid.ToString("B");
		}

        public override void OnAddResultPaneImages(IImageList il)
        {
            // an OCX view isn't allowed to add images for some reason...
            // at least I think...
            //
            // so don't handle this message, but call into our child nodes
            // to see if they want to handle it.
            //

            // Now add all the children images
            foreach(BaseNode node in m_ChildNodes)
                node.OnAddResultPaneImages(il);

            m_events.Fire(OnAddResultPaneImagesKey, this, new AddImagesArgs(il));
        }
    }
}
