using System;
using System.Runtime.Serialization;
using System.IO;
using System.Text;

namespace Ironring.Management.MMC
{
	// MAM: Snapin persistance.

	/// <summary>
	/// Persisted state of a snapin. 
	/// Subclass and add state. NOTE: beware of adding complete object
	/// hierarchies. Stick to simple non-reference types to be sure
	/// it works.
	/// It seems serialization only works with items in the GAC. We don't
	/// want to be forced into this GAC, so we roll our own serialization.
	/// (if serialization is the only reason to be in the GAC, we don't
	///  want serialization: there seems no further need to be in the GAC,
	///  and it raises cost of maintainability and heavier installations). 
	///  
	/// I use it to store ints and strings and use a '|' character
	/// to seperate them. If someone can tell me how to do proper
	/// Serialization I'd use that.
	/// </summary>	
	public class SnapinStateBase 
	{
		protected bool m_dirty = false;
		/// <summary>
		/// Set this to true when the state has changed (is dirty).
		/// </summary>
		public bool Dirty {
			get { return this.m_dirty; }
			set { this.m_dirty = value; }
		}

		/// <summary>
		/// Override to read-back the state of the object.
		/// </summary>
		/// <param name="stream"></param>
		public virtual void ReadState(ComStream stream) {
		}

		/// <summary>
		/// Helper method to read a string back (ended with a '|').
		/// </summary>
		/// <param name="br"></param>
		/// <returns>The string (minus the '|').</returns>
		protected virtual string readString(StreamReader br) {
			StringBuilder retval = new StringBuilder();
			char c;			
			while( (c=(char)br.Read()) != '|' ) {
				System.Diagnostics.Debug.WriteLine("Read: '" + c + "'");
				retval.Append(c);
			}
			System.Diagnostics.Debug.WriteLine("Read string: '" + retval + "'");
			return retval.ToString();
		}

		/// <summary>
		/// Write out this object. Override me.
		/// </summary>
		/// <param name="stream"></param>
		public virtual void WriteState(ComStream stream) {
		}

		/// <summary>
		/// Construct the default state.
		/// </summary>
		public SnapinStateBase()
		{			
			System.Diagnostics.Debug.WriteLine("SnapinStateBase<init>");
		}
	}
}
