using System;
using System.Collections;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

using System.Threading;
using System.Runtime.InteropServices;

using ceh = Ironring.Management.MMC.CustomExceptionHandler;

namespace Ironring.Management.MMC
{
	/// <summary>
	/// Summary description for SnapinStateHash.
	/// </summary>
	public class SnapinStateHash : SnapinStateBase
	{
		private Hashtable Items = new Hashtable();

		public SnapinStateHash() : base()
		{			
		}

		/// <summary>
		/// Sets the specified property key and value to be persisted.
		/// </summary>
		/// <param name="key"></param>
		/// <param name="value"></param>
		public void SetProp(string key, string value) 
		{
			if (!Items.Contains(key))
				Items.Add(key, value);
			else
				Items[key] = value;

			this.Dirty = true;
		}

		/// <summary>
		/// Gets the specified property value by key from persisted state data.
		/// </summary>
		/// <param name="key"></param>
		/// <returns></returns>
		public string GetProp(string key) 
		{
			if (Items.Contains(key))
				return (string)Items[key];
			else
				return null;
		}

		/// <summary>
		/// Reads and deserializes the persisted state data.
		/// </summary>
		/// <param name="stream"></param>
		public override void ReadState(ComStream stream) 
		{
			try 
			{
				BinaryFormatter b = new BinaryFormatter();
				Items = (Hashtable)b.Deserialize((Stream)stream);
			} 
			catch(Exception ex)
			{
				System.Diagnostics.Trace.WriteLine(ex.Message, "EXCEPTION");
				//Added by Alexander Kachalkov
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(ex));
			}
		}

		/// <summary>
		/// Serializes and writes the data to the state data for persistence.
		/// </summary>
		/// <param name="stream"></param>
		public override void WriteState(ComStream stream) 
		{
			try 
			{
				BinaryFormatter b = new BinaryFormatter();
				b.Serialize((Stream)stream, Items);
			} 
			catch(Exception ex)
			{
				System.Diagnostics.Trace.WriteLine(ex.Message, "EXCEPTION");
				//Added by Alexander Kachalkov
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(ex));
			}

			this.Dirty = false;
		}

	}
}
