using System;
using Ironring.Management.MMC;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace Ironring.Management.MMC
{
	/// <summary>
	/// A class for a single page on the wizard
	/// </summary>
	public class WizardPage : System.Windows.Forms.UserControl
	{
		private Panel pnlHeader;
		public PictureBox pictWizLogo;
		protected Label lblExpandedDescription;
		protected Label lblPageCaption;

		private Container components = null;

		/// <summary>
		/// Delegate
		/// </summary>
		public delegate void WizardPageChanged(WizardPage me);

		protected ArrayList listeners = new ArrayList();
		protected WizardBase wizardBase;
		protected BaseNode node;

		/// <summary>
		/// Constructor
		/// </summary>
		public WizardPage()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="Node">The node where we belong to</param>
		public WizardPage(BaseNode Node)
		{
			this.node = Node;
			InitializeComponent();
		}

		/// <summary>
		/// Get the Node we belong to
		/// </summary>
		public BaseNode Node
		{
			get{ return this.node; }
		}

		/// <summary>
		/// Get the wizard where we belong to 
		/// </summary>
		public WizardBase WizardBase
		{
			get { return wizardBase; }
			set { wizardBase = value; }
		}

		/// <summary>
		/// Called to see if this page is done.
		/// </summary>
		public bool PageReady
		{
			get { return true;}
		}

		/// <summary>
		/// The wizard page registers as a listener. Raise
		/// this event to let the wizard know something changed that
		/// could lead to the next/finish button being activated
		/// </summary>
		/// <param name="listener"></param>
		public void Add_WizardPageStateChange(WizardPageChanged listener) 
		{
			this.listeners.Add(listener);
		}

		/// <summary>
		/// Remove a WizardPageStateChange Listener
		/// </summary>
		/// <param name="listener"></param>
		public void Remove_WizardPageStateChange(WizardPageChanged listener) 
		{
			this.listeners.Remove(listener);
		}

		/// <summary>
		/// Call this to let all listeners know we changed state.
		/// </summary>
		public void FirePageChange() 
		{
			object[] list = this.listeners.ToArray();
			for(int i=0, max=list.Length; i<max; i++) 
				Invoke((WizardPageChanged)list[i], new object[] { this });
		}

		/// <summary>
		/// Get the validation status of this page.
		/// </summary>
		public virtual bool WizardPageValid
		{
			get {return true;}
		}

		#region Component Designer generated code
		/**
		 * Clean up any resources being used.
		 */
		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}
	
		/**
		 * Required method for Designer support - do not modify 
		 * the contents of this method with the code editor.
		 */
		private void InitializeComponent()
		{
			this.pnlHeader = new System.Windows.Forms.Panel();
			this.pictWizLogo = new System.Windows.Forms.PictureBox();
			this.lblExpandedDescription = new System.Windows.Forms.Label();
			this.lblPageCaption = new System.Windows.Forms.Label();
			this.pnlHeader.SuspendLayout();
			this.SuspendLayout();
			// 
			// pnlHeader
			// 
			this.pnlHeader.BackColor = System.Drawing.SystemColors.Window;
			this.pnlHeader.Controls.Add(this.pictWizLogo);
			this.pnlHeader.Controls.Add(this.lblExpandedDescription);
			this.pnlHeader.Controls.Add(this.lblPageCaption);
			this.pnlHeader.Location = new System.Drawing.Point(0, 0);
			this.pnlHeader.Name = "pnlHeader";
			this.pnlHeader.Size = new System.Drawing.Size(496, 64);
			this.pnlHeader.TabIndex = 1;
			// 
			// pictWizLogo
			// 
			this.pictWizLogo.Location = new System.Drawing.Point(440, 8);
			this.pictWizLogo.Name = "pictWizLogo";
			this.pictWizLogo.Size = new System.Drawing.Size(48, 48);
			this.pictWizLogo.TabIndex = 2;
			this.pictWizLogo.TabStop = false;
			// 
			// lblExpandedDescription
			// 
			this.lblExpandedDescription.Location = new System.Drawing.Point(48, 32);
			this.lblExpandedDescription.Name = "lblExpandedDescription";
			this.lblExpandedDescription.Size = new System.Drawing.Size(320, 24);
			this.lblExpandedDescription.TabIndex = 1;
			this.lblExpandedDescription.Text = "Wizard page explanation";
			// 
			// lblPageCaption
			// 
			this.lblPageCaption.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lblPageCaption.Location = new System.Drawing.Point(16, 16);
			this.lblPageCaption.Name = "lblPageCaption";
			this.lblPageCaption.Size = new System.Drawing.Size(352, 16);
			this.lblPageCaption.TabIndex = 0;
			this.lblPageCaption.Text = "Wizard page caption";
			// 
			// WizardPage
			// 
			this.Controls.Add(this.pnlHeader);
			this.Name = "WizardPage";
			this.Size = new System.Drawing.Size(496, 328);
			this.pnlHeader.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion
	}
}