using System;
using System.Runtime.InteropServices;

////////////////////////////////////////////////////////////////////////////////////
// 
// Interop Definitions
// 
// NOTE: these are hand rolled definition of MMC interfaces and related interfaces
// structures, enums and constants.  They do not always model the underlying 
// unmanaged object with 100% fidelity but are good enough for use by MMC snapins
// As an Example the IDataObject implementation is slimmed down since we don't
// need most of the services.
// 
// Several of these interfaces were gleaned from the MMC GotDotNet sample.
// 


namespace Ironring.Management.MMC
{

	////////////////////////////////////////////////////////////////////////////////////
	// 
	// Snapin Implemented Interfaces
	// 
	#region IResultDataCompareEx

	[
	StructLayout(LayoutKind.Sequential)
	]
	public struct RDCOMPARE
	{
		public uint cbSize;
		public uint dwFlags;
		public int nColumn;
		public IntPtr lUserParam;
		public IntPtr prdch1;
		public IntPtr prdch2;

	}

	[
	StructLayout(LayoutKind.Sequential)
	]
	public struct RDITEMHDR
	{
		public uint dwFlags;
		public IntPtr cookie;
		public IntPtr lpReserved;
	}

	[ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("96933476-0251-11d3-AEB0-00C04F8ECD78")
	]
	public interface IResultDataCompareEx
	{
		void Compare(ref RDCOMPARE prdc, out int pnResult);
	}

	#endregion

	#region Interfaces of the IPersistStream
	//Definition for interface IPersistStream.
	// Taken from: http://www.codeproject.com/csharp/objqcparameter.asp
	// Set the "allow unsafe" property to yes to use the UCOMIStream part.
	[
	ComImport,
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown),
	Guid("0000010c-0000-0000-C000-000000000046")
	]
	public interface IPersist 
	{
		void GetClassID( /* [out] */ out Guid pClassID);
	};
    
	[
	ComImport,
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown),
	Guid("00000109-0000-0000-C000-000000000046") //This is IPersistStream
	//Guid("7FD52380-4E07-101B-AE2D-08002B2EC713") // IPersistStreamInit
	]	
	public interface IPersistStream : IPersist 
	{
		// Override and reimplement.
		new void GetClassID(out Guid pClassID);

		[PreserveSig]
		int IsDirty( ); 

		void Load([In] UCOMIStream pStm);
		void Save([In] UCOMIStream pStm, [In, MarshalAs(UnmanagedType.Bool)] bool fClearDirty);
		void GetSizeMax(out long pcbSize);

		// MAM: IPersistStreamInit would be better but it just won't seem to work. Shrug.
		// This is from IPersistStreamInit
		//[PreserveSig]
		//int InitNew();
	};
	#endregion

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("955AB28A-5218-11D0-A985-00C04FD8D565")
	]
	public interface IComponentData
	{
		void Initialize([MarshalAs(UnmanagedType.Interface)] Object pUnknown);
		void CreateComponent(out IComponent ppComponent);
		[PreserveSig()]
		int Notify(IDataObject lpDataObject, uint aevent, IntPtr arg, IntPtr param);
		void Destroy();
		void QueryDataObject(int cookie, uint type, out IDataObject ppDataObject);
		void GetDisplayInfo(ref SCOPEDATAITEM ResultDataItem);
		[PreserveSig()]
		int CompareObjects(IDataObject lpDataObjectA, IDataObject lpDataObjectB);
	}

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("43136EB2-D36C-11CF-ADBC-00AA00A80033")
	]
	public interface IComponent
	{
		void Initialize([MarshalAs(UnmanagedType.Interface)]Object lpConsole);
		[PreserveSig()]
		int Notify(IntPtr lpDataObject, uint aevent, IntPtr arg, IntPtr param);
		void Destroy(int cookie);
		void QueryDataObject(int cookie, uint type, out IDataObject ppDataObject);
		[PreserveSig()]
		int GetResultViewType(int cookie, out IntPtr ppViewType, out int pViewOptions);
		void GetDisplayInfo(ref RESULTDATAITEM ResultDataItem);
		[PreserveSig()]
		int CompareObjects(IDataObject lpDataObjectA, IDataObject lpDataObjectB);
	}

	[   
	ComImport,
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("1245208C-A151-11D0-A7D7-00C04FD909DD")
	]
	public interface ISnapinAbout
	{
		void GetSnapinDescription(out IntPtr lpDescription);
		void GetProvider(out IntPtr pName);
		void GetSnapinVersion(out IntPtr lpVersion);
		void GetSnapinImage(out IntPtr hAppIcon);
		void GetStaticFolderImage(out IntPtr hSmallImage, out IntPtr hSmallImageOpen, out IntPtr hLargeImage, out uint cMask);
	}

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("0000010e-0000-0000-C000-000000000046")
	]
	public interface IDataObject
	{
		[PreserveSig()]
		int GetData(ref FORMATETC pFormatEtc, ref STGMEDIUM b);
		void GetDataHere(ref FORMATETC pFormatEtc, ref STGMEDIUM b);
		[PreserveSig()]
		int QueryGetData(IntPtr a);
		[PreserveSig()]
		int GetCanonicalFormatEtc(IntPtr a, IntPtr b);
		[PreserveSig()]
		int SetData(IntPtr a, IntPtr b, int c);
		[PreserveSig()]
		int EnumFormatEtc(uint a, IntPtr b);
		[PreserveSig()]
		int DAdvise(IntPtr a, uint b, IntPtr c, ref uint d);
		[PreserveSig()]
		int DUnadvise(uint a);
		[PreserveSig()]
		int EnumDAdvise(IntPtr a);
	}

	/// <summary>
	/// Modification: mvellon 9/9/2005 - eliminated PreserveSig
	/// </summary>
	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("4861A010-20F9-11d2-A510-00C04FB6DD2C")
	]
	public interface ISnapinHelp2
	{
		void GetHelpTopic(out IntPtr lpCompiledHelpFile);
		void GetLinkedTopics(out IntPtr lpCompiledHelpFiles);
	}


	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("cc593830-b926-11d1-8063-0000f875a9ce")
	]
	public interface IDisplayHelp
	{
		void ShowTopic([MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPWStr)] String pszHelpTopic);
	}


	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("4F3B7A4F-CFAC-11CF-B8E3-00C04FD8D5B0")
	]
	public interface IExtendContextMenu
	{
		//void AddMenuItems(IDataObject piDataObject, IContextMenuCallback piCallback, ref int pInsertionAllowed);
		void AddMenuItems(IntPtr piDataObject, IContextMenuCallback piCallback, ref int pInsertionAllowed);
		void Command(int lCommandID, IntPtr piDataObject);
	}

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("49506520-6F40-11D0-A98B-00C04FD8D565")
	]
	public interface IExtendControlbar
	{
		void SetControlbar(IControlbar pControlbar);
		[PreserveSig()]
		int ControlbarNotify(uint aevent, IntPtr arg, IntPtr param);
	}

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("85DE64DC-EF21-11cf-A285-00C04FD8DBE6")
	]
	public interface IExtendPropertySheet
	{
		[PreserveSig()]
		int CreatePropertyPages(IPropertySheetCallback lpProvider, IntPtr handle, IDataObject lpIDataObject);
		[PreserveSig()]
		int QueryPagesFor(IDataObject lpDataObject);
	}

/*
    [
        ComImport,
        InterfaceType(ComInterfaceType.InterfaceIsIUnknown),
        Guid("338698b1-5a02-11d1-9fec-00600832db4a")
    ]
    public interface IEnumTASK
    {
        HRESULT Next(uint celt,
                     uint sizeparamindex,
                     [MarshalAs(UnmanagedType.LPArray, SizeParamIndex=1)] MMC_TASK[] pceltFetched,
                
    
                     
                     
            [in] ULONG celt,
                     [out, size_is(celt), length_is(*pceltFetched)] MMC_TASK *rgelt,
                                                                        [out] ULONG *pceltFetched);
        HRESULT Skip([in] ULONG celt);
        HRESULT Reset();
        HRESULT Clone([out] IEnumTASK **ppenum);

        // Skip and Clone are not called
        // Next will be called 1 at a time (i.e. celt will always be 1)
        // Reset will be called
    };


    [
        ComImport,
        InterfaceType(ComInterfaceType.InterfaceIsIUnknown),
        Guid("8dee6511-554d-11d1-9fea-00600832db4a")
    ]
    public interface IExtendTaskpad
    {
        // called from script via CIC
//        [PreserveSig()]
//        HRESULT TaskNotify(IDataObject lpDataObject, [in] VARIANT* arg, [in] VARIANT* param);

        [PreserveSig()]
        HRESULT EnumTasks(IDataObject pdo,
                          ([MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPWStr)] String pszHelpTopic,
                          
    [out] IEnumTASK** ppEnumTASK);

    // support for default taskpad
    [helpstring("Title for the default task pad.")]
    HRESULT GetTitle([in,string] LPOLESTR pszGroup, [out,string] LPOLESTR * pszTitle);

    [helpstring("Descriptive Text for the default task pad.")]
    HRESULT GetDescriptiveText([in,string] LPOLESTR pszGroup, [out,string] LPOLESTR * pszDescriptiveText);

    [helpstring("Background for the default task pad.")]
    HRESULT GetBackground([in,string] LPOLESTR pszGroup, [out] MMC_TASK_DISPLAY_OBJECT * pTDO);

    //      [helpstring("Branding for the default task pad.")]
    //      HRESULT GetBranding([in,string] LPOLESTR pszGroup, [out] MMC_TASK_DISPLAY_OBJECT * pTDO);

    [helpstring("Get ListPad Info.")]
    HRESULT GetListPadInfo ([in,string] LPOLESTR pszGroup, [out] MMC_LISTPAD_INFO * lpListPadInfo);
        
}
*/
	////////////////////////////////////////////////////////////////////////////////////
	// 
	// MMC Implemented Interfaces
	// 

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("85DE64DE-EF21-11cf-A285-00C04FD8DBE6")
	]
	public interface IPropertySheetProvider
	{
		void CreatePropertySheet([MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPWStr)] String title, int type, int cookie, IDataObject pIDataObjectm, uint dwOptions);
		void FindPropertySheet(int cookie, IComponent lpComponent, IDataObject lpDataObject);
		void AddPrimaryPages([MarshalAs(UnmanagedType.Interface)]Object lpUnknown, int bCreateHandle, int hNotifyWindow,int bScopePane);
		void AddExtensionPages();
		void Show(int window, int page);
	}

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("85DE64DD-EF21-11cf-A285-00C04FD8DBE6")
	]
	public interface IPropertySheetCallback
	{
		void AddPage(IntPtr hPage);
		void RemovePage(IntPtr hPage);
	}

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("255F18CC-65DB-11D1-A7DC-00C04FD8D565")
	]
	public interface IConsoleNameSpace2
	{
		void InsertItem(ref SCOPEDATAITEM a);
		void DeleteItem(uint a, int b);
		void SetItem(ref SCOPEDATAITEM a);
		void GetItem(ref SCOPEDATAITEM a);
		void GetChildItem(IntPtr hItem, ref IntPtr hItemChild, ref IntPtr childCookie);
		void GetNextItem(IntPtr hItem, ref IntPtr pItemNext, ref IntPtr  plCookie);
		void GetParentItem(uint a, ref uint b, ref int c);
		void Expand(uint a);
		void AddExtension(IntPtr hItem, ref CLSID lpClsid); 
    }

    [StructLayout(LayoutKind.Explicit)]
    public struct CLSID
    {
        [FieldOffset(0)]public uint x;
        [FieldOffset(4)]public ushort s1;
        [FieldOffset(6)]public ushort s2;
        [FieldOffset(8)]public byte c1;
        [FieldOffset(9)]public byte c2;
        [FieldOffset(10)]public byte c3;
        [FieldOffset(11)]public byte c4;
        [FieldOffset(12)]public byte c5;
        [FieldOffset(13)]public byte c6;
        [FieldOffset(14)]public byte c7;
        [FieldOffset(15)]public byte c8;
    }

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("9757abb8-1b32-11d1-a7ce-00c04fd8d565")
	]
	public interface IHeaderCtrl2
	{
		void InsertColumn(int nCol, [MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPWStr)] string title, int nFormat, int nWidth);
		void DeleteColumn(int nCol);
		void SetColumnText(int nCol, [MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPWStr)] string title);
		void GetColumnText(int nCol, out int pText);
		void SetColumnWidth(int nCol, int nWidth);
		void GetColumnWidth(int nCol, out int pWidth);
		void SetChangeTimeOut( uint uTimeout);
		//int SetColumnFilter(uint nColumn, uint dwType, IntPtr pFilterData);
		int SetColumnFilter(uint nColumn, uint dwType, MMC_FILTERDATA data);
		// MAM: the third argument should not be an IntPtr (we cannot cast a struct to an IntPtr).
		int GetColumnFilter(uint nColumn, ref uint pdwType, ref MMC_FILTERDATA data);
	}

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("31DA5FA0-E0EB-11cf-9F21-00AA003CA9F6")
	]
	public interface IResultData 
	{
		void InsertItem(ref RESULTDATAITEM item);
		void DeleteItem(uint itemID, int nCol);
		void FindItemByLParam(int lParam, out uint pItemID);
		void DeleteAllRsltItems();
		void SetItem(ref RESULTDATAITEM item);
		void GetItem(ref RESULTDATAITEM item);
		void GetNextItem(ref RESULTDATAITEM item);
		void ModifyItemState(int nIndex, uint itemID, uint uAdd, uint uRemove);
		void ModifyViewStyle(int add, int remove);
		void SetViewMode(int lViewMode);
		void GetViewMode(out int lViewMode);
		void UpdateItem(uint itemID);
		void Sort(int nColumn, uint dwSortOptions, int lUserParam);
		//void SetDescBarText([MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPWStr)] ref String DescText);
		//Fixed by Attila Kapostyak
		void SetDescBarText([MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPWStr)] String DescText);
		void SetItemCount(int nItemCount, uint dwOptions);
	}

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("43136EB8-D36C-11CF-ADBC-00AA00A80033")
	]
	public interface IImageList
	{
		void ImageListSetIcon(IntPtr pIcon, int nLoc);
		void ImageListSetStrip(int pBMapSm, int pBMapLg, int nStartLoc, int cMask);
	}

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("103D842A-AA63-11D1-A7E1-00C04FD8D565")
	]
	public interface IConsole2
	{
		void SetHeader(ref IHeaderCtrl2 pHeader);
		void SetToolbar([MarshalAs(UnmanagedType.Interface)] ref Object pToolbar); // Needs to be LPTOOLBAR 
		void QueryResultView([MarshalAs(UnmanagedType.Interface)] out Object pUnknown);
		void QueryScopeImageList(out IImageList ppImageList);
		void QueryResultImageList(out IImageList ppImageList);
		void UpdateAllViews(IDataObject lpDataObject, int data, int hint);
		void MessageBox([MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPWStr)] String lpszText, [MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPWStr)] String lpszTitle, uint fuStyle, ref int piRetval);
		void QueryConsoleVerb(out IConsoleVerb ppConsoleVerb);
		void SelectScopeItem(int hScopeItem);
		void GetMainWindow(ref int phwnd);
		void NewWindow(int hScopeItem, uint lOptions);
		void Expand(int hItem, int bExpand);
		void IsTaskpadViewPreferred();
		void SetStatusText([MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPWStr)]String pszStatusText);
	}



	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("43136EB6-D36C-11CF-ADBC-00AA00A80033")
	]
	public interface IContextMenuProvider : IContextMenuCallback
	{
		HRESULT EmptyMenuList();
		HRESULT AddPrimaryExtensionItems([MarshalAs(UnmanagedType.Interface)]Object lpUnknown, IDataObject lpDataObject);
		HRESULT AddThirdPartyExtensionItems(IDataObject lpDataObject);
		HRESULT ShowContextMenu(IntPtr hwndParent, int xPos, int yPos, out long plSelected);
	};


	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("43136EB7-D36C-11CF-ADBC-00AA00A80033")
	]
	public interface IContextMenuCallback
	{
		void AddItem(ref CONTEXTMENUITEM pItem);
	}

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("69FB811E-6C1C-11D0-A2CB-00C04FD909DD")
	]
	public interface IControlbar
	{
		void Create(MMC_CONTROL_TYPE nType, IExtendControlbar pExtendControlbar, [MarshalAs(UnmanagedType.Interface)] out Object ppUnknown);
		void Attach(MMC_CONTROL_TYPE nType, [MarshalAs(UnmanagedType.Interface)] Object lpUnknown);
		void Detach([MarshalAs(UnmanagedType.Interface)] Object lpUnknown);
	}

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("43136EB9-D36C-11CF-ADBC-00AA00A80033")
	]
	public interface IToolbar
	{
		void AddBitmap(int nImages, IntPtr hbmp, int cxSize, int cySize, int crMask);
		void AddButtons(int nButtons,  ref MMCBUTTON lpButtons);
		void InsertButton(int nIndex, ref MMCBUTTON lpButton);
		void DeleteButton(int nIndex);
		void GetButtonState(int idCommand, MMC_BUTTON_STATE nState, out int pState);
		void SetButtonState(int idCommand, MMC_BUTTON_STATE nState, int bState);
	}

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("E49F7A60-74AF-11D0-A286-00C04FD8FE93")
	]
	public interface IConsoleVerb 
	{
		//void GetVerbState(uint eCmdID, uint nState, ref int pState);
		//void SetVerbState(uint eCmdID, uint nState, int bState);
		//Changed by Alexander Kachalkov
		void GetVerbState(MMC_CONSOLE_VERB eCmdID, MMC_BUTTON_STATE nState, ref int pState);
		void SetVerbState(MMC_CONSOLE_VERB eCmdID, MMC_BUTTON_STATE nState, int bState);
		void SetDefaultVerb(MMC_CONSOLE_VERB eCmdID);
		void GetDefaultVerb(ref MMC_CONSOLE_VERB peCmdID);
	}


	//////////////////////////////////////////////////////////////////
	// Shim Control for hosting WinForms User Controls 
	// in this ActiveX Controls

	[
	ComImport, 
	InterfaceType(ComInterfaceType.InterfaceIsIUnknown), 
	Guid("B8F882F8-3D93-401c-A4D4-E3A9943E858D")
	]
	public interface IMMCFormsShim
	{
		System.IntPtr HostUserControl(string Assembly, string Class); 
	}

	//////////////////////////////////////////////////////////////////
	//
	// Enumerations and Constants
	// 

	//Added by Alexander Kachalkov
	public enum MMC
	{
		MULTI_SELECT_COOKIE		= -2
	}


	#region Filtering
	//Added by Alexander Kachalkov
	public enum MMC_FILTER_TYPE 
	{
		MMC_STRING_FILTER = 0, 
		MMC_INT_FILTER = 0x1, 
		MMC_FILTER_NOVALUE = 0x8000
	}

	// MAM: filtering constants.
	public enum MMC_FILTER_CHANGE_CODE 
	{
		MFCC_DISABLE = 0, 
		MFCC_ENABLE = 1, 
		MFCC_VALUE_CHANGE = 2
	}

	public struct MMC_FILTERDATA 
	{  
		public IntPtr pszText; // Was LPOLESTR
		public int cchTextMax;
		public int lValue;		
	}

	#endregion

	// MAM: needed for column sizing.
	public enum MMCLV 
	{
		AUTO                   = -1
	}


	//Added by Alexander Kachalkov
	public enum LVIS
	{
		FOCUSED					= 0x0001
			,SELECTED				= 0x0002
	}

	public enum ColumnHeaderFormat
	{
		LEFT                   = 0x0000,
		RIGHT                  = 0x0001,
		CENTER                 = 0x0002,
		JUSTIFYMASK            = 0x0003,
		IMAGE                  = 0x0800,
		BITMAP_ON_RIGHT        = 0x1000,
		COL_HAS_IMAGES         = 0x8000
	}

	//Added by Alexander Kachalkov
	public enum ColumnHeaderWidth
	{
		AUTO_WIDTH				= -1	// Column width is determined by the string length + padding
		,HIDE_COLUMN			= -4	// Column will be hidden
	}

	//Added by Alexander Kachalkov
	//due to VB.NET incompatibility
	//with using classes public consts
	public enum TBSTATE
	{
		CHECKED               = 0x01,
		PRESSED               = 0x02,
		ENABLED               = 0x04,
		HIDDEN                = 0x08,
		INDETERMINATE         = 0x10,
		WRAP                  = 0x20,
		ELLIPSES              = 0x40,
		MARKED                = 0x80,
	}// class TBSTATE

	//Commented by Alexander Kachalkov
	//	public class TBSTYLE
	//	{
	//		public const byte BUTTON                = 0x0000;
	//		public const byte SEP                   = 0x0001;
	//		public const byte CHECK                 = 0x0002;
	//		public const byte GROUP                 = 0x0004;
	//		public const byte CHECKGROUP            = (GROUP | CHECK);
	//		public const byte DROPDOWN              = 0x0008;
	//		public const byte AUTOSIZE              = 0x0010;
	//		public const byte NOPREFIX              = 0x0020;
	//	}// class TBSTYLE

	//Added by Alexander Kachalkov
	//due to VB.NET incompatibility
	//with using classes public consts
	public enum TBSTYLE
	{
		BUTTON                = 0x0000,
		SEP                   = 0x0001,
		CHECK                 = 0x0002,
		GROUP                 = 0x0004,
		CHECKGROUP            = (GROUP | CHECK),
		DROPDOWN              = 0x0008,
		AUTOSIZE              = 0x0010,
		NOPREFIX              = 0x0020,
	}// class TBSTYLE

	public class EN
	{
		public const uint SETFOCUS           = 0x0100;
		public const uint KILLFOCUS          = 0x0200;
		public const uint CHANGE             = 0x0300;
		public const uint UPDATE             = 0x0400;
		public const uint ERRSPACE           = 0x0500;
		public const uint MAXTEXT            = 0x0501;
		public const uint HSCROLL            = 0x0601;
		public const uint VSCROLL            = 0x0602;
	}// EN

	public enum PSM
	{
		SETCURSEL          = (0x0400 + 101),
		REMOVEPAGE         = (0x0400 + 102),
		ADDPAGE            = (0x0400 + 103),
		CHANGED            = (0x0400 + 104),
	}

	public enum PSNRET
	{
		NOERROR               = 0,
		INVALID               = 1,
		INVALID_NOCHANGEPAGE  = 2,
	}


	public class PSN
	{
		public const uint FIRST              = unchecked((0U-200U));
		public const uint LAST               = unchecked((0U-299U));
		public const uint SETACTIVE          = unchecked((FIRST-0));
		public const uint KILLACTIVE         = unchecked((FIRST-1));
		public const uint APPLY              = unchecked((FIRST-2));
		public const uint RESET              = unchecked((FIRST-3));
		public const uint HELP               = unchecked((FIRST-5));
		public const uint WIZBACK            = unchecked((FIRST-6));
		public const uint WIZNEXT            = unchecked((FIRST-7));
		public const uint WIZFINISH          = unchecked((FIRST-8));
		public const uint QUERYCANCEL        = unchecked((FIRST-9));
		public const uint GETOBJECT          = unchecked((FIRST-10));
	}


	public enum WM
	{
		WINDOWPOSCHANGING   = 0x0046,
		CHILDACTIVATE       = 0x0022,
		WINDOWPOSCHANGED    = 0x0047,
		NOTIFY              = 0x004E,
		SHOWWINDOW          = 0x0018,
		NCPAINT             = 0x0085,
		PAINT               = 0x000F,
		ERASEBKGND          = 0x0014,
		CTLCOLORDLG         = 0x0136,
		INITDIALOG          = 0x0110,
		COMMAND             = 0x0111,
		DESTROY             = 0x0002,
		CREATE              = 0x0001,
		MOVE                = 0x0003,
		SIZE                = 0x0005,
		ACTIVATEAPP         = 0x001C,
		ACTIVATE            = 0x0006,
	}


	// Used in building dialog templates
	public enum DS
	{
		SETFONT               = 0x40,
		FIXEDSYS              = 0x0008,
	}

	// Constants required for the Property Sheet Page Callback function
	public enum PSPCB
	{
		ADDREF                = 0,
		RELEASE               = 1,
		CREATE                = 2,
	}
    
	public enum PSP
	{
		DEFAULT               = 0x00000000,
		DLGINDIRECT           = 0x00000001,
		USEHICON              = 0x00000002,
		USEICONID             = 0x00000004,
		USETITLE              = 0x00000008,
		RTLREADING            = 0x00000010,

		HASHELP               = 0x00000020,
		USEREFPARENT          = 0x00000040,
		USECALLBACK           = 0x00000080,
		PREMATURE             = 0x00000400,

		HIDEHEADER            = 0x00000800,
		USEHEADERTITLE        = 0x00001000,
		USEHEADERSUBTITLE     = 0x00002000,
	}

	public class HRESULT
	{
		public const int S_OK                  = 0;
		public const int S_FALSE               = 1;
		public const int E_NOTIMPL             = unchecked((int)0x80004001);
		public const int STG_E_MEDIUMFULL      = unchecked((int)0x80030070);
	}

	public class IMAGE
	{
		public const int BITMAP                = 0;
		public const int CURSOR                = 1;
		public const int ICON                  = 2;
	}   

	//Commented by Alexander Kachalkov
	/*
		public class CCT
		{
			public const uint SCOPE            = 0x8000; // Data object for scope pane context 
			public const uint RESULT           = 0x8001; // Data object for result pane context 
			public const uint SNAPIN_MANAGER   = 0x8002; // Data object for Snap-in Manager context 
			public const uint UNINITIALIZED    = 0xFFFF;  // Data object has an invalid type 
		}
	*/

	//Added by Alexander Kachalkov
	//due to VB.NET incompatibility
	//with using classes public consts
	public enum CCT
	{
		SCOPE            = 0x8000 // Data object for scope pane context 
			,RESULT           = 0x8001 // Data object for result pane context 
				 ,SNAPIN_MANAGER   = 0x8002 // Data object for Snap-in Manager context 
					  ,UNINITIALIZED    = 0xFFFF  // Data object has an invalid type 
	}

	//Commented by Alexander Kachalkov
	/*
	public class MMC_BUTTON_STATE
	{
		public const int ENABLED                = 0x1;
		public const int CHECKED                = 0x2;
		public const int HIDDEN                 = 0x4;
		public const int INDETERMINATE          = 0x8;
		public const int BUTTONPRESSED          = 0x10;
	}
	*/

	//Added by Alexander Kachalkov
	//due to VB.NET incompatibility
	//with using classes public consts
	public enum MMC_BUTTON_STATE
	{
		ENABLED                = 0x1,
		CHECKED                = 0x2,
		HIDDEN                 = 0x4,
		INDETERMINATE          = 0x8,
		BUTTONPRESSED          = 0x10
	}
	public class MMC_VERB                               
	{                                            
		public const int NONE            = 0x0000;                       
		public const int OPEN            = 0x8000;                       
		public const int COPY            = 0x8001;                       
		public const int PASTE           = 0x8002;                       
		public const int DELETE          = 0x8003;                       
		public const int PROPERTIES      = 0x8004;                       
		public const int RENAME          = 0x8005;                       
		public const int REFRESH         = 0x8006;                       
		public const int PRINT           = 0x8007;                       
		public const int CUT             = 0x8008;
                                                             
		public const int MAX             = 0x8009;                                            
		public const int FIRST           = OPEN;                
		public const int LAST            = MAX - 1;

	}


	public enum MMC_CONSOLE_VERB
	{                                            
		NONE            = 0x0000,                       
		OPEN            = 0x8000,                       
		COPY            = 0x8001,                       
		PASTE           = 0x8002,                       
		DELETE          = 0x8003,                       
		PROPERTIES      = 0x8004,                       
		RENAME          = 0x8005,                       
		REFRESH         = 0x8006,                       
		PRINT           = 0x8007,                       
		CUT             = 0x8008,
                                                             
		MAX             = 0x8009,                                            
		FIRST           = OPEN,                
		LAST            = MAX - 1
	}

	public enum ImageListOptions
	{
		LEAVE_LARGE_ICON = 0x40000000,
		LEAVE_SMALL_ICON =  0x20000000,
	}


	public enum ViewMode
	{
		Icon        = 0,
		Report      = 1,
		SmallIcon   = 2,
		List        = 3,
		Filtered    = 4,
	}


	public enum MMC_CONTROL_TYPE
	{
		TOOLBAR                    = 0,
		MENUBUTTON                = TOOLBAR + 1,
		COMBOBOXBAR                = MENUBUTTON + 1
	}

	//Changed by Alexander Kachalkov
	//The following groups of flags cannot be used together:
	//MF_DISABLED, MF_ENABLED, and MF_GRAYED 
	//MF_CHECKED and MF_UNCHECKED 
	public enum MF
	{
		//Draws a horizontal dividing line. 
		//Only the IContextMenuProvider interface is allowed to add menu items with MF_SEPARATOR set.
		SEPARATOR  = 0x00000800,

		//Places a check mark next to the menu item.
		CHECKED = 0x00000008,

		//Disables the menu item so it cannot be selected, but the flag does not gray it. 
		DISABLED = 0x00000002,

		//Enables the menu item so it can be selected, restoring it from its grayed state.
		ENABLED = 0x00000000,

		//Disables the menu item, graying it so it cannot be selected. 
		GRAYED = 0x00000001,
	
		//Does not place a check mark next to the item (default).																																																																							  MF_MENUBREAK Places the item on a new line (for a menu bar) or in a new column (for a drop-down menu, submenu, or shortcut menu) without separating columns. 
		UNCHECKED = 0x00000000,

		//Added by Oleg Medyanik
		//Specifies that this is a submenu within the context menu. 
		//Menu items, insertion points, and further submenus can be added to this submenu 
		//using its lCommandID as their lInsertionPointID.
		POPUP     = 0x00000010
}

	//Changed by Alexander Kachalkov
	public enum CCM:uint
	{    
		INSERTIONALLOWED_TOP    = 1,
		INSERTIONALLOWED_NEW    = 2,
		INSERTIONALLOWED_TASK    = 4,
		INSERTIONALLOWED_VIEW    = 8,

		INSERTIONPOINTID_PRIMARY_TOP    = 0xa0000000,
		INSERTIONPOINTID_PRIMARY_NEW    = 0xa0000001,
		INSERTIONPOINTID_PRIMARY_TASK    = 0xa0000002,
		INSERTIONPOINTID_PRIMARY_VIEW    = 0xa0000003,
		INSERTIONPOINTID_3RDPARTY_NEW    = 0x90000001,
		INSERTIONPOINTID_3RDPARTY_TASK = 0x90000002,
		INSERTIONPOINTID_ROOT_MENU     = 0x80000000,

		SPECIAL_SEPARATOR = 0x0001
	}

	//Added by Alexander Kachalkov
	public enum CCM_SPECIAL
	{
		//Ignore all other parameters except lInsertionPointID.
		//Add a separator to the end of the menu or at the specified insertion point. Separators placed at the top or bottom of a menu or submenu will not be displayed. Separators with no menu items between them will be collapsed into a single separator. 
		//Only the IContextMenuProvider interface is permitted to add separators, special or otherwise.
		SEPARATOR = 0x0001,
 
		//If this submenu is empty, it will be grayed and disabled. This is only valid for MF_POPUP items.
		SUBMENU = 0x0002,
        
		//This should be the default menu item.
		//If more than one menu item specifies this flag, the last item in each submenu takes precedence.
		DEFAULT_ITEM = 0x0004,
		
		//Ignore all other parameters except lCommandID and lInsertionPointID.
		//This creates a new insertion point at the end of the insertion point or submenu identified by lInsertionPointID.
		INSERTION_POINT = 0x0008,

		//Subsequent calls can use the lCommandID from this call as their lInsertionPointID, and insert their own menu items, submenus, or insertion points at this point in the menu.
		//Validate the item parameters, but do not add the menu item. Return result code indicating whether add would have been successful.
		TESTONLY = 0x0010
	}

/*
	public class CCM
	{    
		public const uint INSERTIONALLOWED_TOP    = 1;
		public const uint INSERTIONALLOWED_NEW    = 2;
		public const uint INSERTIONALLOWED_TASK    = 4;
		public const uint INSERTIONALLOWED_VIEW    = 8;

		public const uint INSERTIONPOINTID_PRIMARY_TOP    = 0xa0000000;
		public const uint INSERTIONPOINTID_PRIMARY_NEW    = 0xa0000001;
		public const uint INSERTIONPOINTID_PRIMARY_TASK    = 0xa0000002;
		public const uint INSERTIONPOINTID_PRIMARY_VIEW    = 0xa0000003;
		public const uint INSERTIONPOINTID_3RDPARTY_NEW    = 0x90000001;
		public const uint INSERTIONPOINTID_3RDPARTY_TASK = 0x90000002;
		public const uint INSERTIONPOINTID_ROOT_MENU     = 0x80000000;
	}
*/

	public enum MMC_VIEW_OPTIONS
	{
		NONE                          = 0,
		CREATENEW                     = 0x00000010,
		EXCLUDE_SCOPE_ITEMS_FROM_LIST = 0x00000040,
		FILTERED                      = 0x00000008,
		LEXICAL_SORT                  = 0x00000080,
		NOLISTVIEWS                   = 0x00000001,
		MULTISELECT                   = 0x00000002, // Not 4!
		OWNERDATALIST                 = 0x00000004, // Not 2!
		USEFONTLINKING                = 0x00000020,
		ACTIVATE                      = 0x00000006,
	}

	/// <summary>
	/// Notifications
	/// </summary>
	public enum MMCN_Notify
	{
		/// <summary>
		///	Is sent when a window is being activated or deactivated.
		///	arg = TRUE if window is activated, false otherwise.
		///	param = Not used.
		/// </summary>
		ACTIVATE = 0x8001,

		/// <summary>
		/// Sent to IComponent to add images for the result pane. The primary snapin
		/// should add images for both folders and leaf items. Extension snapins
		/// should add only folder images. 
		/// arg = ptr to result panes IImageList.
		/// param = HSCOPEITEM of selected/deselected item
		/// </summary>
		ADD_IMAGES = 0x8002,
    
		/// <summary>
		/// This message is sent when a user clicks on a button.
		/// arg   = When sent to ExtendControlbar it is the data object of currently selected,
		/// when sent to IComponent/IComponentData it is 0.
		/// param = CmdID of the button equal to a value of the MMC_COMMANDS enum type.
		/// </summary>
		BTN_CLICK = 0x8003,
		
		/// <summary>
		/// Click
		/// </summary>
		CLICK     = 0x8004,

		/// <summary>
		/// This message is sent when the user clicks on a result listview column header.
		/// dataObject = NULL
		/// arg = Column number
		/// param = Sort option flags (RSI_xxx) 
		/// </summary>
		COLUMN_CLICK = 0x8005,

		/// <summary>
		/// not used
		/// </summary>
		CONTEXTMENU = 0x8006,

		/// <summary>
		/// dataobject: NULL.
		/// arg: pointer to a dataobject. See multi-selection below.
		/// param: unused.
		/// </summary>
		CUTORMOVE = 0x8007,

		//Added by Alexander Kachalkov
		/// <summary>
		/// persist the new column configuration.
		/// </summary>
		COLUMNS_CHANGED    = 0x8022,

		/// <summary>
		/// Double Click
		/// </summary>
		DBLCLICK = 0x8008,

		/// <summary>
		/// Sent to inform the snapin that the item needs to be deleted. As a result of
		/// the user hitting the 'Delete' key or delete button.
		/// dataobject: dataobject of the selected item(s) provided by the snap-in.
		/// arg, param: unused.
		/// </summary>
		DELETE = 0x8009,

		/// <summary>
		/// This message is sent when all items of an owner-data result pane
		/// are deselected.
		/// dataObject = NULL
		/// arg = 0
		/// param = 0
		/// </summary>
		DESELECT_ALL = 0x800A,

		/// <summary>
		/// arg = TRUE => expand, FALSE => contract
		/// param = parents HSCOPEITEM.
		/// </summary>
		EXPAND = 0x800B,

		/// <summary>
		/// NOT USED
		/// </summary>
		HELP = 0x800C,   

		/// <summary>
		/// This message is sent when a user clicks on a button.
		/// arg   = Data object of currently selected.
		/// param = Structure (LPMENUBUTTONDATA).
		/// </summary>
		MENU_BTNCLICK = 0x800D, 

		/// <summary>
		/// Is sent when a window is being minimized or maximized.
		/// arg = TRUE if minimized, false otherwise.
		/// </summary>
		MINIMIZED = 0x800E, 

		/// <summary>
		/// dataobject and arg: are same as for QUERY_PASTE.
		/// param:
		/// NULL for move (as opposed to cut).
		/// For single item paste:
		/// BOOL* pPasted = (BOOL*)param; 
		/// Set this to TRUE here if the item was successfully pasted.
		/// For multi-item paste:
		/// LPDATAOBJECT* ppDataObj = (LPDATAOBJECT*)param;
		/// Use this to return a pointer to a dataobject consisting of the 
		/// items successfully pasted. (see CUTORMOVE below).
		/// </summary>
		PASTE = 0x800F,

		/// <summary>
		/// lpDataObject = NULL
		/// lParam = user object
		/// </summary>
		PROPERTY_CHANGE = 0x8010,

		/// <summary>
		/// dataobject: dataobject of the selected item provided by the snap-in.        
		/// arg: dataobject of the item(s) provided by the source snap-in that needs to be pasted.
		/// param: unused.
		/// Return S_OK if the data can be pasted, S_FALSE otherwise.
		/// </summary>
		QUERY_PASTE = 0x8011,

		/// <summary>
		/// Refresh
		/// </summary>
		REFRESH = 0x8012,

		/// <summary>
		/// Informs the snapin to delete all the cookies it has added below.
		/// arg = HSCOPEITEM of the node whose children needs to be deleted.
		/// param = unused.
		/// return = unused.
		/// </summary>
		REMOVE_CHILDREN = 0x8013,
    
		/// <summary>
		/// This gets called the first time to query for rename and a
		/// second time to do the rename.  For the query S_OK or S_FALSE for the
		/// return type.  After the rename, we will send the new name with a LPOLESTR.
		/// arg = 0 for query, 1 for rename action 
		/// param = LPOLESTR for containing new name   
		/// return = S_OK to allow rename and S_FALSE to disallow rename.                    
		/// </summary>
		RENAME = 0x8014,

		/// <summary>
		/// If sent to IComponent::Notify:
		/// arg:
		/// BOOL bScope = (BOOL) LOWORD(arg);
		/// BOOL bSelect = (BOOL) HIWORD(arg);
		/// bScope:     TRUE if an item the scope pane is selected, 
		/// FALSE if an item in the result pane is selected.
		/// bSelect:    TRUE if the item is selected, 
		/// FALSE if it is de-selected.
		/// param: 
		/// Ignored.                
		///
		/// If sent to IExtendControlbar::ControlbarNotify:
		/// arg:
		/// BOOL bScope = (BOOL) LOWORD(arg);
		/// BOOL bSelect = (BOOL) HIWORD(arg);
		/// bScope:     TRUE if an item the scope pane is selected, 
		/// FALSE if an item in the result pane is selected.
		/// bSelect:    TRUE if the item is selected,
		/// FALSE if it is de-selected.  
		/// param:
		/// LPDATAOBJECT pDataobject = (LPDATAOBJECT)param;
		/// pDataobject data object of item getting selected/de-selected.
		/// </summary>
		SELECT = 0x8015,

		/// <summary>
		/// arg != 0 if selecting, 1 if deselecting
		/// param = HSCOPEITEM of selected/deselected item
		/// </summary>
		SHOW = 0x8016,

		/// <summary>
		/// This message is sent to update all views of a change.
		/// arg = TRUE if Scope Item, FALSE if Result Item
		/// param = ptr to DataObject selected
		/// </summary>
		VIEW_CHANGE = 0x8017,

		/// <summary>
		/// This message is sent when the user requests help about the snapin.
		/// dataObject = NULL
		/// arg = 0
		/// param = 0
		/// </summary>
		SNAPINHELP = 0x8018,

		/// <summary>
		/// This message is sent when the user requests help about a selected item
		/// arg = 0
		/// param = 0 
		/// </summary>
		CONTEXTHELP = 0x8019,

		/// <summary>
		/// Sent to a snap-in when its custom OCX is initialized for the first time.
		/// </summary>
		INITOCX = 0x801A,
	
		/// <summary>
		/// This message is sent when the filter value for a result view column has been changed. 
		/// dataobject= NULL
		/// arg = Filter change code (see MMC_FILTER_CHANGE_CODE enumeration)
		/// param = column number of changed value, if change code is MFCC_VALUE_CHANGE
		/// </summary>
		FILTER_CHANGE = 0x801B,

		/// <summary>
		/// This message is sent to get the filter operators menu for a result view column.
		/// dataobject = NULL
		/// arg = Column number
		/// param = A pointer to a RECT structure (LPRECT) with the coordinates of the filter button.
		/// The snap-in can use the contents of the structure to position the user interface element (usually a menu) used to change the filter operator.
		/// </summary>
		GET_FILTER_MENU = 0x801C,

		/// <summary>
		/// This message is sent when the user selects an entry from a filter operator menu.
		/// dataobject = NULL
		/// arg = Column number
		/// param = Menu item ID  
		/// </summary>
		FILTER_OPERATOR = 0x801D,

		//Added by Attila Kapostyak
		PRINT = 0x801E,			

		//Added by Alexander Kachalkov
		/// <summary>
		/// Sent when the result pane for a scope item must be restored.
		/// </summary>
		MMCN_RESTORE_VIEW = 0x801D,
	}

	public enum SDI
	{
		STR                = 0x2,
		IMAGE              = 0x4,
		OPENIMAGE          = 0x8,
		STATE              = 0x10,
		PARAM              = 0x20,
		CHILDREN           = 0x40,
		PARENT             = 0,
		PREVIOUS           = 0x10000000,
		NEXT               = 0x20000000,
		FIRST              = 0x8000000,
	}

	public enum RDI
	{
		STR                = 0x2,
		IMAGE              = 0x4,
		STATE              = 0x8,
		PARAM              = 0x10,
		INDEX              = 0x20,
		INDENT             = 0x40,
	}

	//Added by Alexander Kachalkov
	/// <summary>
	/// Themes support
	/// </summary>
	public class ACTCTX_FLAG
	{
		public const uint PROCESSOR_ARCHITECTURE_VALID		= 0x001 ;
		public const uint LANGID_VALID						= 0x002;
		public const uint ASSEMBLY_DIRECTORY_VALID			= 0x004;
		public const uint RESOURCE_NAME_VALID				= 0x008;
		public const uint SET_PROCESS_DEFAULT				= 0x010;
		public const uint APPLICATION_NAME_VALID			= 0x020;
		public const uint HMODULE_VALID						= 0x080;
	}// class ACTCTX_FLAG
	
	//////////////////////////////////////////////////////////////////////
	// 
	// Structures 
	// 
	[
	StructLayout(LayoutKind.Sequential)
	]
	public struct RECT
	{
		public int left;
		public int top;
		public int right;
		public int bottom;
	}

	[
	StructLayout(LayoutKind.Sequential)
	]
	public struct MENUBUTTONDATA
	{
		public int idCommand;
		public int x;
		public int y;
	} 


	[
		StructLayout(LayoutKind.Sequential)
	]
	public struct MMCBUTTON
	{
		public int nBitmap;
		public int idCommand;
		public byte fsState;
		public byte fsType;
		[MarshalAs(UnmanagedType.LPWStr)]
		public String lpButtonText;
		[MarshalAs(UnmanagedType.LPWStr)]
		public String lpTooltipText;
		
		//Added by Alexander Kachalkov
		/// <summary>
		/// Overloads of the default ctor with TBSTATE = ENABLED and TBSTYLE = GROUP
		/// </summary>
		/// <param name="nBitmap">Specifies the zero-based index of a button image</param>
		/// <param name="idCommand">Specifies the command identifier returned when a button is clicke.</param>
		/// <param name="lpButtonText">Button text</param>
		/// <param name="lpTooltipText">Tooltip text</param>
		/// <returns></returns>
		public MMCBUTTON(int nBitmap, int idCommand, String lpButtonText, String lpTooltipText)
		{
			this.nBitmap = nBitmap;
			this.idCommand = idCommand;
			this.fsState = (byte)TBSTATE.ENABLED;
			this.fsType = (byte)TBSTYLE.GROUP;
			this.lpButtonText = lpButtonText;
			this.lpTooltipText = lpTooltipText;
		}

		/// <summary>
		/// Overloads of the default ctor with TBSTATE = ENABLED and TBSTYLE = GROUP
		/// </summary>
		/// <param name="nBitmap">Specifies the zero-based index of a button image</param>
		/// <param name="idCommand">Specifies the command identifier returned when a button is clicked</param>
		/// <param name="fsState">Specifies the button-state flags</param>
		/// <param name="fsType">Specifies the button style</param>
		/// <param name="lpButtonText">Button text</param>
		/// <param name="lpTooltipText">Tooltip text</param>
		public MMCBUTTON(int nBitmap, int idCommand, TBSTATE fsState, TBSTYLE fsType, String lpButtonText, String lpTooltipText)
		{
			this.nBitmap = nBitmap;
			this.idCommand = idCommand;
			this.fsState = (byte)fsState;
			this.fsType = (byte)fsType;
			this.lpButtonText = lpButtonText;
			this.lpTooltipText = lpTooltipText;
		}

	}

	[
	StructLayout(LayoutKind.Sequential, 
		CharSet=CharSet.Auto)
	]
	struct LOGFONT
	{
		public int lfHeight;
		public int lfWidth;
		public int lfEscapement;
		public int lfOrientation;
		public int lfWeight;
		public byte lfItalic;
		public byte lfUnderline;
		public byte lfStrikeOut;
		public byte lfCharSet;
		public byte lfOutPrecision;
		public byte lfClipPrecision;
		public byte lfQuality;
		public byte lfPitchAndFamily;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst=32)]
		public string lfFaceName;
	}

	[
	StructLayout(LayoutKind.Sequential, 
		CharSet=CharSet.Auto)
	]
	public struct TEXTMETRIC 
	{ 
		public int tmHeight; 
		public int tmAscent; 
		public int tmDescent; 
		public int tmInternalLeading; 
		public int tmExternalLeading; 
		public int tmAveCharWidth; 
		public int tmMaxCharWidth; 
		public int tmWeight; 
		public int tmOverhang; 
		public int tmDigitizedAspectX; 
		public int tmDigitizedAspectY; 
		char   tmFirstChar; 
		char   tmLastChar; 
		char   tmDefaultChar; 
		char   tmBreakChar; 
		byte   tmItalic; 
		byte   tmUnderlined; 
		byte   tmStruckOut; 
		byte   tmPitchAndFamily; 
		byte   tmCharSet; 
	}


	[
	StructLayout(LayoutKind.Sequential)
	]
	public struct CONTEXTMENUITEM
	{
		[MarshalAs(UnmanagedType.LPWStr)]
		public String   strName;
		[MarshalAs(UnmanagedType.LPWStr)]
		public String   strStatusBarText;
		public int      lCommandID;
		public uint     lInsertionPointID;
		public int      fFlags;
		public int      fSpecialFlags;
	}

	[
	StructLayout(LayoutKind.Sequential)
	]
	public struct SCOPEDATAITEM
	{
		public uint  mask;
		public IntPtr displayname;
		public int  nImage;
		public int  nOpenImage;
		public uint nState;
		public int  cChildren;
		public int  lParam;
		public int relativeID;
		public int ID;
	}

	[
	StructLayout(LayoutKind.Sequential)
	]
	public struct RESULTDATAITEM
	{
		public uint mask;
		public int bScopeItem;
		public int itemID;
		public int nIndex;
		public int nCol;
		public IntPtr str;
		public int nImage;
		public uint nState;
		public int lParam;
		public int iIndent;
	}

	[
	StructLayout(LayoutKind.Sequential)
	]
	public struct FORMATETC
	{
		public int cfFormat;
		public int ptd;
		public uint dwAspect;
		public int  lindex;
		public uint tymed;
	}

	[
	StructLayout(LayoutKind.Sequential)
	]
	public struct PROPSHEETPAGE
	{  
		public uint					dwSize; 
		public uint					dwFlags; 
		public IntPtr				hInstance; 

		// This is a union of the following Data items
		// String					pszTemplate; 
		// IntPtr					pResource;

		public IntPtr				pResource;

		// This is a union of the following Data items
		// int						hIcon; 
		// String					pszIcon; 
		public IntPtr				hIcon;

		public String				pszTitle; 
		public DialogProc			pfnDlgProc;
		public IntPtr				lParam; 

		public PropSheetPageProc	pfnCallback;
		public int					pcRefParent; 

		public String				pszHeaderTitle;
		public String				pszHeaderSubTitle;
	}

	[
	StructLayout(LayoutKind.Sequential, Pack=2, 
		CharSet=CharSet.Auto)
	]
	internal struct DLGTEMPLATE
	{
		internal uint style; 
		internal uint dwExtendedStyle; 
		internal ushort cdit; 
		internal short x; 
		internal short y; 
		internal short cx; 
		internal short cy;
		internal short wMenuResource;
		internal short wWindowClass;
		internal short wTitleArray;
		internal short wFontPointSize;
		[MarshalAs(UnmanagedType.LPWStr)]
		internal String szFontTypeface;
	}

	[
	StructLayout(LayoutKind.Sequential)
	]
	public struct STGMEDIUM
	{
		public uint tymed;
		public int hGlobal;
		public Object pUnkForRelease;
	}

	[
	StructLayout(LayoutKind.Sequential)
	]
	public struct userFLAG_STGMEDIUM
	{
		public int ContextFlags;
		public int fPassOwnership;
		public STGMEDIUM Stgmed;
	}

	[
	StructLayout(LayoutKind.Sequential)
	]
	public struct NMHDR
	{
		public int hwndFrom; 
		public uint idFrom; 
		public uint code; 
	}


	[StructLayout(LayoutKind.Sequential, Pack=1, CharSet=CharSet.Auto)]
	internal struct DIBSECTION
	{
		internal BITMAP            dsBm;
		internal BITMAPINFOHEADER  dsBmih;
		internal uint              dsBitfields0;
		internal uint              dsBitfields1;
		internal uint              dsBitfields2;
		internal IntPtr            dshSection;
		internal uint              dsOffset;
	}
	[StructLayout(LayoutKind.Sequential, Pack=1, CharSet=CharSet.Auto)]
	internal struct BITMAP
	{
		internal int      bmType;
		internal int      bmWidth;
		internal int      bmHeight;
		internal int      bmWidthBytes;
		internal ushort   bmPlanes;
		internal ushort   bmBitsPixel;
		internal IntPtr   bmBits;
	}
	[StructLayout(LayoutKind.Sequential, Pack=1, CharSet=CharSet.Auto)]
	internal struct BITMAPINFOHEADER
	{
		internal int      biSize;
		internal int      biWidth;
		internal int      biHeight;
		internal ushort   biPlanes;
		internal ushort   biBitCount;
		internal uint     biCompression;
		internal uint     biSizeImage;
		internal int      biXPelsPerMeter;
		internal int      biYPelsPerMeter;
		internal uint     biClrUsed;
		internal uint     biClrImportant;
	}
	[StructLayout(LayoutKind.Sequential, Pack=1, CharSet=CharSet.Auto)]
	internal struct BITMAPINFO
	{
		internal BITMAPINFOHEADER  bmiHeader;
		internal long              bmiColors0;
		internal long              bmiColors1;
		internal long              bmiColors2;
		internal long              bmiColors3;
	}

	//Added by Alexander Kachalkov
	//Themes support
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
	public struct ACTCTX 
	{  
		public uint cbSize;  
		public uint dwFlags;  
		public String lpSource;  
		public ushort wProcessorArchitecture;  
		public ushort wLangId;
		public String lpAssemblyDirectory;  
		public String lpResourceName;  
		public String lpApplicationName;  
		public IntPtr hModule;
	}// struct ACTCTX

	//////////////////////////////////////////////////////////////////////
	// 
	// Delegates used for Windows Callbacks
	// 

	public delegate bool DialogProc(IntPtr hwndDlg, uint uMsg, IntPtr wParam, IntPtr lParam); 
	public delegate uint PropSheetPageProc(IntPtr hwnd, uint uMsg, IntPtr lParam);

	
	//////////////////////////////////////////////////////////////////////
	// 
	// Linkage between usercontrols in a form node and the base snapin
	// 

	/// <summary>
	/// ISnapinLink is used to link a usercontrol to snapin objects.  Custom UserControls
	/// can choose to implement this interface called by the framework at control creation
	/// time.  The Base Sanpin instance is navigable from the Context Node
	/// </summary>
	[Guid("320879B0-AFD9-4277-9543-788AB588EDF0")]
	public interface ISnapinLink
	{
		/// <summary>
		/// The framework will provide the implementing object 
		/// with a reference to the node associated with the object.
		/// Implementing objects are likely UserControls use in FormNodes
		/// and propertyPages.
		/// </summary>
		BaseNode ContextNode
		{
			get;
			set;
		}
	}

	/// <summary>
	/// Implemented by UserControls used in propertypages to receive
	/// the PropertyPage object context.  Fromt hat context UserControsl can 
	/// navigate to the propertysheet, snapin node and base snapin object.
	/// </summary>
	public interface IPropertyPageContext
	{
		PropertyPage PropertyPage
		{
			get;
			set;
		}
	}
}
