using System;
using System.Runtime.InteropServices;

using Ironring.Management.MMC;

namespace MMCTest
{
	//Added by Alexander Kachalkov
	/// <summary>
	/// This class represents a new paradigm of using MMC Framework Library
	/// Just inherit from one of the base nodes and overloads needed method
	/// </summary>
	[Guid("936DA01F-9ABD-4d9d-80C7-02AF85C822A8")]
	public class MyReportNode: ReportNode
	{
		public MyReportNode(SnapinBase snapin):base(snapin)
		{
		}

		public MyReportNode(SnapinBase snapin, BaseNode node):base(snapin, node)
		{
		}

		public override string GetResultViewType(ref int pViewOptions)
		{
			// TODO:  Add MyReportNode.GetResultViewType implementation
			pViewOptions = (int)MMC_VIEW_OPTIONS.MULTISELECT;
	        return string.Empty;
		}
	
		public override void OnDelete()
		{
			// TODO:  Add MyReportNode.OnDelete implementation
			RESULTDATAITEM rdi = GetSelectedItem();
			int nRow;

			while (rdi.nIndex != -1)
			{
				nRow = GetTableRowID(rdi.lParam);

				Table.Rows[nRow].Delete();
				DeleteItem(rdi.lParam);

				rdi = GetSelectedItem();
			}

			base.OnDelete ();
		}

		/// <summary>
		/// return the guid for the node
		/// this is for Column Persistence
		/// </summary>
		public override Guid Guid
		{
			get 
			{
				object[] attrs = GetType().GetCustomAttributes(typeof(GuidAttribute), true);
				if (attrs.Length == 0)
					throw new SnapinException("Failed ot find GuidAttribute on MyReportBase class");

				return new Guid(((GuidAttribute)attrs[0]).Value);
			}
		}
	
		public override void OnSelectResult()
		{
			// TODO:  Add MyReportNode.OnSelectResult implementation
			
			//---ask snap-in for the following buttons		
			SetVerbState(MMC_CONSOLE_VERB.REFRESH, MMC_BUTTON_STATE.ENABLED, true);
			SetVerbState(MMC_CONSOLE_VERB.PROPERTIES, MMC_BUTTON_STATE.ENABLED, true);
			SetVerbState(MMC_CONSOLE_VERB.DELETE, MMC_BUTTON_STATE.ENABLED, true);
			
			//---status text
			Snapin.ResultViewConsole.SetStatusText("To delete multiple items selected them and press del");

			base.OnSelectResult ();
		}
	
		public override void Initialize()
		{
			// TODO:  Add MyReportNode.Initialize implementation
			base.Initialize ();

	        base.AddTopMenuItem(new MenuItem("New...", "", new MenuCommandHandler(MenuHandlerNew)));
		}

		public void MenuHandlerNew(object o, BaseNode node)
		{
			System.Windows.Forms.MessageBox.Show("Something new...");
		}

		// MAM: make this node use a Filtered view.
		/// <summary>
		/// Overrideable method to get the actual type of view for this report-node.
		/// </summary>
		/// <returns></returns>
		public override int GetViewType() {
			return (int)ViewMode.Filtered;
		}
	}
}