using System;
using System.Runtime.InteropServices;

using System.Drawing;
using System.Data;
using System.IO;

using Ironring.Management.MMC;

namespace MMCTest
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	/// 
	[
		SnapinIn("MMC Library Test Snapin", "Ironring Software", "2.1.0.100"),
		ProgId("MMCLibrary.TestSnapin"),
		Guid("B7F2CD77-FA26-4245-A086-984C74AF7884"),
        NodeTypeGuid("476e6448-aaff-11d0-b944-00c04fd8d5b0", true),
    ]
	public class TestSnapin : SnapinBase
	{
        protected ImageList m_SharedResultViewImages;

		// Added by Lesley van Zijl
		// We only want to see Snapin Manager Wizard once
		protected bool showWiz = true;
	    private HTMLNode rootnode;

	    public TestSnapin()
		{
            try
            {
                // add images to the global snapin scope image collection

                AddImage("MMCTest.images.Closed.ico");
                AddImage("MMCTest.images.Open.ico");
                AddImage("MMCTest.images.Happy.ico");
				AddImage("MMCTest.images.snapin.Ico");
				
                // populate the node result view collection
                m_SharedResultViewImages = new ImageList();
                m_SharedResultViewImages.Add("MMCTest.images.Open.ico");
                m_SharedResultViewImages.Add("MMCTest.images.Happy.ico");
				m_SharedResultViewImages.Add("MMCTest.images.snapin.Ico");

                // ste the default result view image control 
                BaseNode.DefaultResultPaneImages = m_SharedResultViewImages;

				InitToolbar();

                // create my base node
				//Changed by Wojciech Sobocinski
                HTMLNode rootnode = new CustomHTMLNode(this);
                rootnode.Url = "http://www.centeris.com/";

				rootnode.Pasteable = true;
                rootnode.DisplayName = "My Test Snapin";
				rootnode.OnSelectScopeEvent += new NodeNotificationHandler(RootNode_OnSelect);
				rootnode.OnDeleteEvent += new NodeNotificationHandler(RootNode_OnDelete);
				rootnode.OnControlbarNotifyEvent += new NodeNotificationHandler(RootNode_OnSetToolbar);
				rootnode.OnControlbarCommandEvent += new NodeNotificationHandler(RootNode_OnToolbarCommand);

				base.RootNode = rootnode;

                // create the sub nodes
                CreateHTMLNode(rootnode);
                CreateOCXNode();
                CreateForm1Node();
                CreateReportNode();
                CreateNodeTree();
                CreateForm2Node();
				//Added by Alexander Kachalkov
				//Changed by Lesley: Changed path to currentDir
				CreateDirectoryNode(this, Directory.GetCurrentDirectory(), rootnode); //Or any other path. 
				//Added by Wojciech Sobocinski
				CreateDragDropNodes();
            }
            catch(Exception e)
            {
                System.Diagnostics.Debug.WriteLine("Initialization exception - " + e.ToString());
            }
        }

	    public void CreateDirectoryNode(SnapinBase snapinBase, string
			parentDirectory, BaseNode parentNode) 
		{
			DirectoryInfo directoryInfo = new DirectoryInfo(parentDirectory);
			FolderNode folderNode = new FolderNode(snapinBase, parentDirectory);
			folderNode.DisplayName = directoryInfo.Name;
			parentNode.AddChild(folderNode);
			foreach(DirectoryInfo subDirectoryInfo in directoryInfo.GetDirectories()) 
			{
				CreateDirectoryNode(snapinBase,
									subDirectoryInfo.Parent.FullName + "\\" + subDirectoryInfo.Name,
									folderNode);
			}
		}

		protected void CreateNodeTree()
		{
			BaseNode node = new BaseNode(this);
			node.DisplayName = "The BaseNode forest";
			base.RootNode.AddChild(node);

            BaseNode childnode = null;

			//Added by Lesley van Zijl 
			//Tasks for the Example Wizards
			MenuItem wizTask = new MenuItem("show Wizard", "Shows a wizard", new MenuCommandHandler(WizardMethod));
			MenuItem wizTask2 = new MenuItem("show Wizard", "Shows a wizard", new MenuCommandHandler(WizardMethod2));
            
			childnode = new BaseNode(this);
			childnode.DisplayName = "1";
			//Added by Lesley van Zijl
			//Add the tasks to the first Node in the BaseNode Forest
			childnode.AddTaskMenuItem(wizTask);
			childnode.AddTaskMenuItem(wizTask2);
            node.AddChild(childnode);

			childnode = new BaseNode(this);
			childnode.DisplayName = "2";
            node.AddChild(childnode);

			childnode = new BaseNode(this);
			childnode.DisplayName = "3";
            node.AddChild(childnode);

			childnode = new BaseNode(this);
			childnode.DisplayName = "4";
            node.AddChild(childnode);

            MenuItem itemTask = new MenuItem("A Task", "Do a task", new MenuCommandHandler(MenuHandler));
            MenuItem itemView = new MenuItem("A View Command", "Do something to the view", new MenuCommandHandler(MenuHandler));

			childnode = new BaseNode(this);
			childnode.DisplayName = "5";
            childnode.AddTaskMenuItem(itemTask);
            childnode.AddViewMenuItem(itemView);
            node.AddChild(childnode);
            node = childnode;

			childnode = new BaseNode(this);
			childnode.DisplayName = "6";
            childnode.AddTaskMenuItem(itemTask);
            childnode.AddViewMenuItem(itemView);
            node.AddChild(childnode);

			childnode = new BaseNode(this);
			childnode.DisplayName = "7";
            node.AddChild(childnode);
			node = childnode;

			childnode = new BaseNode(this);
			childnode.DisplayName = "8";
            node.AddChild(childnode);

			childnode = new BaseNode(this);
			childnode.DisplayName = "9";

			node.AddChild(childnode);
		}

        protected HTMLNode CreateHTMLNode(BaseNode parent)
		{
			HTMLNode node = new HTMLNode(this);
			node.DisplayName = "Foo - HTML Node";
			node.Url = "http://www.sf.net/projects/mmclibrary";
            
            parent.AddChild(node);

			// context menu support

			MenuItem item = new MenuItem("Top level menu Item", "Something important because its on top!", new MenuCommandHandler(MenuHandler));
			node.AddTopMenuItem(item);
			item.Handler += new MenuCommandHandler(MenuHandler2); // add a second handler for the event

			item = new MenuItem("New Something", "add a new something", new MenuCommandHandler(NewNodeMenuHandler));
			node.AddNewMenuItem(item);

			item = new MenuItem("A Task", "Do a task", new MenuCommandHandler(MenuHandler));
			node.AddTaskMenuItem(item);

			item = new MenuItem("A View Item", "Something view related", new MenuCommandHandler(MenuHandler));
			node.AddViewMenuItem(item);

            node.AddPropertyPage("Page 1", Type.GetType("MMCTest.UserControl1"));
            node.AddPropertyPage("Page 2", Type.GetType("MMCTest.UserControl2"));
            node.AddPropertyPage("Page 3", Type.GetType("MMCTest.UserControl1"));
            node.AddPropertyPage("Page 4", Type.GetType("MMCTest.UserControl2"));
            node.AddPropertyPage("Page 5", Type.GetType("MMCTest.UserControl1"));

            rootnode = node;
//            rootnode.Guid = new Guid("e6c0d403-5f4b-4820-853f-d5e3ccfd7519");
            rootnode.Guid = new Guid("476e6448-aaff-11d0-b944-00c04fd8d5b0");
            rootnode.OnExpandEvent += new NodeNotificationHandler(RootNode_OnExpand);

            return node;
        }

        // TBD: change the prototype - BaseNode -> MenuHandlerArgs and have BaseNode be a member of that 
		protected void MenuHandler(object sender, BaseNode arg)
		{
            System.Windows.Forms.MessageBox.Show("You did something!");
		}

		protected void MenuHandler2(object sender, BaseNode arg)
		{
            System.Windows.Forms.MessageBox.Show("You did something else!");
        }

		//Added by Lesley van Zijl
		//This is a Method to call a Wizard, the static method ShowWizard in WizardBase is the
		//key to show a wizard, it's all you need to show it. it requires some parameters
		//- title for the title of the wizard
		//- Bitmap for the logo in the top right corner of the wizard, use null for no icon
		//- wizardpages, usercontrols derived from WizardPage in MMCLib
		//- the final parameter is a method that will be executed if the wizards finish button is clicked
        protected void WizardMethod(object sender, BaseNode arg)
		{
			WizardBase.ShowWizard("mmctest Wizard", null, 
				new WizardPage[] {new WzdTest1(), new WzdTest2()}, 
				new WizardBase.WizardAction(FirstWizHandler));
		}

		// this is the method that is executed after WizardMethod's Wizards' Finish button is pressed.
		// to collect data from the wizard (like textfield data) you first get the page out of the array and cast it too the WizardPage derived subclass
        private void FirstWizHandler(WizardPage[] pages)
		{
			System.Windows.Forms.MessageBox.Show("first textbox text was: " + 
				((WzdTest1)pages[0]).textBox1.Text);
			System.Windows.Forms.MessageBox.Show("second textbox text was: " + 
				((WzdTest2)pages[1]).textBox1.Text);
		}

		//this is an example for the use of a bitmap as a logo
		//it gets it's bitmap from an embedded resource
		protected void WizardMethod2(object sender, BaseNode arg)
		{
			WizardBase.ShowWizard("mmctest Wizard with Logo", 
				new System.Drawing.Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("MMCTest.images.SuperToolBar.bmp")),
				new WizardPage[] {new WzdTest1(), new WzdTest2()}, 
				new WizardBase.WizardAction(FirstWizHandler));
		}

        protected void NewNodeMenuHandler(object sender, BaseNode arg)
        {
            // dynamically insert anew node under neigth the sender
            HTMLNode node = CreateHTMLNode(arg);
            node.Insert(arg);
			//BaseNode node = new BaseNode(this);
			//node.Insert(RootNode);
        }

		protected void CreateOCXNode()
		{
			//Changed by Lesley
			base.RootNode.AddChild(new OCXNode(this, 
											   "Bar - OCX Node",
                                                // use the Windows Media Player 10 OCX
				                                new Guid("6BF52A52-394A-11D3-B153-00C04F79FAA6")
								               ));
			//OCXNode node = new OCXNode(this);
			//node.DisplayName = "Bar - OCX Node";
			// setup to use the calendar control
			//node.OCXGuid = new Guid("8E27C92B-1264-101C-8A2F-040224009C02");
			//base.RootNode.AddChild(node);
		}


        protected void CreateForm1Node()
        {
            FormNode node = new FormNode(this);
            node.DisplayName = "Well Formed! - Form Node";
            node.ControlType = Type.GetType("MMCTest.UserControl1");
            base.RootNode.AddChild(node);
        }

        protected void CreateForm2Node()
        {
            FormNode node = new FormNode(this);
            node.DisplayName = "Well Formed again! - Form Node";
            node.ControlType = Type.GetType("MMCTest.UserControl2");
            base.RootNode.AddChild(node);
        }
      
        protected void CreateReportNode()
		{
			DataTable theDataTable = new DataTable();

			// Declare variables for DataColumn and DataRow objects.
			DataColumn myDataColumn;
			DataRow myDataRow;
 
			// Create new DataColumn, set DataType, ColumnName and add to DataTable.    
			myDataColumn = new DataColumn();
			myDataColumn.DataType = System.Type.GetType("System.Int32");
			myDataColumn.ColumnName = "item";
			myDataColumn.ReadOnly = true;
			myDataColumn.Unique = true;
			// Add the Column to the DataColumnCollection.
			theDataTable.Columns.Add(myDataColumn);
 
			// Create second column.
			myDataColumn = new DataColumn();
			myDataColumn.DataType = System.Type.GetType("System.String");
			myDataColumn.ColumnName = "Name";
			myDataColumn.AutoIncrement = false;
			myDataColumn.Caption = "Name";
			myDataColumn.ReadOnly = false;
			myDataColumn.Unique = false;
			// Add the column to the table.
			theDataTable.Columns.Add(myDataColumn);

            // Create third column.
            myDataColumn = new DataColumn();
            myDataColumn.DataType = System.Type.GetType("System.String");
            myDataColumn.ColumnName = "Description";
            myDataColumn.AutoIncrement = false;
            myDataColumn.Caption = "Description";
            myDataColumn.ReadOnly = false;
            myDataColumn.Unique = false;
            // Add the column to the table.
            theDataTable.Columns.Add(myDataColumn);


			// Make the ID column the primary key column.
			DataColumn[] PrimaryKeyColumns = new DataColumn[1];
			PrimaryKeyColumns[0] = theDataTable.Columns["item"];
			theDataTable.PrimaryKey = PrimaryKeyColumns;
 
 
			// Create three new DataRow objects and add them to the DataTable
			for (int i = 0; i <= 200; i++)
			{
				myDataRow = theDataTable.NewRow();
				myDataRow["item"] = i;
				myDataRow["Name"] = "I Am item " + i;
                myDataRow["Description"] = "I Am a desription for item " + i;
                theDataTable.Rows.Add(myDataRow);
			}

			// setup the node
			//Changed by Alexander Kachalkov
			//ReportNode node = new ReportNode(this);
			MyReportNode node = new MyReportNode(this);

			node.DisplayName = "Baz - The Report Node";
			node.Table = theDataTable;

			//Commented by Alexander Kachalkov
			//node.OnSelectResultEvent += new NodeNotificationHandler(ReportNode_OnSelect);
            
			node.AddPropertyPage("Page 1", Type.GetType("MMCTest.UserControl1"));
			node.AddPropertyPage("Page 2", Type.GetType("MMCTest.UserControl2"));
			node.AddPropertyPage("Page 3", Type.GetType("MMCTest.UserControl1"));
			node.AddPropertyPage("Page 4", Type.GetType("MMCTest.UserControl2"));
			node.AddPropertyPage("Page 5", Type.GetType("MMCTest.UserControl1"));
			
			base.RootNode.AddChild(node);
		}

		//Added by Wojciech Sobocinski
		//Changed By Lesley van Zijl
		//made DragDropNode obsolete
		protected void CreateDragDropNodes()
		{
			BaseNode node = new BaseNode(this);
			node.DisplayName = "Drag Me";
			node.Pasteable = true;
			node.Cutable = true;
			base.RootNode.AddChild(node);

			BaseNode node2 = new BaseNode(this);
			node2.Pasteable = true;
			node2.Cutable = true;
			node2.DisplayName = "Drag Me Also";
			node.AddChild(node2);

			BaseNode node3 = new BaseNode(this);
			node3.Pasteable = true;
			node3.Cutable = true;
			node3.DisplayName = "Drag Me Too";
			base.RootNode.AddChild(node3);

		}

        protected void CreateMyComputerNode()
        {                         // 5D6179C8-17EC-11D1-9AA9-00C04FD8FE93

            //            Guid guidNew = new Guid("5D6179C8-17EC-11D1-9AA9-00C04FD8FE93"); // local users and groups
            Guid guidNew = new Guid("58221C69-EA27-11CF-ADCF-00AA00A80033"); // shared folders

            try 
            {
                base.AddExtension(rootnode.HScopeItem, guidNew);
                base.ConsoleNameSpace.Expand(Convert.ToUInt32(rootnode.HScopeItem));
            }
            catch (Exception e)
            {
                System.Windows.Forms.MessageBox.Show(e.Message + "\n" + e.StackTrace);
            }

             System.Windows.Forms.MessageBox.Show("added extension!");
        }

		//Commented by Alexander Kachalkov
		//due to new paradigm
		//see class MyReportNode
//		protected void ReportNode_OnSelect(object sender, NodeEventArgs args)
//		{
//			BaseNode selNode = sender as BaseNode;
//
//			//---ask snap-in for the following buttons		
//			IConsoleVerb icv;       
//			selNode.Snapin.ResultViewConsole.QueryConsoleVerb(out icv);
//			icv.SetVerbState(MMC_CONSOLE_VERB.REFRESH, MMC_BUTTON_STATE.ENABLED, 1);
//			icv.SetVerbState(MMC_CONSOLE_VERB.PROPERTIES, MMC_BUTTON_STATE.ENABLED, 1);
//			icv.SetVerbState(MMC_CONSOLE_VERB.DELETE, MMC_BUTTON_STATE.ENABLED, 1);
//
//			//---status text
//			//selNode.Snapin.ResultViewConsole.SetStatusText("The windows service to host remoting objects based on their configuration in the config file.");
//		}
		
		//Added by Alexander Kachalkov
		#region Node Toolbar Methods And Event Handlers
		protected void InitToolbar()
		{
			base.m_Component = new Component(this);
			base.m_Component.ToolbarImageIndex = AddImage("MMCTest.images.SuperToolBar.bmp", Color.Teal);

			MMCBUTTON[] btns = new MMCBUTTON[5];
			btns[0] = new MMCBUTTON(0, 1, "test", "test");
			btns[1] = new MMCBUTTON(1, 2, "test", "test");
			btns[2] = new MMCBUTTON(2, 3, "test", "test");
			btns[3] = new MMCBUTTON(0, 0, TBSTATE.ENABLED, TBSTYLE.SEP, "test", "test");
			btns[4] = new MMCBUTTON(3, 4, "test", "test");

			base.m_Component.ToolbarButtons = btns;
		}
		protected void RootNode_OnSelect(object sender, NodeEventArgs args)
		{
			BaseNode selNode = sender as BaseNode;

			//---ask snap-in for the following buttons		
			IConsoleVerb icv;       
			selNode.Snapin.ResultViewConsole.QueryConsoleVerb(out icv);
			icv.SetVerbState(MMC_CONSOLE_VERB.REFRESH, MMC_BUTTON_STATE.ENABLED, 1);
			icv.SetVerbState(MMC_CONSOLE_VERB.PROPERTIES, MMC_BUTTON_STATE.ENABLED, 1);
			icv.SetVerbState(MMC_CONSOLE_VERB.DELETE, MMC_BUTTON_STATE.ENABLED, 1);

			//---status text
			selNode.Snapin.ResultViewConsole.SetStatusText("The windows service to host remoting objects based on their configuration in the config file.");
		}
		protected void RootNode_OnDelete(object sender, NodeEventArgs e)
		{
			BaseNode selNode = sender as BaseNode;

			//---delete node and its children
			//selNode.Snapin.ConsoleNameSpace.DeleteItem((uint)selNode.HScopeItem, 1);
			//Added by Alexander Kachalkov
			selNode.Remove(false);

			//---set scope to the root
			//selNode.Snapin.ResultViewConsole.SelectScopeItem(selNode.Snapin.RootNode.HScopeItem);
		}

		protected void RootNode_OnSetToolbar(object sender, NodeEventArgs args)
		{
			BaseNode selNode = (BaseNode)sender;
			SetToolbarArgs e = (SetToolbarArgs)args;

			if (e.Toolbar == null) return;

			if (e.bSelect)
				e.ControlBar.Attach(MMC_CONTROL_TYPE.TOOLBAR, e.Toolbar);
			else
				e.ControlBar.Detach(e.Toolbar);
		}
		protected void RootNode_OnToolbarCommand(object sender, NodeEventArgs args)
		{
			BaseNode selNode = (BaseNode)sender;
			ToolbarCommandArgs e = (ToolbarCommandArgs)args;

			System.Windows.Forms.MessageBox.Show(String.Format("Toolbar button  {0} pressed!", Convert.ToString(e.verb)), "Node_OnToolbarCommandEvent");
        }
		#endregion

        private void RootNode_OnExpand(object sender, NodeEventArgs args)
        {
            CreateMyComputerNode();
        }


		//Added by Lesley van Zijl
		#region CCT.SNAPIN_MANAGER Implemenation
		// This is how it works:
		// If you load a snapin in the MMC (so a not saved snapin(.msc)) the mmc sends a msg to the
		// Snapin with the question 'do you have a SnapinManager for me?'
		// as this happens we set up our wizard, and we make sure that the Wizard is shown once. 
		// after that we let the base Class handle the QueryDataObject
		//
		// you could use a SnapinManager for Startup configuration, like the Microsoft Services plugin
		// it asks if you want to connect to a remote server or want work on the local machine.
		public override void QueryDataObject(int cookie, uint type, out IDataObject ppDataObject)
		{
			if(type == (uint)CCT.SNAPIN_MANAGER && showWiz)
			{
				System.Diagnostics.Debug.WriteLine("Asks for snapin manager in snapinbase");
				// false so that it doesn't show up again
				showWiz = false;
				ShowInitialisationWizard();
			} 
			base.QueryDataObject(cookie, type, out ppDataObject);
		}

		// The Wizard see comment on WizardMethod()
		public void ShowInitialisationWizard()
		{
			WizardBase.ShowWizard("Startup Snapin Manager Wizard", null ,new WizardPage[] {new WzdSnapinManager()}, new WizardBase.WizardAction(ManagerHandler));
		}

		// This is the method that handles the finish of the Wizard, in this example it does nothing
		// but this method could send data to a remote server or could adjust some nodes, be creative :)
		protected virtual void ManagerHandler(WizardPage[] managerPages)
		{
		}
        #endregion
    }

    // TODO: why does this need to be public?
    internal class CustomHTMLNode : HTMLNode
    {
        /// <summary>
        /// Get the clipboard format ids
        /// </summary>
        [DllImport("user32.dll", CharSet=CharSet.Unicode)]
        protected static extern ushort RegisterClipboardFormat(String format);

        private ushort s_SnapinMachineName = RegisterClipboardFormat("MMC_SNAPIN_MACHINE_NAME");

        public CustomHTMLNode(SnapinBase snapin)
            : base(snapin)
        {}

        public override bool HandleCustomClipboardFormat(ushort cf, BinaryWriter writer)
        {
            if (s_SnapinMachineName == cf)
            {
                // note: just calling the "String" method of BinaryWriter::Write() doesn't work
                //       because it will be written in ASCII, rather than unicode.  Using
                //       String::ToCharArray() doesn't work, because we're 
                //
                String hostname = "10.100.1.112";
                writer.Write(hostname.ToCharArray());

                return true;
            }
            return false;
        }
    }
}

