/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

// CenterisRedirect.cpp : Implementation of CCenterisRedirect

#include "stdafx.h"
#include "SpoofReg.h"
#include "RegLieManager.h"
#include "CenterisRedirect.h"

#include "ComUtil.h"

#define LIKEWISE_PORT	9106

// TODO: refactor Computer Management-specific code
//       out from MMC snap-in redirecting COM code?
//

CCenterisRedirect::CCenterisRedirect()
{
    m_pInner = NULL;
    m_cfMachineName = 0;
    m_strMachineName[0] = L'\0';
    m_fIsCenterisBox = FALSE;

    m_fUninitialized = TRUE;
}

//
// a couple of functions that will help us to do our winsock stuff a little
// better than what ATL can do.
//

//
// Quick function to get a host's ip address. We're passed a string and if it's a 
// an ip addrress we'll convert it to a sockaddr and if it's a domain name, we'll
// do a dns lookup on it.
//

BOOL GetHostAddr( char *lpHostName, LPSOCKADDR_IN lpReturnAddr )
{
	SOCKADDR_IN OurSock ;
	LPHOSTENT lpHost ;
	char szDebug[1024] ;

	OutputDebugString("(GetHostAddr) Now entering function") ;

	//
	// quick check..
	//

	if( !(lpHostName && lpReturnAddr) )
	{
		return(FALSE) ;
	}

	//
	// check and see if this a host name or an ip address. if we can convert it.
	// then it's an ip address.
	//

	if((OurSock.sin_addr.s_addr = inet_addr(lpHostName)) == INADDR_NONE)
	{
		lpHost = gethostbyname(lpHostName);

		if(!lpHost)
		{
			sprintf(szDebug, "(GetHostAddr) unable to resolve server name %s", lpHostName) ;
			OutputDebugString(szDebug) ;
			return(FALSE) ;
		}

		OurSock.sin_addr.s_addr = *((ULONG *)lpHost->h_addr) ;

		sprintf(szDebug, "(GetHostName) Now using IP address %s", inet_ntoa(OurSock.sin_addr) ) ;
		OutputDebugString(szDebug) ;
	}

	lpReturnAddr->sin_addr.s_addr = OurSock.sin_addr.s_addr ;

	return(TRUE) ;  // looks like everything went well.

}


//
// This function will allow us to make a connection to a remote machine. This is 
// probably a little more heavy weight than what we need (I pulled this from some
// other code that I wrote for a server app.)
//

BOOL ConnectToServer( 
						char	*lpHostName,	// host name or ip addr
						short	sPort,			// port to connect to
						DWORD	dwTimeOut		// time to wait on connect.
                    )
{

	DWORD dwReturn ;

	int af, type, proto ;

	WSAEVENT eWaitArray[1] ;

	WSANETWORKEVENTS TriggeredEvents ;

	SOCKET sSocket = INVALID_SOCKET ;

	SOCKADDR_IN ConnectTo ;

	WSAEVENT eConnectEvent = WSA_INVALID_EVENT ;

	BOOL bConnectSuccess = FALSE ;

	char szDebug[1024] ;
	
	

	//
	// Note to self.
	//

	OutputDebugString("(ConnectToServer) Now entering function.\n") ;
	
	//
	// quick check.
	//

	if( !lpHostName )
	{
		return(FALSE) ;
	}

	//
	// Try and get the ip address that we're going to try and connect to. Try and do
	// this first because if we can't even do this, there's no reason to do the rest.
	//

	ConnectTo.sin_family      = AF_INET; 
	ConnectTo.sin_addr.s_addr = INADDR_ANY ;
	ConnectTo.sin_port        = htons(sPort) ; 

	if( !GetHostAddr(lpHostName, &ConnectTo) )
	{
		return(FALSE) ;
	}

	//
	// do the grunt work of setting up the socket stuff.
	//

	af    = AF_INET ;
	type  = SOCK_STREAM ;
	proto = IPPROTO_TCP ;

	sSocket = socket(af, type, proto) ;

	if( sSocket == INVALID_SOCKET )
	{
		OutputDebugString("(ConnectToServer) Unable to create a socket\n") ;
		return(FALSE) ;
	}

	//
	// set up the event seclect stuff...
	//

	eConnectEvent = WSACreateEvent() ;

	if( eConnectEvent == WSA_INVALID_EVENT )
	{
		OutputDebugString("(ConnectToServer) Unable to create wait event for connect.\n") ;
		goto Cleanup ;
	}

	WSAEventSelect(sSocket, eConnectEvent, FD_CONNECT ) ;

	eWaitArray[0] = eConnectEvent ;

	//
	// try and connect to the remote host.
	//

	bConnectSuccess = FALSE ;

	WSAResetEvent(eConnectEvent) ;

	WSAConnect(       
					sSocket, 
					(const LPSOCKADDR) &ConnectTo, 
					sizeof(SOCKADDR_IN), 
					NULL, 
					NULL, 
					NULL, 
					NULL 
			  ) ;

	dwReturn = WSAWaitForMultipleEvents(
											1,				// how many events
											eWaitArray,		// events to wait on
											FALSE,			// trigger on any of them
											dwTimeOut,		// how long to wait
											FALSE			// we'll do our own queueing
										) ;
	//
	// check and see if we connected or if we timed out.
	//

	if( dwReturn == WSA_WAIT_EVENT_0 )
	{
		WSAEnumNetworkEvents(sSocket, eConnectEvent, &TriggeredEvents) ;

		if(    (TriggeredEvents.lNetworkEvents & FD_CONNECT)
			&& (TriggeredEvents.iErrorCode[FD_CONNECT_BIT] == 0)
		  )
		{
			bConnectSuccess = TRUE ;
		}
		else
		{
			sprintf(szDebug, "(ConnectToServer) Unable to connect to server %s, error = %d", lpHostName, TriggeredEvents.iErrorCode[FD_CONNECT_BIT] ) ;
			OutputDebugString(szDebug) ;
		}
		

	}

	//
	// if we're here, then we encountered some type of error and we need to clean up
	// and return our general error that says to exit the thread.
	//

	Cleanup: 

	if( eConnectEvent != WSA_INVALID_EVENT ) 
	{
		CloseHandle(eConnectEvent) ;
	}
		
	if( sSocket != INVALID_SOCKET )
	{
		closesocket(sSocket) ;
	}

	return( bConnectSuccess ) ;

}  // end of ConnectToServer






//*****************************************************************
//** ATL methods
//**

HRESULT CCenterisRedirect::FinalConstruct()
{
    HRESULT hr;

    // note: we don't need to release punkOuter, as
    // GetControllingUnknown() doesn't addref()
    //
    IUnknown * punkOuter = GetControllingUnknown();
    hr = CoCreateInstanceIgnoringTreatAs(_uuidof(STOLENCLASS),
                                         punkOuter,
                                         IID_IUnknown,
                                         (LPVOID*)&m_pInner,
                                         &g_hDllInner);

	if (FAILED(hr))
		return hr;

    m_cfMachineName = (CLIPFORMAT)::RegisterClipboardFormat(_T("MMC_SNAPIN_MACHINE_NAME"));

    // Load Winsock
    WSADATA wsaData;
	int retval;
    if ((retval = WSAStartup(MAKEWORD(2,2), &wsaData)) != 0)
    {
        WSACleanup();

		return E_FAIL;
    }

    return hr;
}

void CCenterisRedirect::FinalRelease()
{
    if (m_pInner) {
        m_pInner->Release();
        m_pInner = NULL;
    }

    bool fCanUnload = (1 == _AtlModule.GetLockCount());
    if (g_hDllInner && fCanUnload)
    {
        typedef int (__stdcall *PDLLCANUNLOADNOW)(void);

        // unload the wrapped DLL, if we can
        PDLLCANUNLOADNOW pfnDllCanUnloadNow = NULL;
        pfnDllCanUnloadNow = ::GetProcAddress(g_hDllInner, _T("DllCanUnloadNow"));
        if (pfnDllCanUnloadNow)
        {
            // if this is true, the DLL doesn't have any more COM references
            // in memory.  If it doesn't, unload it.
            //
            HRESULT hr = (*pfnDllCanUnloadNow)();
            if (S_OK == hr)
            {
                ::FreeLibrary(g_hDllInner);
                g_hDllInner = NULL;
            }
        }
    }

	// done with winsock
	WSACleanup();

}

//*****************************************************************
//** Utility functions
//**

// Get the current machine name from the real computer management snap-in.
// The machine name is stored in the field m_strMachineName.
// An empty string "" means the snap-in is connected to the local machine.
//
// Also detects whether the machine is running centeris web services,
// and if it is, it enables m_fIsCenterisBox.
//
HRESULT CCenterisRedirect::UpdateCurrentMachineName(IDataObject *piDataObject)
{
    HRESULT hr = ExtractData(piDataObject,
                             m_cfMachineName,
                             (BYTE*)m_strMachineName, 
                             sizeof(m_strMachineName));

    // we've now updated m_strMachineName with the name of the
    // newly connected machine.  Is it a Centeris box?
    // We tell, by seeing if we can connet to the SystemManger
    // web service.
    //
    setCenterisBoxInfo(isCenterisMachine());
    return hr;
}

//
// Returns true if m_strMachineName refers to a Centeris-enabled server.
// The test for this is somewhat rudimentary: we check if the machine is
// remote and we check if it has the Likewise port available.

BOOL CCenterisRedirect::isCenterisMachine()
{

	BOOL bReturn = FALSE ;

	OutputDebugString("(isCenterisMachine) Now entering function") ;

	//
	// try and do a simple connect to the remote machine to see if it's listening
	// on the Likewise port, which is the com port for webservices.
	//
	// TODO - change the hard coded values here to something that is defined someplace
	// else.
	//

	if( ConnectToServer( CW2A(m_strMachineName), LIKEWISE_PORT, 2000) ) // timeout is 2 seconds
	{
		OutputDebugString("(isCenterisMachine) We were able to connect to remote machine") ;
		bReturn = TRUE ;
	}
	else
	{
		OutputDebugString("(isCenterisMachine) Remote machine is unreachable.") ;
		bReturn = FALSE ;
	}
		
	return(bReturn) ;

	
}

void CCenterisRedirect::setCenterisBoxInfo(BOOL fNewBoxIsCenterisBox)   // TRUE if a centeris linux box
                                                                        // FALSE if a windows box
{
    m_fIsCenterisBox = fNewBoxIsCenterisBox;

    // TODO: HACK -- doesn't work with >1 console
    g_pLieManager->SetShouldLie(fNewBoxIsCenterisBox);

    // we've determined what type of box we're connected to, so enable our flag
    //
    m_fUninitialized = FALSE;
}

// Returns TRUE if the hostname in m_strMachineName
// refers to a remote machine, rather than the local machine.
// (if it's the local machine, it will be the empty string)
//
BOOL CCenterisRedirect::isRemoteMachine()
{
    return (L'\0' != m_strMachineName[0]);
}


//*****************************************************************
//** IComponentData implementation
//**
//** We implement this interface to hook the "Command" method,
//** which is called when the user chooses "Connect To" from the
//** Computer Management context menu.
//**
STDMETHODIMP CCenterisRedirect::AddMenuItems( LPDATAOBJECT piDataObject,
                        LPCONTEXTMENUCALLBACK piCallback,
                        long* pInsertionAllowed )
{

    HRESULT hr = E_UNEXPECTED;

    CComQIPtr<IExtendContextMenu,&IID_IExtendContextMenu> iContextMenuReal(m_pInner);
    if (iContextMenuReal) {

        hr = iContextMenuReal->AddMenuItems(piDataObject,
                                            piCallback,
                                            pInsertionAllowed);
    }
    if (m_fUninitialized)
    {
        // TODO: hook things just once at initialize time?
        //
        UpdateCurrentMachineName(piDataObject);
        m_fUninitialized = FALSE;

        // trace attached computer name
        ATLTRACE2(atlTraceGeneral,
                  0,
                  _T("in AddMenuItems, machine name is %ls\n"),
                  m_strMachineName);
    }

    return hr;
}

STDMETHODIMP CCenterisRedirect::Command(long lCommandID, LPDATAOBJECT piDataObject)
{
    HRESULT hr = E_UNEXPECTED;

    CComQIPtr<IExtendContextMenu,&IID_IExtendContextMenu> iContextMenuReal(m_pInner);
    if (iContextMenuReal) {

        hr = iContextMenuReal->Command(lCommandID,
                                        piDataObject);
    }

    if (s_nConnectToCommandID == lCommandID)
    {
        // the "connect to..." method was called
        //
        UpdateCurrentMachineName(piDataObject);
        ATLTRACE2(atlTraceGeneral,
                    0,
                    _T("in Command, lCommandID=%d, machine name is %ls\n"),
                    lCommandID,
                    m_strMachineName);
    }

    return hr;

}


//*****************************************************************
//** IComponentData implementation
//**
//** We implement this interface to override the QueryDataObject
//** method, which lets us insert our own wrapper for IDataObject
//** around the one that the Computer Management object would
//** return.
//**
STDMETHODIMP CCenterisRedirect::Initialize(/* [in] */ LPUNKNOWN pUnknown)
{
    HRESULT hr = E_UNEXPECTED;

    CComQIPtr<IComponentData,&IID_IComponentData> iComponentDataReal(m_pInner);
    if (iComponentDataReal) {

        hr = iComponentDataReal->Initialize(pUnknown);
    }

    return hr;
}

STDMETHODIMP CCenterisRedirect::CreateComponent(/* [out] */ LPCOMPONENT * ppComponent)
{
    HRESULT hr = E_UNEXPECTED;

    CComQIPtr<IComponentData,&IID_IComponentData> iComponentDataReal(m_pInner);
    if (iComponentDataReal) {

        hr = iComponentDataReal->CreateComponent(ppComponent);
    }

    return hr;
}

STDMETHODIMP CCenterisRedirect::Notify(
        /* [in] */ LPDATAOBJECT lpDataObject,
        /* [in] */ MMC_NOTIFY_TYPE event,
        /* [in] */ LPARAM arg,
        /* [in] */ LPARAM param)
{
    HRESULT hr = E_UNEXPECTED;

    CComQIPtr<IComponentData,&IID_IComponentData> iComponentDataReal(m_pInner);
    if (iComponentDataReal) {

        hr = iComponentDataReal->Notify(lpDataObject, event, arg, param);
    }

    if (m_fUninitialized)
    {
        UpdateCurrentMachineName(lpDataObject);
        m_fUninitialized = FALSE;

        // trace attached computer name
        ATLTRACE2(atlTraceGeneral,
                    0,
                    _T("in Notify, event=%d, machine name is %ls\n"),
                    event,
                    m_strMachineName);
    }

    return hr;
}

STDMETHODIMP CCenterisRedirect::Destroy(void) {
    HRESULT hr = E_UNEXPECTED;

    CComQIPtr<IComponentData,&IID_IComponentData> iComponentDataReal(m_pInner);
    if (iComponentDataReal) {

        hr = iComponentDataReal->Destroy();
    }

    return hr;
}

STDMETHODIMP CCenterisRedirect::QueryDataObject(
        /* [in] */ MMC_COOKIE cookie,
        /* [in] */ DATA_OBJECT_TYPES type,
        /* [out] */ LPDATAOBJECT *ppDataObject)
{
    HRESULT hr = E_UNEXPECTED;

    ATLTRACE2(atlTraceGeneral,
                0,
                _T("QueryDataObject called, our cookie is = 0x%x\n"),
                cookie);

    CComQIPtr<IComponentData,&IID_IComponentData> iComponentDataReal(m_pInner);
    if (iComponentDataReal) {

        hr = iComponentDataReal->QueryDataObject(cookie, type, ppDataObject);
    }

    // wrap pInternalDataObject with our object, and return that in ppDataObject
    //
    if (ppDataObject) {

        if (CCT_SNAPIN_MANAGER == type)
        {
            UpdateCurrentMachineName(*ppDataObject);
        }
    }

    return hr;
}

STDMETHODIMP CCenterisRedirect::GetDisplayInfo(
        /* [out][in] */ SCOPEDATAITEM *pScopeDataItem)
{
    HRESULT hr = E_UNEXPECTED;

    CComQIPtr<IComponentData,&IID_IComponentData> iComponentDataReal(m_pInner);
    if (iComponentDataReal) {

// TODO: how to handle select?        g_pLieManager->SetShouldLie(m_fIsCenterisBox);

        hr = iComponentDataReal->GetDisplayInfo(pScopeDataItem);
    }

    return hr;
}

STDMETHODIMP CCenterisRedirect::CompareObjects(
        /* [in] */ LPDATAOBJECT lpDataObjectA,
        /* [in] */ LPDATAOBJECT lpDataObjectB)
{
    HRESULT hr = E_UNEXPECTED;

    CComQIPtr<IComponentData,&IID_IComponentData> iComponentDataReal(m_pInner);
    if (iComponentDataReal) {

        hr = iComponentDataReal->CompareObjects(lpDataObjectA, lpDataObjectB);
    }

    return hr;
}

