/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

// CenterisRedirect.h : Declaration of the CCenterisRedirect

#pragma once
#include "resource.h"       // main symbols


#include "hackMmcTest.h"
#include "legacyMMC.h"

class ChackMmcTestModule;
extern ChackMmcTestModule _AtlModule;


// CCenterisRedirect

class ATL_NO_VTABLE CCenterisRedirect : 
    public CComObjectRootEx<CComSingleThreadModel>,
    public CComCoClass<CCenterisRedirect, &CLSID_CenterisRedirect>,
    public ICenterisRedirect,
    public IExtendContextMenu,
    public IComponentData
{
public:
    IUnknown * m_pInner;
    CLIPFORMAT m_cfMachineName;

    WCHAR m_strMachineName[MAX_PATH];
    BOOL m_fIsCenterisBox;

    // when true, we haven't set m_strMachineName or m_fIsCenterisBox yet
    //
    BOOL m_fUninitialized;

    // obtained through experimentation
    //
    static const int s_nConnectToCommandID = 251;

    CCenterisRedirect();

    DECLARE_REGISTRY_RESOURCEID(IDR_CENTERISREDIRECT)
/*
    // interfaces that we're not implementing, but could
    COM_INTERFACE_ENTRY(IID_IComponent)
    COM_INTERFACE_ENTRY(IID_IComponentData)
    COM_INTERFACE_ENTRY(IID_IEnumTASK)
    COM_INTERFACE_ENTRY(IID_IExtendContextMenu)
    COM_INTERFACE_ENTRY(IID_IExtendControlbar)
    COM_INTERFACE_ENTRY(IID_IExtendPropertySheet2)
    COM_INTERFACE_ENTRY(IID_IExtendTaskPad)
    COM_INTERFACE_ENTRY(IID_IResultDataCompare)
    COM_INTERFACE_ENTRY(IID_IResultDataCompareEx)
    COM_INTERFACE_ENTRY(IID_IResultOwnerData)
    COM_INTERFACE_ENTRY(IID_ISnapinAbout)
    COM_INTERFACE_ENTRY(IID_ISnapinHelp2)
*/
BEGIN_COM_MAP(CCenterisRedirect)
    COM_INTERFACE_ENTRY(ICenterisRedirect)
    COM_INTERFACE_ENTRY(IExtendContextMenu)
    COM_INTERFACE_ENTRY(IComponentData)

    COM_INTERFACE_ENTRY_AGGREGATE_BLIND(m_pInner)
END_COM_MAP()


    // required for aggregation
    //
    DECLARE_GET_CONTROLLING_UNKNOWN()

    DECLARE_PROTECT_FINAL_CONSTRUCT()

    HRESULT FinalConstruct();
    void FinalRelease();

protected:
    //*****************************************************************
    //** Utility functions
    //**

    HRESULT UpdateCurrentMachineName(IDataObject *piDataObject);
    BOOL isRemoteMachine();
	BOOL isCenterisMachine();

    void setCenterisBoxInfo(BOOL);

public:
    //*****************************************************************
    //** Interface IExtendContextMenu
    //**
    //** Used to hook the "Connect To..." command.
    //**
    STDMETHODIMP AddMenuItems( LPDATAOBJECT piDataObject,
                          LPCONTEXTMENUCALLBACK piCallback,
                          long* pInsertionAllowed );

    STDMETHODIMP Command(long lCommandID, LPDATAOBJECT piDataObject);

    //*****************************************************************
    //** Interface IComponentData
    //**
    //** Used to insert our IDataObject wrapper around the data
    //** objects returned to MMC.  Creating our own data object
    //** wrapper allows us to insert our own clipboard formats,
    //** to provide or Centeris extension snapins with information
    //** about the machine they're managing.
    //**
    STDMETHODIMP Initialize(/* [in] */ LPUNKNOWN pUnknown);
    STDMETHODIMP CreateComponent(/* [out] */ LPCOMPONENT * ppComponent);
    STDMETHODIMP Notify(
            /* [in] */ LPDATAOBJECT lpDataObject,
            /* [in] */ MMC_NOTIFY_TYPE event,
            /* [in] */ LPARAM arg,
            /* [in] */ LPARAM param);
    STDMETHODIMP Destroy(void);
    STDMETHODIMP QueryDataObject(
            /* [in] */ MMC_COOKIE cookie,
            /* [in] */ DATA_OBJECT_TYPES type,
            /* [out] */ LPDATAOBJECT *ppDataObject);
    STDMETHODIMP GetDisplayInfo(
            /* [out][in] */ SCOPEDATAITEM *pScopeDataItem);
    STDMETHODIMP CompareObjects(
            /* [in] */ LPDATAOBJECT lpDataObjectA,
            /* [in] */ LPDATAOBJECT lpDataObjectB);
};


OBJECT_ENTRY_AUTO(__uuidof(CenterisRedirect), CCenterisRedirect)
