/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

#include "stdafx.h"

//
// Big Fat Note - This module is geared to work on NT only; there are a number of issues that
// I've skipped that relate directly to Win9x, simply because we don't need to worry about them.
// There are a number of "gotchas" that still need to be taken care of, such as the weird values
// you'll sometimes run across in dereferencing a header. For the most part this works
// with MMC and all of its components, which is what we really want for this round.
//


#include "hookapi.h"


#pragma warning (disable: 4311) // pointer truncation
#pragma warning (disable: 4312) // conversion to pointer of greater size
#pragma warning (disable: 4244) // conversion from 'SIZE_T' to 'DWORD', possible loss of data



static GET_PROC_ADDRESS_API_PTR g_lpRealGetProcAddress	= NULL;  
static LOAD_LIBRARYA_API_PTR 	g_lpRealLoadLibrary		= NULL; 
static FREE_LIBRARY_API_PTR 	g_lpRealFreeLibrary		= NULL; 


//
// Functions that we need the real entrypoints so that if someone is trying to spoof
// stuff in the kernel, we'll get the real entrypoints before we do any of the 
// hooking stuff.
//

BOOL GetRealKernelFunctions( void )
{

	//
	// Ok, think about this one. We need to make sure that we can *always* do a 
	// LoadLibrary and GetProcAddress without being spoofed. So, first off, get the real
	// entry points and make note of them.
	//

	if(!g_lpRealGetProcAddress) 
	{
		g_lpRealGetProcAddress = (GET_PROC_ADDRESS_API_PTR) GetProcAddress(GetModuleHandle(NT_KERNEL), "GetProcAddress") ;

		if( !g_lpRealGetProcAddress )
		{
			return(FALSE) ;
		}
	}
		
	if(!g_lpRealLoadLibrary) 
	{
		g_lpRealLoadLibrary = (LOAD_LIBRARYA_API_PTR) g_lpRealGetProcAddress(GetModuleHandle(NT_KERNEL), "LoadLibraryA") ;

		if( !g_lpRealLoadLibrary )
		{
			return(FALSE) ;
		}
	}


	if(!g_lpRealFreeLibrary) 
	{
		g_lpRealFreeLibrary = (FREE_LIBRARY_API_PTR) g_lpRealGetProcAddress(GetModuleHandle(NT_KERNEL), "FreeLibrary") ;

		if( !g_lpRealLoadLibrary )
		{
			return(FALSE) ;
		}
	}


	return(TRUE) ;

}



//
// quick function to scan our list of functions in an image entry to find
// the real entrypoint. If we don't find it or it isn't set, we'll return a
// NULL to the caller.
//

FARPROC GetRealEntryPoint( char *lpFunctionName, LPHOOKED_IMAGE lpImageEntry )
{

	DWORD dwLoop ;

	if( !( lpFunctionName && lpImageEntry) )
	{
		return(NULL) ;
	}

	for( dwLoop = 0 ; dwLoop < lpImageEntry->dwHookedFunctionsCount ; dwLoop++ )
	{
		if( !strcmp(lpImageEntry->HookedFunctions[dwLoop].szFunctionName, lpFunctionName) )
		{
			return( (FARPROC)lpImageEntry->HookedFunctions[dwLoop].lpRealEntryPoint ) ;
		}
	}

	//
	// if we're here, then we didn't find it.
	//

	return(NULL) ;

}



//
// This function creates a hooked image entry. were passed a list of
// functions and an image name.
//

LPHOOKED_IMAGE CreateHookedImageEntry( char *szImageName, LPHOOKED_FUNCTION_INFO lpFunctionList )
{
	LPHOOKED_IMAGE lpTmpEntry ;
	
	int iTmpCount, iFunctionCount, iTmpSize ;

	LPHOOKED_FUNCTION_INFO lpLoop ;

	FARPROC lpTmpEntryPoint ;

	BOOL bErrorOccurred ;

	

	//
	// quick check. 
	//

	if( !( szImageName && lpFunctionList) )
	{
		return(NULL) ;
	}

	//
	// Get the real function pointers, just incase we haven't yet.
	//

	if( !GetRealKernelFunctions() )
	{
		return(NULL) ;
	}

	//
	// first up, count the number of functions passed to us in the list. Note,
	// there should be at least one.
	//

	lpLoop			= lpFunctionList ;
	iFunctionCount	= 0 ;

	while(lpLoop->szFunctionName[0] != '\0' )
	{
		iFunctionCount++ ;
		lpLoop++ ;
	}
	
	if( !iFunctionCount )
	{
		return(NULL) ;
	}

	//
	// get the memory for the entire entry and list of functions. Remember, there should be
	// at least one in the list, and we've accounted for in the in struct def.
	//
	
	iTmpSize = sizeof(HOOKED_IMAGE)+(sizeof(HOOKED_FUNCTION_INFO)*(iFunctionCount-1)) ;

	lpTmpEntry = (LPHOOKED_IMAGE)malloc( iTmpSize ) ;

	if( !lpTmpEntry )
	{
		// todo - put some error output here.
		return(NULL) ;
	}

	//
	// Fill in the entry with the info that was passed to us.
	// 
	// TODO - add some error logging, if we need it.
	//

	ZeroMemory( lpTmpEntry, iTmpSize ) ;

	strncpy( lpTmpEntry->szImageName, szImageName, MAX_PATH ) ;
	lpTmpEntry->szImageName[MAX_PATH-1] = '\0' ;

	lpTmpEntry->dwHookedFunctionsCount = iFunctionCount ;

	//
	// Try and load the DLL. If we can't then we're done.
	//

	bErrorOccurred = FALSE ;

	lpTmpEntry->hImageHandle = g_lpRealLoadLibrary(lpTmpEntry->szImageName) ;

	if( lpTmpEntry->hImageHandle == NULL )
	{
		bErrorOccurred = TRUE ;
		goto Cleanup ;
	}

	//
	// loop over the function names again and copy them along with thier real info into the list.
	//

	iTmpCount	= 0 ;
	lpLoop		= lpFunctionList ;
	
	while( lpLoop->szFunctionName[0] != '\0' )
	{
	
		lpTmpEntry->HookedFunctions[iTmpCount] = *lpLoop ;

		lpTmpEntryPoint = g_lpRealGetProcAddress(	
												lpTmpEntry->hImageHandle,
												lpTmpEntry->HookedFunctions[iTmpCount].szFunctionName
											  ) ;

		if( lpTmpEntryPoint )
		{
			lpTmpEntry->HookedFunctions[iTmpCount].lpRealEntryPoint = lpTmpEntryPoint ;
		}
		else
		{
			bErrorOccurred = TRUE ;	// at least one bad entry.
			goto Cleanup ;
		}

		iTmpCount++ ;
		lpLoop++ ;
		
	}
	
	// TODO - add error check / assert for if iTmpCount != iFunctionCount ;

	//
	// Check and see if we were able to resolve all of the addresses correctly and if
	// not, pass back a NULL and let the caller figure out what happened.
	//

	if( bErrorOccurred )
	{
		free( lpTmpEntry ) ;
		lpTmpEntry = NULL  ;
	}

	//
	// Done, return the chunk of memory to the caller or a NULL if there was an error.
	//

	Cleanup :

	return(lpTmpEntry) ;
	

}



//
// In this function, we're going to try and hook all the functions in the list of images that
// were passed. If there is any problem in looking up the original entry point of dll/exe
// to hook, we'll skip it and go on to the next function. The caller should check.
//

DWORD HookImage( char *szHookModuleName, LPHOOKED_IMAGE lpStuffToHook, BOOL bSpoofFunctions )
{

	PIMAGE_IMPORT_DESCRIPTOR lpImageDescriptor ;

	HMODULE hHookModule ;

	DWORD dwError ;
	
	//
	// quick check.
	//

	if( !(szHookModuleName && lpStuffToHook) )
	{
		return(ERROR_INVALID_PARAMETER) ;
	}

	//
	// Before we do anything, make *sure* that we have the real entrypoints
	// to the kernel functions that we need. This is just incase someone
	// is trying to hook them.
	//

	if( !GetRealKernelFunctions() )
	{
		// todo - error logging here.
		return(ERROR_BAD_CONFIGURATION) ;
	}

	//
	// First step, try and load the DLL. If we can't then we're done.
	//

	lpStuffToHook->hImageHandle = g_lpRealLoadLibrary(lpStuffToHook->szImageName) ;

	if( lpStuffToHook->hImageHandle == NULL )
	{
		dwError = GetLastError() ;
		return(dwError) ;
	}

	//
	// Magic Happens Here.
	//

	hHookModule = g_lpRealLoadLibrary(szHookModuleName) ;

	if( !hHookModule )
	{
		dwError = GetLastError() ;
		return(dwError) ;
	}

	lpImageDescriptor = GetImportTable( hHookModule, lpStuffToHook->szImageName) ;

	if( !lpImageDescriptor )
	{
		return(ERROR_NOT_FOUND) ;	// todo - put better error logic/reporting.
	}

	//
	// If we have a valid descriptor, then we'll patch the image's address table to
	// point to where we want it to.
	//

	dwError = PatchAddressTable( hHookModule, lpImageDescriptor, lpStuffToHook, bSpoofFunctions ) ;

	
	//
	// Done, let the caller know what we found out. If we patched the table successfully, then that's
	// what we'll return.
	//

	return(dwError) ;

}



//
// This is one of our "magic happens" functions where we rip apart
// the import table looking for the module that has the functions
// we want to hook.
//

PIMAGE_IMPORT_DESCRIPTOR GetImportTable(HMODULE hHookModule, char *szImportName)
{

	PIMAGE_DOS_HEADER			lpDosHeader			= NULL ;
	PIMAGE_NT_HEADERS 			lpNTHeader			= NULL ;
	PIMAGE_IMPORT_DESCRIPTOR	lpImportDescriptor	= NULL ;

	char *szTmpName ;

	//
	// First off, try and get the dos version of the header and see
	// if it's valid.
	//
	// Note - you can find all of these defines in winnt.h, if you're
	// feeling masochistic.
	//
	
	lpDosHeader = (PIMAGE_DOS_HEADER) hHookModule;

	if(		(IsBadReadPtr(lpDosHeader, sizeof(IMAGE_DOS_HEADER)))
		||	(lpDosHeader->e_magic != IMAGE_DOS_SIGNATURE)
	  )
	{
		return(NULL) ;
	}

	//
	// Ok, now get the NT version of the header, which contains the PE header.
	//

	lpNTHeader = MakePtr(PIMAGE_NT_HEADERS, lpDosHeader, lpDosHeader->e_lfanew);

	if(		(IsBadReadPtr(lpNTHeader, sizeof(IMAGE_NT_HEADERS)))
		||	(lpNTHeader->Signature != IMAGE_NT_SIGNATURE)
	  )
	{
		return(NULL) ;
	}

	//
	// Now check and see if there are any imports for this image.
	//
	
	if( lpNTHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress == 0 )
	{
		return(NULL) ;
	}

	//
	// Ok, at this point we have a vaild image, with the correct headers and a list of files
	// that it imports. We'll now scan the list of imports to try and find the dll that we
	// want to change the function pointers on.
	//
	
	lpImportDescriptor = MakePtr(	PIMAGE_IMPORT_DESCRIPTOR, 
									lpDosHeader,
									lpNTHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress
								);

	//
	// Loop through the list of imports and see if we can find the name of
	// the one that we want to hook.
	//
	// Note - we're doing a lowercase compare here.
	//
	
	while (lpImportDescriptor->Name)
	{
		szTmpName = MakePtr(PSTR, lpDosHeader, lpImportDescriptor->Name);

		if( !(_stricmp(szTmpName, szImportName)) )
		{
			break ;
		}

		lpImportDescriptor++;
	}

	if( lpImportDescriptor->Name != 0 )
	{
		return(lpImportDescriptor) ;
	}

	//
	// if we're here, then we didn't find the module that we want to hook
	// in this image, even though everything was fine.
	//

	return(NULL) ;

}



//
// This is the function that will patch the address table with the new function locations
// that we want the process to use.
//

DWORD PatchAddressTable( 	HMODULE 					hHookModule, 			// handle to IAT to scan
							PIMAGE_IMPORT_DESCRIPTOR	lpImportDescriptor, 	// descriptor table that has function names
							LPHOOKED_IMAGE				lpStuffToHook, 			// our list
							BOOL						bSpoofFunctions 		// patch table with spoofed or real addresses.
						)
{

	PIMAGE_THUNK_DATA lpOriginalThunk, lpAddressThunk ;
  
	PIMAGE_IMPORT_BY_NAME lpImportName ;

	MEMORY_BASIC_INFORMATION MemoryBlock ;

	DWORD dwMemoryBlockSize, dwNameLoop, dwTmp, dwError ;

	//
	// quick check.
	//

	if( !(		lpImportDescriptor 
			&&	lpStuffToHook 
			&&	!(IsBadReadPtr((PIMAGE_THUNK_DATA)hHookModule, sizeof(IMAGE_THUNK_DATA))) 
		 ) 
	  )
	{
		return(ERROR_INVALID_PARAMETER) ;
	}

	//
	// First off, get the original thunk pointer, so we can scan for the names of
	// the functions that we want to change the entry points to. This table hasn't 
	// been updated by the system loader.
	//

	lpOriginalThunk = MakePtr(PIMAGE_THUNK_DATA, hHookModule, lpImportDescriptor->OriginalFirstThunk);

	//
	// The first thunk contains the address that we want to patch. This table should have already been
	// update by the system loader to have the real addresses of the functions that we want to spoof.
	//
	
	lpAddressThunk = MakePtr(PIMAGE_THUNK_DATA, hHookModule, lpImportDescriptor->FirstThunk);

  	//
  	// Now we loop through the list of function names, looking for ones that we want to hook.
  	// When we find one, we change the protection on the memeory where it's located and then
  	// update it. Note, we need to do to the protection queries/changes because the OS will get
  	// cranky if we don't, especially XP.
  	//
	while (lpOriginalThunk->u1.Function)
	{
		//
		// check and see if this function entry is an ordinal and if it is, skip it.
		//

		if( (lpOriginalThunk->u1.Ordinal & IMAGE_ORDINAL_FLAG) != IMAGE_ORDINAL_FLAG )
		{

			lpImportName = MakePtr(PIMAGE_IMPORT_BY_NAME, hHookModule, lpOriginalThunk->u1.AddressOfData) ;

			//
			// if the name is empty, then skip it.
			//
			// Note - Name is a run off struct, like our HOOKED_IMAGE.
			//

			if( lpImportName->Name[0] != '\0' )
			{
				//
				// Loop through the list of funcion names that we want
				// to hook and if we find one, update the address table.
				//

				for( dwNameLoop = 0 ; dwNameLoop < (lpStuffToHook->dwHookedFunctionsCount) ; dwNameLoop++ )
				{
					if( !strcmp( ((char *) lpImportName->Name), lpStuffToHook->HookedFunctions[dwNameLoop].szFunctionName) )
					{
						//
						// Ok, we found the function name, let's geek the protection on the memory and
						// patch the address table to point to our function. Once we've changed the table,
						// change the protection for the memory block back to what it was.
						//
						// Note - from past experience, either this will work all the time or none of the time,
						// so if we fail to patch the table, it'll be on the first call and we'll haven't 
						// affected the table as a whole so we shouldn't need to fix up any pointers as
						// a clean up condition upon leaving.
						//
						// TODO - put in an "unpatch" routine here.
						//

						dwMemoryBlockSize = VirtualQuery(	lpAddressThunk, 					// base address of memory to query
															&MemoryBlock, 						// memory descriptor buffer.
															sizeof(MEMORY_BASIC_INFORMATION)	// how big the buffer is.
														);

						dwError = VirtualProtect(	MemoryBlock.BaseAddress, 	// base address of the region to change.
													MemoryBlock.RegionSize, 	// size of region. remember, this is based on pages.
													PAGE_READWRITE, 			// change the protect to this.
													&MemoryBlock.Protect		// what the protection was before the change.
												) ;

						if( !dwError )
						{
							dwError = GetLastError() ;
							return(dwError) ;	// todo - put more error handling here.
						}

						//
						// update the actual address entry with our spoofed address or the real address.
						//
						
						if( bSpoofFunctions )
						{
							lpAddressThunk->u1.Function = reinterpret_cast<DWORD>(lpStuffToHook->HookedFunctions[dwNameLoop].lpNewEntryPoint) ;
						}
						else
						{
							lpAddressThunk->u1.Function = reinterpret_cast<DWORD>(lpStuffToHook->HookedFunctions[dwNameLoop].lpRealEntryPoint) ;
						}

						//
						// put the protection on the memory pages back to what it was. Note - if it worked once, it
						// should work the second time.
						//

						dwError = VirtualProtect(	MemoryBlock.BaseAddress,	// base address of the region to change.
													MemoryBlock.RegionSize, 	// how big the region is.
													MemoryBlock.Protect, 		// change it back to the protection that it was.
													&dwTmp						// this *must* be a valid pointer, or we'll fail.
												) ;
												
						if( !dwError )
						{
							dwError = GetLastError() ;
							return(dwError) ;	// todo - put more error handling here.
						}


						break ;	// no need to keep looping.
						
					}

				}

			}

			

		}

		//
		// increment through the tables.
		//
		
		lpOriginalThunk++ ;
		
		lpAddressThunk++ ;

  	}

  	//
  	// if we're here, then everything went ok.
  	//

  	return(ERROR_SUCCESS) ;

  }





