/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */


#ifndef _INC_HOOK_API_
#define _INC_HOOK_API_


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>

#ifdef __cplusplus
  extern "C" {
#endif

//
// A couple of quick defines to make our life easier.
//


//
// These are used for always being able to access Win32 API
// that we need without getting spoofed.
//

#define NT_KERNEL "kernel32.dll"

typedef FARPROC (WINAPI *GET_PROC_ADDRESS_API_PTR)(HMODULE, LPCSTR) ;

typedef HINSTANCE (WINAPI *LOAD_LIBRARYA_API_PTR)(LPCSTR) ;

typedef BOOL (WINAPI *FREE_LIBRARY_API_PTR)(HINSTANCE) ;



// MakePtr is a macro that allows you to easily add to values (including
// pointers) together without dealing with C's pointer arithmetic.  It
// essentially treats the last two parameters as DWORDs.  The first
// parameter is used to typecast the result to the appropriate pointer type.

#define MakePtr( cast, ptr, offset ) (cast)( (DWORD)(ptr)+(DWORD)(offset))


//
// This struct conatins the list of function that we want to hook, where we
// want to redirect it to and what the original entrypoint was.
//

typedef struct _HookedFunctionInfo
{
  char szFunctionName[MAX_PATH];	// The name of the function to hook.
  LPVOID lpNewEntryPoint;			// The procedure to blast in.
  LPVOID lpRealEntryPoint ;			// The original entry point.
  
} HOOKED_FUNCTION_INFO , *LPHOOKED_FUNCTION_INFO, *PHOOKED_FUNCTION_INFO;

//
// struct to store the name of the dll and the list of hooked functions
// that we need to process. Note, the variable list is at the end
// of the struct so we can run off it.
//

typedef struct _HookedImage
{
	char szImageName[MAX_PATH] ;				// name of the dll or exe to process.

	HMODULE hImageHandle ;						// handle from loadlibrary. keep ref count up.

	DWORD dwHookedFunctionsCount ;				// number of functions in the list

	HOOKED_FUNCTION_INFO HookedFunctions[1] ;	// array of fucntions to hook.

} HOOKED_IMAGE, *LPHOOKED_IMAGE, *PHOOKED_IMAGE ;


//
// Function forwards...
//

BOOL GetRealKernelFunctions( void ) ;

FARPROC GetRealEntryPoint( char *, LPHOOKED_IMAGE ) ;

LPHOOKED_IMAGE CreateHookedImageEntry( char *, LPHOOKED_FUNCTION_INFO ) ;

DWORD HookImage( char *, LPHOOKED_IMAGE, BOOL ) ;

PIMAGE_IMPORT_DESCRIPTOR GetImportTable(HMODULE, char *) ;

DWORD PatchAddressTable( HMODULE, PIMAGE_IMPORT_DESCRIPTOR, LPHOOKED_IMAGE, BOOL ) ;


#ifdef __cplusplus
}
#endif


#endif // of _INC_HOOK_API


