/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

#include "stdafx.h"


#include "hookapi.h"
#include "spoofreg.h"
#include "RegLieManager.h"


#include "assert.h"


//
// Everything that is in this file is used specifically for hooking calls made to the registry. If
// we need to hook another set of functions in a different DLL, copy this file and change the 
// specific infomation to match what is needed.
//




//
// Here is the list of files (images) that we will patch.
//
// All these images need to be in the directory "%windir%\\system32\\"
//
static LPCSTR g_aryImagesToHook[] = {
                                "mmc.exe",      // todo: do we really need to hook this?
// NOT ON WIN2K!!!                                "mmcbase.dll",
                                "mmcndmgr.dll",
                                "mmcshext.dll", // todo: do we really need to hook this?
                            } ;


RegLieManager* g_pLieManager = NULL;

//
// Here is the image and function names that we want to hook for the app
// doing registry calls.
//

static LPHOOKED_IMAGE g_lpHookRegistryEntry     = NULL ;

char g_szRegFunctionImage[]                     =   "advapi32.dll" ;

HOOKED_FUNCTION_INFO g_RegFunctionsToHook[]     =   {
                                                        {"RegOpenKeyA",     SpoofedRegOpenKeyA,     NULL}, 
                                                        {"RegOpenKeyW",     SpoofedRegOpenKeyW,     NULL}, 
                                                        {"RegOpenKeyExA",   SpoofedRegOpenKeyExA,   NULL}, 
                                                        {"RegOpenKeyExW",   SpoofedRegOpenKeyExW,   NULL}, 
                                                        {"RegCreateKeyA",   SpoofedRegCreateKeyA,   NULL}, 
                                                        {"RegCreateKeyW",   SpoofedRegCreateKeyW,   NULL}, 
                                                        {"RegCreateKeyExA", SpoofedRegCreateKeyExA, NULL}, 
                                                        {"RegCreateKeyExW", SpoofedRegCreateKeyExW, NULL}, 
                                                        {"RegCloseKey",     SpoofedRegCloseKey,     NULL}, 

                                                        { "", NULL, NULL } // this should always be the last one.
                                                    } ;





//
// This is the call to make if you want to hook the registy call listed above in a 
// given .exe or .dll image file.
//

DWORD HookImagesRegistryCalls(char *lpImageToHook )
{

    DWORD dwError ;

    //
    // quick check.
    //

    if( !lpImageToHook )
    {
        return(ERROR_INVALID_PARAMETER) ;
    }

    //
    // set up our hooking stuff and patch the IAT with the calls below.
    //
    
    if( !g_lpHookRegistryEntry )
    {
        g_lpHookRegistryEntry= CreateHookedImageEntry( g_szRegFunctionImage, g_RegFunctionsToHook ) ;
    }

    dwError = HookImage( lpImageToHook, g_lpHookRegistryEntry, TRUE ) ;

    return(dwError) ;
}



//
// In the case we're we're done hooking something and we want to patch the IAT table back
// to the original entry points, we'd call this function.
//

DWORD UnHookImagesRegistryCalls(char *lpImageToHook )
{

    DWORD dwError ;

    //
    // quick check.
    //

    if( !lpImageToHook )
    {
        return(ERROR_INVALID_PARAMETER) ;
    }

    //
    // set up our hooking stuff and patch the IAT with the calls below.
    //
    
    if( !g_lpHookRegistryEntry )
    {
        return(ERROR_BAD_CONFIGURATION) ;
    }

    dwError = HookImage( lpImageToHook, g_lpHookRegistryEntry, FALSE ) ;

    return(dwError) ;


}



const bool HOOK_IMAGES   = true;
const bool UNHOOK_IMAGES = false;

// aryImagesToHook  a list of images to hook
//
// cImages          the number of elements in aryImagesToHook
//
// fAddHook         when HOOK_IMAGES, the images in aryImagesToHook will be hooked
//                  when UNHOOK_IMAGES, the previously-added hook will be removed
//
DWORD HookOrUnhookAllImages(const LPCSTR aryImagesToHook[],
                            UINT cImages,
                            bool fAddHook)
{
    char szSystem32Path[MAX_PATH] ;
    const int cchSystem32Path = ::ExpandEnvironmentStrings("%windir%\\system32\\",
                                                           szSystem32Path,
                                                           MAX_PATH);
    if (!cchSystem32Path || (cchSystem32Path > MAX_PATH))
    {
        return(ERROR_INVALID_PARAMETER) ;
    }

    //
    // setup hooking for all of the components that make up MMC.
    //
    for (UINT i = 0; i < cImages; ++i)
    {
        LPCSTR strImageToHook = aryImagesToHook[i];

        // build a full path to each image by prepending
        // the System32Path to the file name.
        //
        // This is all done in a single buffer -- each time
        // through the new filename is written over top of
        // the old one.
        //
        // note: cchSystem32Path is the length of %windir%\system32\,
        // including a null terminator.
        //
        strcpy(szSystem32Path + cchSystem32Path - 1, strImageToHook);

        DWORD dwError;
        if (HOOK_IMAGES == fAddHook)
        {
            dwError = HookImagesRegistryCalls( szSystem32Path );
        }
        else
        {
            dwError = UnHookImagesRegistryCalls( szSystem32Path );
        }

        if( dwError != ERROR_SUCCESS )
        {
            ATLTRACE("Could not hook image %s.  dwError = %d", szSystem32Path, dwError);
            return(dwError) ;
        }
        else
        {
            ATLTRACE("Successfully hooked %s", szSystem32Path);
        }
    }

    return(ERROR_SUCCESS) ;
}

DWORD HookMmcRegistryCalls()
{
    // init global map table that we'll use
    // TODO: use some synchronization on this
    //
    if (!g_pLieManager)
    {
        g_pLieManager = new RegLieManager();

        g_pLieManager->PopulateFakeNodeType();
    }

    return HookOrUnhookAllImages(g_aryImagesToHook,
                                 sizeof(g_aryImagesToHook)/sizeof(g_aryImagesToHook[0]),
                                 HOOK_IMAGES);
}

DWORD UnhookMmcRegistryCalls()
{
    DWORD dwResult = HookOrUnhookAllImages(g_aryImagesToHook,
                                           sizeof(g_aryImagesToHook)/sizeof(g_aryImagesToHook[0]),
                                           UNHOOK_IMAGES);

    if (g_pLieManager)
    {
        delete g_pLieManager;
        g_pLieManager = NULL;
    }

    return dwResult;
}

HRESULT EnablePermissiveRegistryKeys()
{
    ATLASSERT(g_pLieManager);
    if (g_pLieManager)
    {
        g_pLieManager->PopulateFakeNodeType();

        return g_pLieManager->EnablePermissiveRegistryKeys();
    }
    return E_UNEXPECTED;
}

HRESULT CleanupCenterisNodeTypes()
{
    ATLASSERT(g_pLieManager);
    if (g_pLieManager)
    {
        return g_pLieManager->CleanupCenterisNodeTypes();
    }
    return E_UNEXPECTED;
}

//
// For all of our spoofed functions, the first thing that we'll do is find
// the real entry point of the function that we'll be spoofing.
// 


LONG APIENTRY SpoofedRegOpenKeyA( HKEY hKey, LPCSTR lpSubKey, PHKEY phkResult )  
{
    static REG_OPEN_KEYA_API_PTR lpRealRegOpenKeyA = NULL ;

    if( !lpRealRegOpenKeyA )
    {
         lpRealRegOpenKeyA = (REG_OPEN_KEYA_API_PTR) GetRealEntryPoint( "RegOpenKeyA" , g_lpHookRegistryEntry ) ;

         assert( lpRealRegOpenKeyA ) ;
    }

    //
    // Add the code that does something useful here.
    //

    ATLTRACE("\nHello From SpoofedRegOpenKeyA\n") ;

    return( lpRealRegOpenKeyA(hKey, lpSubKey, phkResult) ) ;
}



LONG APIENTRY SpoofedRegOpenKeyW(HKEY hKey,LPCWSTR lpSubKey,PHKEY phkResult)  
{
    static REG_OPEN_KEYW_API_PTR lpRealRegOpenKeyW = NULL ;

    if( !lpRealRegOpenKeyW )
    {
        lpRealRegOpenKeyW = (REG_OPEN_KEYW_API_PTR) GetRealEntryPoint( "RegOpenKeyW", g_lpHookRegistryEntry ) ;

        assert( lpRealRegOpenKeyW ) ;
    }

    //
    // Add the code that does something useful here.
    //
    
    ATLTRACE("\nHello From SpoofedRegOpenKeyW\n") ;

    return( lpRealRegOpenKeyW(hKey, lpSubKey, phkResult) ) ;
}



LONG APIENTRY SpoofedRegOpenKeyExA(HKEY hKey,LPCSTR lpSubKey,DWORD ulOptions,REGSAM samDesired,PHKEY phkResult)  
{
    static REG_OPEN_KEY_EXA_API_PTR lpRealRegOpenKeyExA = NULL ;

    if( !lpRealRegOpenKeyExA )
    {
        lpRealRegOpenKeyExA = (REG_OPEN_KEY_EXA_API_PTR) GetRealEntryPoint( "RegOpenKeyExA", g_lpHookRegistryEntry ) ;

        assert( lpRealRegOpenKeyExA ) ;
    }

    //
    // Add the code that does something useful here.
    //
    
    ATLTRACE("\nHello From SpoofedRegOpenKeyExA\n") ;

    return( lpRealRegOpenKeyExA(hKey, lpSubKey, ulOptions, samDesired, phkResult) ) ;
}





LONG APIENTRY SpoofedRegOpenKeyExW(HKEY hKey,LPCWSTR lpSubKey,DWORD ulOptions,REGSAM samDesired,PHKEY phkResult)  
{
    static REG_OPEN_KEY_EXW_API_PTR lpRealRegOpenKeyExW = NULL ;

    if( !lpRealRegOpenKeyExW )
    {
        lpRealRegOpenKeyExW = (REG_OPEN_KEY_EXW_API_PTR) GetRealEntryPoint( "RegOpenKeyExW", g_lpHookRegistryEntry ) ;

        assert( lpRealRegOpenKeyExW ) ;
    }

    ATLTRACE("\nHello From SpoofedRegOpenKeyExW\n") ;

    if (g_pLieManager)
    {
        HKEY hkLieResult;
        LONG lLieReturnValue;

        BOOL fLie = g_pLieManager->HandleRegOpenExW(hKey,
                                                    lpSubKey,
                                                    ulOptions,
                                                    samDesired,
                                                    lpRealRegOpenKeyExW,
                                                    &hkLieResult,
                                                    &lLieReturnValue);

        if (fLie)
        {
            // the lie manager has returned a false key for us.  Use that instead
            // of calling the real function
            //
            *phkResult = hkLieResult;
            return lLieReturnValue;
        }
    }

    // pass through the call to the real function
    //
    return( lpRealRegOpenKeyExW(hKey, lpSubKey, ulOptions, samDesired, phkResult) ) ;
}



LONG APIENTRY SpoofedRegCreateKeyA(HKEY hKey, LPCSTR lpSubKey, PHKEY phkResult)  
{
    static REG_CREATE_KEYA_API_PTR lpRealRegCreateKeyA = NULL ;

    if( !lpRealRegCreateKeyA )
    {
        lpRealRegCreateKeyA = (REG_CREATE_KEYA_API_PTR) GetRealEntryPoint( "RegCreateKeyA", g_lpHookRegistryEntry ) ;

        assert( lpRealRegCreateKeyA ) ;
    }

    //
    // Add the code that does something useful here.
    //
    
    ATLTRACE("\nHello From SpoofedRegCreateKeyA\n") ;

    return( lpRealRegCreateKeyA(hKey, lpSubKey, phkResult)  ) ;

}



LONG APIENTRY SpoofedRegCreateKeyW(HKEY hKey, LPCWSTR lpSubKey, PHKEY phkResult)  
{
    static REG_CREATE_KEYW_API_PTR lpRealRegCreateKeyW = NULL ;

    if( !lpRealRegCreateKeyW )
    {
        lpRealRegCreateKeyW = (REG_CREATE_KEYW_API_PTR) GetRealEntryPoint( "RegCreateKeyW", g_lpHookRegistryEntry ) ;

        assert( lpRealRegCreateKeyW ) ;
    }

    //
    // Add the code that does something useful here.
    //
    
    ATLTRACE("\nHello From SpoofedRegCreateKeyW\n") ;

    return( lpRealRegCreateKeyW(hKey, lpSubKey, phkResult) ) ;

}



LONG APIENTRY SpoofedRegCreateKeyExA(HKEY hKey,LPCSTR lpSubKey,DWORD Reserved,LPSTR lpClass,DWORD dwOptions,REGSAM samDesired,LPSECURITY_ATTRIBUTES lpSecurityAttributes,PHKEY phkResult,LPDWORD lpdwDisposition)  
{
    static REG_CREATE_KEY_EXA_API_PTR lpRealRegCreateKeyExA = NULL ;

    if( !lpRealRegCreateKeyExA )
    {
        lpRealRegCreateKeyExA = (REG_CREATE_KEY_EXA_API_PTR) GetRealEntryPoint( "RegCreateKeyExA", g_lpHookRegistryEntry ) ;

        assert( lpRealRegCreateKeyExA ) ;
    }

    //
    // Add the code that does something useful here.
    //
    
    ATLTRACE("\nHello From SpoofedRegCreateKeyExA\n") ;

    return( lpRealRegCreateKeyExA(hKey, lpSubKey, Reserved, lpClass, dwOptions, samDesired, lpSecurityAttributes, phkResult, lpdwDisposition) ) ;

}



LONG APIENTRY SpoofedRegCreateKeyExW(HKEY hKey,LPCWSTR lpSubKey,DWORD Reserved,LPWSTR lpClass,DWORD dwOptions,REGSAM samDesired,LPSECURITY_ATTRIBUTES lpSecurityAttributes,PHKEY phkResult,LPDWORD lpdwDisposition)  
{
    static REG_CREATE_KEY_EXW_API_PTR lpRealRegCreateKeyExW = NULL ;

    if( !lpRealRegCreateKeyExW )
    {
        lpRealRegCreateKeyExW = (REG_CREATE_KEY_EXW_API_PTR) GetRealEntryPoint( "RegCreateKeyExW", g_lpHookRegistryEntry ) ;

        assert( lpRealRegCreateKeyExW ) ;
    }

    //
    // Add the code that does something useful here.
    //
    
    ATLTRACE("\nHello From SpoofedRegCreateKeyExW\n") ;

    return( lpRealRegCreateKeyExW( hKey, lpSubKey, Reserved, lpClass, dwOptions, samDesired, lpSecurityAttributes, phkResult, lpdwDisposition) ) ;

}



LONG APIENTRY SpoofedRegCloseKey(HKEY hKey)  
{
    static REG_CLOSE_KEY_API_PTR lpRealRegCloseKey = NULL ;

    if( !lpRealRegCloseKey )
    {
        lpRealRegCloseKey = (REG_CLOSE_KEY_API_PTR) GetRealEntryPoint( "RegCloseKey", g_lpHookRegistryEntry ) ;

        assert( lpRealRegCloseKey ) ;
    }

    if (g_pLieManager)
    {
        // it doesn't matter if the key was removed or not
        //
        g_pLieManager->HandleRegClose(hKey);
    }

    ATLTRACE("\nHello From SpoofedRegCloseKey\n") ;

    return( lpRealRegCloseKey(hKey) ) ;

}




