/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

// hackMmcTest.cpp : Implementation of DLL Exports.

#include "stdafx.h"
#include "resource.h"
#include "hackMmcTest.h"
#include "dlldatax.h"
#include "legacyMMC.h"
#include "SpoofReg.h"


ChackMmcTestModule _AtlModule;
HMODULE g_hDllInner = NULL;


// DLL Entry Point
extern "C" BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
    switch (dwReason)
    {
    case DLL_PROCESS_ATTACH:
        // hook the registry calls of all the MMC images in this process
        //
        // note: if this fails, our binary won't work very well for its intended
        //       purpose, as we won't be able to create custom nodeTypes for our
        //       own Centeris machines.  In this failure mode it's more polite for
        //       us to continue running, though, so that the user can use the
        //       plain MMC interface, though without our enhancements.
        //
        HookMmcRegistryCalls();
        break;

    case DLL_PROCESS_DETACH:
        // our dll is being unloaded; remove the registry hook
        //
        UnhookMmcRegistryCalls();
        break;
    }

#ifdef _MERGE_PROXYSTUB
    if (!PrxDllMain(hInstance, dwReason, lpReserved))
        return FALSE;
#endif

    hInstance;
    return _AtlModule.DllMain(dwReason, lpReserved); 
}


// Used to determine whether the DLL can be unloaded by OLE
STDAPI DllCanUnloadNow(void)
{
#ifdef _MERGE_PROXYSTUB
    HRESULT hr = PrxDllCanUnloadNow();
    if (FAILED(hr))
        return hr;
#endif
    return _AtlModule.DllCanUnloadNow();
}


// Returns a class factory to create an object of the requested type
STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID* ppv)
{
#ifdef _MERGE_PROXYSTUB
    if (PrxDllGetClassObject(rclsid, riid, ppv) == S_OK)
        return S_OK;
#endif
    return _AtlModule.DllGetClassObject(rclsid, riid, ppv);
}


// DllRegisterServer - Adds entries to the system registry
STDAPI DllRegisterServer(void)
{
    // registers object, typelib and all interfaces in typelib
    HRESULT hr = _AtlModule.DllRegisterServer(FALSE);
#ifdef _MERGE_PROXYSTUB
    if (FAILED(hr))
        return hr;
    hr = PrxDllRegisterServer();
#endif

    if (SUCCEEDED(hr)) {
        // emulate the Computer Manager class
        //
        hr = ::CoTreatAsClass(_uuidof(STOLENCLASS),
                              _uuidof(CenterisRedirect));
    }

    if (SUCCEEDED(hr)) {
        // add MMC node entries into the registry
        //
        hr = _AtlModule.UpdateRegistryFromResourceS(IDR_NODETYPEMAGIC, true);
    }

    // allow anyone to write to our magic registry keys
    // this also populates our fancy registry keys, so they're all ready to go
    // just in case.
    if (SUCCEEDED(hr)) {
        hr = EnablePermissiveRegistryKeys();
    }

    return hr;
}


// DllUnregisterServer - Removes entries from the system registry
STDAPI DllUnregisterServer(void)
{
    HRESULT hr = _AtlModule.DllUnregisterServer(FALSE);
#ifdef _MERGE_PROXYSTUB
    if (FAILED(hr))
        return hr;
    hr = PrxDllRegisterServer();
    if (FAILED(hr))
        return hr;
    hr = PrxDllUnregisterServer();
#endif

    // remove the emulation of the Computer Manager class.  Do this even
    // if other things fail.
    //
    if (SUCCEEDED(hr)) {
        hr = ::CoTreatAsClass(_uuidof(STOLENCLASS),
                              CLSID_NULL);
    }


    // remove subkeys from the crossover nodetype, so the
    // IDR_NODETYPEMAGIC file can delete the parent keys.
    //
    if (SUCCEEDED(hr)) {
        hr = CleanupCenterisNodeTypes();
    }

    if (SUCCEEDED(hr)) {
        // remove MMC node entries from the registry
        //
        hr = _AtlModule.UpdateRegistryFromResourceS(IDR_NODETYPEMAGIC, false);
    }

    return hr;
}



