/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

/*
 * Note: This code was originally based on sample code provided
 * by Microsoft.
 */

#include <winsock2.h>
#include "PPgeExt.h"
#include "resource.h"
#include "globals.h"
#include "bitmaputil.h"
#include <crtdbg.h>


// max chars allowed in the description field
#define cchDescriptionMax 40

// we need to do this to get around MMC.IDL - it explicitly defines
// the clipboard formats as WCHAR types...
#define _T_CCF_DISPLAY_NAME _T("CCF_DISPLAY_NAME")
#define _T_CCF_NODETYPE _T("CCF_NODETYPE")
#define _T_CCF_SNAPIN_CLASSID _T("CCF_SNAPIN_CLASSID")
#define _T_CCF_MMC_SNAPIN_MACHINE_NAME _T("MMC_SNAPIN_MACHINE_NAME")

    // These are the clipboard formats that we must supply at a minimum.
    // mmc.h actually defined these. We can make up our own to use for
    // other reasons. We don't need any others at this time.
UINT CPropSheetExtension::s_cfDisplayName = RegisterClipboardFormat(_T_CCF_DISPLAY_NAME);
UINT CPropSheetExtension::s_cfNodeType    = RegisterClipboardFormat(_T_CCF_NODETYPE);
UINT CPropSheetExtension::s_cfSnapInCLSID = RegisterClipboardFormat(_T_CCF_SNAPIN_CLASSID);
UINT CPropSheetExtension::s_cfMachineName = RegisterClipboardFormat(_T_CCF_MMC_SNAPIN_MACHINE_NAME);

CPropSheetExtension::CPropSheetExtension() : m_cref(0)
{
    m_pSMUtil = NULL;
    // note: we don't need to call CoInitialize, since we're in a COM object
    //       and COM is presuamably already running...
    //
    OBJECT_CREATED

    SMUtil::ISystemManagerUtil * pSMUtil = NULL;
	REFCLSID clsid = __uuidof(SMUtil::SMUtil);
    HRESULT hr = ::CoCreateInstance(clsid,
                                    NULL,
                                    CLSCTX_INPROC_SERVER,
									__uuidof(SMUtil::ISystemManagerUtil),
                                    (LPVOID*)&pSMUtil);
    if (SUCCEEDED(hr)) {
        m_pSMUtil = pSMUtil;
    }

}

CPropSheetExtension::~CPropSheetExtension()
{
    if (m_pSMUtil)
    {
        m_pSMUtil->Release();
        m_pSMUtil = NULL;
    }

    OBJECT_DESTROYED
}

///////////////////////
// IUnknown implementation
///////////////////////

STDMETHODIMP CPropSheetExtension::QueryInterface(REFIID riid, LPVOID *ppv)
{
    if (!ppv)
        return E_FAIL;
    
    *ppv = NULL;
    
    if (IsEqualIID(riid, IID_IUnknown))
        *ppv = static_cast<IExtendPropertySheet *>(this);
    else if (IsEqualIID(riid, IID_IExtendPropertySheet))
        *ppv = static_cast<IExtendPropertySheet *>(this);
    
    if (*ppv) 
    {
        reinterpret_cast<IUnknown *>(*ppv)->AddRef();
        return S_OK;
    }
    
    return E_NOINTERFACE;
}

STDMETHODIMP_(ULONG) CPropSheetExtension::AddRef()
{
    return InterlockedIncrement((LONG *)&m_cref);
}

STDMETHODIMP_(ULONG) CPropSheetExtension::Release()
{
    if (InterlockedDecrement((LONG *)&m_cref) == 0)
    {
        // we need to decrement our object count in the DLL
        delete this;
        return 0;
    }
    
    return m_cref;
}


HBITMAP LoadTransparentLogoBitmap(HWND hwndDlg, int idcControl, int idbBitmap)
{
    HBITMAP hbmMasked;
    DWORD clrDialogBackground = ::GetSysColor(COLOR_BTNFACE);

    HBITMAP hBmpOriginalLogo = LoadBitmap(g_hinst,
                                          MAKEINTRESOURCE(idbBitmap));
    hbmMasked = ReplaceColor(hBmpOriginalLogo,
                                RGB(255, 0, 255),
                                clrDialogBackground,
                                NULL); // replace blue by green

    DeleteObject(hBmpOriginalLogo);

    LRESULT lResult = SendDlgItemMessage(hwndDlg,  // dialog box window handle 
        idcControl,				// icon identifier 
        STM_SETIMAGE,           // message to send 
        (WPARAM) IMAGE_BITMAP,  // image type 
        (LPARAM) hbmMasked);    // icon handle 

    return hbmMasked;
}


BOOL CALLBACK CPropSheetExtension::DialogProc(
                                              HWND hwndDlg,  // handle to dialog box
                                              UINT uMsg,     // message
                                              WPARAM wParam, // first message parameter
                                              LPARAM lParam  // second message parameter
                                              )
{
    static CPropSheetExtension *pThis = NULL;
    static HGDIOBJ s_hGdiHollowBrush = INVALID_HANDLE_VALUE;
    static HBITMAP s_hbmMasked = 0;
	static HBITMAP s_hbmMasked2 = 0;
    
    switch (uMsg) {
    case WM_INITDIALOG:
        pThis = reinterpret_cast<CPropSheetExtension *>(reinterpret_cast<PROPSHEETPAGE *>(lParam)->lParam);

        // a hollow brush will let us see the XP-themed
        // gradient on the background.
        //
        // note: it is not necessary to free this handle
        //
        s_hGdiHollowBrush = ::GetStockObject(HOLLOW_BRUSH);

        // place our bitmaps logo on our static picture control
        s_hbmMasked = LoadTransparentLogoBitmap(hwndDlg, IDS_COMPUTER_PICTURE, IDB_CENTERIS_LOGO);
        s_hbmMasked2 = LoadTransparentLogoBitmap(hwndDlg, IDS_WIDELOGO, IDB_BITMAPLOGO);

		// fetch the machine information

        // init dialog from saved and web service info
        {
			BSTR bstrHostname = ::SysAllocString(pThis->m_pszComputerName);
			if (bstrHostname!=NULL)
			{
				SAFEARRAY *pInfo=NULL;
				HRESULT hr = pThis->m_pSMUtil->centerisSystemInfo(bstrHostname, &pInfo);
				if (SUCCEEDED(hr) && ::SafeArrayGetDim(pInfo)==1)
				{
					LONG cElements=0;
					::SafeArrayGetUBound(pInfo, 1, &cElements);
					if (cElements>=11)
					{
						for (LONG i = 0; i < (4 + 4 + 5); ++i)
						{
							BSTR bstrData=NULL;
							if (i < cElements)
							{
								::SafeArrayGetElement(pInfo, &i, &bstrData);
								if (bstrData!=NULL)
								{
									::SetDlgItemTextW(hwndDlg, 52 + i, bstrData);
									::SysFreeString(bstrData);
								}
							}

						}
					}

					// free the data
					::SafeArrayDestroy(pInfo);
				}
				else
				{
					// show error label, hide others
					HWND hwnd;
					hwnd = ::GetDlgItem(hwndDlg, IDC_LABEL_SYSTEM);
					::ShowWindow(hwnd, SW_HIDE);
					hwnd = ::GetDlgItem(hwndDlg, IDC_LABEL_REGISTERED);
					::ShowWindow(hwnd, SW_HIDE);
					hwnd = ::GetDlgItem(hwndDlg, IDC_LABEL_COMPUTER);
					::ShowWindow(hwnd, SW_HIDE);
					hwnd = ::GetDlgItem(hwndDlg, IDC_LABEL_ERROR);
					::ShowWindow(hwnd, SW_NORMAL);
				}

				// cleanup
				::SysFreeString(bstrHostname);
			}

        }
        break;

    case WM_NCDESTROY:
        DeleteObject(s_hbmMasked);
        DeleteObject(s_hbmMasked2);
        s_hbmMasked = 0;
        s_hbmMasked2 = 0;

        // note: we don't need to free s_hGdiHollowBrush
        break;

    case WM_SYSCOLORCHANGE:
        {
            // the system color scheme has changed, so we need to
            // recompute our logo bitmap (which we had earlier modified
            // by replacing purple with the dialog background color)
            //

            // bugbug: is there a problem deleting this when we
            //         haven't yet removed it from the picture control?
            //
			DeleteObject(s_hbmMasked);
			DeleteObject(s_hbmMasked2);
			s_hbmMasked = 0;
			s_hbmMasked2 = 0;

        s_hbmMasked = LoadTransparentLogoBitmap(hwndDlg, IDS_COMPUTER_PICTURE, IDB_CENTERIS_LOGO);
        s_hbmMasked2 = LoadTransparentLogoBitmap(hwndDlg, IDS_WIDELOGO, IDB_BITMAPLOGO);
       }
        break;

    case WM_CTLCOLORMSGBOX:
    case WM_CTLCOLORSTATIC:
        {
            // note: checkboxes don't work properly,
            //       but I don't really care since our dialog
            //       won't have any...
            ///

            // Set the background mode to transparent
            HDC hDC = (HDC)wParam;
            ::SetBkMode(hDC, TRANSPARENT);

            // we want to use the hollow brush for our picture
            // controls, but for nothing else
            //
            HWND hWndControl = (HWND)lParam;
            int controlID = ::GetDlgCtrlID(hWndControl);
            if (IDS_COMPUTER_PICTURE == controlID || IDS_WIDELOGO == controlID)
            {
                INT_PTR pResult = (INT_PTR)s_hGdiHollowBrush;
                return pResult;
            }
        }
        break;

    case WM_CTLCOLORDLG:
        {
            // yes, MSDN really says to do this...
            INT_PTR pResult = (INT_PTR)s_hGdiHollowBrush;
            return pResult;
        }
        break;


    case WM_COMMAND:
        if (HIWORD(wParam) == EN_CHANGE ||
            HIWORD(wParam) == CBN_SELCHANGE)
        {
            SendMessage(GetParent(hwndDlg), PSM_CHANGED, (WPARAM)hwndDlg, 0);
        }
        else if (HIWORD(wParam) == BN_CLICKED)
        {
            switch (LOWORD(wParam)) {
                case IDC_BUTTON1:
                    {
                        BSTR bstrHostname = ::SysAllocString(pThis->m_pszComputerName);
                        if (bstrHostname)
                        {
                            LONG lDialogResult = 0;
                            HRESULT hr = pThis->m_pSMUtil->ShowShutdownDialog((long)hwndDlg, bstrHostname, &lDialogResult);

                            ::SysFreeString(bstrHostname);
                            bstrHostname = NULL;
                        }
                    }
                    break;
            }
        }

        break;
        
    case WM_DESTROY:
        // we don't free the notify handle for property sheets
        // MMCFreeNotifyHandle(pThis->m_ppHandle);
        break;
        
    case WM_NOTIFY:
        switch (((NMHDR *) lParam)->code) {
        case PSN_APPLY:
            // don't notify the primary snap-in that Apply
            // has been hit...
            // MMCPropertyChangeNotify(pThis->m_ppHandle, (long)pThis);
            return PSNRET_NOERROR;
        }
        break;
    }
    
    return FALSE;
}

BOOL CALLBACK CPropSheetExtension::NameDialogProc(
                                              HWND hwndDlg,  // handle to dialog box
                                              UINT uMsg,     // message
                                              WPARAM wParam, // first message parameter
                                              LPARAM lParam  // second message parameter
                                              )
{
    static CPropSheetExtension *pThis = NULL;

    switch (uMsg) {
    case WM_INITDIALOG:
        pThis = reinterpret_cast<CPropSheetExtension *>(reinterpret_cast<PROPSHEETPAGE *>(lParam)->lParam);

		// limit the size of the description field
		::SendDlgItemMessage(hwndDlg, IDC_DESCRIPTION, EM_SETLIMITTEXT, cchDescriptionMax, 0);

        // init dialog from saved and web service info
        if (pThis->m_pSMUtil!=NULL) 
		{
			BSTR bstrHostname = ::SysAllocString(pThis->m_pszComputerName);

			if (bstrHostname!=NULL)
			{
				BSTR bstrDomain=NULL;
				HRESULT hr = pThis->m_pSMUtil->GetConfiguredDomain(bstrHostname, &bstrDomain);

				if (SUCCEEDED(hr) && bstrDomain!=NULL)
				{
					::SetDlgItemText(hwndDlg, IDC_DOMAIN, bstrDomain);

					// compose the FQDN since m_pszComputerName is not FQ
					WCHAR wszFQDN[MAX_PATH];
					wcscpy(wszFQDN, pThis->m_pszComputerName);
					wcscat(wszFQDN, L".");
					wcscat(wszFQDN, bstrDomain);
					::SetDlgItemTextW(hwndDlg, IDC_FQDN, wszFQDN);

					// free the returned string
					::SysFreeString(bstrDomain);
				}
				else
				{
					::SetDlgItemText(hwndDlg, IDC_DOMAIN, L"");
					::SetDlgItemText(hwndDlg, IDC_FQDN, L"");
				}

				// get the description
				BSTR bstrDescription=NULL;
				hr = pThis->m_pSMUtil->GetConfiguredDescription(bstrHostname, &bstrDescription);
				if (SUCCEEDED(hr) && bstrDescription!=NULL)
				{
					::SetDlgItemText(hwndDlg, IDC_DESCRIPTION, bstrDescription);
					::SysFreeString(bstrDescription);
				}
				else
					::SetDlgItemText(hwndDlg, IDC_DESCRIPTION, L"");

				::SysFreeString(bstrHostname);
			}
        }
        break;

    case WM_COMMAND:
        if (HIWORD(wParam) == EN_CHANGE ||
            HIWORD(wParam) == CBN_SELCHANGE)
        {
            SendMessage(GetParent(hwndDlg), PSM_CHANGED, (WPARAM)hwndDlg, 0);
        }
        else if (HIWORD(wParam) == BN_CLICKED)
        {
        }

        break;
        
    case WM_DESTROY:
        // we don't free the notify handle for property sheets
        // MMCFreeNotifyHandle(pThis->m_ppHandle);
        break;
        
    case WM_NOTIFY:
        switch (((NMHDR *) lParam)->code) {
        case PSN_APPLY:
            // don't notify the primary snap-in that Apply
            // has been hit...
            // MMCPropertyChangeNotify(pThis->m_ppHandle, (long)pThis);

            return PSNRET_NOERROR;
        }
        break;
    }
    
    return FALSE;
}

///////////////////////////////
// Interface IExtendPropertySheet
///////////////////////////////
HRESULT CPropSheetExtension::CreatePropertyPages( 
                                                 /* [in] */ LPPROPERTYSHEETCALLBACK lpProvider,
                                                 /* [in] */ LONG_PTR handle,
                                                 /* [in] */ LPDATAOBJECT lpIDataObject)
{
    PROPSHEETPAGE psp;
    HPROPSHEETPAGE hPage = NULL;

    // Get the hostname of the computer we're connected to.
    // If we're managing the local machine, this will be the
    // empty string, instead of an actual hostname.
    //
    HRESULT hr = ExtractString(lpIDataObject,
                                s_cfMachineName,
                                m_pszComputerName,
                                sizeof(m_pszComputerName));
    if (FAILED(hr)) {
        return hr;
    }

    // we don't cache this handle like in a primary snap-in
    // the handle value here is always 0
    // m_ppHandle = handle;

    psp.dwSize = sizeof(psp);
    psp.dwFlags = PSP_DEFAULT;
    psp.hInstance = g_hinst;
    psp.pszTemplate = MAKEINTRESOURCE(IDD_PROPGENERAL);
	psp.pfnDlgProc = DialogProc;
    psp.lParam = reinterpret_cast<LPARAM>(this);

    // this line should be a no-op, but it should make sure that
    // we are using the proper version of PROPSHEETPAGE.
    // If this doesn't work, ensure that ISOLATION_AWARE_ENABLED
    // is defined.
    //
    psp.hActCtx = 0;
    
    hPage = CreatePropertySheetPage(&psp);
    _ASSERT(hPage);

    hr = lpProvider->AddPage(hPage);

	psp.pszTemplate = MAKEINTRESOURCE(IDD_PROPNAME);
	psp.pfnDlgProc = NameDialogProc;
    hPage = CreatePropertySheetPage(&psp);
    _ASSERT(hPage);

    hr = lpProvider->AddPage(hPage);
    return hr;
}

HRESULT CPropSheetExtension::QueryPagesFor( 
                                           /* [in] */ LPDATAOBJECT lpDataObject)
{
    return S_OK;
}
